/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Carte.h"

Carte::Carte()
{
    int i,j;

    setDecalageFenetreX(0);
    setDecalageFenetreY(0);

    // Initialiser le typeBlocTraversable de refTilset (configuration en défaut : tile non traversable) et le type sol
	refTileset = new position*[62];
	for(i=0;i<62;i++)
	{
	    refTileset[i] = new position[62];
		for(j=0;j<62;j++)
		{
	        refTileset[i][j].x = i;
	        refTileset[i][j].y = j;
	        refTileset[i][j].typeBlocTraversable = 15;
	        refTileset[i][j].typeSol = 0;
		}
	}

    // Initialisation des différentes tables (taille 120 tiles * 60 tiles max)
	sol = new position*[121];
	table = new position*[121];
	air = new position*[121];

	for(i=0;i<121;i++)
	{
	    sol[i] = new position[61];
	    table[i] = new position[61];
	    air[i] = new position[61];
	}

    // Initialisation des SDL_Surface utilisées par la carte
    initSDLSurface(1); // PNJ
    initSDLSurface(4); // Objets

    initSDLRect(1);
    initSDLRect(4);

    // Initialisations diverses
    setNombreBlocSurX(20);
    setNombreBlocSurY(15);
    setInterRouge(false);

    // Init des sons de la carte
	sonsCarte[0] = Mix_LoadWAV("sons/item.ogg"); // Chargement du son du joueur qui obtient un item

}

Carte::~Carte()
{
    int i;

	for(i=0;i<120;i++)
	{
	    delete sol[i];
	    delete table[i];
	    delete air[i];
	}
	delete sol;
	delete table;
	delete air;

	for(i=0;i<62;i++)
	{
        delete refTileset[i];
	}
	delete refTileset;

    freeMem(1);
    freeMem(4);

    // Libération des sons du joueur
    Mix_FreeChunk(sonsCarte[0]);
}

void Carte::setNumero(int N) {numero = N;}
int Carte::getNumero() {return numero;}

void Carte::setDimensionX(int X) {dimensionX = X;}
int Carte::getDimensionX() {return dimensionX;}
void Carte::setDimensionY(int Y) {dimensionY = Y;}
int Carte::getDimensionY() {return dimensionY;}

void Carte::setNombreBlocSurX(int X) {nombreBlocSurX = X;}
int Carte::getNombreBlocSurX() {return nombreBlocSurX;}
void Carte::setNombreBlocSurY(int Y) {nombreBlocSurY = Y;}
int Carte::getNombreBlocSurY() {return nombreBlocSurY;}

void Carte::setDecalageFenetreX(int DFX) {decalageFenetreX = DFX;}
int Carte::getDecalageFenetreX() {return decalageFenetreX;}
void Carte::setDecalageFenetreY(int DFY) {decalageFenetreY = DFY;}
int Carte::getDecalageFenetreY() {return decalageFenetreY;}

void Carte::setTableauSol(position** TS) {sol = TS;}
position** Carte::getTableauSol() {return sol;}
void Carte::afficherTableauSol(SDL_Surface* screenCarte) {} // A compléter

void Carte::setTableauTable(position** TT) {table = TT;}
position** Carte::getTableauTable() {return table;}
void Carte::afficherTableauTable(SDL_Surface* screenCarte) {} // A compléter

void Carte::setTableauAir(position** TA) {air = TA;}
position** Carte::getTableauAir() {return air;}
void Carte::afficherTableauAir(SDL_Surface* screenCarte) {} // A compléter

void Carte::setRefTileset(position** RF) {refTileset = RF;}
position** Carte::getRefTileset() {return refTileset;}

void Carte::setPnjEnnemi(list<Ennemi> E) {pnjEnnemi = E;}
list<Ennemi> Carte::getPnjEnnemi() {return pnjEnnemi;}

void Carte::setPnjAmi(list<Ami> A) {pnjAmi = A;}
list<Ami> Carte::getPnjAmi() {return pnjAmi;}

void Carte::setPnjProjectile(list<Projectile> P) {pnjProjectile = P;}
list<Projectile> Carte::getPnjProjectile() {return pnjProjectile;}

void Carte::setPnjObjets(list<Objet> O) {pnjObj = O;}
list<Objet> Carte::getPnjObjets() {return pnjObj;}

void Carte::setObjetRecoltables(list<ObjetRecolte> OR) {objRecoltes = OR;}
list<ObjetRecolte> Carte::getObjetRecoltables() {return objRecoltes;}

void Carte::setLienCartes(list<lienCarte> LC) {liaisonsUnidirectionnelles = LC;}
list<lienCarte> Carte::getLienCartes() {return liaisonsUnidirectionnelles;}

void Carte::selectionCarte(int numCarte)
{
    setNumero(numCarte);

    // IMPORTANT : a chaque changement de carte, initialiser les PNJ et tout ce qui doit l'être sur la nouvelle carte
    initCarte();
    chargerCarte();
    chargerModifsCarte();

}

void Carte::afficherCarte(SDL_Surface* screenCarte,SDL_Surface* tilesetA,SDL_Surface* tilesetB,SDL_Surface* tilesetC,SDL_Surface* tilesetD,position** Table)
{
    // Cette fonction est appelée pour chaque tableau (sol, table, air)

	int i,j;
	SDL_Rect Rect_source;
	SDL_Rect Rect_dest;

	Rect_source.w = LARGEUR_TILE;
	Rect_source.h = HAUTEUR_TILE;
	Rect_dest.w = LARGEUR_TILE;
	Rect_dest.h = HAUTEUR_TILE;

	for(i = getDecalageFenetreX()/16;i<(getDecalageFenetreX()+LARGEUR_ECRAN+15)/16;i++)
	{
		for(j = getDecalageFenetreY()/16;j<(getDecalageFenetreY()+HAUTEUR_ECRAN+15)/16;j++)
		{
            if(Table[i][j].x !=39 || Table[i][j].y != 61) // La tile exclue est celle totalement transparente du tileset de référence (couleur #0000ff)
			{

                Rect_source.x = ((Table[i][j].x)*LARGEUR_TILE);
                Rect_dest.x = i*LARGEUR_TILE- getDecalageFenetreX();
                Rect_dest.y = j*HAUTEUR_TILE- getDecalageFenetreY();

			    switch((Table[i][j].y/15))
			    {
			        case 0:
                        Rect_source.y = ((Table[i][j].y)*HAUTEUR_TILE);
                        SDL_BlitSurface(tilesetA,&Rect_source,screenCarte,&Rect_dest);
                        break;

			        case 1:
                        Rect_source.y = ((Table[i][j].y-15)*HAUTEUR_TILE);
                        SDL_BlitSurface(tilesetB,&Rect_source,screenCarte,&Rect_dest);
                        break;

			        case 2:
                        Rect_source.y = ((Table[i][j].y-30)*HAUTEUR_TILE);
                        SDL_BlitSurface(tilesetC,&Rect_source,screenCarte,&Rect_dest);
                        break;

			        default:
                        Rect_source.y = ((Table[i][j].y-45)*HAUTEUR_TILE);
                        SDL_BlitSurface(tilesetD,&Rect_source,screenCarte,&Rect_dest);
                        break;
			    }
			}
		}
	}
}

void Carte::chargerCarte()
{
	Uint32 pixSol,pixTable,pixAir;
	SDL_Surface* sol0;
	SDL_Surface* table0 ;
	SDL_Surface* air0;
	SDL_Surface* corr;
	int i,j;

	sol0 = chargerImage(chargerNomCarteSol().c_str());// carte de la zone en code couleur
	table0 = chargerImage(chargerNomCarteCol().c_str());// carte de la zone en code couleur
	air0 = chargerImage(chargerNomCarteAir().c_str());// carte de la zone en code couleur
    corr = chargerImage( FICHIER_CORRESPONDANCE_REFTILESET );// fichier de correspondance code couleur

    setNombreBlocSurX(table0->w);
    setNombreBlocSurY(table0->h);

	for(i=0;i<getNombreBlocSurX();i++)
	{
		for(j=0;j<getNombreBlocSurY();j++)
		{
			pixSol = getPixel32(sol0,i,j);
			sol[i][j] = localiserPixDansCorr(corr,pixSol);

			pixTable = getPixel32(table0,i,j);
            table[i][j] = localiserPixDansCorr(corr,pixTable);

			pixAir = getPixel32(air0,i,j);
			air[i][j] = localiserPixDansCorr(corr,pixAir);
		}
	}

	chargerObjets(true); // Une fois la carte chargée, on charge les objets particuliers de la carte (modifie l'init précédente si ajout d'objet)

	SDL_FreeSurface(sol0);
	SDL_FreeSurface(table0);
	SDL_FreeSurface(air0);
	SDL_FreeSurface(corr);

    // Implémenter une fonction modifiant air0 et table0 à partir de rêgles prédéfinies
    // Charger ensuite te typeBlocTraversable de "table" en copiant celui qui convient de "tableTile"
    for(i=0;i<getNombreBlocSurX();i++)
    {
        for(j=0;j<getNombreBlocSurY();j++)
        {
            sol[i][j].compteurTile = 0;
            table[i][j].compteurTile = 0;
            air[i][j].compteurTile = 0;
        }
    }

}

string  Carte::chargerNomCarteSol()
{
    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence adéquate en position 0
            if(ligne.find("carte_" + oss.str() + "_sol ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                // retourner la chaine contenant l'adresse du fichier de sol
                return infosUnitaires.at(1);
            }
        }
    }

    return "";

}

string  Carte::chargerNomCarteCol()
{
    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence adéquate en position 0
            if(ligne.find("carte_" + oss.str() + "_col ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                // retourner la chaine contenant l'adresse du fichier de sol
                return infosUnitaires.at(1);
            }
        }
    }

    return "";
}

string  Carte::chargerNomCarteAir()
{
    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence adéquate en position 0
            if(ligne.find("carte_" + oss.str() + "_air ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                // retourner la chaine contenant l'adresse du fichier de sol
                return infosUnitaires.at(1);
            }
        }
    }

    return "";
}

SDL_Surface* Carte::chargerImage(const char* fichierImage)
{
	SDL_Surface* imageResultat = NULL;
	SDL_Surface* imageRam = IMG_Load(fichierImage);	// charge l'image dans image_ram en RAM
	if (imageRam==NULL)
	{
		printf("Image %s introuvable !! \n",fichierImage);
		SDL_Quit();
		exit(-1);
	}

    imageResultat=SDL_CreateRGBSurface(SDL_HWSURFACE, imageRam->w, imageRam->h, 32, 0, 0, 0, 0);  // cree une image en RAM

	SDL_BlitSurface(imageRam,NULL,imageResultat,NULL);	// copie l'image image_ram de moins de 32 bits vers image_result qui fait 32 bits
	SDL_SetColorKey( imageResultat, SDL_RLEACCEL | SDL_SRCCOLORKEY, SDL_MapRGB( imageResultat->format, 0x00, 0x00, 0xff ) ); // Optimisation de l'image
	SDL_FreeSurface(imageRam);      // supprime la surface image_ram : inutile maintenant --> libere la mémoire

	return imageResultat;
}

Uint32 Carte::getPixel32(SDL_Surface* image,int i,int j)
{
	if (i<0 || i>image->w-1 || j<0 || j>image->h-1)
        return 0;
	return ((Uint32*)(image->pixels))[j*(image->pitch/4)+i];   // lecture directe des pixels
}

position Carte::localiserPixDansCorr(SDL_Surface* corr,Uint32 pix)
{
    position posXY;
	int i,j;
	for(i=0;i<corr->w;i++)
	{
        //for(j=0;j<corr->h;j++)
        for(j=corr->h - 1;j>-1;j--)
        {
            if (getPixel32(corr,i,j) == pix)
            {
                posXY.x = i;
                posXY.y = j;
            }
        }
	}
	return posXY;
}

position** Carte::initRefTileset()
{
    int x,y,val;

    // ouverture du fichier contenant les informations du tileset
    ifstream fichier( FICHIER_REFTILESET );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence adéquate en position 0
            if(ligne.find("typeBlocTraversable ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                stringstream flux1(infosUnitaires.at(1));
                flux1 >> x;

                stringstream flux2(infosUnitaires.at(2));
                flux2 >> y;

                stringstream flux3(infosUnitaires.at(3));
                flux3 >> val;

                // Initialisation du typeBlocTraversable de la position du reftileset
                refTileset[x][y].typeBlocTraversable = val;
            }

            // Tester si l'une des lignes contient la séquence adéquate en position 0
            else if(ligne.find("typeSol ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                stringstream flux1(infosUnitaires.at(1));
                flux1 >> x;

                stringstream flux2(infosUnitaires.at(2));
                flux2 >> y;

                stringstream flux3(infosUnitaires.at(3));
                flux3 >> val;

                // Initialisation du typeSol de la position du reftileset
                refTileset[x][y].typeSol = val;
            }
        }
    }

    return refTileset;
}

void Carte::miseAJourCarte()
{
    int i,j;
    for(i=getDecalageFenetreX()/LARGEUR_TILE;i<(getDecalageFenetreX()+LARGEUR_ECRAN+LARGEUR_TILE-1)/LARGEUR_TILE;i++)
    {
        for(j=getDecalageFenetreY()/HAUTEUR_TILE;j<(getDecalageFenetreY()+HAUTEUR_ECRAN+HAUTEUR_TILE-1)/HAUTEUR_TILE;j++)
        {
            // Mise à jour typeSol et typeBlocTraversable
            table[i][j].typeBlocTraversable = refTileset[table[i][j].x][table[i][j].y].typeBlocTraversable;
            table[i][j].typeSol = refTileset[table[i][j].x][table[i][j].y].typeSol;
        }
    }
}

void Carte::initCarte()
{
    miseAJourLiensCarte();
    chargerObjets(true);
    chargerObjetsRecoltables();
    chargerEnnemis();
    chargerAmis();
}

void Carte::miseAJourLiensCarte()
{
    // Suppression des liens précédents
    liaisonsUnidirectionnelles.clear();
    list<lienCarte> C;
    C.clear();

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        int xD0,xD1,yD0,yD1,dxD0,dxD1,dyD0,dyD1; // informations de départ du lien
        int carteA,xA,yA,dxA,dyA; // information de l'arrivée du lien

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence adéquate en position 0
            if(ligne.find("carte_" + oss.str() + "_lien ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                // Attribuer ces informations au lien qui sera ajouté à la carte
                stringstream flux1(infosUnitaires.at(1));
                flux1 >> xD0;
                stringstream flux2(infosUnitaires.at(2));
                flux2 >> xD1;

                stringstream flux3(infosUnitaires.at(3));
                flux3 >> yD0;
                stringstream flux4(infosUnitaires.at(4));
                flux4 >> yD1;

                stringstream flux5(infosUnitaires.at(5));
                flux5 >> dxD0;
                stringstream flux6(infosUnitaires.at(6));
                flux6 >> dxD1;

                stringstream flux7(infosUnitaires.at(7));
                flux7 >> dyD0;
                stringstream flux8(infosUnitaires.at(8));
                flux8 >> dyD1;

                stringstream flux9(infosUnitaires.at(9));
                flux9 >> carteA;

                stringstream flux10(infosUnitaires.at(10));
                flux10 >> xA;

                stringstream flux11(infosUnitaires.at(11));
                flux11 >> yA;

                stringstream flux12(infosUnitaires.at(12));
                flux12 >> dxA;

                stringstream flux13(infosUnitaires.at(13));
                flux13 >> dyA;
/*
                LUD1.positionDepartX = xD; // Position sur X au départ du lien
                LUD1.positionDepartY = yD; // Position sur y au départ du lien
                LUD1.decalageDepartX = dxD; // Décalage sur X au départ du lien
                LUD1.decalageDepartY = dyD; // Décalage sur Y au départ du lien

                LUD1.carteArrive = carteA; // Carte où le lien mène
                LUD1.positionArriveX = xA; // Position sur X à l'arrivée du lien
                LUD1.positionArriveY = yA; // Position sur y à l'arrivée du lien
                LUD1.decalageArriveX = dxA; // Décalage sur X à l'arrivée du lien
                LUD1.decalageArriveY = dyA; // Décalage sur Y à l'arrivée du lien*/

                LUD1.positionDepartX0 = xD0; // Position la plus à gauche sur X au départ du lien
                LUD1.positionDepartX1 = xD1; // Position la plus à droite sur X au départ du lien
                LUD1.positionDepartY0 = yD0; // Position la plus haute sur y au départ du lien
                LUD1.positionDepartY1 = yD1; // Position la plus basse sur y au départ du lien
                LUD1.decalageDepartX0 = dxD0; // Décalage le plus à gauche sur X au départ du lien
                LUD1.decalageDepartX1 = dxD1; // Décalage le plus à droite sur X au départ du lien
                LUD1.decalageDepartY0 = dyD0; // Décalage le plus haut sur Y au départ du lien
                LUD1.decalageDepartY1 = dyD1; // Décalage le plus bas sur Y au départ du lien

                LUD1.carteArrive = carteA; // Carte où le lien mène
                LUD1.positionArriveX = xA; // Position la plus à gauche sur X à l'arrivée du lien
                LUD1.positionArriveY = yA; // Position la plus haute sur y à l'arrivée du lien
                LUD1.decalageArriveX = dxA; // Décalage le plus à gauche sur X à l'arrivée du lien
                LUD1.decalageArriveY = dyA; // Décalage le plus haut sur Y à l'arrivée du lien

                C.push_back(LUD1);
            }
        }
    }

    setLienCartes(C);
}


void Carte::chargerAmis()
{
    pnjAmi.clear();

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        string nom;
        int type;
        int x,y;
        int narration;

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence adéquate en position 0
            if(ligne.find("carte_" + oss.str() + "_ami ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                // Attribuer ces informations à l'ami qui sera ajouté à la carte
                nom = infosUnitaires.at(1);

                stringstream flux1(infosUnitaires.at(2));
                flux1 >> type;

                stringstream flux2(infosUnitaires.at(3));
                flux2 >> x;

                stringstream flux3(infosUnitaires.at(4));
                flux3 >> y;

                stringstream flux4(infosUnitaires.at(5));
                flux4 >> narration;

                A1.setNom(nom);
                A1.setType(type);
                A1.initType();
                A1.initPosition(x,y);
                A1.setNumNarration(narration);
                pnjAmi.push_back(A1);
            }
        }
    }
}

void Carte::chargerEnnemis()
{
    pnjEnnemi.clear();

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        int type;
        int x,y;

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "ennemi " (espace compris)
            if(ligne.find("carte_" + oss.str() + "_ennemi ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                // Attribuer ces informations à l'ennemi
                stringstream flux1(infosUnitaires.at(1));
                flux1 >> type;

                stringstream flux2(infosUnitaires.at(2));
                flux2 >> x;

                stringstream flux3(infosUnitaires.at(3));
                flux3 >> y;

                E1.setType(type);
                E1.initType();
                E1.initPosition(x,y);
                pnjEnnemi.push_back(E1);
            }
        }
    }
}

void Carte::chargerObjets(bool premierChargement)
{
    pnjObj.clear();

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // si premier chargement de la carte (et non un rechargement intermédiaire), on supprime l'ajout de "_modifie " aux noms des objets
    if(premierChargement)
    {
        suppressionModificationsTemporairesObjet();
    }

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        string nom;
        int type;
        int x,y;
        int contenuObj,valContenuObj;

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "ennemi " (espace compris)
            if(ligne.find("carte_" + oss.str() + "_objet ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                nom = infosUnitaires.at(1);

                // Attribuer ces informations à l'ennemi
                stringstream flux1(infosUnitaires.at(2));
                flux1 >> type;

                stringstream flux2(infosUnitaires.at(3));
                flux2 >> x;

                stringstream flux3(infosUnitaires.at(4));
                flux3 >> y;

                stringstream flux4(infosUnitaires.at(5));
                flux4 >> contenuObj;

                stringstream flux5(infosUnitaires.at(6));
                flux5 >> valContenuObj;

                // Modifier la carte pour y inclure l'objet
                switch(type)
                {
                    case 8: // Jarre sur socle
                    case 9: // petite tête de mort sur socle
                    case 10: // coffre fermé
                    case 11: // petit coffre ouvert
                        O1.setNumNarration(-1);
                        O1.setTypeObjetRecolte(contenuObj);
                        O1.setValeurObjetRecolte(valContenuObj);
                        break;

                    // Panneaux au sol
                    case 15:
                    case 16:
                    case 17:
                        O1.setNumNarration(contenuObj);
                        O1.setTypeObjetRecolte(-1);
                        O1.setValeurObjetRecolte(-1);
                        break;

                    default:
                        break;
                }

                O1.setNom(nom);
                O1.setType(type);
                O1.setEtat(0);
                O1.initType();
                O1.initPosition(x*16,y*16);

                pnjObj.push_back(O1);

                // Mise à jour carte

                switch(type)
                {
                    case 8: // Jarre sur socle
                        table[x][y].x = 30;
                        table[x][y].y = 0;
                        break;

                    case 9: // petite tête de mort sur socle
                        table[x][y].x = 31;
                        table[x][y].y = 0;
                        break;

                    case 10: // Coffre fermé
                        table[x][y].x = 23;
                        table[x][y].y = 7;
                        break;

                    case 11: // Coffre ouvert
                        table[x][y].x = 24;
                        table[x][y].y = 7;
                        break;

                    case 15: // pancarte sol sable
                        table[x][y].x = 34;
                        table[x][y].y = 0;
                        break;

                    case 16: // pancarte sol herbe
                        table[x][y].x = 33;
                        table[x][y].y = 0;
                        break;

                    case 17: // pancarte sol terre
                        table[x][y].x = 36;
                        table[x][y].y = 0;
                        break;

                    default:
                        break;
                }
            }
        }
    }
}

void Carte::chargerObjetsRecoltables()
{
    objRecoltes.clear();

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        string nom;
        int type;
        int x,y;

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "ennemi " (espace compris)
            if(ligne.find("carte_" + oss.str() + "_objetRecoltable ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                nom = infosUnitaires.at(1);

                // Attribuer ces informations à l'ennemi
                stringstream flux1(infosUnitaires.at(2));
                flux1 >> type;

                stringstream flux2(infosUnitaires.at(3));
                flux2 >> x;

                stringstream flux3(infosUnitaires.at(4));
                flux3 >> y;

                OR1.setNom(nom);
                OR1.setType(type);
                OR1.initType();
                OR1.initPosition(x,y);
                objRecoltes.push_back(OR1);
            }
        }
    }
}

void Carte::modificationObjet(string nomObjet)
{
    string Buffer = ""; //Variable contenant le texte à réécrire dans le fichier

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichierLecture( FICHIER_SAUVEGARDE_TMP );

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    if (fichierLecture) //Si le fichier est trouvé
    {
        int typeObjet;
        int x,y;

        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        while ( getline( fichierLecture, ligne ) ) //on parcours le fichier et on initialise line à la ligne actuelle
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "ennemi " (espace compris)
            if(ligne.find("carte_" + oss.str() + "_objet ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                if(nomObjet != infosUnitaires.at(1)) Buffer += ligne + "\n"; // On ajoute le contenu de la ligne dans le contenu à réécrire
                else
                {
                    stringstream flux1(infosUnitaires.at(2));
                    flux1 >> typeObjet;

                    stringstream flux2(infosUnitaires.at(3));
                    flux2 >> x;

                    stringstream flux3(infosUnitaires.at(4));
                    flux3 >> y;

                    switch(typeObjet)
                    {

                        /*
                        POUR INFO
                        nom = infosUnitaires.at(1);

                        // Attribuer ces informations
                        stringstream flux1(infosUnitaires.at(2));
                        flux1 >> type;

                        stringstream flux2(infosUnitaires.at(3));
                        flux2 >> x;

                        stringstream flux3(infosUnitaires.at(4));
                        flux3 >> y;

                        stringstream flux4(infosUnitaires.at(5));
                        flux4 >> contenuObj;

                        stringstream flux5(infosUnitaires.at(6));
                        flux5 >> valContenuObj;*/



                        case 8: // Jarre sur son socle -> contenu supprimé et modification de la carte table pour afficher un socle vide
                        case 9: // Petite tête de mort sur son socle -> contenu supprimé et modification de la carte table pour afficher un socle vide
                            Buffer += infosUnitaires.at(0) + "_modifie " + infosUnitaires.at(1) + " " + infosUnitaires.at(2) + " " + infosUnitaires.at(3) + " " + infosUnitaires.at(4) + " -3 0\n";
                            table[x][y].x = 0;
                            table[x][y].y = 1;
                            break;

                        case 10: // coffre fermé -> coffre ouvert et contenu supprimé
                            Buffer += infosUnitaires.at(0) + " " + infosUnitaires.at(1) + " 11 " + infosUnitaires.at(3) + " " + infosUnitaires.at(4) + " -3 0\n";
                            break;

                        case 15: // Pancarte sur fond desertique (reste le trou)
                            Buffer += infosUnitaires.at(0) + "_modifie " + infosUnitaires.at(1) + " " + infosUnitaires.at(2) + " " + infosUnitaires.at(3) + " " + infosUnitaires.at(4) + " -3 0\n";
                            table[x][y].x = 3;
                            table[x][y].y = 1;
                            break;

                        case 16: // Pancarte sur fond vert (reste le trou)
                            Buffer += infosUnitaires.at(0) + "_modifie " + infosUnitaires.at(1) + " " + infosUnitaires.at(2) + " " + infosUnitaires.at(3) + " " + infosUnitaires.at(4) + " -3 0\n";
                            table[x][y].x = 2;
                            table[x][y].y = 1;
                            break;

                        case 17: // Pancarte sur fond marron (reste le trou)
                            Buffer += infosUnitaires.at(0) + "_modifie " + infosUnitaires.at(1) + " " + infosUnitaires.at(2) + " " + infosUnitaires.at(3) + " " + infosUnitaires.at(4) + " -3 0\n";
                            table[x][y].x = 5;
                            table[x][y].y = 1;
                            break;

                        default:
                            Buffer += ligne + "\n"; // On ajoute le contenu de la ligne dans le contenu à réécrire
                            break;
                    }
                }
            }
            else Buffer += ligne + "\n"; // On ajoute obligatoirement le contenu de la ligne dans le contenu à réécrire
        }
    }
    fichierLecture.close(); //On ferme le fichier en lecture

    ofstream fichierEcriture( FICHIER_SAUVEGARDE_TMP ); //On ouvre ce même fichier en écriture
    fichierEcriture << Buffer; //On écris le texte dedans
    fichierEcriture.close(); //et on ferme le fichier
}

void Carte::suppressionModificationsTemporairesObjet()
{
    string Buffer = ""; //Variable contenant le texte à réécrire dans le fichier

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichierLecture( FICHIER_SAUVEGARDE_TMP );

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    if (fichierLecture) //Si le fichier est trouvé
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        while ( getline( fichierLecture, ligne ) ) //on parcours le fichier et on initialise line à la ligne actuelle
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "ennemi " (espace compris)
            if(ligne.find("carte_" + oss.str() + "_objet_modifie ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                Buffer += "carte_" + oss.str() + "_objet " + infosUnitaires.at(1) + " " + infosUnitaires.at(2) + " " + infosUnitaires.at(3) + " " + infosUnitaires.at(4) + " " + infosUnitaires.at(5) + " " + infosUnitaires.at(6) + "\n";
            }
            else Buffer += ligne + "\n"; // On ajoute obligatoirement le contenu de la ligne dans le contenu à réécrire
        }
    }
    fichierLecture.close(); //On ferme le fichier en lecture

    ofstream fichierEcriture( FICHIER_SAUVEGARDE_TMP ); //On ouvre ce même fichier en écriture
    fichierEcriture << Buffer; //On écris le texte dedans
    fichierEcriture.close(); //et on ferme le fichier
}

void Carte::suppressionObjetRecolte(string nomObjetRecolte)
{
    string Buffer = ""; //Variable contenant le texte à réécrire dans le fichier

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichierLecture( FICHIER_SAUVEGARDE_TMP );

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    if (fichierLecture) //Si le fichier est trouvé
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        while ( getline( fichierLecture, ligne ) ) //on parcours le fichier et on initialise line à la ligne actuelle
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "ennemi " (espace compris)
            if(ligne.find("carte_" + oss.str() + "_objetRecoltable ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                if(nomObjetRecolte != infosUnitaires.at(1)) Buffer += ligne + "\n"; // On ajoute le contenu de la ligne dans le contenu à réécrire

            }
            else Buffer += ligne + "\n"; // On ajoute obligatoirement le contenu de la ligne dans le contenu à réécrire
        }
    }
    fichierLecture.close(); //On ferme le fichier en lecture

    ofstream fichierEcriture( FICHIER_SAUVEGARDE_TMP ); //On ouvre ce même fichier en écriture
    fichierEcriture << Buffer; //On écris le texte dedans
    fichierEcriture.close(); //et on ferme le fichier
}

void Carte::ajoutObjetRecoltable(int type, int x, int y, bool fictif)
{
    // Si le type d'objet recoltable est inférieur à 0, il permet d'identifier différents tirages aléatoires

    // Si Objet recoltable aléatoire (tirage de type 1 -> Ennemi normal battu)
    if(type == -1)
    {
        type = rand()%26;

        // Augmenter la propabilité de sortie de quelques items par rapport aux autres
        /*if(type < 5) type = 0;
        else if(type > 30) type = 23;*/

        switch(type)
        {
            case 0:
            case 14:
            case 15:
            case 22:
            case 24:
                OR1.setNom("ObjetGagne");
                OR1.setType(type);
                OR1.initType();
                OR1.initPosition(x+4,y+3); // décalage fixé en dur pour essai uniquement ; reprendre ultérieurement et plus proprement
                break;

            case 23:
            case 25:
                OR1.setNom("ObjetGagne");
                OR1.setType(type);
                OR1.initType();
                OR1.initPosition(x+6,y+3); // décalage fixé en dur pour essai uniquement ; reprendre ultérieurement et plus proprement
                break;

            default:
                OR1.setNom("FauxObjetGagne");
                OR1.setType(-1);
                OR1.setValeur(0);
                OR1.initType();
                OR1.initPosition(x+6,y+3); // décalage fixé en dur pour essai uniquement ; reprendre ultérieurement et plus proprement
                break;
        }
        objRecoltes.push_back(OR1);
    }

    // Si Objet recoltable aléatoire (tirage de type 2 -> Elément de décors cassé)
    else if(type == -2)
    {
        type = rand()%35;

        // Augmenter la propabilité de sortie de quelques items par rapport aux autres
        if(type > 15 && type < 24) type = 23;
        else if(type > 24 && type < 30) type = 25;

        switch(type)
        {
            case 23:
            case 25:
                OR1.setNom("ObjetGagne");
                OR1.setType(type);
                OR1.initType();
                OR1.initPosition(x,y); // décalage fixé en dur pour essai uniquement ; reprendre ultérieurement et plus proprement
                break;

            default:
                OR1.setNom("FauxObjetGagne");
                OR1.setType(-1);
                OR1.setValeur(0);
                OR1.initType();
                OR1.initPosition(x,y); // décalage fixé en dur pour essai uniquement ; reprendre ultérieurement et plus proprement
                break;
        }
        objRecoltes.push_back(OR1);
    }

    // Si plus d'objet à recolter (jarre vide par exemple)
    else if(type == -3)
    {
        // Rien à faire
    }

    // Dans le cas d'un objet à simplement afficher (sorti d'un coffre par exemple)
    else if(fictif)
    {
        OR1.setNom("ObjetGagneFictif");
        OR1.setType(type);
        OR1.initType();
        OR1.setValeur(1); // pas de valeur normalement
        OR1.setTimeToLive(30); // initialisation du TTL à une valeur > 0
        OR1.initPosition(x,y-10);
        objRecoltes.push_back(OR1);
    }

    // Sinon, objets récoltable et prédéterminé
    else
    {
        OR1.setNom("ObjetGagne");
        OR1.setType(type);
        OR1.initType();
        OR1.initPosition(x,y);
        objRecoltes.push_back(OR1);
    }
}



void Carte::sauvegardeModifsCarte(int niveauCarte, int xCarte, int yCarte, int xTileset, int yTileset)
{
    string Buffer = "";

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichierLecture( FICHIER_SAUVEGARDE_TMP );

    // créer les flux de sortie et écrire les bonnes valeurs dedans

        // Numéro de la carte
        std::ostringstream oss;
        oss << getNumero();

        // Niveau où on effectue la modification (Sol, Table ou Air)
        std::ostringstream niv;
        niv << niveauCarte;

        // Coordonnée en x à traiter sur la carte
        std::ostringstream xC;
        xC << xCarte;

        // Coordonnée en y à traiter sur la carte
        std::ostringstream yC;
        yC << yCarte;

        // Coordonnée en x de la nouvelle tuile dans le tileset de référence
        std::ostringstream xT;
        xT << xTileset;

        // Coordonnée en y de la nouvelle tuile dans le tileset de référence
        std::ostringstream yT;
        yT << yTileset;

    if (fichierLecture) //Si le fichier est trouvé
    {
        string ligne; // variable contenant chaque ligne lue

        while ( getline( fichierLecture, ligne ) ) //on parcours le fichier et on initialise line à la ligne actuelle
        {
            Buffer += ligne + "\n"; // On ajoute obligatoirement le contenu de la ligne dans le contenu à réécrire
        }
    }
    fichierLecture.close(); //On ferme le fichier en lecture

    Buffer += "carte_" + oss.str() + "_modification " + niv.str() + " " + xC.str() + " " + yC.str() + " " + xT.str() + " " + yT.str() + "\n";

    ofstream fichierEcriture( FICHIER_SAUVEGARDE_TMP ); //On ouvre le même fichier en écriture
    fichierEcriture << Buffer; //On écris le texte dedans
    fichierEcriture.close(); //et on ferme le fichier
}

void Carte::chargerModifsCarte()
{
    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << getNumero();

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_SAUVEGARDE_TMP );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne

        int niveau;
        int xC,yC,xT,yT;

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "ennemi " (espace compris)
            if(ligne.find("carte_" + oss.str() + "_modification ") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                // Attribuer ces informations à l'ennemi
                stringstream flux1(infosUnitaires.at(1));
                flux1 >> niveau;

                stringstream flux2(infosUnitaires.at(2));
                flux2 >> xC;

                stringstream flux3(infosUnitaires.at(3));
                flux3 >> yC;

                stringstream flux4(infosUnitaires.at(4));
                flux4 >> xT;

                stringstream flux5(infosUnitaires.at(5));
                flux5 >> yT;

                switch(niveau)
                {
                    case 0: // Sol
                        sol[xC][yC].x = xT;
                        sol[xC][yC].y = yT;
                        break;

                    case 1: // Table (pour les collisions)
                        table[xC][yC].x = xT;
                        table[xC][yC].y = yT;
                        break;

                    case 2: // Air
                        air[xC][yC].x = xT;
                        air[xC][yC].y = yT;
                        break;

                    default:

                        break;
                }
            }
        }
    }
}

void Carte::gestionAffichageJoueur(SDL_Surface* screenCarte,Hero* perso,bool pause, bool affichageMenu)
{
    SDL_Rect sprite = perso->selectionSprite(pause,affichageMenu);
    SDL_Rect screen;

    // Cas particulier lors de la perte totale d'énergie par le perso
    if(perso->getEnergie() <= 0 && (!perso->getTouche() || perso->getCompteurTouche() <= TEMPS_INTOUCHABLE - TEMPS_RECUL))
    {
        screen.y = perso->getPositionY() + 10;
        screen.x = perso->getPositionX() - 4;
    }
    // Décalage de l'affichage du sprite nécessaire car plus large et risque de chevauchement entre la tête du perso et le décors si déplacmeent vers la droite
    else if(perso->getNage() && perso->getOrientation() == 3)
    {
        screen.y = perso->getPositionY();
        screen.x = perso->getPositionX() - 8;
    }
    // Si encore au moins un cœur d'énergie
    else
    {
        screen.y = perso->getPositionY();
        screen.x = perso->getPositionX();
    }
    screen.w = 16;
    screen.h = 24;

    if(!perso->getTouche() || perso->getCompteurTouche()%2 == 0)
    {
        if(perso->getOrientation() == 0)
        {
            perso->afficherArme(screenCarte,screen); // Affichage de l'arme utilisée par le perso
            perso->afficherEntite(screenCarte,sprite,screen); // Héro affiché tout le temps s'il n'est pas touché ou une fois sur deux lorsqu'il est touché
        }
        else
        {
            perso->afficherEntite(screenCarte,sprite,screen); // Héro affiché tout le temps s'il n'est pas touché ou une fois sur deux lorsqu'il est touché
            perso->afficherArme(screenCarte,screen); // Affichage de l'arme utilisée par le perso
        }
    }
}

void Carte::gestionAffichageAmis(SDL_Surface* screenCarte,Hero* perso,bool hauteurCarte,bool pause)
{
    // Traitement des PNJ amis
    int Y = perso->getPositionY();

    SDL_Rect rectAmi;
    SDL_Rect rectScreen;
    list<Ami> pnjA = getPnjAmi();
    list<Ami>::iterator itAmi;

    for(itAmi=pnjA.begin() ; itAmi!=pnjA.end() ; itAmi++)
    {
        rectAmi = (*itAmi).selectionSprite(pause);

        rectScreen.x = (*itAmi).getPositionX() - getDecalageFenetreX();
        rectScreen.y = (*itAmi).getPositionY() - getDecalageFenetreY();
        rectScreen.w = (*itAmi).getDimensionX();
        rectScreen.h = (*itAmi).getDimensionY();

        if((((*itAmi).getPositionY() - getDecalageFenetreY()) <= Y &&  hauteurCarte) || (((*itAmi).getPositionY() - getDecalageFenetreY()) > Y &&  !hauteurCarte))
        {
            if((*itAmi).getPositionX() - getDecalageFenetreX() >= - (*itAmi).getDimensionX()/2
               && (*itAmi).getPositionX() - getDecalageFenetreX() <= LARGEUR_ECRAN + (*itAmi).getDimensionX()/2
               && (*itAmi).getPositionY() - getDecalageFenetreY() >= - (*itAmi).getDimensionY()/2
               && (*itAmi).getPositionY() - getDecalageFenetreY() <= HAUTEUR_ECRAN - (*itAmi).getDimensionY()/2)
            {
                (*itAmi).afficherEntite(screenCarte,rectAmi,rectScreen);
            }
        }
    }

    // Mise à jour des Amis sur la carte
    setPnjAmi(pnjA);
}


void Carte::gestionAffichageEnnemis(SDL_Surface* screenCarte,Hero* perso,bool hauteurCarte,bool pause)
{
    // Traitement des PNJ Ennemis
    int Y = perso->getPositionY();
    SDL_Rect rectEnnemi;
    SDL_Rect rectScreen;
    list<Ennemi> pnjE = getPnjEnnemi();
    list<Ennemi>::iterator itEnnemi;

    for(itEnnemi=pnjE.begin() ; itEnnemi!=pnjE.end() ; itEnnemi++)
    {
        rectEnnemi = (*itEnnemi).selectionSprite(pause,getInterRouge());

        rectScreen.x = (*itEnnemi).getPositionX() - getDecalageFenetreX();
        rectScreen.y = (*itEnnemi).getPositionY() - getDecalageFenetreY();
        rectScreen.w = (*itEnnemi).getDimensionX();
        rectScreen.h = (*itEnnemi).getDimensionY();

        if((((*itEnnemi).getPositionY() - getDecalageFenetreY()) <= Y &&  hauteurCarte) || (((*itEnnemi).getPositionY() - getDecalageFenetreY()) > Y &&  !hauteurCarte))
        {
            if((*itEnnemi).getPositionX() - getDecalageFenetreX() >= - (*itEnnemi).getDimensionX()/2
               && (*itEnnemi).getPositionX() - getDecalageFenetreX() <= LARGEUR_ECRAN + (*itEnnemi).getDimensionX()/2
               && (*itEnnemi).getPositionY() - getDecalageFenetreY() >= - (*itEnnemi).getDimensionY()/2
               && (*itEnnemi).getPositionY() - getDecalageFenetreY() <= HAUTEUR_ECRAN - (*itEnnemi).getDimensionY()/2)
               {
                    if((*itEnnemi).getEnergie() < 1 && (*itEnnemi).getTouche() && (*itEnnemi).getCompteurDeplacement() > TEMPS_RECUL && (*itEnnemi).getType() != 0)
                    {
                        // Ajustement de la position de l'ennemi s'il n'est pas encore du type "explosion"
                        (*itEnnemi).setPositionX((*itEnnemi).getDimensionX()/2 - 14);
                        (*itEnnemi).setPositionY((*itEnnemi).getDimensionY()/2 - 14);

                        // Modification du type (passage au type "explosion") et initialisation
                        (*itEnnemi).setType(0);
                        (*itEnnemi).setFeuilleSprite(explo);
                        (*itEnnemi).setVitesseDeplacementSurX(0);
                        (*itEnnemi).setVitesseDeplacementSurY(0);
                    }
                    else if((*itEnnemi).getType() == 0 && (*itEnnemi).getCompteurDeplacement() == TEMPS_RECUL + TEMPS_IMMOBILISATION - 1)
                    {
                        ajoutObjetRecoltable(-1,(*itEnnemi).getPositionX(), (*itEnnemi).getPositionY(), false);
                    }

                    // Affichage de l'ennemi
                    if(!(*itEnnemi).getTouche() || (*itEnnemi).getCompteurDeplacement() > TEMPS_RECUL || (*itEnnemi).getCompteurDeplacement()%2 == 0)
                    {
                        (*itEnnemi).afficherEntite(screenCarte,rectEnnemi,rectScreen);
                    }
               }
        }
    }

    // Mise à jour des Ennemis sur la carte
    setPnjEnnemi(pnjE);
}


void Carte::gestionAffichageProjectiles(SDL_Surface* screenCarte,Hero* perso,bool hauteurCarte,bool pause)
{
    // Traitement des projectiles
    int Y = perso->getPositionY();
    SDL_Rect rectProjectile;
    SDL_Rect rectScreen;
    list<Projectile> pnjP = getPnjProjectile();
    list<Projectile>::iterator itProjectile;

    for(itProjectile=pnjP.begin() ; itProjectile!=pnjP.end() ; itProjectile++)
    {
        rectProjectile = (*itProjectile).selectionSprite(pause);

        rectScreen.x = (*itProjectile).getPositionX() - getDecalageFenetreX();
        rectScreen.y = (*itProjectile).getPositionY() - getDecalageFenetreY();
        rectScreen.w = (*itProjectile).getDimensionX();
        rectScreen.h = (*itProjectile).getDimensionY();

        if((((*itProjectile).getPositionY() - getDecalageFenetreY()) <= Y && hauteurCarte) || (((*itProjectile).getPositionY() - getDecalageFenetreY()) > Y &&  !hauteurCarte))
        {
            if((*itProjectile).getPositionX() - getDecalageFenetreX() >= - (*itProjectile).getDimensionX()/2
               && (*itProjectile).getPositionX() - getDecalageFenetreX() <= LARGEUR_ECRAN + (*itProjectile).getDimensionX()/2
               && (*itProjectile).getPositionY() - getDecalageFenetreY() >= - (*itProjectile).getDimensionY()/2
               && (*itProjectile).getPositionY() - getDecalageFenetreY() <= HAUTEUR_ECRAN - (*itProjectile).getDimensionY()/2)
               {
                    (*itProjectile).afficherEntite(screenCarte,rectProjectile,rectScreen);
               }

            if(!pause)
            {
                if((*itProjectile).getCompteurAnim() > 300) (*itProjectile).setTimeToLive(0);

                else if(((*itProjectile).getType() == 4) && (*itProjectile).getTimeToLive() < 2)
                {
                    if(collisionAABB(perso->getPositionX() + getDecalageFenetreX(),
                                     perso->getPositionX() + perso->getDimensionX() + getDecalageFenetreX(),
                                     perso->getPositionY() + getDecalageFenetreY(),
                                     perso->getPositionY() + perso->getDimensionY() + getDecalageFenetreY(),
                                     (*itProjectile).getPositionX(),
                                     (*itProjectile).getPositionX() + (*itProjectile).getDimensionX(),
                                     (*itProjectile).getPositionY(),
                                     (*itProjectile).getPositionY() + (*itProjectile).getDimensionY()))
                                     {
                                         (*itProjectile).setTimeToLive(0);
                                     }
                    else (*itProjectile).setTimeToLive(3);
                }

                else (*itProjectile).setTimeToLive((*itProjectile).getTimeToLive() - 1);
            }
        }

    }

    // Mise à jour des Projectiles sur la carte
    setPnjProjectile(pnjP);
}

void Carte::gestionAffichageObjets(SDL_Surface* screenCarte,Hero* perso,bool hauteurCarte,bool pause)
{
    // Traitement des objets
    int Y = perso->getPositionY();
    SDL_Rect rectObjets;
    SDL_Rect rectScreen;
    list<Objet> ob = getPnjObjets();
    list<Objet>::iterator itObjet;

    for(itObjet=ob.begin() ; itObjet!=ob.end() ; itObjet++)
    {
        // Par défaut, les objets ne sont pas affichés par cette méthode.
        // La carte est alors simplement modifiée pour afficher un objet au sol.

        // Par contre, lorsqu'un objet n'est plus au sol ou aligné sur la grille (objet porté ou lancé par le joueur, objet se déplaçant au sol), c'est cette fonction qui gère son affichage.
        if((*itObjet).getEtat() > 0)
        {
            rectObjets = (*itObjet).selectionSprite(pause);

            rectScreen.x = (*itObjet).getPositionX() - getDecalageFenetreX();
            rectScreen.y = (*itObjet).getPositionY() - getDecalageFenetreY();
            rectScreen.w = (*itObjet).getDimensionX();
            rectScreen.h = (*itObjet).getDimensionY();

            if((((*itObjet).getPositionY() - getDecalageFenetreY()) <= Y && hauteurCarte) || (((*itObjet).getPositionY() - getDecalageFenetreY()) > Y &&  !hauteurCarte))
            {
                if((*itObjet).getPositionX() - getDecalageFenetreX() >= - (*itObjet).getDimensionX()/2
                   && (*itObjet).getPositionX() - getDecalageFenetreX() <= LARGEUR_ECRAN + (*itObjet).getDimensionX()/2
                   && (*itObjet).getPositionY() - getDecalageFenetreY() >= - (*itObjet).getDimensionY()/2
                   && (*itObjet).getPositionY() - getDecalageFenetreY() <= HAUTEUR_ECRAN - (*itObjet).getDimensionY()/2)
                   {
                        (*itObjet).afficherEntite(screenCarte,rectObjets,rectScreen);
                   }
            }
        }
    }

    // Mise à jour des objets sur la carte
    setPnjObjets(ob);
}


void Carte::gestionAffichageObjetsRecoltes(SDL_Surface* screenCarte,Hero* perso,bool hauteurCarte,bool pause)
{
    // Traitement des objets à récolter
    int Y = perso->getPositionY();
    SDL_Rect rectObjetsRecoltables;
    SDL_Rect rectScreen;
    list<ObjetRecolte> objRec = getObjetRecoltables();
    list<ObjetRecolte>::iterator itObjetRecolte;

    for(itObjetRecolte=objRec.begin() ; itObjetRecolte!=objRec.end() ; itObjetRecolte++)
    {
        rectObjetsRecoltables = (*itObjetRecolte).selectionSprite(pause);

        rectScreen.x = (*itObjetRecolte).getPositionX() - getDecalageFenetreX();
        rectScreen.y = (*itObjetRecolte).getPositionY() - getDecalageFenetreY();
        rectScreen.w = (*itObjetRecolte).getDimensionX();
        rectScreen.h = (*itObjetRecolte).getDimensionY();

        if((((*itObjetRecolte).getPositionY() - getDecalageFenetreY()) <= Y && hauteurCarte) || (((*itObjetRecolte).getPositionY() - getDecalageFenetreY()) > Y && !hauteurCarte))
        {
            if((*itObjetRecolte).getPositionX() - getDecalageFenetreX() >= - (*itObjetRecolte).getDimensionX()/2
               && (*itObjetRecolte).getPositionX() - getDecalageFenetreX() <= LARGEUR_ECRAN + (*itObjetRecolte).getDimensionX()/2
               && (*itObjetRecolte).getPositionY() - getDecalageFenetreY() >= - (*itObjetRecolte).getDimensionY()/2
               && (*itObjetRecolte).getPositionY() - getDecalageFenetreY() <= HAUTEUR_ECRAN - (*itObjetRecolte).getDimensionY()/2)
               {
                    (*itObjetRecolte).afficherEntite(screenCarte,rectObjetsRecoltables,rectScreen);
               }
        }

        // Pour chaque objet recoltable, on décrémente le TTL s'il est > 0 (ces objets récoltables sont donc par définition temporaires, un TTL à 0 signifiant qu'ils doivent être supprimés de la carte)
        if((*itObjetRecolte).getTimeToLive() > 0 && !hauteurCarte)
        {
            (*itObjetRecolte).setTimeToLive((*itObjetRecolte).getTimeToLive() - 1);
        }
        else if((*itObjetRecolte).getTimeToLive() == 0)
        {
            (*itObjetRecolte).setValeur(0);
        }

    }

    // Mise à jour des objets recoltables sur la carte
    setObjetRecoltables(objRec);
}



void Carte::gestionInterractionsEntiteEntite(Hero* perso)
{
    applicationIaEntites();

    gestionInterractionsJoueurEnnemis(perso);
    gestionInterractionsJoueurAmis(perso);
    gestionInterractionsJoueurObjets(perso);
    gestionInterractionsJoueurObjetsRecoltables(perso);
    gestionInterractionsJoueurCarte(perso);

    gestionInterractionsArmeJoueurEnnemis(perso);
    gestionInterractionsArmeJoueurProjectiles(perso);
    gestionInterractionsArmeJoueurCarte(perso);

    gestionInterractionsEnnemisEnnemis();
    gestionInterractionsEnnemisAmis();
    gestionInterractionsEnnemisObjets();
    gestionInterractionsEnnemisObjetsRecoltales();
    gestionInterractionsEnnemisProjectiles();

    gestionInterractionsAmisAmis();
    gestionInterractionsAmisObjets();
    gestionInterractionsAmisObjetsRecoltables();
    gestionInterractionsAmisProjectiles();

    gestionInterractionsObjetsObjetsRecoltables();
    gestionInterractionsObjetsProjectiles();

    gestionInterractionsObjetsRecoltablesProjectiles();

    gestionInterractionsObjetsProjectilesCarte();
}


void Carte::applicationIaEntites()
{
    list<Ennemi>::iterator itE;
    list<Ami>::iterator itA;
    list<Objet>::iterator itO;
    list<ObjetRecolte>::iterator itOR;
    list<Projectile>::iterator itP;

    // Gestion des déplacements automatisés des entités
    // et gestion des déplacements des entités
    for(itE=pnjEnnemi.begin();itE != pnjEnnemi.end();itE++)
    {
        (*itE).intelligenceArtificielle();
        (*itE).traitementDeplacementsPNJ(getTableauTable(),getDecalageFenetreX(),getDecalageFenetreY());
    }

    for(itA=pnjAmi.begin();itA != pnjAmi.end();itA++)
    {
        (*itA).intelligenceArtificielle();
        (*itA).traitementDeplacementsPNJ(getTableauTable(),getDecalageFenetreX(),getDecalageFenetreY());
    }

    for(itO=pnjObj.begin();itO != pnjObj.end();itO++)
    {
        (*itO).intelligenceArtificielle();
        (*itO).traitementDeplacementsPNJ(getTableauTable(),getDecalageFenetreX(),getDecalageFenetreY());
    }

    for(itOR=objRecoltes.begin();itOR != objRecoltes.end();itOR++)
    {
        (*itOR).intelligenceArtificielle();
        (*itOR).traitementDeplacementsPNJ(getTableauTable(),getDecalageFenetreX(),getDecalageFenetreY());
    }

    for(itP=pnjProjectile.begin();itP != pnjProjectile.end();itP++)
    {
        (*itP).intelligenceArtificielle();
        (*itP).traitementDeplacementsPNJ(getTableauTable(),getDecalageFenetreX(),getDecalageFenetreY());
    }

    // Elimination des entités lorsque cela devient nécessaire
    pnjEnnemi.remove_if (ennemiASortir());
    pnjObj.remove_if (objetPorteASortir());
    objRecoltes.remove_if (objetRecolteASortir());
    pnjProjectile.remove_if (projectileASortir());
}

void Carte::gestionInterractionsJoueurEnnemis(Hero* perso)
{
    list<Ennemi>::iterator it;

    for(it=pnjEnnemi.begin();it != pnjEnnemi.end();it++)
    {
        if(!perso->getTouche()
           && (*it).getEnergie() > 0
           && collisionAABB(perso->getPositionX()+getDecalageFenetreX()+3,
                            perso->getPositionX()+getDecalageFenetreX()+LARGEUR_PERSO-3,
                            perso->getPositionY()+getDecalageFenetreY()+11,
                            perso->getPositionY()+getDecalageFenetreY()+HAUTEUR_PERSO-11,
                            (*it).getPositionX() + 2,
                            (*it).getPositionX() + (*it).getDimensionX() - 2,
                            (*it).getPositionY() + 11,
                            (*it).getPositionY() + (*it).getDimensionY() - 2
                            ) )
                            {
                                if(perso->getPositionX()+getDecalageFenetreX() > (*it).getPositionX()-15 && perso->getPositionX()+getDecalageFenetreX() < (*it).getPositionX())
                                {
                                    perso->setBougeDroite(false);
                                    (*it).setBougeGauche(false);
                                }
                                else if((*it).getPositionX() > perso->getPositionX()-15+getDecalageFenetreX() && (*it).getPositionX() < perso->getPositionX()+getDecalageFenetreX())
                                {
                                    (*it).setBougeDroite(false);
                                    perso->setBougeGauche(false);
                                }

                                if(perso->getPositionY()+getDecalageFenetreY() > (*it).getPositionY()-16 && perso->getPositionY()+getDecalageFenetreY() < (*it).getPositionY())
                                {
                                    perso->setBougeBas(false);
                                    (*it).setBougeHaut(false);
                                }
                                else if((*it).getPositionY() > perso->getPositionY()-16+getDecalageFenetreY() && (*it).getPositionY() < perso->getPositionY()+getDecalageFenetreY())
                                {
                                    (*it).setBougeBas(false);
                                    perso->setBougeHaut(false);
                                }

                                // Si joueur jamais touché
                                if(!perso->getTouche() && perso->getEnergie() > 0)
                                {
                                    perso->setTouche(true);
                                    perso->setCompteurTouche(TEMPS_INTOUCHABLE);
                                    perso->setEnergie(perso->getEnergie() - 1);
                                }
                            }
    }

    // Gestion du mode Attaque de l'ennemi
    for(it=pnjEnnemi.begin();it != pnjEnnemi.end();it++)
    {
        (*it).setDecalageXAvecLeHero(perso->getPositionX()+getDecalageFenetreX());
        (*it).setDecalageYAvecLeHero(perso->getPositionY()+getDecalageFenetreY());

        if(!(*it).getAttaque() && abs((*it).getDecalageXAvecLeHero()) < 60 && abs((*it).getDecalageYAvecLeHero()) < 60)
        {
            (*it).setAttaque(true);
        }

        else if((*it).getAttaque() && !(*it).getTouche() && (abs((*it).getDecalageXAvecLeHero()) > 120 || abs((*it).getDecalageYAvecLeHero()) > 120))
        {
            (*it).setAttaque(false);
            (*it).setCompteurDeplacement(-1); // initialisation du compteur de déplacement
        }

        // Si le Héro n'a plus d'énergie, le mode attaque repasse à faux (permet de libérer l'attention des PNJ ennemis sur le joueur)
        if(perso->getEnergie() <= 0) (*it).setAttaque(false);
    }
}

void Carte::gestionInterractionsJoueurAmis(Hero* perso)
{
    list<Ami>::iterator it;

    // Gestion globale des collisions entre amis et joueur
    for(it=pnjAmi.begin();it != pnjAmi.end();it++)
    {
        if(collisionAABB(perso->getPositionX()+getDecalageFenetreX(),
                        perso->getPositionX()+getDecalageFenetreX()+LARGEUR_PERSO,
                        perso->getPositionY()+getDecalageFenetreY()+8,
                        perso->getPositionY()+getDecalageFenetreY()+HAUTEUR_PERSO-8,
                        (*it).getPositionX(),
                        (*it).getPositionX() + (*it).getDimensionX(),
                        (*it).getPositionY()+8,
                        (*it).getPositionY() +
                        (*it).getDimensionY()-8))
                        {
                            if(perso->getPositionX()+getDecalageFenetreX() > (*it).getPositionX()-15 && perso->getPositionX()+getDecalageFenetreX() < (*it).getPositionX())
                            {
                                perso->setBougeDroite(false);
                                (*it).setBougeGauche(false);
                            }
                            else if((*it).getPositionX() > perso->getPositionX()-15+getDecalageFenetreX() && (*it).getPositionX() < perso->getPositionX()+getDecalageFenetreX())
                            {
                                (*it).setBougeDroite(false);
                                perso->setBougeGauche(false);
                            }


                            if(perso->getPositionY()+getDecalageFenetreY() > (*it).getPositionY()-16 && perso->getPositionY()+getDecalageFenetreY() < (*it).getPositionY())
                            {
                                perso->setBougeBas(false);
                                (*it).setBougeHaut(false);
                            }
                            else if((*it).getPositionY() > perso->getPositionY()-16+getDecalageFenetreY() && (*it).getPositionY() < perso->getPositionY()+getDecalageFenetreY())
                            {
                                (*it).setBougeBas(false);
                                perso->setBougeHaut(false);
                            }
                        }
    }
}

void Carte::gestionInterractionsJoueurObjets(Hero* perso)
{
    // Zone d'influence du perso vis à vis des objets
    int x0 = 0;
    int x1 = 0;
    int y0 = 0;
    int y1 = 0;

    bool rechargerObj1 = false;
    bool rechargerObj2 = false;
    int numObjetPorteAAjouter = -1;

    // Définition de la zone d'influence du joueur vis à vis des objets qu'il peut rencontrer
    gestionZoneInfluenceJoueurSurObjets(perso,&x0,&x1,&y0,&y1);

    // Gestion de l'état des différents PNJ objet
    rechargerObj1 = gestionModificationEtatObjetsManipules(perso,&x0,&x1,&y0,&y1,&numObjetPorteAAjouter);

    // Gestion transformation éléments de la carte en objets
    rechargerObj2 = gestionTransformationElementCarteEnObjetPorte(perso,&numObjetPorteAAjouter);

    // Rechargement des objets de la carte afin de prendre en compte directement les modifications
    if(rechargerObj1 || rechargerObj2)
    {
        chargerObjets(false);

        if(numObjetPorteAAjouter > -1)
        {
            // Ajout de l'objet porté par le joueur sur la carte
            O1.setNom("ObjetPorte");
            O1.setType(numObjetPorteAAjouter);
            O1.setEtat(1);
            O1.initType();
            O1.initPosition(perso->getPositionX() + getDecalageFenetreX(),perso->getPositionY() + getDecalageFenetreY() - 8);
            pnjObj.push_back(O1);
        }
    }
}

void Carte::gestionZoneInfluenceJoueurSurObjets(Hero* perso,int* x0,int* x1,int* y0,int* y1)
{
    // Permet d'utilser ensuite la fonction de gestion des collision AABB

    switch(perso->getOrientation())
    {
        case 0:
            *x0 = perso->getPositionX() + getDecalageFenetreX() + 6;
            *x1 = perso->getPositionX() + getDecalageFenetreX() + 7;
            *y0 = perso->getPositionY() + getDecalageFenetreY() + 6;
            *y1 = perso->getPositionY() + getDecalageFenetreY() + 7;
            break;

        case 1:
            *x0 = perso->getPositionX() + getDecalageFenetreX() + 6;
            *x1 = perso->getPositionX() + getDecalageFenetreX() + 7;
            *y0 = perso->getPositionY() + getDecalageFenetreY() + 24;
            *y1 = perso->getPositionY() + getDecalageFenetreY() + 25;
            break;

        case 2:
            *x0 = perso->getPositionX() + getDecalageFenetreX() - 2;
            *x1 = perso->getPositionX() + getDecalageFenetreX() - 1;
            *y0 = perso->getPositionY() + getDecalageFenetreY() + 16;
            *y1 = perso->getPositionY() + getDecalageFenetreY() + 17;
            break;

        case 3:
            *x0 = perso->getPositionX() + getDecalageFenetreX() + 16;
            *x1 = perso->getPositionX() + getDecalageFenetreX() + 17;
            *y0 = perso->getPositionY() + getDecalageFenetreY() + 16;
            *y1 = perso->getPositionY() + getDecalageFenetreY() + 17;
            break;

        default:
            break;
    }
}

// Voir si factorisation réellement nécessaire
bool Carte::gestionModificationEtatObjetsManipules(Hero* perso,int* x0,int* x1,int* y0,int* y1,int* numObjetPorteAAjouter)
{
    list<Objet>::iterator it;
    bool rechargerObj = false;


    // Gestion des déplacements automatisés des PNJ objet
    for(it=pnjObj.begin();it != pnjObj.end();it++)
    {
        // Test de collision avec un objet dans un état inactif (excluant les objets en train d'être lancé par exemple)
        if((*it).getEtat() == 0 && collisionAABB(*x0, *x1, *y0, *y1, (*it).getPositionX() + 1, (*it).getPositionX() + (*it).getDimensionX() - 1, (*it).getPositionY() + 1, (*it).getPositionY() + (*it).getDimensionY() - 1))
        {
            // Si le perso est en état "action = vrai", il peut agir sur l'objet
            if(perso->getAction())
            {
                // Selon le type d'objet :
                switch((*it).getType())
                {
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                        // Si le joueur est en dessous et regarde vers le haut, alors il peut agir sur l'objet
                        if(perso->getPositionY()+decalageFenetreY+HAUTEUR_PERSO+1 > (*it).getPositionY() + (*it).getDimensionY() && perso->getOrientation() == 0)
                        {
                            // Les stats du joueur évoluent en fonction de l'objet récolté.
                            perso->setMAJObjetsRecolte((*it).getTypeObjetRecolte(), (*it).getValeurObjetRecolte());
                            // L'objet recolté est simplement affiché et en léger mouvement vers le haut.
                            ajoutObjetRecoltable((*it).getTypeObjetRecolte(), (*it).getPositionX(), (*it).getPositionY(), true);
                            // Le fichier temporaire est modifié pour prendre en compte que l'objet est ouvert et/ou ne contient plus d'objet récoltable
                            modificationObjet((*it).getNom());
                        }
                        break;

                    case 15: // pancartes (leur comportement varie en fonction de l'orientation du joueur devant elles)
                    case 16:
                    case 17:
                        // Si le joueur est en dessous et regarde vers le haut, alors il peut agir sur l'objet
                        if(perso->getPositionY()+decalageFenetreY+HAUTEUR_PERSO+1 > (*it).getPositionY() + (*it).getDimensionY() && perso->getOrientation() == 0)
                        {
                            // Les stats du joueur évoluent en fonction de l'objet récolté.
                            perso->setMAJObjetsRecolte((*it).getTypeObjetRecolte(), (*it).getValeurObjetRecolte());
                            // L'objet recolté est simplement affiché et en léger mouvement vers le haut.
                            //ajoutObjetRecoltable((*it).getTypeObjetRecolte(), (*it).getPositionX(), (*it).getPositionY(), true);
                        }
                        else if(perso->getOrientation() >= 2) // gauche ou droite
                        {
                            // Modification du numéro d'objet porté à ajouter
                            if(!perso->getPorteObjet()) *numObjetPorteAAjouter = 7; // pancarte nue
                            // On élimine la pancarte au sol
                            (*it).setEtat(-1);
                            // Le fichier temporaire est modifié pour prendre en compte que l'objet est ouvert et/ou ne contient plus d'objet récoltable
                            modificationObjet((*it).getNom());
                            // Temporaire à cet endroit -> modif de l'état "porte objet" du perso
                            perso->setPorteObjet(true);
                        }
                        break;

                    default:
                        // L'objet est remplacé par l'objet recoltable qu'il contient (et c'est au personnage d'aller ensuite chercher cette objet en se mettant dessus)
                        ajoutObjetRecoltable((*it).getTypeObjetRecolte(), (*it).getPositionX(), (*it).getPositionY(), false);
                        // Le fichier temporaire est modifié pour prendre en compte que l'objet est ouvert et/ou ne contient plus d'objet récoltable
                        modificationObjet((*it).getNom());
                        // Modification du numéro d'objet porté à ajouter
                        if(!perso->getPorteObjet()) *numObjetPorteAAjouter = (*it).getType()-4;
                        // Temporaire à cet endroit -> modif de l'état "porte objet" du perso
                        perso->setPorteObjet(true);
                        break;

                }

                // Puisque le perso a effectué une action sur l'un des objets de la carte, on indique qu'il faudra recharger les objet présents sur la carte
                // (l'un de ces objets ayant été modifié)
                rechargerObj = true;
            }
        }

        // Si l'objet est en train d'exploser
        if((*it).getEtat() == 3)
        {
            switch((*it).getType())
            {
                case 1: // Les buissons explosent plus longtemps que les autres objets
                case 2:
                case 3:
                    if((*it).getCompteurAnim() > 76) (*it).setEtat(-1);
                    break;

                default:
                    if((*it).getCompteurAnim() > 20) (*it).setEtat(-1);
                    break;
            }

        }

        // Si l'objet est lancé
        else if((*it).getEtat() == 2)
        {
            (*it).incrementationCompteurDeplacement();

            switch((*it).getOrientation())
            {
                case 0:
                    (*it).initPosition((*it).getPositionX(),(*it).getPositionY() - 4 + (*it).getCompteurDeplacement()/5);
                    break;

                case 1:
                    (*it).initPosition((*it).getPositionX(),(*it).getPositionY() + 5 - (*it).getCompteurDeplacement()/5);
                    break;

                case 2:
                    (*it).initPosition((*it).getPositionX()-4,(*it).getPositionY() + (*it).getCompteurDeplacement()/5);
                    break;

                case 3:
                    (*it).initPosition((*it).getPositionX()+4,(*it).getPositionY() + (*it).getCompteurDeplacement()/5);
                    break;

                default:
                    break;
            }

            if((*it).getCompteurDeplacement() > 13)
            {
                (*it).setEtat(3); // on passe à l'état "explosé"

                if((*it).getType() == 1 || (*it).getType() == 2 || (*it).getType() == 3) (*it).setFeuilleSprite(buisson); // Uniquement les buissons
                else (*it).setFeuilleSprite(objets1); // on applique la nouvelle feuille de style

                (*it).initCompteurAnim();
            }
        }

        // Si l'objet est porté
        else if((*it).getEtat() == 1)
        {
            switch(perso->getOrientation())
            {
                case 0:
                    (*it).initPosition(perso->getPositionX() + getDecalageFenetreX(),perso->getPositionY() + getDecalageFenetreY() - 6);
                    break;

                case 1:
                    (*it).initPosition(perso->getPositionX() + getDecalageFenetreX(),perso->getPositionY() + getDecalageFenetreY() - 7);
                    break;

                case 2:
                    (*it).initPosition(perso->getPositionX() + getDecalageFenetreX() - 3,perso->getPositionY() + getDecalageFenetreY() - 6);
                    break;

                case 3:
                    (*it).initPosition(perso->getPositionX() + getDecalageFenetreX() + 3,perso->getPositionY() + getDecalageFenetreY() - 6);
                    break;

                default:
                    break;
            }

            if(!perso->getPorteObjet())
            {
                (*it).setEtat(2);
                (*it).setCompteurDeplacement(0);
                (*it).setOrientation(perso->getOrientation()); // On fixe l'orientation de l'objet lors qu'il passe à l'état "lancé"
            }
        }
    }

    return rechargerObj;
}


bool Carte::gestionTransformationElementCarteEnObjetPorte(Hero* perso,int* numObjetPorteAAjouter)
{
    int xPointInfluense = 0;
    int yPointInfluense = 0;
    bool ajouterObj = false;
    bool rechargerObj = false;
    int typeObjetPorte = 0;
    std::string nomObjetPorte = "";

    // Permet de définir un seul point en fonction de l'orientation du joueur.
    // Un seul objet du décors peut être transformé à la fois en objet porté.
    // Le joueur ne peut en effet porter qu'un seul objet à la fois
    switch(perso->getOrientation())
    {
        case 0:
            xPointInfluense = perso->getPositionX() + 8 + getDecalageFenetreX();
            yPointInfluense = perso->getPositionY() + 23 + getDecalageFenetreY();
            break;

        case 1:
            xPointInfluense = perso->getPositionX() + 8 + getDecalageFenetreX();
            yPointInfluense = perso->getPositionY() + 41 + getDecalageFenetreY();
            break;

        case 2:
            xPointInfluense = perso->getPositionX() - 2  + getDecalageFenetreX();
            yPointInfluense = perso->getPositionY() + 30 + getDecalageFenetreY();
            break;

        case 3:
            xPointInfluense = perso->getPositionX() + 17 + getDecalageFenetreX();
            yPointInfluense = perso->getPositionY() + 30 + getDecalageFenetreY();
            break;

        default:
            break;
    }

    // Si le perso est en état "action = vrai", il teste s'il peut agir sur un élément de décors
    if(perso->getAction() && !perso->getPorteObjet() && !perso->getCoupEpee())
    {
        // Si "action" devant un bosquet vert
        if(table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].x == 26 && table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].y == 0)
        {
            // On remplace le bosquet par une souche de bosquet
            table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].x = 29;
            typeObjetPorte = 1;
            nomObjetPorte = "Bosquet vert";
            ajouterObj = true;
        }

        // Si "action" devant un bosquet mauve
        else if(table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].x == 25 && table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].y == 9)
        {
            // On remplace le bosquet par une souche de bosquet
            table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].x = 26;
            typeObjetPorte = 2;
            nomObjetPorte = "Bosquet mauve";
            ajouterObj = true;
        }

        // Si "action" devant un bosquet marron
        else if(table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].x == 8 && table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].y == 1)
        {
            // On remplace le bosquet par une souche de bosquet
            table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].x = 9;
            typeObjetPorte = 3;
            nomObjetPorte = "Bosquet marron";
            ajouterObj = true;
        }

        // Si "action" devant une pierre gris clair
        else if(table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].x == 32 && table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].y == 0)
        {
            // On remplace le bosquet par une souche de bosquet
            table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].x = 1;
            table[xPointInfluense/LARGEUR_TILE][yPointInfluense/HAUTEUR_TILE - 1].y = 1;
            typeObjetPorte = 6;
            nomObjetPorte = "Pierre gris clair";
            ajouterObj = true;
        }

        // Dans le cas où l'une des conditions précédentes s'est vérifiée
        if(ajouterObj)
        {
            // ajoute un bosquet à la liste des objets de la carte
            O1.setNom(nomObjetPorte);
            O1.setType(typeObjetPorte);
            O1.setEtat(0);
            O1.initType();
            O1.initPosition(yPointInfluense,yPointInfluense);

            pnjObj.push_back(O1);

            // Modification du numéro d'objet porté à ajouter
            *numObjetPorteAAjouter = typeObjetPorte;

            // Temporaire à cet endroit -> modif de l'état "porte objet" du perso
            perso->setPorteObjet(true);

            // Puisque le perso a effectué une action sur l'un des objets de la carte, on indique qu'il faudra recharger les objet présents sur la carte
            // (l'un de ces objets ayant été modifié)
            rechargerObj = true;
        }

    }

    return rechargerObj;
}



void Carte::gestionInterractionsJoueurObjetsRecoltables(Hero* perso)
{
    // Si collision entre un objet récoltable
    list<ObjetRecolte> listeObjetsRecoltables = getObjetRecoltables();
    list<ObjetRecolte>::iterator it;

    for(it=listeObjetsRecoltables.begin();it != listeObjetsRecoltables.end();it++)
    {
        if(collisionAABB(perso->getPositionX()+getDecalageFenetreX()+3,
                        perso->getPositionX()+getDecalageFenetreX()+LARGEUR_PERSO-3,
                        perso->getPositionY()+getDecalageFenetreY()+11,
                        perso->getPositionY()+getDecalageFenetreY()+HAUTEUR_PERSO-11,
                        (*it).getPositionX(),
                        (*it).getPositionX() + (*it).getDimensionX(),
                        (*it).getPositionY(),
                        (*it).getPositionY() + (*it).getDimensionY()))
                        {
                            perso->setListeItemsDisponibles((*it).getType(), (*it).getValeur());
                            perso->setMAJObjetsRecolte((*it).getType(), (*it).getValeur());
                            (*it).setValeur(0);

                            // Eliminer l'objet récolté de la sauvegarde (n'a d'effet que si l'objet est bel et bien dans le fichier de sauvegarde
                            suppressionObjetRecolte((*it).getNom());
                        }
    }
    setObjetRecoltables(listeObjetsRecoltables);
}



// A factoriser éventuellement si la fonction grossit
void Carte::gestionInterractionsJoueurCarte(Hero* perso)
{
    int x0;
    int x1;
    int y0;
    int y1;
    int i,j;

        // Gestion des interrupteurs au sol (actionnés lorsque le personnage est dessus)

    // Initialisation de la zone d'influence au sol du joueur (les objets sur lesquels il peut marcher)
    x0 = perso->getPositionX() + 2 + getDecalageFenetreX();
    x1 = perso->getPositionX() + 13 + getDecalageFenetreX();
    y0 = perso->getPositionY() + 10 + getDecalageFenetreY();
    y1 = perso->getPositionY() + 21 + getDecalageFenetreY();

    // Initialisation des interrupteurs au sol
    for(i=getDecalageFenetreX()/LARGEUR_TILE;i<(getDecalageFenetreX() + LARGEUR_ECRAN)/LARGEUR_TILE;i++)
    {
        for(j=getDecalageFenetreY()/HAUTEUR_TILE;j<(getDecalageFenetreX() + HAUTEUR_ECRAN)/HAUTEUR_TILE;j++)
        {
            if(table[i][j].x == 0 && table[i][j].y == 44)
            {
                table[i][j].x = 39;
                table[i][j].y = 43;
            }
        }
    }

    // Nouvelle vérification d'actionnement des interrupteurs au sol
    if(table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 39 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 43)
    {
        table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 0;
        table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y = 44;
    }
    if(table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 39 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 43)
    {
        table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 0;
        table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE].y = 44;
    }
    if(table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 39 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 43)
    {
        table[x0/LARGEUR_TILE][y1/HAUTEUR_TILE].x = 0;
        table[x0/LARGEUR_TILE][y1/HAUTEUR_TILE].y = 44;
    }
    if(table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 39 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 43)
    {
        table[x1/LARGEUR_TILE][y1/HAUTEUR_TILE].x = 0;
        table[x1/LARGEUR_TILE][y1/HAUTEUR_TILE].y = 44;
    }

    // Passage des portes (ouverture)
    if(table[(x0)/LARGEUR_TILE][(y0)/HAUTEUR_TILE].x == 25 && table[(x0)/LARGEUR_TILE][(y0)/HAUTEUR_TILE].y == 17)
    {
        table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 31;
    }
    if(table[(x1)/LARGEUR_TILE][(y0)/HAUTEUR_TILE].x == 26 && table[(x1)/LARGEUR_TILE][(y0)/HAUTEUR_TILE].y == 17)
    {
        table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 32;
    }

    // Passage des portes (fermeture)
    if(table[(x0)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 1].x == 31 && table[(x0)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 1].y == 17)
    {
        table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 1].x = 25;
    }
    if(table[(x1)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 1].x == 32 && table[(x1)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 1].y == 17)
    {
        table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 1].x = 26;
    }


    // Initialisation de la zone d'influence au sol du joueur (les objets sur lesquels il peut marcher)
    x0 = perso->getPositionX() + 0 + getDecalageFenetreX();
    x1 = perso->getPositionX() + 16 + getDecalageFenetreX();
    y0 = perso->getPositionY() + 10 + getDecalageFenetreY();
    y1 = perso->getPositionY() + 21 + getDecalageFenetreY();

    // Passages des grandes portes (ouverture)
    if(table[(x0)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 1].x == 38 && table[(x0)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 1].y == 22 &&
            table[(x1)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 1].x == 39 && table[(x1)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 1].y == 22)
    {
        table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 1].x = 5;
        table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 1].y = 23;
        table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 1].x = 6;
        table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 1].y = 23;

        table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 2].x = 3;
        table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 2].y = 23;
        table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 2].x = 4;
        table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 2].y = 23;
    }
    if(x0 > 16 && y0 > 16)
    {
        if(table[(x0)/LARGEUR_TILE - 1][(y0)/HAUTEUR_TILE - 1].x == 38 && table[(x0)/LARGEUR_TILE - 1][(y0)/HAUTEUR_TILE - 1].y == 22 &&
                table[(x1)/LARGEUR_TILE - 1][(y0)/HAUTEUR_TILE - 1].x == 39 && table[(x1)/LARGEUR_TILE - 1][(y0)/HAUTEUR_TILE - 1].y == 22)
        {
            table[x0/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 1].x = 5;
            table[x0/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 1].y = 23;
            table[x1/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 1].x = 6;
            table[x1/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 1].y = 23;

            table[x0/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 2].x = 3;
            table[x0/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 2].y = 23;
            table[x1/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 2].x = 4;
            table[x1/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 2].y = 23;
        }

        // Passages des grandes portes (fermeture)
        if((table[(x0)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 2].x == 5 && table[(x0)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 2].y == 23) &&
                (table[(x1)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 2].x == 6 && table[(x1)/LARGEUR_TILE][(y0)/HAUTEUR_TILE - 2].y == 23))
        {
            table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 2].x = 38;
            table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 2].y = 22;
            table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 2].x = 39;
            table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 2].y = 22;

            table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 3].x = 34;
            table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE - 3].y = 22;
            table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 3].x = 35;
            table[x1/LARGEUR_TILE][y0/HAUTEUR_TILE - 3].y = 22;
        }
        if((table[(x0)/LARGEUR_TILE - 1][(y0)/HAUTEUR_TILE - 2].x == 5 && table[(x0)/LARGEUR_TILE - 1][(y0)/HAUTEUR_TILE - 2].y == 23) &&
                (table[(x1)/LARGEUR_TILE - 1][(y0)/HAUTEUR_TILE - 2].x == 6 && table[(x1)/LARGEUR_TILE - 1][(y0)/HAUTEUR_TILE - 2].y == 23))
        {
            table[x0/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 2].x = 38;
            table[x0/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 2].y = 22;
            table[x1/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 2].x = 39;
            table[x1/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 2].y = 22;

            table[x0/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 3].x = 34;
            table[x0/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 3].y = 22;
            table[x1/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 3].x = 35;
            table[x1/LARGEUR_TILE - 1][y0/HAUTEUR_TILE - 3].y = 22;
        }
    }

}

void Carte::gestionInterractionsArmeJoueurEnnemis(Hero* perso)
{
    list<Ennemi>::iterator it;

    // Déclaration des variables de la zone d'influence des armes
    int x0 = perso->getArmeActive().getZoneActive().x;
    int x1 = perso->getArmeActive().getZoneActive().y;
    int y0 = 0;
    int y1 = 0;

    // Gestion des collisions entre arme joueur et ennemis
    for(it=pnjEnnemi.begin();it != pnjEnnemi.end();it++)
    {
        // Si utilisation de l'épée ou du marteau (9)
        if(perso->getCoupEpee() || (perso->getUtilisationArme() && perso->getItemActif() == 9))
        {
            // Définition de la zone d'influence de l'arme (calqué sur sa position à l'écran)
            // VOIR S'IL NE FAUDRAIT PAS REDEFINIR CES VALEURS POUR CERTAINES ARMES COMME LE MAILLET
            x0 = perso->getArmeActive().getZoneActive().x;
            y0 = perso->getArmeActive().getZoneActive().y;
            x1 = x0 + perso->getArmeActive().getZoneActive().w;
            y1 = y0 + perso->getArmeActive().getZoneActive().h;

            if(!(*it).getTouche() && collisionAABB(x0,
                                                x1,
                                                y0,
                                                y1,
                                                (*it).getPositionX() - getDecalageFenetreX(),
                                                (*it).getPositionX() - getDecalageFenetreX() + (*it).getDimensionX(),
                                                (*it).getPositionY() - getDecalageFenetreY(),
                                                (*it).getPositionY() - getDecalageFenetreY() + (*it).getDimensionY()))
                                                {
                                                    (*it).setTouche(true);
                                                }
        }
    }
}

void Carte::gestionInterractionsArmeJoueurProjectiles(Hero* perso)
{
    int x0;
    int x1;
    int y0;
    int y1;

    // Actualisation des informations du PNJ projectile
    gestionActualisationInformationsProjectiles(perso);

    // Utilisation d'une arme secondaire
    if(perso->getUtilisationArme())
    {
        list<Projectile>::iterator it;
        int nbProjectileIdentique = 1;

        for(it=pnjProjectile.begin();it != pnjProjectile.end();it++)
        {
            if(perso->getItemActif() == (*it).getType()) nbProjectileIdentique++;
        }

        // Initialisation de la position du nouveau projectile
        initPositionNouveauProjectile(perso);

        // Finalisation de l'initialisation du projectile et ajout sur la carte
        switch(perso->getItemActif())
        {
            case 1: // Arc (Création d'un projectile)
                finalisationCreationNouveauProjectileFleche(perso,nbProjectileIdentique);
                break;

            /*case 2: // Grappin
                // Création d'un projectile

                // METTRE EN PLACE LE VERROU POUR UN SEUL GRAPPIN

                P1.setType(2);
                P1.initType(grappin);
                pnjProjectile.push_back(P1);
                perso->setUtilisationArme(false);
                perso->setEnvoiProjectile(false);
                break;*/

            case 3: // Bombe
                finalisationCreationNouveauProjectileBombe(perso,nbProjectileIdentique);
                break;

            case 4: // Boomrang (Création d'un projectile)
                finalisationCreationNouveauProjectileBoomrang(perso,nbProjectileIdentique);
                break;

            case 5: // Baguette de feu (Création d'un projectile)
                finalisationCreationNouveauProjectileFeu(perso,nbProjectileIdentique);
                break;

            case 6: // Baguette de glace (Création d'un projectile)
                finalisationCreationNouveauProjectileGlace(perso,nbProjectileIdentique);
                break;

            case 7: // Lanterne (Création d'un projectile)
                finalisationCreationNouveauProjectileFeuLampe(perso,nbProjectileIdentique);
                break;

            default:
                break;
        }
    }
}

void Carte::gestionActualisationInformationsProjectiles(Hero* perso)
{
    list<Projectile>::iterator it;

    for(it=pnjProjectile.begin();it != pnjProjectile.end();it++)
    {
        (*it).setDecalageXAvecLeHero(perso->getPositionX()+getDecalageFenetreX());
        (*it).setDecalageYAvecLeHero(perso->getPositionY()+getDecalageFenetreY());

        // Détermine si le projectile est à l'arrêt (état utilisé ensuite pas l'IA qui déterminera si cela est en contradiction avec la consigne donnée au projectile
        if(!(*it).getBougeHaut() && !(*it).getBougeBas() && !(*it).getBougeGauche() && !(*it).getBougeDroite())
        {
            (*it).setArretProjectile(true);
        }
    }
}

void Carte::initPositionNouveauProjectile(Hero* perso)
{
    switch(perso->getOrientation())
    {
        case 0:
            if(perso->getItemActif() == 1 || perso->getItemActif() == 5 || perso->getItemActif() == 6)
            {
                P1.initPosition(perso->getPositionX() + getDecalageFenetreX(),((perso->getPositionY() + getDecalageFenetreY() - 8)/8)*8);
            }
            else
            {
                P1.initPosition(perso->getPositionX() + getDecalageFenetreX(),perso->getPositionY() + getDecalageFenetreY() - 8);
            }
            break;

        case 1:
            if(perso->getItemActif() == 1 || perso->getItemActif() == 5 || perso->getItemActif() == 6)
            {
                P1.initPosition(perso->getPositionX() + getDecalageFenetreX(),((perso->getPositionY() + getDecalageFenetreY() + 24)/8)*8);
            }
            else
            {
                P1.initPosition(perso->getPositionX() + getDecalageFenetreX(),perso->getPositionY() + getDecalageFenetreY() + 20);
            }
            break;

        case 2:
            if(perso->getItemActif() == 1 || perso->getItemActif() == 5 || perso->getItemActif() == 6)
            {
                P1.initPosition(((perso->getPositionX() + getDecalageFenetreX() - 16)/8)*8,perso->getPositionY() + getDecalageFenetreY() + 8);
            }
            else
            {
                P1.initPosition(perso->getPositionX() + getDecalageFenetreX() - 16,perso->getPositionY() + getDecalageFenetreY() + 8);
            }
            break;

        case 3:
            if(perso->getItemActif() == 1 || perso->getItemActif() == 5 || perso->getItemActif() == 6)
            {
                P1.initPosition(((perso->getPositionX() + getDecalageFenetreX() + 16)/8)*8,perso->getPositionY() + getDecalageFenetreY() + 8);
            }
            else
            {
                P1.initPosition(perso->getPositionX() + getDecalageFenetreX() + 16,perso->getPositionY() + getDecalageFenetreY() + 8);
            }
            break;

        default:
            break;
    }
}

void Carte::finalisationCreationNouveauProjectileFleche(Hero* perso, int nbProjectileIdentique)
{
    // Permet d'introduire un décalage entre le moment où l'arme s'affiche et le moment où le projectile est lancé
    if(perso->getCompteurAnim() > 8)
    {
        // METTRE EN PLACE LE VERROU POUR UNE FLECHE A LA FOIS
        if(nbProjectileIdentique < 3 && perso->getEnvoiProjectile() && perso->getFleche() > 0)
        {
            P1.setType(1);
            P1.initType(projectiles1);
            P1.setPuissance(perso->getValeurItemActif());
            P1.setOrientation(perso->getOrientation());
            pnjProjectile.push_back(P1);

            // On enlève une flèche portée par le joueur
            perso->setFleche(perso->getFleche() - 1);
        }
        perso->setEnvoiProjectile(false);
    }
}

void Carte::finalisationCreationNouveauProjectileGrappin(Hero* perso, int nbProjectileIdentique)
{

}

void Carte::finalisationCreationNouveauProjectileBombe(Hero* perso, int nbProjectileIdentique)
{
    if(perso->getBombe() > 0 && nbProjectileIdentique < 4)
    {
        P1.setType(3);
        P1.initType(bombe);
        pnjProjectile.push_back(P1);
        perso->setBombe(perso->getBombe() - 1);
    }
    perso->setUtilisationArme(false);
    perso->setEnvoiProjectile(false);
}

void Carte::finalisationCreationNouveauProjectileBoomrang(Hero* perso, int nbProjectileIdentique)
{
    if(nbProjectileIdentique == 1 && perso->getEnvoiProjectile())
    {
        P1.setType(4);
        P1.initType(boomerang);
        P1.setPuissance(perso->getValeurItemActif());
        P1.setOrientation(perso->getOrientation());
        pnjProjectile.push_back(P1);
    }
    perso->setUtilisationArme(false);
    perso->setEnvoiProjectile(false);
}

void Carte::finalisationCreationNouveauProjectileFeu(Hero* perso, int nbProjectileIdentique)
{
    if(perso->getCompteurAnim() > 8)
    {
        // METTRE EN PLACE LE VERROU POUR PAS PLUS DEUX PROJECTILES A LA FOIS
        if(perso->getMagie() > 0 && nbProjectileIdentique < 3 && perso->getEnvoiProjectile())
        {
            P1.setType(5);
            P1.initType(magie);
            P1.setPuissance(perso->getValeurItemActif());
            P1.setOrientation(perso->getOrientation());
            pnjProjectile.push_back(P1);
            perso->setMagie(perso->getMagie() - 1);
        }
        //perso->setUtilisationArme(false);
        perso->setEnvoiProjectile(false);
    }
}

void Carte::finalisationCreationNouveauProjectileGlace(Hero* perso, int nbProjectileIdentique)
{
    // Permet d'introduire un décalage entre le moment où l'arme s'affiche et le moment où le projectile est lancé
    if(perso->getCompteurAnim() > 8)
    {
        // METTRE EN PLACE LE VERROU POUR PAS PLUS DEUX PROJECTILES A LA FOIS
        if(perso->getMagie() > 0 && nbProjectileIdentique < 3 && perso->getEnvoiProjectile())
        {
            P1.setType(6);
            P1.initType(magie);
            P1.setPuissance(perso->getValeurItemActif());
            P1.setOrientation(perso->getOrientation());
            pnjProjectile.push_back(P1);
            perso->setMagie(perso->getMagie() - 1);
        }
        //perso->setUtilisationArme(false);
        perso->setEnvoiProjectile(false);
    }
}

void Carte::finalisationCreationNouveauProjectileFeuLampe(Hero* perso, int nbProjectileIdentique)
{
    if(perso->getMagie() > 0 && nbProjectileIdentique == 1 && perso->getEnvoiProjectile())
    {
        P1.setType(7);
        P1.initType(feu);
        pnjProjectile.push_back(P1);
        perso->setMagie(perso->getMagie() - 1);
    }
    perso->setUtilisationArme(false);
    perso->setEnvoiProjectile(false);
}

// A factoriser
void Carte::gestionInterractionsArmeJoueurCarte(Hero* perso)
{
    int x0;
    int x1;
    int y0;
    int y1;

    gestionZoneInfluenceArmeJoueurCarte(perso,&x0,&x1,&y0,&y1);

    // Utilisation de l'épée
    if(perso->getCoupEpee())
    {
        // Action sur les bornes interrupteur rouge ou bleu
        if((table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 11 || table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 12) && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 61 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile >= 0)
        {
            if(getInterRouge()) setInterRouge(false);
            else setInterRouge(true);
            table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = -20;
        }
        if((table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 11 || table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 12) && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 61 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile >= 0)
        {
            if(getInterRouge()) setInterRouge(false);
            else setInterRouge(true);
            table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = -20;
        }
        if((table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 11 || table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 12) && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 61 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile >= 0)
        {
            if(getInterRouge()) setInterRouge(false);
            else setInterRouge(true);
            table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = -20;
        }
        if((table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 11 || table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 12) && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 61 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile >= 0)
        {
            if(getInterRouge()) setInterRouge(false);
            else setInterRouge(true);
            table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = -20;
        }

        // Action sur les bosquets vert (l'action sur "table" entraine un effet sur "air" cas le bosquet qui explose doit être au dessus de link)
        if(table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 26 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 0)
        {
            table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 29;
            table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = 1;
        }
        if(table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 26 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 0)
        {
            table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 29;
            table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = 1;
        }
        if(table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 26 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 0)
        {
            table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 29;
            table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = 1;
        }
        if(table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 26 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 0)
        {
            table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 29;
            table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = 1;
        }

        // Action sur les bosquets marron
        if(table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 8 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 1) {table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 9;table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = 1;}
        if(table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 8 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 1) {table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 9;table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = 1;}
        if(table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 8 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 1) {table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 9;table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = 1;}
        if(table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 8 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 1) {table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 9;table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = 1;}

        // Action sur les bosquets mauve
        if(table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 25 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 9) {table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 26;table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = 1;}
        if(table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 25 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 9) {table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 26;table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = 1;}
        if(table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 25 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 9) {table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 26;table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = 1;}
        if(table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 25 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 9) {table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 26;table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = 1;}
    }

    // Utilisation d'une arme secondaire
    if(perso->getUtilisationArme())
    {
        // Finalisation de l'initialisation du projectile et ajout sur la carte
        switch(perso->getItemActif())
        {
            case 9: // Utilisation du maillet (Pas de créaton de projectile)

                // Action sur les plots 1
                if(table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 6 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 3)
                {
                    table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 7;
                    sauvegardeModifsCarte(1,x0/LARGEUR_TILE,y0/HAUTEUR_TILE,7,3);
                }
                if(table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 6 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 3)
                {
                    table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 7;
                    sauvegardeModifsCarte(1,x1/LARGEUR_TILE,y0/HAUTEUR_TILE,7,3);
                }
                if(table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 6 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 3)
                {
                    table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 7;
                    sauvegardeModifsCarte(1,x0/LARGEUR_TILE,y1/HAUTEUR_TILE,7,3);
                }
                if(table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 6 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 3)
                {
                    table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 7;
                    sauvegardeModifsCarte(1,x1/LARGEUR_TILE,y1/HAUTEUR_TILE,7,3);
                }

                // Action sur les plots 2
                if(table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 33 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 9) table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 34;
                if(table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 33 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 9) table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 34;
                if(table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 33 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 9) table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 34;
                if(table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 33 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 9) table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 34;

                // Action sur les plots 3
                if(table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 27 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 24) table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 28;
                if(table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 27 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 24) table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 28;
                if(table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 27 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 24) table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 28;
                if(table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 27 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 24) table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 28;

                // Action sur les plots 4
                if(table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 1 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 44) table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 2;
                if(table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 1 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 44) table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x = 2;
                if(table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 1 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 44) table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 2;
                if(table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 1 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 44) table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x = 2;

                // Action sur les bornes interrupteur rouge ou bleu
                if((table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 11 || table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 12) && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 61 && table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile >= 0)
                {
                    if(getInterRouge()) setInterRouge(false);
                    else setInterRouge(true);
                    table[x0/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = -20;
                }
                if((table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 11 || table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].x == 12) && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].y == 61 && table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile >= 0)
                {
                    if(getInterRouge()) setInterRouge(false);
                    else setInterRouge(true);
                    table[(x1)/LARGEUR_TILE][y0/HAUTEUR_TILE].compteurTile = -20;
                }
                if((table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 11 || table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 12) && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 61 && table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile >= 0)
                {
                    if(getInterRouge()) setInterRouge(false);
                    else setInterRouge(true);
                    table[x0/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = -20;
                }
                if((table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 11 || table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].x == 12) && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].y == 61 && table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile >= 0)
                {
                    if(getInterRouge()) setInterRouge(false);
                    else setInterRouge(true);
                    table[(x1)/LARGEUR_TILE][(y1)/HAUTEUR_TILE].compteurTile = -20;
                }
            break;

            default:
                break;
        }
    }
}

void Carte::gestionZoneInfluenceArmeJoueurCarte(Hero* perso,int* x0,int* x1,int* y0,int* y1)
{
    // Note importante : la zone d'influence des armes vis à vis des ennemis ne sera pas nécessairement identique
    switch(perso->getOrientation())
    {
        case 0:
            *x0 = perso->getPositionX() + 5 + getDecalageFenetreX();
            *x1 = perso->getPositionX() + 11 + getDecalageFenetreX();
            *y0 = perso->getPositionY() - 8 + getDecalageFenetreY();
            *y1 = perso->getPositionY() + 8 + getDecalageFenetreY();
            break;

        case 1:
            *x0 = perso->getPositionX() + 5 + getDecalageFenetreX();
            *x1 = perso->getPositionX() + 11 + getDecalageFenetreX();
            *y0 = perso->getPositionY() + 16 + getDecalageFenetreY();
            *y1 = perso->getPositionY() + 32 + getDecalageFenetreY();
            break;

        case 2:
            *x0 = perso->getPositionX() - 8 + getDecalageFenetreX();
            *x1 = perso->getPositionX() + 8 + getDecalageFenetreX();
            *y0 = perso->getPositionY() + 13 + getDecalageFenetreY();
            *y1 = perso->getPositionY() + 19 + getDecalageFenetreY();
            break;

        case 3:
            *x0 = perso->getPositionX() + 8 + getDecalageFenetreX();
            *x1 = perso->getPositionX() + 24 + getDecalageFenetreX();
            *y0 = perso->getPositionY() + 13 + getDecalageFenetreY();
            *y1 = perso->getPositionY() + 19 + getDecalageFenetreY();
            break;

        default:
            break;
    }
}

void Carte::gestionInterractionsEnnemisEnnemis()
{
    list<Ennemi>::iterator it;
    list<Ennemi>::iterator it2;

    for(it=pnjEnnemi.begin();it != pnjEnnemi.end();it++)
    {
        for(it2=it;it2 != pnjEnnemi.end();it2++)
        {
            // Gestion des collisions entre ennemis uniquement
            if(it2 != it && !(*it).getTouche() && !(*it2).getTouche())
            {
                if((*it).getPositionY() == (*it2).getPositionY())
                {
                    if((*it).getPositionX() > (*it2).getPositionX()-15 && (*it).getPositionX() < (*it2).getPositionX())
                    {
                        (*it).setBougeDroite(false);
                        (*it2).setBougeGauche(false);
                    }
                    else if((*it2).getPositionX() > (*it).getPositionX()-15 && (*it2).getPositionX() < (*it).getPositionX())
                    {
                        (*it).setBougeGauche(false);
                        (*it2).setBougeDroite(false);
                    }
                }

                if((*it).getPositionX() == (*it2).getPositionX())
                {
                    if((*it).getPositionY() > (*it2).getPositionY()-16 && (*it).getPositionY() < (*it2).getPositionY())
                    {
                        (*it).setBougeBas(false);
                        (*it2).setBougeHaut(false);
                    }
                    else if((*it2).getPositionY() > (*it).getPositionY()-16 && (*it2).getPositionY() < (*it).getPositionY())
                    {
                        (*it).setBougeHaut(false);
                        (*it2).setBougeBas(false);
                    }
                }

                if((*it).getPositionX() == (*it2).getPositionX() && (*it).getPositionY() == (*it2).getPositionY())
                {
                    (*it2).setImmobiliser();
                }
            }
        }
    }
}

void Carte::gestionInterractionsEnnemisAmis()
{
    // Cas improbable (ces deux types de PNJ ne se trouvant pas sur les mêmes cartes)
}

void Carte::gestionInterractionsEnnemisObjets()
{
    list<Ennemi>::iterator it;
    list<Objet>::iterator it2;

    // Gestion des collisions des PNJ ennemis avec les objets sur la carte
    for(it=pnjEnnemi.begin();it != pnjEnnemi.end();it++)
    {
        for(it2=pnjObj.begin();it2 != pnjObj.end();it2++)
        {
            if(collisionAABB((*it).getPositionX(),
                        (*it).getPositionX() + (*it).getDimensionX(),
                        (*it).getPositionY(),
                        (*it).getPositionY() + (*it).getDimensionY(),
                        (*it2).getPositionX(),
                        (*it2).getPositionX() + (*it2).getDimensionX(),
                        (*it2).getPositionY(),
                        (*it2).getPositionY() + (*it2).getDimensionY()))
                        {
                            if((*it).getPositionX() > (*it2).getPositionX() - (*it).getDimensionX() && (*it).getPositionX() < (*it2).getPositionX())
                            {
                                (*it).setBougeDroite(false);
                                (*it2).setBougeGauche(false);
                            }
                            else if((*it2).getPositionX() > (*it).getPositionX() - (*it2).getDimensionX() && (*it2).getPositionX() < (*it).getPositionX())
                            {
                                (*it2).setBougeDroite(false);
                                (*it).setBougeGauche(false);
                            }

                            if((*it).getPositionY() > (*it2).getPositionY() - (*it).getDimensionY() && (*it).getPositionY() < (*it2).getPositionY())
                            {
                                (*it).setBougeBas(false);
                                (*it2).setBougeHaut(false);
                            }
                            else if((*it2).getPositionY() > (*it).getPositionY() - (*it2).getDimensionX() + 8  && (*it2).getPositionY() < (*it).getPositionY())
                            {
                                (*it2).setBougeBas(false);
                                (*it).setBougeHaut(false);
                            }

                            // Dans le cas où l'objet est lancé, sa collision avec un ennemi cause des dégâts à l'ennemi
                            if((*it2).getEtat() == 2 && !(*it).getTouche() )
                            {
                                (*it2).setEtat(3);
                                if((*it2).getType() == 1 || (*it2).getType() == 2 || (*it2).getType() == 3) (*it2).setFeuilleSprite(buisson); // Uniquement les buissons
                                else (*it2).setFeuilleSprite(objets1); // on applique la nouvelle feuille de style
                                (*it2).initCompteurAnim();
                                (*it).setTouche(true);
                            }
                        }
        }
    }
}

void Carte::gestionInterractionsEnnemisObjetsRecoltales()
{

}

// A factoriser ultérieurement (confirmer si réellement utile)
void Carte::gestionInterractionsEnnemisProjectiles()
{
    int x0, x1, y0, y1;

    list<Projectile>::iterator it;
    list<Ennemi>::iterator it2;

    for(it=pnjProjectile.begin();it != pnjProjectile.end();it++)
    {
        switch((*it).getType())
        {
            case 1: // flèches

                // Initialisation de la zone d'influence au sol de la flèche
                x0 = (*it).getPositionX() + 4;
                x1 = (*it).getPositionX() + (*it).getDimensionX() - 4;
                y0 = (*it).getPositionY() + 4;
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 4;

                // Test de la présence de chaque PNJ ennemis dans la zone d'influence de la flèche
                for(it2=pnjEnnemi.begin();it2 != pnjEnnemi.end();it2++)
                {
                    if((*it2).getEnergie() > 0 && !(*it2).getTouche() && collisionAABB(
                                                                            x0,
                                                                            x1,
                                                                            y0,
                                                                            y1,
                                                                            (*it2).getPositionX(),
                                                                            (*it2).getPositionX() + (*it2).getDimensionX(),
                                                                            (*it2).getPositionY(),
                                                                            (*it2).getPositionY() + (*it2).getDimensionY()))
                    {
                        // adaptation de l'orientation du pnj avant qu'il se fasse toucher
                        if((*it).getBougeHaut()) (*it2).setOrientation(1);
                        else if((*it).getBougeBas()) (*it2).setOrientation(0);
                        else if((*it).getBougeGauche()) (*it2).setOrientation(3);
                        else if((*it).getBougeDroite()) (*it2).setOrientation(2);

                        (*it).setArretProjectile(true);
                        (*it2).setTouche(true);
                    }
                }
                break;

            case 3: // bombe

                // Initialisation de la zone d'influence au sol de la bombe
                x0 = (*it).getPositionX() - 8;
                x1 = (*it).getPositionX() + (*it).getDimensionX() + 32;
                y0 = (*it).getPositionY() - 8;
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 32;

                // Test des éléments de décors dans la zone d'influence de la bombe lorsqu'elle explose
                if((*it).getTimeToLive() == 10)
                {
                    // Test de la présence de chaque PNJ ennemis dans la zone d'influence de la bombe
                    for(it2=pnjEnnemi.begin();it2 != pnjEnnemi.end();it2++)
                    {
                        // Test si les PNJ sont affectés ou non par l'explosion de la bombe
                        if((*it2).getEnergie() > 0 && !(*it2).getTouche() && collisionAABB(
                                                                                x0,
                                                                                x1,
                                                                                y0,
                                                                                y1,
                                                                                (*it2).getPositionX(),
                                                                                (*it2).getPositionX() + (*it2).getDimensionX(),
                                                                                (*it2).getPositionY(),
                                                                                (*it2).getPositionY() + (*it2).getDimensionY()))
                        {
                            // Immobilisation du PNJ
                            (*it2).setImmobiliser();

                            // Mise à jour de l'orientation des PNJ s'il sont touchés lors de l'explosion d'une bombe
                            if((*it2).getPositionX() > x1) (*it2).setOrientation(2); // PNJ à droite de la bombe
                            else if((*it2).getPositionX() + (*it2).getDimensionX() < x0) (*it2).setOrientation(3); // PNJ à gauche de la bombe
                            if((*it2).getPositionY() > y1) (*it2).setOrientation(0); // PNJ plus bas que la bombe
                            else if((*it2).getPositionY() + (*it2).getDimensionY() < y0) (*it2).setOrientation(1); // PNJ plus haut que la bombe

                            // Sortie du mode attaque
                            (*it2).setAttaque(false);

                            // Le passer au statut touché
                            (*it2).setTouche(true);
                        }
                    }
                }
                break;

            case 4: // Boomerang

                // Initialisation de la zone d'influence au sol du boomerang
                x0 = (*it).getPositionX();
                x1 = (*it).getPositionX() + (*it).getDimensionX();
                y0 = (*it).getPositionY();
                y1 = (*it).getPositionY() + (*it).getDimensionY();

                // Test de la présence de chaque PNJ ennemis dans la zone d'influence du boomerang
                for(it2=pnjEnnemi.begin();it2 != pnjEnnemi.end();it2++)
                {
                    if((*it2).getEnergie() > 0 && !(*it2).getTouche() && collisionAABB(x0,
                                                                            x1,
                                                                            y0,
                                                                            y1,
                                                                            (*it2).getPositionX(),
                                                                            (*it2).getPositionX() + (*it2).getDimensionX(),
                                                                            (*it2).getPositionY(),
                                                                            (*it2).getPositionY() + (*it2).getDimensionY()))
                    {
                        // adaptation de l'orientation du pnj avant qu'il se fasse toucher
                        if((*it).getBougeHaut()) (*it2).setOrientation(1);
                        else if((*it).getBougeBas()) (*it2).setOrientation(0);
                        else if((*it).getBougeGauche()) (*it2).setOrientation(3);
                        else if((*it).getBougeDroite()) (*it2).setOrientation(2);

                        (*it).setArretProjectile(true);
                        (*it2).setTouche(true);
                    }
                }

                break;

            case 5: // Flamme baguette de feu

                // Initialisation de la zone d'influence au sol
                x0 = (*it).getPositionX() - 1;
                x1 = (*it).getPositionX() + (*it).getDimensionX() + 16;
                y0 = (*it).getPositionY() - 1;
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 16;

                // Test de la présence de chaque PNJ ennemis dans la zone d'influence
                for(it2=pnjEnnemi.begin();it2 != pnjEnnemi.end();it2++)
                {
                    if((*it2).getEnergie() > 0 && !(*it2).getTouche() && collisionAABB(x0,
                                                                            x1,
                                                                            y0,
                                                                            y1,
                                                                            (*it2).getPositionX(),
                                                                            (*it2).getPositionX() + (*it2).getDimensionX(),
                                                                            (*it2).getPositionY(),
                                                                            (*it2).getPositionY() + (*it2).getDimensionY()))
                    {
                        // adaptation de l'orientation du pnj avant qu'il se fasse toucher
                        if((*it).getBougeHaut()) (*it2).setOrientation(1);
                        else if((*it).getBougeBas()) (*it2).setOrientation(0);
                        else if((*it).getBougeGauche()) (*it2).setOrientation(3);
                        else if((*it).getBougeDroite()) (*it2).setOrientation(2);

                        (*it).setArretProjectile(true);
                        (*it2).setTouche(true);
                    }
                }

                break;

            case 6: // Glace baguette de glace

                // Initialisation de la zone d'influence au sol
                x0 = (*it).getPositionX() - 1;
                x1 = (*it).getPositionX() + (*it).getDimensionX() + 16;
                y0 = (*it).getPositionY() - 1;
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 16;

                // Test de la présence de chaque PNJ ennemis dans la zone d'influence
                for(it2=pnjEnnemi.begin();it2 != pnjEnnemi.end();it2++)
                {
                    if((*it2).getEnergie() > 0 && !(*it2).getTouche() && collisionAABB(x0,
                                                                                x1,
                                                                                y0,
                                                                                y1,
                                                                                (*it2).getPositionX(),
                                                                                (*it2).getPositionX() + (*it2).getDimensionX(),
                                                                                (*it2).getPositionY(),
                                                                                (*it2).getPositionY() + (*it2).getDimensionY()))
                    {
                        // adaptation de l'orientation du pnj avant qu'il se fasse toucher
                        if((*it).getBougeHaut()) (*it2).setOrientation(1);
                        else if((*it).getBougeBas()) (*it2).setOrientation(0);
                        else if((*it).getBougeGauche()) (*it2).setOrientation(3);
                        else if((*it).getBougeDroite()) (*it2).setOrientation(2);

                        (*it).setArretProjectile(true);
                        (*it2).setTouche(true);
                    }
                }

                break;

            case 7: // Feu de la lampe

                // Initialisation de la zone d'influence au sol de la flamme de la lampe
                x0 = (*it).getPositionX();
                x1 = (*it).getPositionX() + (*it).getDimensionX() + 8;
                y0 = (*it).getPositionY();
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 8;

                // Moment où la flamme est active
                if((*it).getTimeToLive() == 5 )
                {
                    // Test de la présence de chaque PNJ ennemis dans la zone d'influence de la flamme de la lampe
                    for(it2=pnjEnnemi.begin();it2 != pnjEnnemi.end();it2++)
                    {
                        if((*it2).getEnergie() > 0 && !(*it2).getTouche() && collisionAABB(x0,
                                                                                    x1,
                                                                                    y0,
                                                                                    y1,
                                                                                    (*it2).getPositionX(),
                                                                                    (*it2).getPositionX() + (*it2).getDimensionX(),
                                                                                    (*it2).getPositionY(),
                                                                                    (*it2).getPositionY() + (*it2).getDimensionY()))
                        {
                            (*it2).setTouche(true);
                        }
                    }
                }
                break;

            default:
                break;
        }
    }
}


void Carte::gestionInterractionsAmisAmis()
{
    list<Ami>::iterator it;
    list<Ami>::iterator it2;

    // Gestion globale des collisions entre amis
    for(it=pnjAmi.begin();it != pnjAmi.end();it++)
    {
        for(it2=it;it2 != pnjAmi.end();it2++)
        {
            if(it2 != it)
            {
                if((*it).getPositionY() == (*it2).getPositionY())
                {
                    if((*it).getPositionX() > (*it2).getPositionX()-15 && (*it).getPositionX() < (*it2).getPositionX())
                    {
                        (*it).setBougeDroite(false);
                        (*it2).setBougeGauche(false);
                    }
                    else if((*it2).getPositionX() > (*it).getPositionX()-15 && (*it2).getPositionX() < (*it).getPositionX())
                    {
                        (*it2).setBougeDroite(false);
                        (*it).setBougeGauche(false);
                    }
                }

                if((*it).getPositionX() == (*it2).getPositionX())
                {
                    if((*it).getPositionY() > (*it2).getPositionY()-16 && (*it).getPositionY() < (*it2).getPositionY())
                    {
                        (*it).setBougeBas(false);
                        (*it2).setBougeHaut(false);
                    }
                    else if((*it2).getPositionY() > (*it).getPositionY()-16 && (*it2).getPositionY() < (*it).getPositionY())
                    {
                        (*it2).setBougeBas(false);
                        (*it).setBougeHaut(false);
                    }
                }

                if((*it).getPositionX() == (*it2).getPositionX() && (*it).getPositionY() == (*it2).getPositionY())
                {
                    (*it2).setImmobiliser();
                }
            }
        }
    }
}

void Carte::gestionInterractionsAmisObjets()
{
    // Cas improbable
}

void Carte::gestionInterractionsAmisObjetsRecoltables()
{
    // Cas improbable
}

void Carte::gestionInterractionsAmisProjectiles()
{
    // Cas improbable
}

void Carte::gestionInterractionsObjetsObjetsRecoltables()
{

}

void Carte::gestionInterractionsObjetsProjectiles()
{

}

// A refactoriser ensuite
void Carte::gestionInterractionsObjetsProjectilesCarte()
{
    int x0, x1, y0, y1;
    int i, j;

    list<Projectile>::iterator it;

    for(it=pnjProjectile.begin();it != pnjProjectile.end();it++)
    {
        switch((*it).getType())
        {
            case 1: // flèches

                // Initialisation de la zone d'influence au sol de la flèche
                x0 = (*it).getPositionX() + 4;
                x1 = (*it).getPositionX() + (*it).getDimensionX() - 4;
                y0 = (*it).getPositionY() + 4;
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 4;

                break;

            case 3: // bombe

                // Initialisation de la zone d'influence au sol de la bombe
                x0 = (*it).getPositionX() - 8;
                x1 = (*it).getPositionX() + (*it).getDimensionX() + 32;
                y0 = (*it).getPositionY() - 8;
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 32;

                // Test des éléments de décors dans la zone d'influence de la bombe lorsqu'elle explose
                if((*it).getTimeToLive() == 10)
                {
                    // Test et modification des éléments de décors dans la zone d'influence de la bombe
                    for(i=x0/LARGEUR_TILE;i<x1/LARGEUR_TILE;i++)
                    {
                        for(j=y0/HAUTEUR_TILE;j<y1/HAUTEUR_TILE;j++)
                        {
                            // Grotte falaise #1 (gauche)
                            if(table[i][j].x == 12 && table[i][j].y == 13)
                            {
                                table[i][j].x = 16;
                                sauvegardeModifsCarte(1,i,j,16,13);

                                table[i+1][j].x = 17;
                                sauvegardeModifsCarte(1,i+1,j,17,13);

                                table[i][j-1].x = 14;
                                sauvegardeModifsCarte(1,i,j-1,14,13);

                                table[i+1][j-1].x = 15;
                                sauvegardeModifsCarte(1,i+1,j-1,15,13);

                                air[i][j-1].x = 41;
                                air[i][j-1].y = 8;
                                sauvegardeModifsCarte(2,i,j-1,41,8);

                                air[i+1][j-1].x = 42;
                                air[i+1][j-1].y = 8;
                                sauvegardeModifsCarte(2,i+1,j-1,42,8);
                            }
                            // Grotte falaise #1 (droite)
                            if(table[i][j].x == 13 && table[i][j].y == 13)
                            {
                                table[i-1][j].x = 16;
                                sauvegardeModifsCarte(1,i-1,j,16,13);

                                table[i][j].x = 17;
                                sauvegardeModifsCarte(1,i,j,17,13);

                                table[i-1][j-1].x = 14;
                                sauvegardeModifsCarte(1,i-1,j-1,14,13);

                                table[i][j-1].x = 15;
                                sauvegardeModifsCarte(1,i,j-1,15,13);

                                air[i-1][j-1].x = 41;
                                air[i-1][j-1].y = 8;
                                sauvegardeModifsCarte(2,i-1,j-1,41,8);

                                air[i][j-1].x = 42;
                                air[i][j-1].y = 8;
                                sauvegardeModifsCarte(2,i,j-1,42,8);
                            }

                            // Grotte falaise #2 (gauche)
                            if(table[i][j].x == 30 && table[i][j].y == 19)
                            {
                                table[i][j].x = 32;
                                sauvegardeModifsCarte(1,i,j,32,19);

                                table[i+1][j].x = 33;
                                sauvegardeModifsCarte(1,i+1,j,33,19);

                                table[i][j-1].x = 14;
                                sauvegardeModifsCarte(1,i,j-1,14,13);

                                table[i+1][j-1].x = 15;
                                sauvegardeModifsCarte(1,i+1,j-1,15,13);

                                air[i][j-1].x = 41;
                                air[i][j-1].y = 8;
                                sauvegardeModifsCarte(2,i,j-1,41,8);

                                air[i+1][j-1].x = 42;
                                air[i+1][j-1].y = 8;
                                sauvegardeModifsCarte(2,i+1,j-1,42,8);
                            }
                            // Grotte falaise #2 (droite)
                            if(table[i][j].x == 31 && table[i][j].y == 19)
                            {
                                table[i-1][j].x = 32;
                                sauvegardeModifsCarte(1,i-1,j,32,19);

                                table[i][j].x = 33;
                                sauvegardeModifsCarte(1,i,j,33,19);

                                table[i-1][j-1].x = 14;
                                sauvegardeModifsCarte(1,i-1,j-1,14,13);

                                table[i][j-1].x = 15;
                                sauvegardeModifsCarte(1,i,j-1,15,13);

                                air[i-1][j-1].x = 41;
                                air[i-1][j-1].y = 8;
                                sauvegardeModifsCarte(2,i-1,j-1,41,8);

                                air[i][j-1].x = 42;
                                air[i][j-1].y = 8;
                                sauvegardeModifsCarte(2,i,j-1,42,8);
                            }

                            // Action sur un bosquet vert
                            if(table[i][j].x == 26 && table[i][j].y == 0)
                            {
                                table[i][j].x = 29;
                                table[i][j].compteurTile = 1;
                            }

                            // Vasque à enflammer
                            if(table[i][j].x == 20 && table[i][j].y == 29) table[i][j].x = 21;

                        }
                    }
                }
                break;

            case 4: // Boomerang

                // Initialisation de la zone d'influence au sol du boomerang
                x0 = (*it).getPositionX();
                x1 = (*it).getPositionX() + (*it).getDimensionX();
                y0 = (*it).getPositionY();
                y1 = (*it).getPositionY() + (*it).getDimensionY();

                break;

            case 5: // Flamme baguette de feu

                // Initialisation de la zone d'influence au sol
                x0 = (*it).getPositionX() - 1;
                x1 = (*it).getPositionX() + (*it).getDimensionX() + 16;
                y0 = (*it).getPositionY() - 1;
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 16;

                // Test des éléments de décors dans la zone d'influence de la flamme de la lampe
                for(i=x0/LARGEUR_TILE;i<x1/LARGEUR_TILE;i++)
                {
                    for(j=y0/HAUTEUR_TILE;j<y1/HAUTEUR_TILE;j++)
                    {
                        // Vasque à enflammer
                        if(table[i][j].x == 20 && table[i][j].y == 29) table[i][j].x = 21;
                    }
                }

                break;

            case 6: // Glace baguette de glace

                // Initialisation de la zone d'influence au sol
                x0 = (*it).getPositionX() - 1;
                x1 = (*it).getPositionX() + (*it).getDimensionX() + 16;
                y0 = (*it).getPositionY() - 1;
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 16;

                // Test des éléments de décors dans la zone d'influence de la flamme de la lampe
                for(i=x0/LARGEUR_TILE;i<x1/LARGEUR_TILE;i++)
                {
                    for(j=y0/HAUTEUR_TILE;j<y1/HAUTEUR_TILE;j++)
                    {
                        // Vasque à éteindre
                        if(table[i][j].x == 21 && table[i][j].y == 29) table[i][j].x = 20;
                        if(table[i][j].x == 22 && table[i][j].y == 29) table[i][j].x = 20;
                        if(table[i][j].x == 23 && table[i][j].y == 29) table[i][j].x = 20;
                    }
                }

                break;

            case 7: // Feu de la lampe

                // Initialisation de la zone d'influence au sol de la flamme de la lampe
                x0 = (*it).getPositionX();
                x1 = (*it).getPositionX() + (*it).getDimensionX() + 8;
                y0 = (*it).getPositionY();
                y1 = (*it).getPositionY() + (*it).getDimensionY() + 8;

                // Moment où la flamme est active
                if((*it).getTimeToLive() == 5 )
                {
                    // Test des éléments de décors dans la zone d'influence de la flamme de la lampe
                    for(i=x0/LARGEUR_TILE;i<x1/LARGEUR_TILE;i++)
                    {
                        for(j=y0/HAUTEUR_TILE;j<y1/HAUTEUR_TILE;j++)
                        {
                            // Vasque à enflammer
                            if(table[i][j].x == 20 && table[i][j].y == 29) table[i][j].x = 21;
                        }
                    }
                }
                break;

            default:
                break;
        }
    }
}

void Carte::gestionInterractionsObjetsRecoltablesProjectiles()
{

}



bool Carte::collisionAABB(int pX0, int pX1, int pY0, int pY1, int pX2, int pX3, int pY2, int pY3)
{
    bool collision = false;

    /*
    pX0 pX1 pY0 pY1 => boite 1
    pX2 pX3 pY2 pY3 => boite 2
    */

    if(!(pX0 > pX3 || pX1 < pX2 || pY0 > pY3 || pY1 < pY2)) collision = true;

    return collision;
}

void Carte::miseAJourParticulariteSurface(Hero* perso)
{
    int x = (perso->getPositionX()+getDecalageFenetreX()+8)/16;
    int y = (perso->getPositionY()+getDecalageFenetreY()+8)/16;

    if(!perso->getNage()) perso->initInertie();

    perso->setNage(false);
    perso->setMonteEchelle(false);

    if(getTableauTable()[x][y].typeSol == 1) perso->setNage(true); // nage
    else if(getTableauTable()[x][y].typeSol == 2) perso->setMonteEchelle(true); // échelle
    else if(getTableauTable()[x][y].typeSol == 3) // glace
    {

    }
    else if(getTableauTable()[x][y].typeSol == 4) // chute
    {

    }
    else
    {

    }
}

void Carte::ordonnerPnjSurY()
{
    list<Ami> pnjA = getPnjAmi();
    list<Ennemi> pnjE = getPnjEnnemi();

    // Tri sur Y
    pnjA.sort(triSurY());
    pnjE.sort(triSurY());

    // Mise à jour des PNJ sur la carte
    setPnjAmi(pnjA);
    setPnjEnnemi(pnjE);
}

void Carte::immobiliserPnj(bool P)
{
    // Cette fonction a pour but d'arrêter l'animation des PNJ de la carte

    list<Ami> pnjA = getPnjAmi();
    list<Ami>::iterator itA;

    list<Ennemi> pnjE = getPnjEnnemi();
    list<Ennemi>::iterator itE;

    for(itA=pnjA.begin();itA != pnjA.end();itA++)
    {
        (*itA).setPauseAnimation(P);
    }

    for(itE=pnjE.begin();itE != pnjE.end();itE++)
    {
        (*itE).setPauseAnimation(P);
    }

    // Mise à jour des PNJ sur la carte
    setPnjAmi(pnjA);
    setPnjEnnemi(pnjE);
}


void Carte::miseAJourTexteAffichable(Hero* perso, Texte* lecture)
{
    lecture->setNumTexte(-1);

    list<Ami>::iterator it;
    list<Objet>::iterator it2;

    for(it=pnjAmi.begin();it != pnjAmi.end();it++)
    {
        if(lecture->getAutorisationAffichageTexte() && collisionAABB(perso->getPositionX()+getDecalageFenetreX(), perso->getPositionX()+getDecalageFenetreX()+LARGEUR_PERSO, perso->getPositionY()+getDecalageFenetreY()+8, perso->getPositionY()+getDecalageFenetreY()+HAUTEUR_PERSO-8, (*it).getPositionX(), (*it).getPositionX() + (*it).getDimensionX(), (*it).getPositionY()+8, (*it).getPositionY() + (*it).getDimensionY()-8))
        {
            lecture->setNumTexte((*it).getNumNarration()); // Si pas de texte affiché à l'écran
            immobiliserPnj(true);
            perso->setImmobiliser();
            perso->setPauseAnimation(true);
            perso->setAction(true); // Permet d'interdire les autres actions durant la lecture
        }
    }

    for(it2=pnjObj.begin();it2 != pnjObj.end();it2++)
    {
        // si l'un des objets présents sur la carte est dans un état autre qu'inactif, cela signifie que le joueur est en train d'agir dessus et qu'il n'est donc pas en mesure de lire le texte affichable
        if((*it2).getEtat() > 0) lecture->setAutorisationAffichageTexte(false);

        if(lecture->getAutorisationAffichageTexte() && perso->getOrientation() == 0 && collisionAABB(perso->getPositionX()+getDecalageFenetreX(), perso->getPositionX()+getDecalageFenetreX()+LARGEUR_PERSO, perso->getPositionY()+getDecalageFenetreY()+8, perso->getPositionY()+getDecalageFenetreY()+HAUTEUR_PERSO-8, (*it2).getPositionX() + 3, (*it2).getPositionX() + (*it2).getDimensionX() - 3, (*it2).getPositionY()+8, (*it2).getPositionY() + (*it2).getDimensionY()))
        {
            lecture->setNumTexte((*it2).getNumNarration()); // Si pas de texte affiché à l'écran
            immobiliserPnj(true);
            perso->setImmobiliser();
            perso->setPauseAnimation(true);
            perso->setAction(true); // Permet d'interdire les autres actions durant la lecture
        }
    }

    // Réinitialisation du texte s'il n'a pas évolué
    if(lecture->getNumTexte() == -1)
    {
        immobiliserPnj(false);
        perso->setPauseAnimation(false);
        lecture->setAutorisationAffichageTexte(false);
        perso->setAction(false); // Autorise de nouveau les actions si pas de texte à afficher
    }
}

void Carte::miseAJourAnimationsTuiles()
{
    int i,j,X,Y,x0,y0;
    position** S = getTableauSol();
    position** T = getTableauTable();
    position** A = getTableauAir();

    X = getDecalageFenetreX()/LARGEUR_TILE;
    Y = getDecalageFenetreY()/HAUTEUR_TILE;

    // On ne traite que les animations à afficher
    for(i=X;i<X+LARGEUR_ECRAN/LARGEUR_TILE;i++)
    {
        for(j=Y;j<Y+HAUTEUR_ECRAN/HAUTEUR_TILE;j++)
        {
            if(S[i][j].compteurTile == 0)
            {
                // Mise à jour "sol"
                x0 = S[i][j].x;
                y0 = S[i][j].y;
            }

            if(T[i][j].compteurTile == 0)
            {
                // Mise à jour "table"
                x0 = T[i][j].x;
                y0 = T[i][j].y;

                // fleurs qui bougent au vent
                if (x0 == 1 && y0 == 0) {T[i][j].x = 8; T[i][j].y = 3;T[i][j].compteurTile = -10;}
                else if (x0 == 8 && y0 == 3) {T[i][j].x = 9;T[i][j].compteurTile = -25;}
                else if (x0 == 9 && y0 == 3) {T[i][j].x = 1; T[i][j].y = 0;T[i][j].compteurTile = -10;}

                // Vasque au sol avec flamme
                if (x0 == 21 && y0 == 29) {T[i][j].x = 22;T[i][j].compteurTile = -10;}
                else if (x0 == 22 && y0 == 29) {T[i][j].x = 23;T[i][j].compteurTile = -10;}
                else if (x0 == 23 && y0 == 29) {T[i][j].x = 21;T[i][j].compteurTile = -10;}

                // bord de l'eau #1
                if (x0 == 25 && y0 == 10) {T[i][j].x = 26;T[i][j].compteurTile = -25;}
                else if (x0 == 26 && y0 == 10) {T[i][j].x = 27;T[i][j].compteurTile = -25;}
                else if (x0 == 27 && y0 == 10) {T[i][j].x = 25;T[i][j].compteurTile = -25;}
                // bord de l'eau #2
                if (x0 == 28 && y0 == 10) {T[i][j].x = 29;T[i][j].compteurTile = -25;}
                else if (x0 == 29 && y0 == 10) {T[i][j].x = 30;T[i][j].compteurTile = -25;}
                else if (x0 == 30 && y0 == 10) {T[i][j].x = 28;T[i][j].compteurTile = -25;}
                // bord de l'eau #3
                if (x0 == 31 && y0 == 10) {T[i][j].x = 32;}
                else if (x0 == 32 && y0 == 10) {T[i][j].x = 33;T[i][j].compteurTile = -25;}
                else if (x0 == 33 && y0 == 10) {T[i][j].x = 31;T[i][j].compteurTile = -25;}
                // bord de l'eau #4
                if (x0 == 34 && y0 == 10) {T[i][j].x = 35;T[i][j].compteurTile = -25;}
                else if (x0 == 35 && y0 == 10) {T[i][j].x = 36;T[i][j].compteurTile = -25;}
                else if (x0 == 36 && y0 == 10) {T[i][j].x = 34;T[i][j].compteurTile = -25;}
                // bord de l'eau #6
                if (x0 == 0 && y0 == 11) {T[i][j].x = 1;T[i][j].compteurTile = -25;}
                else if (x0 == 1 && y0 == 11) {T[i][j].x = 2;T[i][j].compteurTile = -25;}
                else if (x0 == 2 && y0 == 11) {T[i][j].x = 0;T[i][j].compteurTile = -25;}
                // bord de l'eau #8
                if (x0 == 6 && y0 == 11) {T[i][j].x = 7;T[i][j].compteurTile = -25;}
                else if (x0 == 7 && y0 == 11) {T[i][j].x = 8;T[i][j].compteurTile = -25;}
                else if (x0 == 8 && y0 == 11) {T[i][j].x = 6;T[i][j].compteurTile = -25;}
                // bord de l'eau #10
                if (x0 == 12 && y0 == 11) {T[i][j].x = 13;T[i][j].compteurTile = -25;}
                else if (x0 == 13 && y0 == 11) {T[i][j].x = 14;T[i][j].compteurTile = -25;}
                else if (x0 == 14 && y0 == 11) {T[i][j].x = 12;T[i][j].compteurTile = -25;}
                // bord de l'eau #12
                if (x0 == 20 && y0 == 11) {T[i][j].x = 21;T[i][j].compteurTile = -25;}
                else if (x0 == 21 && y0 == 11) {T[i][j].x = 22;T[i][j].compteurTile = -25;}
                else if (x0 == 22 && y0 == 11) {T[i][j].x = 20;T[i][j].compteurTile = -25;}

                // dans l'eau #1
                if (x0 == 23 && y0 == 11) {T[i][j].x = 24;T[i][j].compteurTile = -25;}
                else if (x0 == 24 && y0 == 11) {T[i][j].x = 25;T[i][j].compteurTile = -25;}
                else if (x0 == 25 && y0 == 11) {T[i][j].x = 23;T[i][j].compteurTile = -25;}
                // dans l'eau #2
                if (x0 == 26 && y0 == 11) {T[i][j].x = 27;T[i][j].compteurTile = -25;}
                else if (x0 == 27 && y0 == 11) {T[i][j].x = 28;T[i][j].compteurTile = -25;}
                else if (x0 == 28 && y0 == 11) {T[i][j].x = 26;T[i][j].compteurTile = -25;}

                // bord de l'eau #5
                if (x0 == 37 && y0 == 10) {T[i][j].x = 38;T[i][j].compteurTile = -25;}
                else if (x0 == 38 && y0 == 10) {T[i][j].x = 39;T[i][j].compteurTile = -25;}
                else if (x0 == 39 && y0 == 10) {T[i][j].x = 37;T[i][j].compteurTile = -25;}
                // bord de l'eau #7
                if (x0 == 3 && y0 == 11) {T[i][j].x = 4;T[i][j].compteurTile = -25;}
                else if (x0 == 4 && y0 == 11) {T[i][j].x = 5;T[i][j].compteurTile = -25;}
                else if (x0 == 5 && y0 == 11) {T[i][j].x = 3;T[i][j].compteurTile = -25;}
                // bord de l'eau #9
                if (x0 == 9 && y0 == 11) {T[i][j].x = 10;T[i][j].compteurTile = -25;}
                else if (x0 == 10 && y0 == 11) {T[i][j].x = 11;T[i][j].compteurTile = -25;}
                else if (x0 == 11 && y0 == 11) {T[i][j].x = 9;T[i][j].compteurTile = -25;}
                // bord de l'eau #11
                if (x0 == 17 && y0 == 11) {T[i][j].x = 18;T[i][j].compteurTile = -25;}
                else if (x0 == 18 && y0 == 11) {T[i][j].x = 19;T[i][j].compteurTile = -25;}
                else if (x0 == 19 && y0 == 11) {T[i][j].x = 17;T[i][j].compteurTile = -25;}

                // bord de l'eau #13
                if (x0 == 20 && y0 == 13) {T[i][j].x = 21;T[i][j].compteurTile = -25;}
                else if (x0 == 21 && y0 == 13) {T[i][j].x = 22;T[i][j].compteurTile = -25;}
                else if (x0 == 22 && y0 == 13) {T[i][j].x = 20;T[i][j].compteurTile = -25;}
                // bord de l'eau #14
                if (x0 == 23 && y0 == 13) {T[i][j].x = 24;T[i][j].compteurTile = -25;}
                else if (x0 == 24 && y0 == 13) {T[i][j].x = 25;T[i][j].compteurTile = -25;}
                else if (x0 == 25 && y0 == 13) {T[i][j].x = 23;T[i][j].compteurTile = -25;}

                // Cascade
                if (x0 == 41 && y0 == 18) {T[i][j].x = 42;T[i][j].compteurTile = -40;}
                else if (x0 == 42 && y0 == 18) {T[i][j].x = 41;T[i][j].compteurTile = -10;}

            }

            if(A[i][j].compteurTile == 0)
            {
                // Mise à jour "air"
                x0 = A[i][j].x;
                y0 = A[i][j].y;

                // hélice girouette haut
                if (x0 == 0 && y0 == 19) {A[i][j].x = 3;A[i][j].compteurTile = -10;}
                else if (x0 == 3 && y0 == 19) {A[i][j].x = 5;A[i][j].compteurTile = -10;}
                else if (x0 == 5 && y0 == 19) {A[i][j].x = 0;A[i][j].compteurTile = -10;}
                // hélice girouette bas
                if (x0 == 2 && y0 == 19) {A[i][j].x = 4;A[i][j].compteurTile = -10;}
                else if (x0 == 4 && y0 == 19) {A[i][j].x = 6;A[i][j].compteurTile = -10;}
                else if (x0 == 6 && y0 == 19) {A[i][j].x = 2;A[i][j].compteurTile = -10;}

                // Vasque en hauteur avec flamme
                if (x0 == 33 && y0 == 29) {A[i][j].x = 34;A[i][j].compteurTile = -10;}
                else if (x0 == 34 && y0 == 29) {A[i][j].x = 35;A[i][j].compteurTile = -10;}
                else if (x0 == 35 && y0 == 29) {A[i][j].x = 33;A[i][j].compteurTile = -10;}

            }

            // Incrémentation de chaque compteur (après initialisation des compteurs si besoin)
            if(S[i][j].compteurTile == 0) S[i][j].compteurTile = -2;
            if(T[i][j].compteurTile == 0) T[i][j].compteurTile = -2;
            if(A[i][j].compteurTile == 0) A[i][j].compteurTile = -2;

            S[i][j].compteurTile++;
            T[i][j].compteurTile++;
            A[i][j].compteurTile++;
        }
    }

    setTableauSol(S);
    setTableauTable(T);
    setTableauAir(A);
}

// Gère les animations conditionnées sur la carte (tout ce qui est déclenché par les évènements qui surviennent sur la carte ; action du joueur sur le décors par exemple)
void Carte::miseAJourAnimationsConditionnees(SDL_Surface* screenCarte)
{
    int i,j,x0,y0; // à utiliser pour les tests et les modifications de la partie de la carte visible.

    for(i=getDecalageFenetreX()/LARGEUR_TILE;i<(getDecalageFenetreX()+LARGEUR_ECRAN+15)/LARGEUR_TILE;i++)
    {
        for(j=getDecalageFenetreY()/HAUTEUR_TILE;j<(getDecalageFenetreY()+HAUTEUR_ECRAN+15)/HAUTEUR_TILE;j++)
        {
            x0 = table[i][j].x;
            y0 = table[i][j].y;

            if (table[i][j].compteurTile > 0)
            {
                // Animation lorsqu'on coupe un petit bosquet vert
                if (x0 == 29 && y0 == 0)
                {
                    if(table[i][j].compteurTile == 1)
                    {
                        ajoutObjetRecoltable(-2,i*16,j*16,false); // temporaire sous cette forme
                    }

                    SDL_Rect offset;
                    offset.x = i*LARGEUR_TILE-getDecalageFenetreX()-5;
                    offset.y = j*HAUTEUR_TILE-getDecalageFenetreY()-13-table[i][j].compteurTile/12;
                    SDL_BlitSurface(buisson, &feuillesExplose[table[i][j].compteurTile/12][0], screenCarte, &offset );

                    table[i][j].compteurTile++;
                    if(table[i][j].compteurTile>76) table[i][j].compteurTile = 0;
                }

                // Animation lorsqu'on coupe un petit bosquet marron
                if (x0 == 26 && y0 == 9)
                {
                    SDL_Rect offset;
                    offset.x = i*LARGEUR_TILE-getDecalageFenetreX()-5;
                    offset.y = j*HAUTEUR_TILE-getDecalageFenetreY()-13-table[i][j].compteurTile/12;
                    SDL_BlitSurface(buisson, &feuillesExplose[table[i][j].compteurTile/12][1], screenCarte, &offset );

                    table[i][j].compteurTile++;
                    if(table[i][j].compteurTile>76) table[i][j].compteurTile = 0;
                }

                // Animation lorsqu'on coupe un petit bosquet mauve
                if (x0 == 9 && y0 == 1)
                {
                    SDL_Rect offset;
                    offset.x = i*LARGEUR_TILE-getDecalageFenetreX()-5;
                    offset.y = j*HAUTEUR_TILE-getDecalageFenetreY()-13-table[i][j].compteurTile/12;
                    SDL_BlitSurface(buisson, &feuillesExplose[table[i][j].compteurTile/12][2], screenCarte, &offset );

                    table[i][j].compteurTile++;
                    if(table[i][j].compteurTile>76) table[i][j].compteurTile = 0;
                }
            }
        }
    }
}

// Fonction qui modifie la carte en fonction de l'état de l'interrupteur Rouge
void Carte::setInterRouge(bool IR)
{
    interRouge = IR;

    // Mise à jour des éléments de la carte qui sont à modifier si on actionne cet interrupteur
    // ATTENTION, TOUT LES ELEMENT DE LA CARTE SONT EVALUES MAIS FONCTION APPELE UNIQUEMENT LORSQUE L'ON AGIT SUR L'INTERRUPTEUR
    int i,j;
    for(i=0;i<getNombreBlocSurX();i++)
    {
        for(j=0;j<getNombreBlocSurY();j++)
        {
            // Traitement des bornes interrupteur
            if(table[i][j].y == 61)
            {
                switch(table[i][j].x)
                {
                    case 11:
                        if(interRouge) table[i][j].x = 12;
                        break;
                    case 12:
                        if(!interRouge) table[i][j].x = 11;
                        break;
                    default:
                        break;
                }
            }

            // Plots qui montent et descendent
            // BAS       HAUT
            //25 33 <-> 27 33 -- Rouge
            //26 33 <-> 28 33 -- Bleu
            if(table[i][j].y == 33)
            {
                switch(table[i][j].x)
                {
                    case 25:
                        if(interRouge) table[i][j].x = 27;
                        break;
                    case 26:
                        if(!interRouge) table[i][j].x = 28;
                        break;
                    case 27:
                        if(!interRouge) table[i][j].x = 25;
                        break;
                    case 28:
                        if(interRouge) table[i][j].x = 26;
                        break;
                    default:
                        break;
                }
            }
        }
    }
}

bool Carte::getInterRouge() {return interRouge;}

void Carte::setDejaVisite(bool DV) {dejaVisite = DV;}
bool Carte::getDejaVisite() {return dejaVisite;}
