/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CARTE_H_INCLUDED
#define CARTE_H_INCLUDED

#include "position.h"
#include "Ennemi.h"
#include "Ami.h"
#include "Hero.h"
#include "Objet.h"
#include "ObjetRecolte.h"
#include "Texte.h"
#include "ImagesPartie.h"
#include "Projectile.h"

// Définition du foncteur servant à trier nos objets selon le nombre
struct triSurY
{
    bool operator ()(Pnj& a1,Pnj& a2)
    {
       return a1.getPositionY() < a2.getPositionY();
    }
};

struct ennemiASortir
{
    bool operator ()(Ennemi& a1)
    {
        return (a1.getEnergie()<1 && !a1.getTouche());
    }
};

struct projectileASortir
{
    bool operator ()(Projectile& p1)
    {
        return (p1.getTimeToLive()<1);
    }
};

struct objetRecolteASortir
{
    // On fois qu'un objet est récolté, il est nécessaire de placer savaleur à 0 pour qu'il soit éliminé
    bool operator ()(ObjetRecolte& or1)
    {
        return (or1.getValeur()<1);
    }
};

struct objetPorteASortir
{
    // On fois qu'un objet est récolté, il est nécessaire de placer sa valeur à 0 pour qu'il soit éliminé
    bool operator ()(Objet& o1)
    {
        return (o1.getEtat() == -1);
    }
};

// Classe gérant indépendamment chaque carte du jeu
class Carte : public ImagesPartie
{
    public:

        Carte();
        virtual ~Carte();

        void setNumero(int N);
        int getNumero();

        void setDimensionX(int X);
        int getDimensionX();
        void setDimensionY(int Y);
        int getDimensionY();

        void setNombreBlocSurX(int X);
        int getNombreBlocSurX();
        void setNombreBlocSurY(int Y);
        int getNombreBlocSurY();

        void setDecalageFenetreX(int DFX);
        int getDecalageFenetreX();
        void setDecalageFenetreY(int DFY);
        int getDecalageFenetreY();

        void setTableauSol(position** TS);
        position** getTableauSol();
        void afficherTableauSol(SDL_Surface* screenCarte);

        void setTableauTable(position** TT);
        position** getTableauTable();
        void afficherTableauTable(SDL_Surface* screenCarte);

        void setTableauAir(position** TA);
        position** getTableauAir();
        void afficherTableauAir(SDL_Surface* screenCarte);

        void setRefTileset(position** RF);
        position** getRefTileset();

        void setPnjEnnemi(list<Ennemi> E);
        list<Ennemi> getPnjEnnemi();

        void setPnjAmi(list<Ami> A);
        list<Ami> getPnjAmi();

        void setPnjProjectile(list<Projectile> P);
        list<Projectile> getPnjProjectile();

        void setPnjObjets(list<Objet> O);
        list<Objet> getPnjObjets();

        void setObjetRecoltables(list<ObjetRecolte> OR);
        list<ObjetRecolte> getObjetRecoltables();

        void setLienCartes(list<lienCarte> LC);
        list<lienCarte> getLienCartes();

        void selectionCarte(int numCarte);

        void afficherCarte(SDL_Surface* screenCarte,SDL_Surface* tilesetA,SDL_Surface* tilesetB,SDL_Surface* tilesetC,SDL_Surface* tilesetD,position** table);
        position** initRefTileset();
        void miseAJourCarte();


        void gestionAffichageJoueur(SDL_Surface* screenCarte, Hero* perso, bool pause, bool affichageMenu);
        void gestionAffichageAmis(SDL_Surface* screenCarte, Hero* perso, bool hauteurCarte, bool pause);
        void gestionAffichageEnnemis(SDL_Surface* screenCarte, Hero* perso, bool hauteurCarte, bool pause);
        void gestionAffichageProjectiles(SDL_Surface* screenCarte, Hero* perso, bool hauteurCarte, bool pause);
        void gestionAffichageObjets(SDL_Surface* screenCarte, Hero* perso, bool hauteurCarte, bool pause);
        void gestionAffichageObjetsRecoltes(SDL_Surface* screenCarte, Hero* perso, bool hauteurCarte, bool pause);


        void gestionInterractionsEntiteEntite(Hero* perso);


        void ordonnerPnjSurY();
        void immobiliserPnj(bool P);

        void miseAJourTexteAffichable(Hero* perso, Texte* lecture); // En fonction de la position du joueur, cette fonction détermine et met à jour le texte affichable
        void miseAJourAnimationsTuiles(); // Gère les animations automatiques sur la carte
        void miseAJourAnimationsConditionnees(SDL_Surface* screenCarte); // Gère les animations conditionnées sur la carte

        void setInterRouge(bool IR);
        bool getInterRouge();

        void setDejaVisite(bool DV);
        bool getDejaVisite();

        bool collisionAABB(int pX0, int pX1, int pY0, int pY1, int pX2, int pX3, int pY2, int pY3);
        void miseAJourParticulariteSurface(Hero* perso);

    protected:

    private:

        void chargerCarte();
        string chargerNomCarteSol();
        string chargerNomCarteCol();
        string chargerNomCarteAir();

        SDL_Surface* chargerImage(const char* fichierImage);
        Uint32 getPixel32(SDL_Surface* image,int i,int j);
        position localiserPixDansCorr(SDL_Surface* corr,Uint32 pix);

        void initCarte();

        void miseAJourLiensCarte();
        void chargerAmis();
        void chargerEnnemis();
        void chargerObjets(bool premierChargement);
        void chargerObjetsRecoltables();

        void modificationObjet(string nomObjet);
        void suppressionModificationsTemporairesObjet();
        void suppressionObjetRecolte(string nomObjetRecolte);
        void ajoutObjetRecoltable(int type, int x, int y, bool fictif);

        void sauvegardeModifsCarte(int niveauCarte, int xCarte, int yCarte, int xTileset, int yTileset);
        void chargerModifsCarte();



        void applicationIaEntites();

        void gestionInterractionsJoueurEnnemis(Hero* perso);
        void gestionInterractionsJoueurAmis(Hero* perso);
        void gestionInterractionsJoueurObjets(Hero* perso);
            void gestionZoneInfluenceJoueurSurObjets(Hero* perso,int* x0,int* x1,int* y0,int* y1);
            bool gestionModificationEtatObjetsManipules(Hero* perso,int* x0,int* x1,int* y0,int* y1,int* numObjetPorteAAjouter);
            bool gestionTransformationElementCarteEnObjetPorte(Hero* perso,int* numObjetPorteAAjouter);
        void gestionInterractionsJoueurObjetsRecoltables(Hero* perso);
        void gestionInterractionsJoueurCarte(Hero* perso);

        void gestionInterractionsArmeJoueurEnnemis(Hero* perso);
        void gestionInterractionsArmeJoueurProjectiles(Hero* perso);
            void gestionActualisationInformationsProjectiles(Hero* perso);
            void initPositionNouveauProjectile(Hero* perso);
            void finalisationCreationNouveauProjectileFleche(Hero* perso, int nbProjectileIdentique);
            void finalisationCreationNouveauProjectileGrappin(Hero* perso, int nbProjectileIdentique);
            void finalisationCreationNouveauProjectileBombe(Hero* perso, int nbProjectileIdentique);
            void finalisationCreationNouveauProjectileBoomrang(Hero* perso, int nbProjectileIdentique);
            void finalisationCreationNouveauProjectileFeu(Hero* perso, int nbProjectileIdentique);
            void finalisationCreationNouveauProjectileGlace(Hero* perso, int nbProjectileIdentique);
            void finalisationCreationNouveauProjectileFeuLampe(Hero* perso, int nbProjectileIdentique);
        void gestionInterractionsArmeJoueurCarte(Hero* perso);
            void gestionZoneInfluenceArmeJoueurCarte(Hero* perso,int* x0,int* x1,int* y0,int* y1);

        void gestionInterractionsEnnemisEnnemis();
        void gestionInterractionsEnnemisAmis();
        void gestionInterractionsEnnemisObjets();
        void gestionInterractionsEnnemisObjetsRecoltales();
        void gestionInterractionsEnnemisProjectiles();

        void gestionInterractionsAmisAmis();
        void gestionInterractionsAmisObjets();
        void gestionInterractionsAmisObjetsRecoltables();
        void gestionInterractionsAmisProjectiles();

        void gestionInterractionsObjetsObjetsRecoltables();
        void gestionInterractionsObjetsProjectiles();

        void gestionInterractionsObjetsRecoltablesProjectiles();

        void gestionInterractionsObjetsProjectilesCarte();


        // Attribut

        int numero; // Numéro de la carte en cours d'affichage

        int dimensionX; // Dimension sur X de la carte exprimée en pixels
        int dimensionY; // Dimension sur Y de la carte exprimée en pixels
        int nombreBlocSurX; // Dimension sur X de la carte exprimée en nombre de tuiles
        int nombreBlocSurY; // Dimension sur Y de la carte exprimée en nombre de tuiles

        int decalageFenetreX;
        int decalageFenetreY;

        position** sol; // Tableau à deux dimensions de type "position" (structure) contenant les tuiles au niveau du sol et n'entrant pas en compte dans la gestion des collisions. Utile lorsqu'une tuile superposée est partiellement ou totalement transparente.
        position** table; // Tableau à deux dimensions de type "position" (structure) contenant les tuiles au niveau du sol. C'est à ce niveau que sont gérées les collisions.
        position** air; // Tableau à deux dimensions de type "position" (structure) contenant les tuiles au dessus du personnage et n'entrant pas en compte dans la gestion des collisions.
        position** refTileset; // Tableau à deux dimensions de type "position" (structure) contenant le tileset de référence (soit toutes les tuiles disponibles du jeu)

        list<Ennemi> pnjEnnemi; // Contient tout les ennemis présents sur la carte
        list<Ami> pnjAmi; // Contient tout les amis ou personnages neutres présents sur la carte
        list<Projectile> pnjProjectile; // Contient tout les projectiles présents sur la carte
        list<Objet> pnjObj; // Contient tout les objets présents sur la carte
        list<ObjetRecolte> objRecoltes; // Contient tout les objets récoltables présents sur la carte
        list<lienCarte> liaisonsUnidirectionnelles; // Liste regroupant l'ensemble des liaisons unidirectionnelles de la carte en cours d'affichage

        Ami A1; // Ami à initialiser autant de fois que l'on veut avant de le placer sur une carte
        Ennemi E1; // Ennemi à initialiser autant de fois que l'on veut avant de le placer sur une carte
        Projectile P1; // Projectile à initialiser autant de fois que l'on veut avant de le placer sur une carte
        Objet O1; // Objet à initialiser autant de fois que l'on veut avant de le placer sur une carte
        ObjetRecolte OR1; // ObjetRecolte à initialiser autant de fois que l'on veut avant de le placer sur une carte
        lienCarte LUD1; // Lien unidirectionnel

        bool interRouge; // Mémorise, pour une carte donnée, l'état des interrupteurs "bleu" ou "rouge". Vrai si rouge, faux sinon.
        bool dejaVisite; // Vrai si la carte a déjà été visitée par le joueur.

        Mix_Chunk* sonsCarte[10]; // Création du tableau pointeurs de type Mix_Chunk
};



#endif // CARTE_H_INCLUDED
