/*
    This program is a clone à Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Clavier.h"

Clavier::Clavier()
{

    setDerniereToucheEnfoncee(-1);
    setAvantDerniereToucheEnfoncee(-1);

    setFlecheHaut(false);
    setFlecheBas(false);
    setFlecheGauche(false);
    setFlecheDroite(false);

}

Clavier::~Clavier()
{

}

void Clavier::ecouteEventMenuSelectionActif(SDL_Event event,bool &done,int &numPartie,bool &effacerPartie,bool &validation)
{

    while( SDL_PollEvent( &event ) )
    {
        switch (event.type) // Attente de signaux
        {
        case SDL_QUIT: // Sortie si la fenêtre est fermée
            done = true;
            break;

        case SDL_KEYDOWN: // Gestion des touches enfoncées
            switch(event.key.keysym.sym)
            {
                case SDLK_ESCAPE: // Sortie du jeu si appui sur echap
                    done = true;
                    break;

                // Touches de direction
                case SDLK_UP:
                    numPartie--;
                    if(numPartie != 3) effacerPartie = false;
                    break;

                case SDLK_DOWN:
                    numPartie++;
                    if(numPartie != 3) effacerPartie = false;
                    break;

                case SDLK_LEFT:
                case SDLK_RIGHT:
                    if(numPartie == 3)
                    {
                        if(effacerPartie) effacerPartie = false;
                        else effacerPartie = true;
                    }
                    break;

                case SDLK_RETURN:
                    validation = true;
                    break;

                // Par défaut, on ne fait rien
                default:
                    break;
            }
            break;

        // Gestion des touches relachées
        case SDL_KEYUP:
            switch(event.key.keysym.sym)
            {
                default:
                    break;
            }
            break;

        default :
            break;
        }
    }

}

void Clavier::ecouteEventMenuCreationNouvellePartie(SDL_Event event, bool &done, bool &retour, bool &valider, bool &suppr, int &x, int &y)
{

    while( SDL_PollEvent( &event ) )
    {
        switch (event.type) // Attente de signaux
        {
        case SDL_QUIT: // Sortie si la fenêtre est fermée
            done = true;
            break;

        case SDL_KEYDOWN: // Gestion des touches enfoncées
            switch(event.key.keysym.sym)
            {
                // Touche echap
                case SDLK_ESCAPE: // Sortie du jeu si appui sur echap
                    retour = true;
                    break;

                // Touches de direction
                case SDLK_UP:
                    y--;
                    break;

                case SDLK_DOWN:
                    y++;
                    break;

                case SDLK_LEFT:
                    x--;
                    break;

                case SDLK_RIGHT:
                    x++;
                    break;

                // Return
                case SDLK_RETURN:
                    valider = true;
                    break;

                // Suppr
                case SDLK_DELETE:
                    suppr = true;
                    break;

                // Par défaut, on ne fait rien
                default:
                    break;
            }
            break;

        // Gestion des touches relachées
        case SDL_KEYUP:
            switch(event.key.keysym.sym)
            {
                default:
                    break;
            }
            break;

        default:
            break;
        }
    }

}

void Clavier::ecouteEventMenuOptionsPartie(SDL_Event event,bool &done,bool &retour)
{

    while( SDL_PollEvent( &event ) )
    {
        switch (event.type) // Attente de signaux
        {
        case SDL_QUIT: // Sortie si la fenêtre est fermée
            done = true;
            break;

        case SDL_KEYDOWN: // Gestion des touches enfoncées
            switch(event.key.keysym.sym)
            {
                case SDLK_ESCAPE: // Sortie du jeu si appui sur echap
                    retour = true;
                    break;

                // Par défaut, on ne fait rien
                default:
                    break;
            }
            break;

        // Gestion des touches relachées
        case SDL_KEYUP:
            switch(event.key.keysym.sym)
            {
                default:
                    break;
            }
            break;

        default:
            break;
        }
    }

}

void Clavier::ecouteEventMenuSuppressionPartie(SDL_Event event,bool &done,int &numPartie,bool &retour,bool & validation)
{

    while( SDL_PollEvent( &event ) )
    {
        switch (event.type) // Attente de signaux
        {
        case SDL_QUIT: // Sortie si la fenêtre est fermée
            done = true;
            break;

        case SDL_KEYDOWN: // Gestion des touches enfoncées
            switch(event.key.keysym.sym)
            {
                case SDLK_ESCAPE: // Sortie du jeu si appui sur echap
                    retour = true;
                    break;

                // Touches de direction
                case SDLK_UP:
                    numPartie--;
                    break;

                case SDLK_DOWN:
                    numPartie++;
                    break;

                // Return
                case SDLK_RETURN:
                    validation = true;
                    break;

                // Par défaut, on ne fait rien
                default:
                    break;
            }
            break;

        // Gestion des touches relachées
        case SDL_KEYUP:
            switch(event.key.keysym.sym)
            {
                default:
                    break;
            }
            break;

        default:
            break;
        }
    }

}

void Clavier::ecouteEventDeroulementPartie(SDL_Event event,bool &done,Hero &joueur,Texte &lecture,Menu &menuJeu,Carte &map,bool &quitter,bool &valider)
{
    // Initialisation état joueur
    joueur.setAction(false);

    // Mise à jour des touches enfoncée si le héro ne cours pas ou s'il ne vient pas juste d'être touché
    if(!joueur.getDeplacementRapide() && joueur.getCompteurTouche() <= TEMPS_INTOUCHABLE - TEMPS_RECUL )
    {
        if(!joueur.getNage())
        {
            joueur.setBougeHaut(getFlecheHaut());
            joueur.setBougeBas(getFlecheBas());
            joueur.setBougeGauche(getFlecheGauche());
            joueur.setBougeDroite(getFlecheDroite());
        }
        else
        {
            joueur.setInertieVerticale(getFlecheHaut(),getFlecheBas());
            joueur.setInertieHorizontale(getFlecheGauche(),getFlecheDroite());
        }
    }
    else
    {
        joueur.setImmobiliser();
        switch(joueur.getOrientation())
        {
            case 0:
                joueur.setBougeHaut(true);
                break;
            case 1:
                joueur.setBougeBas(true);
                break;
            case 2:
                joueur.setBougeGauche(true);
                break;
            case 3:
                joueur.setBougeDroite(true);
                break;
            default:
                break;
        }
        setDerniereToucheEnfoncee(joueur.getOrientation());
        setAvantDerniereToucheEnfoncee(-1);
    }



    while( SDL_PollEvent( &event ) )
    {
        switch (event.type) // Attente de signaux
        {
        case SDL_QUIT: // Sortie si la fenêtre est fermée
            done = true;
            break;

        case SDL_KEYDOWN: // Gestion des touches enfoncées
            switch(event.key.keysym.sym)
            {
                case SDLK_ESCAPE:
                    if(quitter) {quitter = false;} // Quitte le menu de fin de partie et/ou de sauvegarde
                    else if(menuJeu.getAffichageMenu()) {menuJeu.setAffichageMenu(false);} // Quitte le menu de sélection d'items et de visualisation de l'état de la partie du joueur
                    else if(lecture.getNumTexte() != 1004) {quitter = true; lecture.setNumTexte(1000);} // Autorise l'affichage du menu de sauvegarde et de fin de partie
                    else {done = true;}
                    break;

                // Touches de direction
                case SDLK_UP:
                    setFlecheHaut(true);
                    setAvantDerniereToucheEnfoncee(getDerniereToucheEnfoncee());
                    setDerniereToucheEnfoncee(0);
                    if(quitter)
                    {
                        lecture.setNumTexte(lecture.getNumTexte() - 1);
                        if(lecture.getNumTexte() < 1000) lecture.setNumTexte(1002);
                    }
                    else if(menuJeu.getAffichageMenu())
                    {
                        menuJeu.setCurseurY(menuJeu.getCurseurY() - 1);
                        if(menuJeu.getCurseurY() < 0) menuJeu.setCurseurY(3);
                    }
                    break;

                case SDLK_DOWN:
                    setFlecheBas(true);
                    setAvantDerniereToucheEnfoncee(getDerniereToucheEnfoncee());
                    setDerniereToucheEnfoncee(1);
                    if(quitter)
                    {
                        lecture.setNumTexte(lecture.getNumTexte() + 1);
                        if(lecture.getNumTexte() > 1002) lecture.setNumTexte(1000);
                    }
                    else if(menuJeu.getAffichageMenu())
                    {
                        menuJeu.setCurseurY(menuJeu.getCurseurY() + 1);
                        if(menuJeu.getCurseurY() > 3) menuJeu.setCurseurY(0);
                    }
                    break;

                case SDLK_LEFT:
                    setFlecheGauche(true);
                    setAvantDerniereToucheEnfoncee(getDerniereToucheEnfoncee());
                    setDerniereToucheEnfoncee(2);
                    if(menuJeu.getAffichageMenu())
                    {
                        menuJeu.setCurseurX(menuJeu.getCurseurX() - 1);
                        if(menuJeu.getCurseurX() < 0) menuJeu.setCurseurX(3);
                    }
                    break;

                case SDLK_RIGHT:
                    setFlecheDroite(true);
                    setAvantDerniereToucheEnfoncee(getDerniereToucheEnfoncee());
                    setDerniereToucheEnfoncee(3);
                    if(menuJeu.getAffichageMenu())
                    {
                        menuJeu.setCurseurX(menuJeu.getCurseurX() + 1);
                        if(menuJeu.getCurseurX() > 3) menuJeu.setCurseurX(0);
                    }
                    break;

                case SDLK_RETURN:
                    valider = true;
                    if(menuJeu.getAffichageMenu())
                    {
                        if (menuJeu.getValeurItem(menuJeu.getCurseurX(),menuJeu.getCurseurY()) > 0)
                        {
                            joueur.setItemActif(menuJeu.getNumItem(menuJeu.getCurseurX(),menuJeu.getCurseurY()));
                            menuJeu.setAffichageMenu(false);
                        }
                    }
                    else if(lecture.getNumTexte() == 1004) {done = true;}
                    else menuJeu.setAffichageMenu(true);
                    break;




                case SDLK_q: // Windows
                //case SDLK_a: // Linux
                    if(!joueur.getCoupEpee() && !joueur.getAction() && !joueur.getUtilisationArme() && !joueur.getNage() && !joueur.getPorteObjet())
                    {
                        joueur.setCoupEpee(true);
                        joueur.setAction(true);
                    }
                    break;

                case SDLK_w: // Windows
                //case SDLK_z: // Linux
                    if(!joueur.getCoupEpee() && !joueur.getAction() && !joueur.getUtilisationArme() && !joueur.getNage() && !joueur.getPorteObjet())
                    {
                        joueur.setUtilisationArme(true);
                        joueur.setEnvoiProjectile(true);
                    }
                    break;

                case SDLK_a:  // Windows
                //case SDLK_q:  // Linux
                    if(!joueur.getNage() && !joueur.getPorteObjet())
                    {
                        joueur.setDeplacementRapide(true);
                    }
                    break;


                // Gestion de l'affichage des dialogues et des pancartes ; ouverture des coffres
                case SDLK_s:
                case SDLK_SPACE:
                    if(!lecture.getAutorisationAffichageTexte()) // Pas de texte à afficher
                    {
                        lecture.setAutorisationAffichageTexte(true);
                        lecture.setFinAffichageTexte(false);
                    }
                    else if(!lecture.getFinAffichagePage()) // Permet d'aller à la fin de la page si le texte est encore en train defiler
                    {
                        lecture.setFinAffichagePage(true);
                    }
                    else if(lecture.getNumTexte() != -1)
                    {
                        lecture.setPageSuivante(); // Demande la page suivante
                        lecture.setFinAffichagePage(false);
                    }

                    // Si le joueur ne fait rien, appuyer sur la touche lui fait changer d'état à action = vrai
                    if(!joueur.getCoupEpee() && !joueur.getAction() && !joueur.getUtilisationArme() && !joueur.getPorteObjet())
                    {
                        joueur.setAction(true);
                    }

                    if(joueur.getPorteObjet())
                    {
                        joueur.setPorteObjet(false);
                    }

                    break;


                // Par défaut, on ne fait rien
                default:
                    break;
            }
            break;

        // Gestion des touches relachées
        case SDL_KEYUP:
            switch(event.key.keysym.sym)
            {
                case SDLK_UP:
                    setFlecheHaut(false);
                    break;
                case SDLK_DOWN:
                    setFlecheBas(false);
                    break;
                case SDLK_LEFT:
                    setFlecheGauche(false);
                    break;
                case SDLK_RIGHT:
                    setFlecheDroite(false);
                    break;

                case SDLK_a:  // Windows
                //case SDLK_q:  // Linux
                    joueur.setDeplacementRapide(false);
                    break;

                default:
                    break;

                setDerniereToucheEnfoncee(getAvantDerniereToucheEnfoncee());
                setAvantDerniereToucheEnfoncee(-1);
            }
            break;
        }
    }

}

void Clavier::setFlecheHaut(bool FH){flecheHaut = FH;}
bool Clavier::getFlecheHaut(){return flecheHaut;}


void Clavier::setFlecheBas(bool FB){flecheBas = FB;}
bool Clavier::getFlecheBas(){return flecheBas;}

void Clavier::setFlecheGauche(bool FG){flecheGauche = FG;}
bool Clavier::getFlecheGauche(){return flecheGauche;}

void Clavier::setFlecheDroite(bool FD) {flecheDroite = FD;}
bool Clavier::getFlecheDroite(){return flecheDroite;}

bool* Clavier::getFleches()
{
    fleches[0] = flecheHaut;
    fleches[1] = flecheBas;
    fleches[2] = flecheGauche;
    fleches[3] = flecheDroite;

    return fleches;
}

void Clavier::setFlechesInactives()
{
    setFlecheHaut(false);
    setFlecheBas(false);
    setFlecheGauche(false);
    setFlecheDroite(false);
}

void Clavier::setDerniereToucheEnfoncee(int DTE){derniereToucheEnfoncee = DTE;}
int Clavier::getDerniereToucheEnfoncee(){return derniereToucheEnfoncee;}

void Clavier::setAvantDerniereToucheEnfoncee(int ADTE){avantDerniereToucheEnfoncee = ADTE;}
int Clavier::getAvantDerniereToucheEnfoncee(){return avantDerniereToucheEnfoncee;}
