/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "DeroulementPartie.h"

DeroulementPartie::DeroulementPartie()
{
    int i;

    setPartieEnCours(true);
    setQuitter(false);

    // Création des objets (au sens POO) de la partie
    map = new Carte();
    clav = new Clavier();
    joueur = new Hero();
    stat = new Statut();
    menuJeu = new Menu();
    lecture = new Texte();
    donjon1 = new Donjon(1);

    // Initialisation des SDL_Surface du tileset uniquement
    initSDLSurface(0);

    // Initialisation des propriétés traversantes des tiles
    refTileset = map->initRefTileset();
    map->setRefTileset(refTileset);

    // crée et initialise le tableau contenant l'état d'avancement des quêtes ---> UTILITE A PROUVER !!!!!!!!!!!!!
    avancementQuetes = new int[2];
    for(i=0;i<2;i++)
    {
        avancementQuetes[i] = 0;
    }
    // Position par défaut sur la carte
    setDecalageSurX(0);
    setDecalageSurY(0);

    // init de l'alpha du fond
    fondu = 255;

    // init de la musique de la carte
    nomMusique = "";
}

DeroulementPartie::~DeroulementPartie()
{
    delete map;
    delete clav;
    delete joueur;
    delete stat;
    delete menuJeu;
    delete lecture;
    delete donjon1;
    delete avancementQuetes;
    freeMem(0);
}

void DeroulementPartie::setQuitter(bool Q) {quitter = Q;}
bool DeroulementPartie::getQuitter() {return quitter;}

void DeroulementPartie::setPlan(bool P) {plan = P;}
bool DeroulementPartie::getPlan(){return plan;}

void DeroulementPartie::setMenuEtatItem(bool MEI) {menuEtatItem = MEI;}
bool DeroulementPartie::getMenuEtatItem() {return menuEtatItem;}

void DeroulementPartie::setPartieEnCours(bool PEC) {partieEnCours = PEC;}
bool DeroulementPartie::getPartieEnCours() {return partieEnCours;}
void DeroulementPartie::gestionPartieEnCours(SDL_Surface* screenPartieEnCours, Mix_Chunk* musique,bool* sortieProgramme)
{
    // Gestion des événements clavier
    bool pause = getQuitter();
    bool validation = false;
    clav->ecouteEventDeroulementPartie(event,*sortieProgramme,*joueur,*lecture,*menuJeu,*map,pause,validation);
    setQuitter(pause);

    // PRE-TRAITEMENT
    preTraitementAffichagePartieEnCours(&validation, musique, sortieProgramme);
    gestionMusiqueDeFond(musique, map, numPartie);

    // AFFICHAGE
    afficherPartieEnCours(screenPartieEnCours);
}

void DeroulementPartie::preTraitementAffichagePartieEnCours(bool* validation, Mix_Chunk* musique,bool* sortieProgramme)
{
    // TEMPORAIRE POUR DEV
    /*cout << "Position X du joueur : " << joueur->getPositionX() << endl;
    cout << "Position Y du joueur : " << joueur->getPositionY() << endl;
    cout << "Décalage sur X : " << getDecalageSurX() << endl;
    cout << "Décalage sur Y : " << getDecalageSurY() << endl;*/

    // Sélection de la carte si on doit en changer
    // Vérifie si le joueur se trouve placé au point d'entrée d'une liaison avec une autre carte.
    // Si c'est le cas, le joueur doit changer de carte et sa position est mise à jour.
    testLiensCarte(getDecalageSurX(),getDecalageSurY(),joueur,map);

    // Animations automatiques de la carte
    map->miseAJourAnimationsTuiles();

    // Les joueurs et les autres persos peuvent évoluer sur des surfaces particulières (eau, glace...)
    map->miseAJourParticulariteSurface(joueur);

    // Partie d'action du jeu
    if(!getQuitter() && !menuJeu->getAffichageMenu())
    {
        map->miseAJourCarte(); // Mise à jour de la carte /---------> Traiter directement les 3 tables dans la fonction
        map->gestionInterractionsEntiteEntite(joueur); // Gestion des interractions sur la carte
        map->miseAJourTexteAffichable(joueur,lecture); // Gestion du texte affichable (dialogues, texte pancarte)

        joueur->gestionEtatsTemporaires(); // Gestion des états temporaires du joueur
        joueur->traitementDeplacements(&decalageSurX,&decalageSurY,map->getNombreBlocSurX()*LARGEUR_TILE,map->getNombreBlocSurY()*HAUTEUR_TILE,map->getTableauTable(),clav->getDerniereToucheEnfoncee(),clav->getFleches()); // Gestion des collisions

        // Mise à jour du décalage sur X et Y de l'affichage de la carte
        map->setDecalageFenetreX(getDecalageSurX());
        map->setDecalageFenetreY(getDecalageSurY());

        // --> Gestion de l'ordre d'affichage des PNJ (tri selon l'axe Y)
        map->ordonnerPnjSurY();

        // --> Mise à jour du statut
        majStats();

        // --> Sélection du texte à afficher lors de la fin de la partie
        if(joueur->getEnergie() <= 0)
        {
            joueur->setImmobiliser();
            lecture->setNumTexte(1004);
            if(*validation) *sortieProgramme = true;
        }
    }

    // Traitement du menu de sortie du programme
    else if(getQuitter() && *validation)
    {
        if(lecture->getNumTexte() == 1000) *sortieProgramme = true;
        else if(lecture->getNumTexte() == 1001)
        {
            enregistrerSauvegarde();
            *sortieProgramme = true;
        }
        else
        {
            *validation = false;
            setQuitter(false);
        }
    }
}

void DeroulementPartie::majStats()
{
    stat->setNbCoeurs(joueur->getEnergie());
    stat->setNbCoeursMax(joueur->getEnergieMax());
    stat->setNumItem(joueur->getItemActif());
    stat->setNbRubis(joueur->getRubis());
    stat->setNbBombes(joueur->getBombe());
    stat->setNbFleches(joueur->getFleche());
    stat->setQuantiteMagie(joueur->getMagie());
    stat->setJaugeMagieDoublee(joueur->getJaugeMagieDoublee());
    stat->setValeurItem(menuJeu->getValeurItem(menuJeu->getCurseurX(),menuJeu->getCurseurY()));
    stat->setNbCles(0); // Temporaire sous cette forme (à terme, lire les données du donjon)
}

void DeroulementPartie::afficherPartieEnCours(SDL_Surface* screenPartieEnCours)
{
    // Affichage du tableau sol et du tableau table
    map->afficherCarte(screenPartieEnCours,tilesetA,tilesetB,tilesetC,tilesetD,map->getTableauSol());
    map->afficherCarte(screenPartieEnCours,tilesetA,tilesetB,tilesetC,tilesetD,map->getTableauTable());

    // --> PNJ et Objets
    map->gestionAffichageObjetsRecoltes(screenPartieEnCours,joueur,true,getQuitter());
    map->gestionAffichageObjetsRecoltes(screenPartieEnCours,joueur,false,getQuitter());
    map->gestionAffichageAmis(screenPartieEnCours,joueur,true,getQuitter());
    map->gestionAffichageEnnemis(screenPartieEnCours,joueur,true,getQuitter());
    map->gestionAffichageProjectiles(screenPartieEnCours,joueur,true,getQuitter());

    // --> Joueur
    map->gestionAffichageJoueur(screenPartieEnCours,joueur,getQuitter(),menuJeu->getAffichageMenu());

    // --> PNJ et Objets restants (restants sur l'axe Y)
    map->gestionAffichageObjets(screenPartieEnCours,joueur,true,getQuitter());
    map->gestionAffichageObjets(screenPartieEnCours,joueur,false,getQuitter());
    map->gestionAffichageAmis(screenPartieEnCours,joueur,false,getQuitter());
    map->gestionAffichageEnnemis(screenPartieEnCours,joueur,false,getQuitter());
    map->gestionAffichageProjectiles(screenPartieEnCours,joueur,false,getQuitter());

    // Affichage des animations conditionnées
    map->miseAJourAnimationsConditionnees(screenPartieEnCours);

    // Affichage du tableau air
    map->afficherCarte(screenPartieEnCours,tilesetA,tilesetB,tilesetC,tilesetD,map->getTableauAir());


    // Affichage fondu
    // Chargement de l'image de debut
    if(fondu > 10)
    {
        SDL_Surface* png = SDL_CreateRGBSurface(SDL_SWSURFACE, screenPartieEnCours->w, screenPartieEnCours->h, 32, 0, 0, 0, 0);
        SDL_SetAlpha( png, SDL_SRCALPHA | SDL_RLEACCEL, fondu );

        // Affichage de l'image de debut
        SDL_BlitSurface(png, NULL, screenPartieEnCours, NULL );

        // Libération de l'espace mémoire occupé par l'image png
        SDL_FreeSurface(png);
        fondu -= 15;
    }
    else
    {
        fondu = 0;
    }

    // Affichage du texte sélectionné
    lecture->afficherTexte(screenPartieEnCours);

    if(menuJeu->getAffichageMenu())
    {
        // AVANT D'AFFICHER LE MENU, LE METTRE A JOUR EN FONCTION DES ITEMS DISPONIBLES POUR LE JOUEUR
        menuJeu->mAJMenu(joueur);

        menuJeu->afficherMenu(screenPartieEnCours); // Afficher le menu de la partie
        joueur->setValeurItemActif(menuJeu->getValeurItem(menuJeu->getCurseurX(),menuJeu->getCurseurY()));
    }
    else stat->afficherStatut(screenPartieEnCours); // Afficher le statut du joueur

}



void DeroulementPartie::setDecalageSurX(int X) {decalageSurX = X;}
int DeroulementPartie::getDecalageSurX() {return decalageSurX;}
void DeroulementPartie::setDecalageSurY(int Y) {decalageSurY = Y;}
int DeroulementPartie::getDecalageSurY() {return decalageSurY;}

void DeroulementPartie::chargerSauvegarde(int NP)
{
    std::string NJoueur = "";
    int NC = 0;
    int VM = 32;
    bool JM = 0;

    int NBRUBIS = 0, NBBOMBES = 0, NBFLECHES = 0;

    int QUETE1 = 0,QUETE2 = 0;

    int NUMEPEE = 0;

    int ARC = 0,GRAPPIN = 0,BOMBE = 0,BOOMERANG = 0;
    int FEU = 0,GLACE = 0,LAMPE = 0,FLUTE = 0;
    int MARTEAU = 0,PALME = 0,GANT = 0,FILET = 0;
    int FLACON1 = 0,FLACON2 = 0,FLACON3 = 0,LIVRE = 0;

    numPartie = NP;

    // Détermination du bon fichier de sauvegarde à charger
    std::string nomFichier = "sauvegardes/sauvegarde";
    switch (NP)
    {
        case 0:
            nomFichier += "0.txt";
            break;
        case 1:
            nomFichier += "1.txt";
            break;
        default:
            nomFichier += "2.txt";
            break;
    }


    // fichier où sera copié la sauvegarde temporaire du joueur
    // (c'est ensuite sur ce fichier que le programme va travailler tant que le joueur n'aura pas explicitement demandé une sauvegarde de sa partie)
    std::string nomFichierTMP = "sauvegardes/sauvegardeTMP.txt";


    // Traitement du fichier de sauvegarde et extraction des données
    std::ifstream fichier(nomFichier.c_str()); // Lecture
    std::ofstream fichierTMP(nomFichierTMP.c_str()); // Ecriture

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        std::string ligne; // variable contenant chaque ligne lue

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( std::getline( fichier, ligne ) )
        {

            // Tester si l'une des lignes contient la séquence "nom_joueur " (espace compris)
            if(ligne.find("nom_joueur ") == 0)
            {
                // Si c'est le cas, éliminer cette séquence de la ligne et convertir ce qui reste en String
                ligne.replace(0,11,"");

                // Attribuer cette valeur à NJoueur
                stringstream flux(ligne);
                flux >> NJoueur;
            }

            // Tester si l'une des lignes contient la séquence "coeur " (espace compris)
            else if(ligne.find("coeur ") == 0)
            {
                // Si c'est le cas, éliminer cette séquence de la ligne et convertir ce qui reste en int
                ligne.replace(0,6,"");

                // Attribuer cette valeur à *nbCoeurs
                stringstream flux(ligne);
                flux >> NC;
            }

            else if(ligne.find("magie ") == 0)
            {
                ligne.replace(0,6,"");
                stringstream flux(ligne);
                flux >> VM;
            }

            else if(ligne.find("jaugeMagieDoublee ") == 0)
            {
                ligne.replace(0,18,"");
                stringstream flux(ligne);
                flux >> JM;
            }

            else if(ligne.find("nbRubis ") == 0)
            {
                ligne.replace(0,8,"");
                stringstream flux(ligne);
                flux >> NBRUBIS;
            }

            else if(ligne.find("nbBombes ") == 0)
            {
                ligne.replace(0,9,"");
                stringstream flux(ligne);
                flux >> NBBOMBES;
            }

            else if(ligne.find("nbFleches ") == 0)
            {
                ligne.replace(0,10,"");
                stringstream flux(ligne);
                flux >> NBFLECHES;
            }

            else if(ligne.find("quete1 ") == 0)
            {
                ligne.replace(0,7,"");
                stringstream flux(ligne);
                flux >> QUETE1;
            }

            else if(ligne.find("quete2 ") == 0)
            {
                ligne.replace(0,7,"");
                stringstream flux(ligne);
                flux >> QUETE2;
            }

            else if(ligne.find("num epee ") == 0)
            {
                ligne.replace(0,9,"");
                stringstream flux(ligne);
                flux >> NUMEPEE;
            }

            else if(ligne.find("num arc ") == 0)
            {
                ligne.replace(0,8,"");
                stringstream flux(ligne);
                flux >> ARC;
            }

            else if(ligne.find("grappin ") == 0)
            {
                ligne.replace(0,8,"");
                stringstream flux(ligne);
                flux >> GRAPPIN;
            }

            else if(ligne.find("bombe ") == 0)
            {
                ligne.replace(0,6,"");
                stringstream flux(ligne);
                flux >> BOMBE;
            }

            else if(ligne.find("boomerang ") == 0)
            {
                ligne.replace(0,10,"");
                stringstream flux(ligne);
                flux >> BOOMERANG;
            }

            else if(ligne.find("feu ") == 0)
            {
                ligne.replace(0,4,"");
                stringstream flux(ligne);
                flux >> FEU;
            }

            else if(ligne.find("glace ") == 0)
            {
                ligne.replace(0,6,"");
                stringstream flux(ligne);
                flux >> GLACE;
            }

            else if(ligne.find("lampe ") == 0)
            {
                ligne.replace(0,6,"");
                stringstream flux(ligne);
                flux >> LAMPE;
            }

            else if(ligne.find("flute ") == 0)
            {
                ligne.replace(0,6,"");
                stringstream flux(ligne);
                flux >> FLUTE;
            }

            else if(ligne.find("marteau ") == 0)
            {
                ligne.replace(0,8,"");
                stringstream flux(ligne);
                flux >> MARTEAU;
            }

            else if(ligne.find("palme ") == 0)
            {
                ligne.replace(0,6,"");
                stringstream flux(ligne);
                flux >> PALME;
            }

            else if(ligne.find("num gant ") == 0)
            {
                ligne.replace(0,9,"");
                stringstream flux(ligne);
                flux >> GANT;
            }

            else if(ligne.find("filet papillon ") == 0)
            {
                ligne.replace(0,15,"");
                stringstream flux(ligne);
                flux >> FILET;
            }

            else if(ligne.find("flacon 1 ") == 0)
            {
                ligne.replace(0,9,"");
                stringstream flux(ligne);
                flux >> FLACON1;
            }

            else if(ligne.find("flacon 2 ") == 0)
            {
                ligne.replace(0,9,"");
                stringstream flux(ligne);
                flux >> FLACON2;
            }

            else if(ligne.find("flacon 3 ") == 0)
            {
                ligne.replace(0,9,"");
                stringstream flux(ligne);
                flux >> FLACON3;
            }

            else if(ligne.find("livre ") == 0)
            {
                ligne.replace(0,6,"");
                stringstream flux(ligne);
                flux >> LIVRE;
            }

            else
            {
                // Copie ligne à ligne des autres infos de la sauvegarde du joueur dans le fichier de travail
                fichierTMP << ligne << "\n";
            }

            // Tests suivants...
        }
    }

    fichier.close();
    fichierTMP.close();

    // Initialisation des objets du jeu (carte, joueur...)
    joueur->setEnergieMax(NC);
    joueur->setEnergie(NC - 3);
    if(joueur->getEnergie() < 3) joueur->setEnergie(3);

    joueur->setNom(NJoueur);
    joueur->setJaugeMagieDoublee(JM);
    joueur->setRubis(NBRUBIS);
    joueur->setBombe(NBBOMBES);
    joueur->setFleche(NBFLECHES);
    joueur->setMagie(VM);
    joueur->setNiveauEpee(NUMEPEE);


    /*
           case 0: // bombe
        case 7: // cristal
           case 8: // lampe
           case 9: // baguette de feu
        case 10: // grande épée plantée
          case 11: // livre vert
        case 12: // medaillon rouge
           case 13: // gant 1
        case 16: // petite épée plantée
           case 17: // gant 2
           case 18: // palme
           case 19: // baguette de glace
        case 20: // medaillon vert
           case 27: // Grappin
           case 28: // Maillet
           case 29: // Arc normal
        case 30: // medaillon bleu
        case 31: // bottes
        case 32: // coupe
        case 33: // tunique bleue
        case 34: // tunique rouge
           case 35: // arc argent
           case 36: // flute
        case 37: // masque
        case 38: // bourse

    */

    // Gestion des quêtes retravailler
    setAvancementQuete(QUETE1,0);
    setAvancementQuete(QUETE2,1);

    joueur->setListeItemsDisponibles(29,ARC);
    joueur->setListeItemsDisponibles(27,GRAPPIN);
    joueur->setListeItemsDisponibles(0,BOMBE);
    //joueur->setListeItemsDisponibles(???,BOOMERANG);

    joueur->setListeItemsDisponibles(9,FEU);
    joueur->setListeItemsDisponibles(19,GLACE);
    joueur->setListeItemsDisponibles(8,LAMPE);
    joueur->setListeItemsDisponibles(36,FLUTE);

    joueur->setListeItemsDisponibles(28,MARTEAU);
    joueur->setListeItemsDisponibles(18,PALME);
    joueur->setListeItemsDisponibles(13,GANT);
    //joueur->setListeItemsDisponibles(???,FILET);

    //joueur->setListeItemsDisponibles(0,FLACON1);
    //joueur->setListeItemsDisponibles(0,FLACON2);
    //joueur->setListeItemsDisponibles(0,FLACON3);
    joueur->setListeItemsDisponibles(11,LIVRE);

    // Mise à jour du menu du joueur
    menuJeu->mAJMenu(joueur);

    // Sélection de la carte à charger par défaut
    map->selectionCarte(CARTE_DEPART);

    // Initialisation du donjon actif
    setDonjonActif(getNumDonjonEnCours(map->getNumero()));
}


void DeroulementPartie::enregistrerSauvegarde()
{

    // Fichier de sauvegarde temporaire
    std::string nomFichierTMP = "sauvegardes/sauvegardeTMP.txt";

    // Détermination du bon fichier pour mémoriser la sauvegarde
    std::string nomFichier = "sauvegardes/sauvegarde";
    switch (numPartie)
    {
        case 0:
            nomFichier += "0.txt";
            break;
        case 1:
            nomFichier += "1.txt";
            break;
        default:
            nomFichier += "2.txt";
            break;
    }

    // Traitement du fichier de sauvegarde et extraction des données
    std::ifstream fichierTMP(nomFichierTMP.c_str()); // Lecture
    std::ofstream fichier(nomFichier.c_str()); // Ecriture

    if ( fichier && fichierTMP ) // ce test échoue si le fichier n'est pas ouvert
    {
        std::string ligne; // variable contenant chaque ligne lue

        // Ecriture de la totalité des données à sauvegarder
        fichier << "nom_joueur " << joueur->getNom() << "\n" ;
        fichier << "coeur " << joueur->getEnergieMax() << "\n" ;
        fichier << "magie " << joueur->getMagie() << "\n" ;
        fichier << "jaugeMagieDoublee " << joueur->getJaugeMagieDoublee() << "\n" ;

        fichier << "nbRubis " << joueur->getRubis() << "\n" ;
        fichier << "nbBombes " << joueur->getBombe() << "\n" ;
        fichier << "nbFleches " << joueur->getFleche() << "\n" ;

        fichier << "quete1 " << getAvancementQuetes()[0] << "\n" ;
        fichier << "quete2 " << getAvancementQuetes()[1] << "\n" ;
        fichier << "num epee " << joueur->getNiveauEpee() << "\n" ;

        fichier << "num arc " << joueur->getListeItemsDisponibles(29) << "\n" ;
        fichier << "grappin " << joueur->getListeItemsDisponibles(27) << "\n" ;
        fichier << "bombe " << joueur->getListeItemsDisponibles(0) << "\n" ;
        fichier << "boomerang " << "0" << "\n" ;

        fichier << "feu " << joueur->getListeItemsDisponibles(9) << "\n" ;
        fichier << "glace " << joueur->getListeItemsDisponibles(19) << "\n" ;
        fichier << "lampe " << joueur->getListeItemsDisponibles(8) << "\n" ;
        fichier << "flute " << joueur->getListeItemsDisponibles(36) << "\n" ;

        fichier << "marteau " << joueur->getListeItemsDisponibles(28) << "\n" ;
        fichier << "palme " << joueur->getListeItemsDisponibles(18) << "\n" ;
        fichier << "num gant " << joueur->getListeItemsDisponibles(13) << "\n" ;
        fichier << "filet papillon " << "0" << "\n" ;

        fichier << "flacon 1 " << "0" << "\n" ;
        fichier << "flacon 2 " << "0" << "\n" ;
        fichier << "flacon 3 " << "0" << "\n" ;
        fichier << "livre " << joueur->getListeItemsDisponibles(11) << "\n" ;


        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( std::getline( fichierTMP, ligne ) )
        {
            // Copie ligne à ligne de la sauvegarde temporaire du joueur (uniquement les données permettant de modifier la population des cartes)
            fichier << ligne << "\n";
        }


    }

    fichier.close();
    fichierTMP.close();

}

void DeroulementPartie::testLiensCarte(int decalageSurX,int decalageSurY,Hero* joueur, Carte* map)
{
    // teste simplement si le joueur se trouve sur le départ d'un lien connu de la carte (via test de collision entre le jeurs et les points (ou les lignes) de départ)
    list<lienCarte> liensCarteEnCours = map->getLienCartes();
    list<lienCarte>::iterator it;

    int plageSurX;
    int plageSurY;

    int largeurNouvelleCarte, hauteurNouvelleCarte;
    int xA, yA, dxA, dyA; // Variables locales contenant la position di joueur sur la carte d'arrivée
    int x,y; // Variables locales mémorisant la position absolues du joueur sur l'axe considéré

    for(it=liensCarteEnCours.begin();it != liensCarteEnCours.end();it++)
    {
        for(plageSurX = 0; plageSurX <= (*it).positionDepartX1 + (*it).decalageDepartX1 - (*it).positionDepartX0 - (*it).decalageDepartX0; plageSurX += 8)
        {
            for(plageSurY = 0; plageSurY <= (*it).positionDepartY1 + (*it).decalageDepartY1 - (*it).positionDepartY0 - (*it).decalageDepartY0; plageSurY += 8)
            {
                if(map->collisionAABB(joueur->getPositionX()+decalageSurX,
                                joueur->getPositionX()+decalageSurX+LARGEUR_PERSO,
                                joueur->getPositionY()+decalageSurY+8,
                                joueur->getPositionY()+decalageSurY+HAUTEUR_PERSO-8,
                                (*it).positionDepartX0 + (*it).decalageDepartX0 + plageSurX - 1,
                                (*it).positionDepartX0 + (*it).decalageDepartX0 + plageSurX + 1,
                                (*it).positionDepartY0 + (*it).decalageDepartY0 + plageSurY - 1,
                                (*it).positionDepartY0 + (*it).decalageDepartY0 + plageSurY + 1))
                                {
                                    joueur->setPorteObjet(false);

                                    // Si le joueur se trouve effectivement au départ d'un lien, mettre à jour la carte et la position du joueur à l'arrivée
                                    map->selectionCarte((*it).carteArrive);

                                    // Lire les informations relatives à la nouvelle carte (taille essentiellement)
                                    largeurNouvelleCarte = map->getNombreBlocSurX()*LARGEUR_TILE;
                                    hauteurNouvelleCarte = map->getNombreBlocSurY()*HAUTEUR_TILE;

                                    // Déterminer la position cible du joueur
                                        // Sur X
                                        if((*it).positionDepartX0 != (*it).positionDepartX1 || (*it).decalageDepartX0 != (*it).decalageDepartX1)
                                        {
                                            x = (*it).positionArriveX + (*it).decalageArriveX + plageSurX;
                                            // Joueur se trouve dans la partie gauche de l'écran
                                            if(x < LARGEUR_ECRAN/2) xA = x;
                                            // Joueur se trouve dans la partie droite de l'écran
                                            else if(x > largeurNouvelleCarte - LARGEUR_ECRAN/2 - 0) xA = x - largeurNouvelleCarte + LARGEUR_ECRAN;
                                            // Joueur se trouve sur la ligne verticale au centre de l'écran
                                            else xA = LARGEUR_ECRAN/2;

                                            dxA = x - xA;
                                        }
                                        else
                                        {
                                            xA = (*it).positionArriveX;
                                            dxA = (*it).decalageArriveX;
                                        }

                                        // Sur Y
                                        if((*it).positionDepartY0 != (*it).positionDepartY1 || (*it).decalageDepartY0 != (*it).decalageDepartY1)
                                        {
                                            y = (*it).positionArriveY + (*it).decalageArriveY + plageSurY;
                                            // Joueur se trouve dans la partie haute de l'écran
                                            if(y < HAUTEUR_ECRAN/2) yA = y;
                                            // Joueur se trouve dans la partie basse de l'écran
                                            else if(y > hauteurNouvelleCarte - HAUTEUR_ECRAN/2 - 0) yA = y - hauteurNouvelleCarte + HAUTEUR_ECRAN;
                                            // Joueur se trouve sur la ligne horizontale au centre de l'écran
                                            else yA = HAUTEUR_ECRAN/2;

                                            dyA = y - yA;
                                        }
                                        else
                                        {
                                            yA = (*it).positionArriveY;
                                            dyA = (*it).decalageArriveY;
                                        }

                                    // Placer le joueur sur la nouvelle carte
                                    joueur->initPosition(xA,yA);
                                    setDecalageSurX(dxA);
                                    setDecalageSurY(dyA);

                                    fondu = 255;

                                    plageSurX == (*it).positionDepartX1 + (*it).decalageDepartX1 + 8;
                                    plageSurY == (*it).positionDepartY1 + (*it).decalageDepartY1 + 8;

                                }
            }

        }
    }

/*
    // Test si changement de carte, pour mise à jour de la variable contenant le numéro du donjon
    if(map->getNumero() != carteActuelle)
    {
        setDonjonActif(getNumDonjonEnCours(map->getNumero()));
    }*/

}


void DeroulementPartie::gestionMusiqueDeFond(Mix_Chunk* musique, Carte* map, int NP)
{
    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << map->getNumero();

    // Variable contenant l'information sur la musique qui doit être jouée sur la carte en cours
    string musiqueCarteActuelle = "";

    // Détermination du bon fichier de sauvegarde à charger
    std::string nomFichier = "sauvegardes/sauvegarde";
    switch (NP)
    {
        case 0:
            nomFichier += "0.txt";
            break;
        case 1:
            nomFichier += "1.txt";
            break;
        default:
            nomFichier += "2.txt";
            break;
    }

    // Traitement du fichier de sauvegarde et extraction des données
    std::ifstream fichier(nomFichier.c_str()); // Lecture

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne


        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( std::getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "nom_joueur " (espace compris)
            if(ligne.find("carte_" + oss.str() + "_musique") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, ' ' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                musiqueCarteActuelle = infosUnitaires.at(1);

            }
        }
    }

    fichier.close();

    if(musiqueCarteActuelle != nomMusique)
    {
        // Gestion de la nouvelle musique
        Mix_FreeChunk(musique); //Libération de la musique
        musique = Mix_LoadWAV(musiqueCarteActuelle.c_str()); //Chargement de la musique
        //Mix_PlayMusic(musique, -1); //Jouer infiniment la musique
        Mix_PlayChannel(1,musique, -1); //Jouer infiniment la musique

        nomMusique = musiqueCarteActuelle;
    }
}


void DeroulementPartie::setAvancementQuete(int AQ,int numQuete) {avancementQuetes[numQuete] = AQ;}
int* DeroulementPartie::getAvancementQuetes() {return avancementQuetes;}

// A modifier : n'appeler cette fonction que lors d'un changement de carte et conserver le résultat dans une variable autrement
int DeroulementPartie::getNumDonjonEnCours(int numCarte)
{
    if(donjon1->getCarteDansDonjon(numCarte)) return 1;
    else return 0;
}

void DeroulementPartie::setDonjonActif(int DA)
{
    donjonActif = DA;
}

int DeroulementPartie::getDonjonActif()
{
    return donjonActif;
}
