/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ENTITE_H
#define ENTITE_H

#include "position.h"
#include "ImagesPartie.h"

using namespace std;

// Regroupe tout ce qui est positionnable en X et Y sur la carte. Classe permettant de définir un type et une orientation à l'entité (certaines sous classes peuvent se passer de l'un et/ou l'autre ces deux attribus)
class Entite : public ImagesPartie
{
    public:

        Entite();
        virtual ~Entite();

        void setNom(string N);
        string getNom();

        void setPositionX(int X);
        int getPositionX();

        void setPositionY(int Y);
        int getPositionY();

        void initPosition(int X, int Y);

        void setOrientation(int O);
        int getOrientation();

        void setType(int T);
        int getType();

        void setFeuilleSprite(SDL_Surface* S);

        void setDimensionX(int X);
        int getDimensionX();

        void setDimensionY(int Y);
        int getDimensionY();

        void initCompteurAnim();
        void setIncrCompteurAnim(); // Incrémentation de 1 du compteur d'animation
        int getCompteurAnim();

        void afficherEntite(SDL_Surface* screenEntite, SDL_Rect decoupeSprite, SDL_Rect positionScreen);

        void setPauseAnimation(bool PA);
        bool getPauseAnimation();

        virtual void initType();

    protected:

        string nom; // Nom donné (si besoin) à l'entité au début de la partie

        int positionX; // Position X de l'entité sur la carte
        int positionY; // Position Y de l'entité sur la carte
        int orientation; // S'applique au entités ayant plusieurs orientations possibles
        int type; // Définit le type de l'entité. Chaque sous classe doit définir la signification des valeurs de type. Une même valeur de type aura une signification différente selon ce qu'elle désigne (item, objet, ami, etc...).
        SDL_Surface* feuilleSprite; // Stock la feuille de sprites de l'entité
        int dimensionX; // dimension sur X de l'entité
        int dimensionY; // dimension sur Y de l'entité
        int compteurAnimation; // Compteur utilisable par chaque entité pour gérer son ou ses animations
        bool pauseAnimation; // Permet de mettre une animation en pause

    private:
};

#endif // ENTITE_H
