/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Hero.h"

Hero::Hero()
{
    //setPositionAbsolue(0,0);
    positionX = 152; // 152
    positionY = 140; // 70

    setOrientation(1);
    initCompteurAnim();

    setCompteurTouche(0);

    // Initialisation des SDL_Surface* et SDL_Rect propres au héro
    initSDLSurface(2);
    initSDLRect(2);
    initSDLRect(3);
    feuilleSprite = link1;

    // Arme du héro
    setCoupEpee(false);
    setAction(false);
    setNage(false);
    setMonteEchelle(false);
    setPorteObjet(false);
    setUtilisationArme(false);
    setEnvoiProjectile(false);
    epeeEtArmesSecondaires = new Arme();
    setItemActif(0);

    // Initialisation objet "consommables" du joueur (valeurs max, quantité disponible, etc...)
    energie = -1;
    magie = -1;
    rubis = -1;
    bombe = -1;
    fleche = -1;
    setRubisMax(999);
    setBombeMax(20);
    setFlecheMax(30);

    initInertie(); // inertie dans l'eau (ou dans un autre milieu)

    // init des sons emis par le joueur
	sonsJoueur[0] = Mix_LoadWAV("sons/sword.ogg"); // Chargement du son de l'épée
	sonsJoueur[1] = Mix_LoadWAV("sons/porte.ogg"); // Chargement du son du joueur lorsqu'il porte un objet
	sonsJoueur[2] = Mix_LoadWAV("sons/hurt.ogg"); // Chargement du son du joueur lorsqu'il est blessé
	sonsJoueur[3] = Mix_LoadWAV("sons/burn.ogg"); // Chargement du son de la lampe et de la baguette de feu
	sonsJoueur[4] = Mix_LoadWAV("sons/magic.ogg"); // Chargement du son du joueur utilise un objet magique
	sonsJoueur[5] = Mix_LoadWAV("sons/item.ogg"); // Chargement du son du joueur qui obtient un item
	sonsJoueur[6] = Mix_LoadWAV("sons/pics.ogg"); // Chargement du son du joueur qui obtient un item

}

Hero::~Hero()
{
    freeMem(2);
    epeeEtArmesSecondaires->freeMem(3);
    delete epeeEtArmesSecondaires;

    // Libération des sons du joueur
    Mix_FreeChunk(sonsJoueur[0]);
    Mix_FreeChunk(sonsJoueur[1]);
    Mix_FreeChunk(sonsJoueur[2]);
    Mix_FreeChunk(sonsJoueur[3]);
    Mix_FreeChunk(sonsJoueur[4]);
    Mix_FreeChunk(sonsJoueur[5]);
    Mix_FreeChunk(sonsJoueur[6]);
}

void Hero::setEnergie(int E)
{
    //if(E > energie && energie > -1) Mix_PlayChannel(2,sonsJoueur[6], 0); // Joue le son du joueur qui récolte un rubis

    energie = E;
    if(energie > energieMax) energie = energieMax;
}
int Hero::getEnergie() {return energie;}
void Hero::setEnergieMax(int EM)
{
    energieMax = EM;
    if(energieMax > 20) energieMax = 20;
}
int Hero::getEnergieMax() {return energieMax;}

void Hero::setMagie(int M)
{
    if(M > magie && magie > -1) Mix_PlayChannel(2,sonsJoueur[6], 0); // Joue le son du joueur qui récolte une dose de magie

    magie = M;
    if(magie > getMagieMax()) magie = getMagieMax();
}
int Hero::getMagie() {return magie;}
void Hero::setMagieMax(int MM) {magieMax = MM;}
int Hero::getMagieMax() {return magieMax;}
void Hero::setJaugeMagieDoublee(bool JMD)
{
    jaugeMagieDoublee = JMD;
    if(jaugeMagieDoublee) setMagieMax(32);
    else setMagieMax(16);
}
bool Hero::getJaugeMagieDoublee() {return jaugeMagieDoublee;}

void Hero::setFleche(int F)
{
    if(F > fleche && fleche > -1) Mix_PlayChannel(2,sonsJoueur[6], 0); // Joue le son du joueur qui récolte une fleche

    fleche = F;
    if(fleche > getFLecheMax()) fleche = getFLecheMax();
}
int Hero::getFleche() {return fleche;}
void Hero::setFlecheMax(int FM) {flecheMax = FM;}
int Hero::getFLecheMax() {return flecheMax;}

void Hero::setBombe(int B)
{
    if(B > bombe && bombe > -1) Mix_PlayChannel(2,sonsJoueur[6], 0); // Joue le son du joueur qui récolte une bombe

    bombe = B;
    if(bombe > getBombeMax()) bombe = getBombeMax();
}
int Hero::getBombe() {return bombe;}
void Hero::setBombeMax(int BM) {bombeMax = BM;}
int Hero::getBombeMax() {return bombeMax;}

void Hero::setRubis(int R)
{
    if(R > rubis && rubis > -1) Mix_PlayChannel(2,sonsJoueur[5], 0); // Joue le son du joueur qui récolte un rubis

    rubis = R;
    if(rubis > getRubisMax()) rubis = getRubisMax();
}
int Hero::getRubis() {return rubis;}
void Hero::setRubisMax(int RM) {rubisMax = RM;}
int Hero::getRubisMax() {return rubisMax;}

void Hero::setQuartDeCoeur(int QDC) {quartDeCoeur = QDC;}
int Hero::getQuartDeCoeur() {return quartDeCoeur;}

void Hero::setMAJObjetsRecolte(int type, int valeur)
{
    switch(type)
    {
        case 0: // bombe
            setBombe(getBombe()+valeur);
            break;

        case 4: // grand cœur
            setEnergieMax(getEnergieMax()+valeur);
            setEnergie(getEnergie()+valeur);
            break;

        case 14: // rubis vert
        case 15: // rubis bleu
        case 22: // rubis rouge
            setRubis(getRubis()+valeur);
            break;

        case 23: // petit cœur
            setEnergie(getEnergie()+valeur);
            break;

        case 24: // flèche normale
            setFleche(getFleche()+valeur);
            break;

        case 25: // petite réserve magie
            setMagie(getMagie()+valeur);
            break;

        case 26: // grande réserve magie
            setMagie(getMagieMax());
            break;

        default:
            break;
    }
}

void Hero::setTouche(bool T)
{
    touche = T;
    if(touche) Mix_PlayChannel(2,sonsJoueur[2], 0); // Joue le son du joueur touché
}

bool Hero::getTouche() {return touche;}
void Hero::setCompteurTouche(int CT) {compteurTouche = CT;}
int Hero::getCompteurTouche() {return compteurTouche;}

void Hero::setCoupEpee(bool CE)
{
    coupEpee = CE;
    if(coupEpee)
    {
        initCompteurAnim();
        Mix_PlayChannel(2,sonsJoueur[0], 0); // Joue le son de l'épée
    }
}
bool Hero::getCoupEpee() {return coupEpee;}
void Hero::setNiveauEpee(int NE) {niveauEpee = NE;}
int Hero::getNiveauEpee() {return niveauEpee;}

void Hero::setUtilisationArme(bool UA)
{
    utilisationArme = UA;
    if(utilisationArme)
    {
        initCompteurAnim();

        switch(getItemActif())
        {
            case 5:
            case 7:
                Mix_PlayChannel(2,sonsJoueur[3], 0); // Joue le son enflammation
                break;
            case 6:
                Mix_PlayChannel(2,sonsJoueur[4], 0); // Joue le son de magie
                break;

            default:
                break;
        }
    }
}
bool Hero::getUtilisationArme() {return utilisationArme;}

void Hero::setEnvoiProjectile(bool EP) {envoiProjectile=EP;}
bool Hero::getEnvoiProjectile() {return envoiProjectile;}

void Hero::setListeItemsDisponibles(int numItem, int valeur)
{
    // A compléter
    switch(numItem)
    {
        // setListeItemsDisponibles(int numItem, int valeur)
        case 0: // bombe
        case 7: // cristal
        case 8: // lampe
        case 9: // baguette de feu
        case 10: // grande épée plantée
        case 11: // livre vert
        case 12: // medaillon rouge
        case 13: // gant 1
        case 16: // petite épée plantée
        //case 17: // gant 2
        case 18: // palme
        case 19: // baguette de glace
        case 20: // medaillon vert
        case 27: // Grappin
        case 28: // Maillet
        case 29: // Arc normal
        case 30: // medaillon bleu
        case 31: // bottes
        case 32: // coupe
        case 33: // tunique bleue
        case 34: // tunique rouge
        //case 35: // arc argent
        case 36: // flute
        case 37: // masque
        case 38: // bourse
            listeItemsDisponibles[numItem] = valeur;
            //cout << "num item : " << numItem << " - Valeur : " << valeur << endl;
            break;

        default:
            break;
    }
    /*

    // items restant à traiter
        case 305:
            return OR_QuartDeCoeur;
            break;

        case 321:
            return OR_FlecheArgent;
            break;

        case 324:
            return OR_Fleche;
            break;

        case 339:
            return OR_EpeeBleue;
            break;

        case 340:
            return OR_PetiteEpeeBlanche;
            break;

        case 341:
            return OR_PetitBouclier;
            break;

        case 342:
            return OR_FlaconVide;
            break;

        case 343:
            return OR_FlaconRouge;
            break;

        case 344:
            return OR_FlaconVert;
            break;

        case 345:
            return OR_FlaconBleu;
            break;

        case 346:
            return OR_Bouclier2;
            break;

        case 347:
            return OR_EpeeRouge;
            break;

        case 348:
            return OR_EpeeJaune;
            break;

        case 349:
            return OR_Bouclier3;
            break;

        case 351:
            return OR_PetiteEpeeBleue;
            break;

        case 352:
            return OR_TriForce;
            break;

    */

}

int Hero::getListeItemsDisponibles(int numItem) {return listeItemsDisponibles[numItem];}

void Hero::setItemActif(int IA) {itemActif = IA;}
int Hero::getItemActif() {return itemActif;}
void Hero::setValeurItemActif(int VIA) {valeurItemActif = VIA;}
int Hero::getValeurItemActif() {return valeurItemActif;}
Arme Hero::getArmeActive() {return *epeeEtArmesSecondaires;}

void Hero::setAction(bool A) {action = A;}
bool Hero::getAction() {return action;}

void Hero::setNage(bool N)
{
    nage = N;
    if(nage)
    {
        setDeplacementRapide(false);
        setAction(false);
        setUtilisationArme(false);
    }
}
bool Hero::getNage() {return nage;}

void Hero::setMonteEchelle(bool ME)
{
    monteEchelle = ME;
    if(monteEchelle) setDeplacementRapide(false);
}

bool Hero::getMonteEchelle() {return monteEchelle;}

void Hero::setPorteObjet(bool PO)
{
    porteObjet = PO;
    if(porteObjet)
    {
        setDeplacementRapide(false);
        Mix_PlayChannel(2,sonsJoueur[1], 0); // Joue le son
    }
}

bool Hero::getPorteObjet() {return porteObjet;}

// Le traitement des déplacements est la fonction qui réalise les déplacements du héro (à partir des déplacements demandés et du traitement qui en est fait par la gestion des collisions).
void Hero::traitementDeplacements(int *decalageFenetreX,int *decalageFenetreY,int largeurCarte,int hauteurCarte,position **table, int derniereToucheEnfoncee, bool *fleches)
{
    bool testVertical; // Permet de définir la priorité de traitement des déplacements du perso en fonction de la dernière touche appuyée

    // Gestion des déplacements du joueur s'il est touché (recul)
    if(getCompteurTouche() > TEMPS_INTOUCHABLE - TEMPS_RECUL) // Recul du héro s'il se fait toucher
    {
        setImmobiliser();
        switch(getOrientation())
        {
            case 0:
                setBougeBas(true);
                break;
            case 1:
                setBougeHaut(true);
                break;
            case 2:
                setBougeDroite(true);
                break;
            case 3:
                setBougeGauche(true);
                break;
            default:
                break;
        }
    }

    setDeplacementDemandeParCorrection(false); // Par défaut les déplacements demandés ne le sont pas par le système de gestion des collisions

    // Certaines actions (mais pas toutes) impliquent que le joueur soit immobilisé
    if(getCoupEpee() || getUtilisationArme()) setImmobiliser();

    // Cas classique où le héro dispose encore d'un peu d'énergie.
    if(getEnergie() > 0 || getCompteurTouche() > TEMPS_INTOUCHABLE - TEMPS_RECUL)
    {

        // On comence le traitement des collisions par l'axe sur lequel se situe le dernier déplacement demandé. Ce déplacement est prioritaire sur celui qui lui est orthogonal (s'il existe)
        if(derniereToucheEnfoncee < 2 && (getBougeHaut() || getBougeBas())) {testVertical = true;}
        else {testVertical = false;}

        // On effectue un premier test de collisions
        gestionCollisions(*decalageFenetreX,*decalageFenetreY,table,false,testVertical,getVitesseDeplacementSurX(),getVitesseDeplacementSurY());

        // Gestion de la vitesse de déplacement du héro suite au premier traitement de gestion des collisions
        gestionVitesseDeplacement(true);

        // On traite en priorité la dernière touche enfoncée si link est autorisé à bouger dans la même direction
        gestionDerniersDeplacementsDemandesVersLeHaut(decalageFenetreX, decalageFenetreY, hauteurCarte, derniereToucheEnfoncee, true);
        gestionDerniersDeplacementsDemandesVersLeBas(decalageFenetreX, decalageFenetreY, hauteurCarte, derniereToucheEnfoncee, true);
        gestionDerniersDeplacementsDemandesVersLaGauche(decalageFenetreX, decalageFenetreY, largeurCarte, derniereToucheEnfoncee, true);
        gestionDerniersDeplacementsDemandesVersLaDroite(decalageFenetreX, decalageFenetreY, largeurCarte, derniereToucheEnfoncee, true);

        // Une fois les déplacements effectués sur l'axe à traiter prioritairement, on traite l'autre axe de manière similaire
        if(testVertical) {testVertical = false;}
        else {testVertical = true;}

        // On effectue un second test de collisions
        gestionCollisions(*decalageFenetreX,*decalageFenetreY,table,false,testVertical,getVitesseDeplacementSurX(),getVitesseDeplacementSurY());

        // Nouvel ajustement des vitesse de déplacement en fonction des divers cas possibles (la précédente gestion des collisions pouvant avoir modifié le cas)
        gestionVitesseDeplacement(false);

        // On traite ensuite l'avant dernière touche enfoncée si link est autorisé à bouger dans la même direction
        gestionDerniersDeplacementsDemandesVersLeHaut(decalageFenetreX, decalageFenetreY, hauteurCarte, derniereToucheEnfoncee, false);
        gestionDerniersDeplacementsDemandesVersLeBas(decalageFenetreX, decalageFenetreY, hauteurCarte, derniereToucheEnfoncee, false);
        gestionDerniersDeplacementsDemandesVersLaGauche(decalageFenetreX, decalageFenetreY, largeurCarte, derniereToucheEnfoncee, false);
        gestionDerniersDeplacementsDemandesVersLaDroite(decalageFenetreX, decalageFenetreY, largeurCarte, derniereToucheEnfoncee, false);

        // Il faut ensuite que l'orientation du héro soit correcte. Ce qui suit doit couvrir tout les cas de figure.
        gestionOrientation(fleches);

    }

    // Protection pour que le héro ne décroche pas du centre de l'écran
    gestionPositionJoueurSurEcran(decalageFenetreX, decalageFenetreY, largeurCarte, hauteurCarte);
}



void Hero::gestionVitesseDeplacement(bool apresPremiereGestionCollisions)
{
    // Ajustement des vitesses de déplacement en fonction des divers cas possibles
    if(apresPremiereGestionCollisions)
    {
        if(getDeplacementDemandeParCorrection())
        {
            setVitesseDeplacementSurX(1);
            setVitesseDeplacementSurY(1);
        }

        else if(getMonteEchelle())
        {
            setVitesseDeplacementSurX(getCompteurAnim()%2);
            setVitesseDeplacementSurY(getCompteurAnim()%2);
        }

        else if((getBougeHaut() && getBougeGauche()) || (getBougeHaut() && getBougeDroite()) || (getBougeBas() && getBougeGauche()) || (getBougeBas() && getBougeDroite()))
        {
            setVitesseDeplacementSurX(1); //2
            setVitesseDeplacementSurY(1); //2
        }

        else
        {
            if(getDeplacementRapide())
            {
                setVitesseDeplacementSurX(3); //4
                setVitesseDeplacementSurY(3);
            }
            else
            {
                setVitesseDeplacementSurX(1); //3
                setVitesseDeplacementSurY(1); //2
            }
        }
    }

    else
    {
        if(getDeplacementDemandeParCorrection())
        {
            setVitesseDeplacementSurX(1);
            setVitesseDeplacementSurY(1);
        }

        else if((getBougeHaut() && getBougeGauche()) || (getBougeHaut() && getBougeDroite()) || (getBougeBas() && getBougeGauche()) || (getBougeBas() && getBougeDroite()))
        {
            setVitesseDeplacementSurX(1); //2
            setVitesseDeplacementSurY(1); //2
        }
    }
}

void Hero::gestionDerniersDeplacementsDemandesVersLeHaut(int *decalageFenetreX,int *decalageFenetreY,int hauteurCarte, int derniereToucheEnfoncee, bool dernierDeplacementDemande)
{
    if(getBougeHaut() && ((derniereToucheEnfoncee == 0 && dernierDeplacementDemande) || (derniereToucheEnfoncee != 0 && !dernierDeplacementDemande)))
    {
        if (getPositionY() == 0) setPositionY(0); // empêche Link de sortir de la carte par le haut
        else if (getPositionY() != 120) setPositionY(-getVitesseDeplacementSurY()); // Cas où on se trouve près du bord haut ou bas de la carte lorsque Link se déplace vers le haut
        else if (*decalageFenetreY == 0) setPositionY(-getVitesseDeplacementSurY()); // Lorsque la carte ne bouge plus, on initie le déplacement de Link vers le haut
        else *decalageFenetreY -= getVitesseDeplacementSurY(); // Cas par défaut lorsqu'on ne se trouve pas près du bord haut ou bas -> Link au centre et la carte se déplace vers le bas

        // Ajustement pour les déplacements à grande vitesse vers le haut
        if (*decalageFenetreY < 0)
        {
            setPositionY(-getVitesseDeplacementSurY() + *decalageFenetreY);
            *decalageFenetreY = 0;
        }

        else if (getPositionY() < 120 && *decalageFenetreY > 0 && *decalageFenetreY < hauteurCarte - 239)
        {
            *decalageFenetreY = hauteurCarte - 240;
            setPositionY(120-getPositionY());
        }
    }
}

void Hero::gestionDerniersDeplacementsDemandesVersLeBas(int *decalageFenetreX,int *decalageFenetreY,int hauteurCarte, int derniereToucheEnfoncee, bool dernierDeplacementDemande)
{
    if(getBougeBas() && ((derniereToucheEnfoncee == 1 && dernierDeplacementDemande) || (derniereToucheEnfoncee != 1 && !dernierDeplacementDemande)))
    {
        if (getPositionY() == 216) setPositionY(0); // empêche Link de sortir de la carte par le bas
        else if (getPositionY() != 120) setPositionY(getVitesseDeplacementSurY()); // Cas où on se trouve près du bord haut ou bas de la carte lorsque Link se déplace vers le bas
        else if (*decalageFenetreY == hauteurCarte - 240) setPositionY(getVitesseDeplacementSurY()); // Lorsque la carte ne bouge plus, on initie le déplacement de Link vers le bas
        else *decalageFenetreY += getVitesseDeplacementSurY(); // Cas par défaut lorsqu'on ne se trouve pas près du bord haut ou bas -> Link au centre et la carte se déplace vers le haut

        // Ajustement pour les déplacements à grande vitesse vers le bas
        if (*decalageFenetreY > hauteurCarte - 240)
        {
            setPositionY(getVitesseDeplacementSurY() - *decalageFenetreY + hauteurCarte - 240);
            *decalageFenetreY = hauteurCarte - 240;
        }
    }
}

void Hero::gestionDerniersDeplacementsDemandesVersLaGauche(int *decalageFenetreX,int *decalageFenetreY,int largeurCarte, int derniereToucheEnfoncee, bool dernierDeplacementDemande)
{
    if(getBougeGauche() && ((derniereToucheEnfoncee == 2 && dernierDeplacementDemande) || (derniereToucheEnfoncee != 2 && !dernierDeplacementDemande)))
    {
        if (getPositionX() == 0) setPositionX(0); // empêche Link de sortir de la carte par la gauche
        else if (getPositionX() != 160) setPositionX(-getVitesseDeplacementSurX()); // Cas où on se trouve près du bord droite ou gauche de la carte lorsque Link se déplace vers la gauche
        else if (*decalageFenetreX == 0) setPositionX(-getVitesseDeplacementSurX()); // Lorsque la carte ne bouge plus, on initie le déplacement de Link vers la gauche
        else *decalageFenetreX -= getVitesseDeplacementSurX(); // Cas par défaut lorsqu'on ne se trouve pas près du bord droite ou gauche -> Link au centre et la carte se déplace vers la droite

        // Ajustement pour les déplacements à grande vitesse vers la droite
        if (*decalageFenetreX < 0)
        {
            setPositionX(-getVitesseDeplacementSurX() + *decalageFenetreX);
            *decalageFenetreX = 0;
        }

        else if (getPositionX() < 160 && *decalageFenetreX > 0 && *decalageFenetreX < largeurCarte - 319)
        {
            *decalageFenetreX = largeurCarte - 320;
            setPositionX(160-getPositionX());
        }
    }
}

void Hero::gestionDerniersDeplacementsDemandesVersLaDroite(int *decalageFenetreX,int *decalageFenetreY,int largeurCarte, int derniereToucheEnfoncee, bool dernierDeplacementDemande)
{
    if(getBougeDroite() && ((derniereToucheEnfoncee == 3 && dernierDeplacementDemande) || (derniereToucheEnfoncee != 3 && !dernierDeplacementDemande)))
    {
        if (getPositionX() >= LARGEUR_ECRAN - LARGEUR_PERSO - getVitesseDeplacementSurX()) setPositionX(0); // empêche Link de sortir de la carte par la droite
        else if (getPositionX() != LARGEUR_ECRAN/2) setPositionX(getVitesseDeplacementSurX()); // Cas où on se trouve près du bord droite ou gauche de la carte lorsque Link se déplace vers la droite
        else if (*decalageFenetreX == largeurCarte - LARGEUR_ECRAN) setPositionX(getVitesseDeplacementSurX()); // Lorsque la carte ne bouge plus, on initie le déplacement de Link vers la droite
        else *decalageFenetreX += getVitesseDeplacementSurX();// Cas par défaut lorsqu'on ne se trouve pas près du bord droite ou gauche -> Link au centre et la carte se déplace vers la gauche

        // Ajustement pour les déplacements à grande vitesse vers la droite
        if (*decalageFenetreX > largeurCarte - LARGEUR_ECRAN)
        {
            setPositionX(getVitesseDeplacementSurX() - *decalageFenetreX + largeurCarte - LARGEUR_ECRAN);
            *decalageFenetreX = largeurCarte - LARGEUR_ECRAN;
        }
    }
}


void Hero::gestionOrientation(bool *fleches)
{
    if(!getPauseAnimation())
    {
        // Cas où trois touches sont appuyées en même temps
        if (getBougeGauche() && getBougeDroite() && getBougeHaut()) setOrientation(0);
        if (getBougeGauche() && getBougeDroite() && getBougeBas()) setOrientation(1);
        if (getBougeGauche() && getBougeHaut() && getBougeBas()) setOrientation(2);
        if (getBougeDroite() && getBougeHaut() && getBougeBas()) setOrientation(3);

        // Cas où une seule direction (maximum) est donnée et que cette direction (s'il y en a une) correspond à une touche effectivement enfoncée -> mise à jour de l'orientation du personnage
        if (!getBougeDroite() && !getBougeGauche() && !getBougeHaut() && fleches[1]) setOrientation(1);
        if (!getBougeBas() && !getBougeGauche() && !getBougeHaut() && fleches[3]) setOrientation(3);
        if (!getBougeBas() && !getBougeDroite() && !getBougeHaut() && fleches[2]) setOrientation(2);
        if (!getBougeBas() && !getBougeDroite() && !getBougeGauche() && fleches[0]) setOrientation(0);
    }
}

void Hero::gestionPositionJoueurSurEcran(int *decalageFenetreX,int *decalageFenetreY,int largeurCarte,int hauteurCarte)
{
    /*
    Fonction permettant au joueur de rester au centre de l'écran s'il se trouve au milieu d'une grande carte
    ou de se rapprocher des bords de l'écran lorsqu"il est placé aux limites de la carte.
    */

    int a;

    // Cas où on doit bouger sur X
    if(largeurCarte - 320 > 0)
    {
        if(getPositionX() > 160 && *decalageFenetreX != largeurCarte - 320)
        {
            a = getPositionX()-160;
            setPositionX(-a);
            *decalageFenetreX = a;
        }

        if(getPositionX() < 160 && *decalageFenetreX != 0)
        {
            a = getPositionX()-160;
            setPositionX(a);
            *decalageFenetreX = -a;
        }
    }

    // Cas où on doit bouger sur Y
    if(hauteurCarte - 240 > 0)
    {
        if(getPositionY() > 120 && *decalageFenetreY != hauteurCarte - 240)
        {
            a = getPositionY()-120;
            setPositionY(-a);
            *decalageFenetreY = a;
        }

        if(getPositionY() < 120 && *decalageFenetreY != 0)
        {
            a = getPositionY()-120;
            setPositionY(a);
            *decalageFenetreY = -a;
        }
    }
}

SDL_Rect Hero::selectionSprite(bool quitter,bool affichageMenuJeu)
{
    SDL_Rect sprite;

    // Link à terre
    if(getEnergie() <= 0 && (!getTouche() || getCompteurTouche() <= TEMPS_INTOUCHABLE - TEMPS_RECUL))
    {
        sprite = getSDLRect( 3 , 0 , 0 );
    }

    // Link nage
    else if(getNage())
    {
        if(getBougeHaut() || getBougeBas() || getBougeGauche() || getBougeDroite() || inertieVerticale != 24 || inertieHorizontale != 24)
        {
            sprite = getSDLRect( 1 , getOrientation() , 2+(getCompteurAnim()/15)%3 );
        }
        else
        {
            sprite = getSDLRect( 1 , getOrientation() , (getCompteurAnim()/15)%2 );
        }
    }

    // Porte un objet
    else if(getPorteObjet())
    {
        if(getBougeHaut() || getBougeBas() || getBougeGauche() || getBougeDroite())
        {
            sprite = getSDLRect( 2 , getOrientation() , (getCompteurAnim()/TEMPS_ANIMATION_ARME)%4 );
        }
        else
        {
            sprite = getSDLRect( 2 , getOrientation() , 0 );
        }
    }

    // Coup d'épée
    else if(getCoupEpee())
    {
        sprite = getSDLRect( 0 , getOrientation()+4 , (getCompteurAnim()/TEMPS_ANIMATION_ARME)%5 );
    }

    // Utilisation d'une arme / item secondaire
    else if(getUtilisationArme())
    {
        switch(getItemActif())
        {
            case 1: // arc
                // A retravailler
                sprite = getSDLRect( 0 , getOrientation()+8 , 5+(getCompteurAnim()/4) );
                break;

            case 5: // Baguette de feu
            case 6: // Baguette de glace
            case 9: // Maillet / marteau
                sprite = getSDLRect( 0 , getOrientation()+12 , getCompteurAnim()/(2*TEMPS_ANIMATION_ARME) );
                break;

            default: // Rien de particulier
                sprite = getSDLRect( 0 , getOrientation() , 1+(getCompteurAnim()/3)%8 );
                break;
        }
    }

    // Recul automatique
    else if(getTouche() && getCompteurTouche() > TEMPS_INTOUCHABLE - TEMPS_RECUL)
    {
        sprite = getSDLRect( 0 , getOrientation() + 4 , 8 );
    }

    // Pas d'action et ne bouge pas
    else if(!getBougeHaut() && !getBougeBas() && !getBougeGauche() && !getBougeDroite())
    {
        sprite = getSDLRect( 0 , getOrientation() , 0 );
    }

    // Déplacement rapide
    else if(getDeplacementRapide())
    {
        sprite = getSDLRect( 0 , getOrientation() , 1+(getCompteurAnim()/2)%8 );
    }

    // Perso à l'arrêt
    else if(quitter || affichageMenuJeu)
    {
        sprite = getSDLRect( 0 , getOrientation() , 0 );
    }

    // Déplacement normal
    else
    {
        sprite = getSDLRect( 0 , getOrientation() , 1+(getCompteurAnim()/4)%8 );
    }

    return sprite;
}


void Hero::afficherArme(SDL_Surface* screenPartieEnCours,SDL_Rect screen)
{
    SDL_Rect sprite;
    int positionXObjet = 0, positionYObjet = 0;

    // Si le perso ne nage pas et ne porte pas d'objet, on peut afficher l'arme ou le bouclier
    if(!getNage() && !getPorteObjet())
    {
        if(getCoupEpee())
        {
            afficherEpee(&sprite, &positionXObjet, &positionYObjet);
        }

        // Autres armes
        else if(getUtilisationArme())
        {
            afficherArmeSecondaire(&sprite, &positionXObjet, &positionYObjet);
        }

        // Bouclier
        else
        {
            afficherBouclier(&sprite, &positionXObjet, &positionYObjet);
        }

        // Une fois l'arme à afficher sélectionnée et paramétrée, on l'affiche où il faut à l'écran
        screen.x += positionXObjet;
        screen.y += positionYObjet;
        if(getEnergie() > 0) epeeEtArmesSecondaires->afficherEntite(screenPartieEnCours,sprite,screen);

        // Pour facilité la gestion des collisions entre l'arme utilisée et les ennemis ou les éléments du décors, la position et les dimensions de l'arme doit être mémorisée
        epeeEtArmesSecondaires->setZoneActive(screen.x,screen.y,sprite.w,sprite.h);
    }
}

// A factoriser
void Hero::afficherEpee(SDL_Rect* sprite, int* positionXObjet, int* positionYObjet)
{
    // Sélection de la bonne immage de l'épée
    switch(getNiveauEpee())
    {
        case 2:
            epeeEtArmesSecondaires->selectionFeuilleSpritesArme(2);
            break;
        case 3:
            epeeEtArmesSecondaires->selectionFeuilleSpritesArme(3); // A adapter
            break;
        default:
            epeeEtArmesSecondaires->selectionFeuilleSpritesArme(1);
            break;
    }

    if(getNiveauEpee() == 1 || getNiveauEpee() == 2)
    {
        switch(getOrientation())
        {
            case 0:
                switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)%5)
                {
                    case 0:
                        *positionXObjet = 11;
                        *positionYObjet = 7;
                        break;
                    case 1:
                        *positionXObjet = 8;
                        *positionYObjet = -5;
                        break;
                    case 2:
                        *positionXObjet = 6;
                        *positionYObjet = -10;
                        break;
                    case 3:
                        *positionXObjet = -1;
                        *positionYObjet = -8;
                        break;
                    case 4:
                        *positionXObjet = -12;
                        *positionYObjet = 3;
                        break;
                }
                break;

            case 1:
                switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)%5)
                {
                    case 0:
                        *positionXObjet = -8;
                        *positionYObjet = 11;
                        break;
                    case 1:
                        *positionXObjet = -4;
                        *positionYObjet = 12;
                        break;
                    case 2:
                        *positionXObjet = 9;
                        *positionYObjet = 19;
                        break;
                    case 3:
                        *positionXObjet = 11;
                        *positionYObjet = 13;
                        break;
                    case 4:
                        *positionXObjet = 16;
                        *positionYObjet = 12;
                        break;
                }
                break;

            case 2:
                switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)%5)
                {
                    case 0:
                        *positionXObjet = -2;
                        *positionYObjet = 5;
                        break;
                    case 1:
                        *positionXObjet = -8;
                        *positionYObjet = 6;
                        break;
                    case 2:
                        *positionXObjet = -10;
                        *positionYObjet = 13;
                        break;
                    case 3:
                        *positionXObjet = -10;
                        *positionYObjet = 14;
                        break;
                    case 4:
                        *positionXObjet = -4;
                        *positionYObjet = 18;
                        break;
                }
                break;

            case 3:
                switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)%5)
                {
                    case 0:
                        *positionXObjet = 11;
                        *positionYObjet = 5;
                        break;
                    case 1:
                        *positionXObjet = 12;
                        *positionYObjet = 4;
                        break;
                    case 2:
                        *positionXObjet = 15;
                        *positionYObjet = 13;
                        break;
                    case 3:
                        *positionXObjet = 14;
                        *positionYObjet = 14;
                        break;
                    case 4:
                        *positionXObjet = 7;
                        *positionYObjet = 17;
                        break;
                }
                break;

            default:
                break;
        }

        // SDL_Rect de l'épée de taille normale
        *sprite = getSDLRect( 4 , getOrientation() , (getCompteurAnim()/TEMPS_ANIMATION_ARME)%5 );

    }

    else if(getNiveauEpee() == 3)
    {
        switch(getOrientation())
        {
            case 0:
                switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)%5)
                {
                    case 0:
                        *positionXObjet = 14;
                        *positionYObjet = 7;
                        break;
                    case 1:
                        *positionXObjet = 10;
                        *positionYObjet = -11;
                        break;
                    case 2:
                        *positionXObjet = 2;
                        *positionYObjet = -16;
                        break;
                    case 3:
                        *positionXObjet = -12;
                        *positionYObjet = -3;
                        break;
                    case 4:
                        *positionXObjet = -13;
                        *positionYObjet = 3;
                        break;
                }
                break;

            case 1:
                switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)%5)
                {
                    case 0:
                        *positionXObjet = -11;
                        *positionYObjet = 12;
                        break;
                    case 1:
                        *positionXObjet = -4;
                        *positionYObjet = 17;
                        break;
                    case 2:
                        *positionXObjet = 8;
                        *positionYObjet = 20;
                        break;
                    case 3:
                        *positionXObjet = 13;
                        *positionYObjet = 14;
                        break;
                    case 4:
                        *positionXObjet = 16;
                        *positionYObjet = 13;
                        break;
                }
                break;

            case 2:
                switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)%5)
                {
                    case 0:
                        *positionXObjet = -3;
                        *positionYObjet = 3;
                        break;
                    case 1:
                        *positionXObjet = -11;
                        *positionYObjet = 5;
                        break;
                    case 2:
                        *positionXObjet = -20;
                        *positionYObjet = 11;
                        break;
                    case 3:
                        *positionXObjet = -11;
                        *positionYObjet = 15;
                        break;
                    case 4:
                        *positionXObjet = -4;
                        *positionYObjet = 19;
                        break;
                }
                break;

            case 3:
                switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)%5)
                {
                    case 0:
                        *positionXObjet = 15;
                        *positionYObjet = 3;
                        break;
                    case 1:
                        *positionXObjet = 12;
                        *positionYObjet = 5;
                        break;
                    case 2:
                        *positionXObjet = 16;
                        *positionYObjet = 11;
                        break;
                    case 3:
                        *positionXObjet = 13;
                        *positionYObjet = 15;
                        break;
                    case 4:
                        *positionXObjet = 6;
                        *positionYObjet = 19;
                        break;
                }
                break;

            default:
                break;
        }

        // SDL_Rect de l'épée de grande taille
        *sprite = getSDLRect( 21 , getOrientation() , (getCompteurAnim()/TEMPS_ANIMATION_ARME)%5 );
    }
}

void Hero::afficherArmeSecondaire(SDL_Rect* sprite, int* positionXObjet, int* positionYObjet)
{
   // Sélection de la bonne partie du SDL_Rect associé à l'arme secondaire utilisée
    switch(getItemActif())
    {
        case 1: // Arc
        // A retravailler
            //sprite = getSDLRect( 25 , getOrientation() , getCompteurAnim()/(3*TEMPS_ANIMATION_ARME) );
            *sprite = getSDLRect( 25 , getOrientation() , getCompteurAnim()/8 );
            break;

        case 5: // Baguette de feu
            *sprite = getSDLRect( 7 , getOrientation() , getCompteurAnim()/(2*TEMPS_ANIMATION_ARME) );
            break;

        case 6: // Baguette de glace
            *sprite = getSDLRect( 8 , getOrientation() , getCompteurAnim()/(2*TEMPS_ANIMATION_ARME) );
            break;

        case 9: // Maillet / marteau
            *sprite = getSDLRect( 6 , getOrientation() , getCompteurAnim()/(2*TEMPS_ANIMATION_ARME) );
            break;

        default:
            break;

    }

    // Mise à jour du type de l'item actif
    epeeEtArmesSecondaires->setType(getItemActif());

    // Sélection de la position à l'écran du sprite à afficher
    switch(getItemActif())
    {
        case 1: // Arc
            epeeEtArmesSecondaires->selectionFeuilleSpritesArme(4);
            switch(getOrientation())
            {
                case 0:
                    switch(getCompteurAnim()/8)
                    {
                        case 0:
                            *positionXObjet = -2;
                            *positionYObjet = 7;
                            break;
                        case 1:
                            *positionXObjet = -4;
                            *positionYObjet = 1;

                        default:
                            break;
                    }
                    break;

                case 1:
                    switch(getCompteurAnim()/8)
                    {
                        case 0:
                            *positionXObjet = 8;
                            *positionYObjet = 8;
                            break;
                        case 1:
                            *positionXObjet = 4;
                            *positionYObjet = 16;

                        default:
                            break;
                    }
                    break;

                case 2:
                    switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)/3)
                    {
                        case 0:
                            *positionXObjet = -2;
                            *positionYObjet = 8;
                            break;
                        case 1:
                            *positionXObjet = -4;
                            *positionYObjet = 6;

                        default:
                            break;
                    }
                    break;

                case 3:
                    switch((getCompteurAnim()/TEMPS_ANIMATION_ARME)/3)
                    {
                        case 0:
                            *positionXObjet = 8;
                            *positionYObjet = 8;
                            break;
                        case 1:
                            *positionXObjet = 12;
                            *positionYObjet = 6;

                        default:
                            break;
                    }
                    break;

                default:
                    break;
            }
            break;


        case 5: // Baquette de feu
        case 6: // Baguette de glace
        case 9: // Marteau

            epeeEtArmesSecondaires->selectionFeuilleSpritesArme(4);
            switch(getOrientation())
            {
                case 0:
                    switch(getCompteurAnim()/(2*TEMPS_ANIMATION_ARME))
                    {
                        case 0:
                            *positionXObjet = -2;
                            *positionYObjet = -4;
                            break;
                        case 1:
                            *positionXObjet = -2;
                            *positionYObjet = -2;
                            break;
                        case 2:
                            *positionXObjet = -2;
                            *positionYObjet = 1;
                            break;
                    }
                    break;

                case 1:
                    switch(getCompteurAnim()/(2*TEMPS_ANIMATION_ARME))
                    {
                        case 0:
                            *positionXObjet = 10;
                            *positionYObjet = -2;
                            break;
                        case 1:
                            *positionXObjet = 10;
                            *positionYObjet = 12;
                            break;
                        case 2:
                            *positionXObjet = 8;
                            *positionYObjet = 19;
                            break;
                    }
                    break;

                case 2:
                    switch(getCompteurAnim()/(2*TEMPS_ANIMATION_ARME))
                    {
                        case 0:
                            *positionXObjet = 0;
                            *positionYObjet = -3;
                            break;
                        case 1:
                            *positionXObjet = -10;
                            *positionYObjet = 6;
                            break;
                        case 2:
                            *positionXObjet = -10;
                            *positionYObjet = 14;
                            break;
                    }
                    break;

                case 3:
                    switch(getCompteurAnim()/(2*TEMPS_ANIMATION_ARME))
                    {
                        case 0:
                            *positionXObjet = 8;
                            *positionYObjet = -3;
                            break;
                        case 1:
                            *positionXObjet = 15;
                            *positionYObjet = 6;
                            break;
                        case 2:
                            *positionXObjet = 13;
                            *positionYObjet = 14;
                            break;
                    }
                    break;

                default:
                    break;
            }

            break;

        default: // Item non affichage (et sans doute à ajouter donc...)
            break;
    }
}

void Hero::afficherBouclier(SDL_Rect* sprite, int* positionXObjet, int* positionYObjet)
{
    epeeEtArmesSecondaires->selectionFeuilleSpritesArme(0);

    switch(getOrientation())
    {
        case 0:
            *positionXObjet = 10;
            *positionYObjet = 9;
            break;

        case 1:
            *positionXObjet = -1;
            *positionYObjet = 13;
            break;

        case 2:
            *positionXObjet = -3;
            *positionYObjet = 9;
            break;

        case 3:
            *positionXObjet = 15;
            *positionYObjet = 9;
            break;

        default:
            break;

    }

    *sprite = getSDLRect( 5 , getOrientation() , 0 );
}


void Hero::setInertieVerticale(bool H, bool B)
{
    setBougeHaut(false);
    setBougeBas(false);

    // Déplacement demandé vers le haut uniquement
    if(H && !B)
    {
        if(inertieVerticale < 48) inertieVerticale += 1;
    }
    // Déplacement demandé vers le bas uniquement
    else if(!H && B)
    {
        if(inertieVerticale > 0) inertieVerticale -= 1;
    }
    // Pas d'actions au clavier ou actions qui s'annulent
    else
    {
        // Retour progressif à l'équilibre
        if(inertieVerticale > 24) inertieVerticale -= 1;
        else if(inertieVerticale < 24) inertieVerticale += 1;
    }

    // En fonction de la somme, l'un ou l'autre des déplacement peut être autorisé
    switch (inertieVerticale)
    {
        case 0:
        case 2:
        case 3:
        case 4:
        case 6:
        case 7:
        case 9:
        case 11:
        case 14:
        case 18:
        case 23:
            setBougeBas(true);
            break;

        case 25:
        case 29:
        case 34:
        case 37:
        case 39:
        case 41:
        case 42:
        case 44:
        case 45:
        case 46:
        case 48:
            setBougeHaut(true);
            break;

        default:
            break;
    }
}

void Hero::setInertieHorizontale(bool G, bool D)
{
    setBougeGauche(false);
    setBougeDroite(false);

    // Déplacement demandé vers le haut uniquement
    if(D && !G)
    {
        if(inertieHorizontale < 48) inertieHorizontale += 1;
    }
    // Déplacement demandé vers le bas uniquement
    else if(!D && G)
    {
        if(inertieHorizontale > 0) inertieHorizontale -= 1;
    }
    // Pas d'actions au clavier ou actions qui s'annulent
    else
    {
        // Retour progressif à l'équilibre
        if(inertieHorizontale > 24) inertieHorizontale -= 1;
        else if(inertieHorizontale < 24) inertieHorizontale += 1;
    }

    // En fonction de la somme, l'un ou l'autre des déplacement peut être autorisé
    switch (inertieHorizontale)
    {
        case 0:
        case 2:
        case 3:
        case 4:
        case 6:
        case 7:
        case 9:
        case 11:
        case 14:
        case 18:
        case 23:
            setBougeGauche(true);
            break;

        case 25:
        case 29:
        case 34:
        case 37:
        case 39:
        case 41:
        case 42:
        case 44:
        case 45:
        case 46:
        case 48:
            setBougeDroite(true);
            break;

        default:
            break;
    }
}

void Hero::initInertie()
{
    inertieVerticale = 24;
    inertieHorizontale = 24;
}


void Hero::gestionEtatsTemporaires()
{
    if(getCoupEpee())
    {
        // Fin du cou d'épée
        if(getCompteurAnim() > (TEMPS_ANIMATION_ARME*5)-2)
        {
            setCoupEpee(false);
            setAction(false);
        }
    }

    else if(getUtilisationArme())
    {
        switch(getItemActif())
        {
            case 1: // Arc
                if(getCompteurAnim() > (TEMPS_ANIMATION_ARME*5)-2)
                {
                    setUtilisationArme(false);
                }
                break;

            case 5: // Baguette de feu
            case 6: // Baguette de glace
            case 9: // Marteau
                if(getCompteurAnim() > (TEMPS_ANIMATION_ARME*6)-2)
                {
                    setUtilisationArme(false);
                }
                break;

            default:
                if(getCompteurAnim() > (TEMPS_ANIMATION_ARME*5)-2)
                {
                    setUtilisationArme(false);
                }
                break;
        }
    }

    if(getCompteurAnim() > 160) initCompteurAnim();
    setIncrCompteurAnim();

    // Gestion de l'état touché du joueur
    if(getCompteurTouche() == 0) setTouche(false);
    else if(getTouche()) setCompteurTouche(getCompteurTouche() - 1);// On décrémente le compteurTouche du Héro

}



