/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef HERO_H_INCLUDED
#define HERO_H_INCLUDED

#include "Personnage.h"
#include "Arme.h"


// Le personnage que le joueur incarne et contrôle
class Hero : public Personnage
{
    public:

        Hero();
        virtual ~Hero();

        void setEnergie(int E);
        int getEnergie();
        void setEnergieMax(int EM);
        int getEnergieMax();

        void setMagie(int M);
        int getMagie();
        int getMagieMax();
        void setJaugeMagieDoublee(bool JMD);
        bool getJaugeMagieDoublee();

        void setFleche(int F);
        int getFleche();
        void setFlecheMax(int FM);
        int getFLecheMax();

        void setBombe(int B);
        int getBombe();
        void setBombeMax(int BM);
        int getBombeMax();

        void setRubis(int R);
        int getRubis();
        void setRubisMax(int RM);
        int getRubisMax();

        void setQuartDeCoeur(int QDC);
        int getQuartDeCoeur();

        void setMAJObjetsRecolte(int type, int valeur); // fonction permettant de mettre à jour le status du joueur en fonction des objets qu'il récolte

        void setTouche(bool T);
        bool getTouche();
        void setCompteurTouche(int CT);
        int getCompteurTouche();

        void setCoupEpee(bool CE);
        bool getCoupEpee();
        void setNiveauEpee(int NE);
        int getNiveauEpee();

        void setUtilisationArme(bool UA);
        bool getUtilisationArme();
        void setEnvoiProjectile(bool EP);
        bool getEnvoiProjectile();

        void setListeItemsDisponibles(int numItem, int valeur);
        int getListeItemsDisponibles(int numItem); // fonction renvoyant la valeur de l'item sélectionné
        void setItemActif(int IA);
        int getItemActif();
        void setValeurItemActif(int VIA);
        int getValeurItemActif();
        Arme getArmeActive();

        void setAction(bool A);
        bool getAction();

        void setNage(bool N);
        bool getNage();

        void setMonteEchelle(bool ME);
        bool getMonteEchelle();

        void setPorteObjet(bool PO);
        bool getPorteObjet();

        void traitementDeplacements(int *decalageFenetreX,int *decalageFenetreY,int largeurCarte,int hauteurCarte,position **table, int derniereToucheEnfoncee, bool *fleches);

        SDL_Rect selectionSprite(bool quitter,bool affichageMenuJeu);

        void afficherArme(SDL_Surface* screenPartieEnCours,SDL_Rect screen);

        // permet d'ajouter de l'inertie entre ce qui est demandé par le joueur au clavier et le mouvement effectué par le perso
        void setInertieVerticale(bool H, bool B);
        void setInertieHorizontale(bool G, bool D);
        void initInertie();

        void gestionEtatsTemporaires(); // gère les états temporaires du joueur et le retour à l'état "normal" (l'utilisation d'une arme place la joueur dans un état emporaire par exemple)

    protected:

    private:

        void setMagieMax(int MM);

        void gestionVitesseDeplacement(bool apresPremiereGestionCollisions);

        void gestionDerniersDeplacementsDemandesVersLeHaut(int *decalageFenetreX,int *decalageFenetreY,int hauteurCarte, int derniereToucheEnfoncee, bool dernierDeplacementDemande);
        void gestionDerniersDeplacementsDemandesVersLeBas(int *decalageFenetreX,int *decalageFenetreY,int hauteurCarte, int derniereToucheEnfoncee, bool dernierDeplacementDemande);
        void gestionDerniersDeplacementsDemandesVersLaGauche(int *decalageFenetreX,int *decalageFenetreY,int largeurCarte, int derniereToucheEnfoncee, bool dernierDeplacementDemande);
        void gestionDerniersDeplacementsDemandesVersLaDroite(int *decalageFenetreX,int *decalageFenetreY,int largeurCarte, int derniereToucheEnfoncee, bool dernierDeplacementDemande);

        void gestionOrientation(bool *fleches);

        void gestionPositionJoueurSurEcran(int *decalageFenetreX,int *decalageFenetreY,int largeurCarte,int hauteurCarte);

        void afficherEpee(SDL_Rect* sprite, int* positionXObjet, int* positionYObjet);
        void afficherArmeSecondaire(SDL_Rect* sprite, int* positionXObjet, int* positionYObjet);
        void afficherBouclier(SDL_Rect* sprite, int* positionXObjet, int* positionYObjet);

        int energie; // Niveau d'énergie actuel du héro
        int energieMax; // Niveau d'énergie max du héro
        int magie; // Niveau de magie en cours du héro
        int magieMax; // Niveau de magie max du héro
        bool jaugeMagieDoublee; // Indique si la jauge de magie est doublée ou non
        int fleche; // Nombre de flèches que détient le héro
        int flecheMax; // Nombre de flèches max que peut détenir le héro
        int bombe; // Nombre de bombes que détient le héro
        int bombeMax; // Nombre de bombes max que peut détenir le héro
        int rubis; // Nombre de rubis que détient le héro
        int rubisMax; // Nombre de rubis max que peut détenir le héro
        int quartDeCoeur; // nombre de quarts de cœur acquis en vu d'ajouter un nouveau cœur de vie au joueur

        bool touche; // indique que le joueur est actuellement touché (cet état est prévu pour durer un temps donné, temps durant lequel il n'est pas possible de faire perdre de l'énergie au héro)
        int compteurTouche; // Compteur utilisé pour générer le comportement du héro lorsqu'il est touché
        bool coupEpee; // Vrai tant que l'épée est sortie de son fourreau
        int niveauEpee; // Arme permanente du héro mais dont le niveau varie (de 0 si non active à 3 sur puissance max). Le dégât de l'épée sont proportionnels à son niveau
        bool utilisationArme; // Vrai durant un temps donné si utilisation d'une arme (permet d'interdire temporairement l'activation d'une autre arme que celle utilisée).
        bool envoiProjectile; // Vrai durant un temps donné si envoi d'un projectile en cours (permet d'interdire temporairement l'activation d'une autre arme que celle utilisée).
        int itemActif; // Contient le numéro associé à l'item actif
        int valeurItemActif; //Renvoi la valeur représentative de la puissance de l'arme secondaire dont dispose le joueur.
        bool action; // Vrai tant qu'une action demandée par le joueur est en cours (lire, parler,monter un objet au dessus de sa tête) (A)
        bool nage; // Vrai si link nage
        bool monteEchelle; // Vrai si link monte une échelle
        bool porteObjet; // Vrai si le perso est en train de porterun objet

        Arme* epeeEtArmesSecondaires; // Gère l'arme affichée (que ce soit l'épée ou n'importe quelle arme secondaire)
        int listeItemsDisponibles[52]; // Ensemble des items que le joueur peut utiliser ; par convention, la valeur 0 signifie que l'item n'est disponible

        int inertieVerticale; // Gère l'inertie du mouvement vertical du joueur dans l'eau
        int inertieHorizontale; // Gère l'inertie du mouvement horizontal du joueur dans l'eau

        Mix_Chunk* sonsJoueur[10]; // Création du tableau pointeurs de type Mix_Chunk

};



#endif // HERO_H_INCLUDED
