/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "ImagesPartie.h"

ImagesPartie::ImagesPartie()
{

}

ImagesPartie::~ImagesPartie()
{

}

SDL_Surface* ImagesPartie::chargerImage( std::string nomFichier )
{
    //L'image qui est chargée
    SDL_Surface* imageChargee = NULL;

    //L'image optimisée qu'on va utiliser
    SDL_Surface* imageOptimisee = NULL;

    //Chargement de l'image
    imageChargee = IMG_Load( nomFichier.c_str() );

    //Si l'image est chargée
    if( imageChargee != NULL )
    {
        //Création de l'image optimisée
        imageOptimisee = SDL_DisplayFormat( imageChargee );

        //Libération de l'espace mémoire occupé par l'ancienne image
        SDL_FreeSurface( imageChargee );

        //Si la création de l'image optimisée s'est bien passée
        if( imageOptimisee != NULL ) SDL_SetColorKey( imageOptimisee, SDL_RLEACCEL | SDL_SRCCOLORKEY, SDL_MapRGB( imageOptimisee->format, 0x00, 0x00, 0xff ));
    }

    //On retourne l'image optimisée
    return imageOptimisee;
}

void ImagesPartie::initSDLSurface(int numSelectionSprite)
{
    switch(numSelectionSprite)
    {
        // Tileset à charger
        case 0:
            tilesetA = chargerImage("images/motifs/motifsA.png");
            tilesetB = chargerImage("images/motifs/motifsB.png");
            tilesetC = chargerImage("images/motifs/motifsC.png");
            tilesetD = chargerImage("images/motifs/motifsD.png");
            break;

        // PNJ et autres images présents sur la carte
        case 1:
            ennemi1 = chargerImage("images/personnages/ennemi1.png");
            ennemi2 = chargerImage("images/personnages/ennemi2.png");
            ennemi3 = chargerImage("images/personnages/ennemi3.png");
            ennemi4 = chargerImage("images/personnages/ennemi4.png");
            ennemi5 = chargerImage("images/personnages/ennemi5.png");
            ennemi6 = chargerImage("images/personnages/ennemi6.png");
            ennemi7 = chargerImage("images/personnages/ennemi7.png");
            ennemi8 = chargerImage("images/personnages/ennemi8.png");
            ennemi9 = chargerImage("images/personnages/ennemi9.png");
            ennemi10 = chargerImage("images/personnages/ennemi10.png");
            ennemi11 = chargerImage("images/personnages/ennemi11.png");
            ennemi12 = chargerImage("images/personnages/ennemi12.png");
            ennemi13 = chargerImage("images/personnages/ennemi13.png");
            ennemi14 = chargerImage("images/personnages/ennemi14.png");
            ennemi15 = chargerImage("images/personnages/ennemi15.png");
            ennemi16 = chargerImage("images/personnages/ennemi16.png");
            ennemi17 = chargerImage("images/personnages/ennemi17.png");
            ennemi18 = chargerImage("images/personnages/ennemi18.png");
            ennemi19 = chargerImage("images/personnages/ennemi19.png");
            ennemi20 = chargerImage("images/personnages/ennemi20.png");
            ennemi21 = chargerImage("images/personnages/ennemi21.png");
            ennemi22 = chargerImage("images/personnages/ennemi22.png");
            ennemi23 = chargerImage("images/personnages/ennemi23.png");
            ennemi24 = chargerImage("images/personnages/ennemi24.png");
            ennemi25 = chargerImage("images/personnages/ennemi25.png");
            ennemi26 = chargerImage("images/personnages/ennemi26.png");
            ennemi27 = chargerImage("images/personnages/ennemi27.png");
            ennemi28 = chargerImage("images/personnages/ennemi28.png");
            ennemi29 = chargerImage("images/personnages/ennemi29.png");
            ennemi30 = chargerImage("images/personnages/ennemi30.png");
            ennemi31 = chargerImage("images/personnages/ennemi31.png");
            ennemi32 = chargerImage("images/personnages/ennemi32.png");
            ennemi33 = chargerImage("images/personnages/ennemi33.png");
            ennemi34 = chargerImage("images/personnages/ennemi34.png");
            ennemi35 = chargerImage("images/personnages/ennemi35.png");
            ennemi36 = chargerImage("images/personnages/ennemi36.png");
            ennemi37 = chargerImage("images/personnages/ennemi37.png");
            ennemi38 = chargerImage("images/personnages/ennemi38.png");
            ennemi39 = chargerImage("images/personnages/ennemi39.png");
            ennemi40 = chargerImage("images/personnages/ennemi40.png");
            ennemi41 = chargerImage("images/personnages/ennemi41.png");
            ennemi42 = chargerImage("images/personnages/ennemi42.png");
            ennemi43 = chargerImage("images/personnages/ennemi43.png");
            ennemi44 = chargerImage("images/personnages/ennemi44.png");
            ennemi45 = chargerImage("images/personnages/ennemi45.png");
            ennemi46 = chargerImage("images/personnages/ennemi46.png");
            ennemi47 = chargerImage("images/personnages/ennemi47.png");
            ennemi48 = chargerImage("images/personnages/ennemi48.png");
            ennemi49 = chargerImage("images/personnages/ennemi49.png");
            ennemi50 = chargerImage("images/personnages/ennemi50.png");
            ennemi51 = chargerImage("images/personnages/ennemi51.png");
            ennemi52 = chargerImage("images/personnages/ennemi52.png");
            personnage1 = chargerImage("images/personnages/personnage1.png");
            personnage2 = chargerImage("images/personnages/personnage2.png");
            personnage3 = chargerImage("images/personnages/personnage3.png");
            personnage4 = chargerImage("images/personnages/personnage4.png");
            personnage5 = chargerImage("images/personnages/personnage5.png");
            personnage6 = chargerImage("images/personnages/personnage6.png");
            personnage7 = chargerImage("images/personnages/personnage7.png");
            personnage8 = chargerImage("images/personnages/personnage8.png");
            personnage9 = chargerImage("images/personnages/personnage9.png");
            personnage10 = chargerImage("images/personnages/personnage10.png");
            personnage11 = chargerImage("images/personnages/personnage11.png");
            personnage12 = chargerImage("images/personnages/personnage12.png");
            personnage13 = chargerImage("images/personnages/personnage13.png");
            personnage14 = chargerImage("images/personnages/personnage14.png");
            personnage15 = chargerImage("images/personnages/personnage15.png");
            personnage16 = chargerImage("images/personnages/personnage16.png");
            personnage17 = chargerImage("images/personnages/personnage17.png");
            personnage18 = chargerImage("images/personnages/personnage18.png");
            personnage19 = chargerImage("images/personnages/personnage19.png");
            personnage20 = chargerImage("images/personnages/personnage20.png");
            personnage21 = chargerImage("images/personnages/personnage21.png");
            personnage22 = chargerImage("images/personnages/personnage22.png");
            personnage23 = chargerImage("images/personnages/personnage23.png");
            personnage24 = chargerImage("images/personnages/personnage24.png");
            personnage25 = chargerImage("images/personnages/personnage25.png");
            personnage26 = chargerImage("images/personnages/personnage26.png");
            personnage27 = chargerImage("images/personnages/personnage27.png");
            personnage28 = chargerImage("images/personnages/personnage28.png");
            personnage29 = chargerImage("images/personnages/personnage29.png");
            personnage30 = chargerImage("images/personnages/personnage30.png");
            personnage31 = chargerImage("images/personnages/personnage31.png");
            personnage32 = chargerImage("images/personnages/personnage32.png");
            explo = chargerImage("images/personnages/explo.png");
            break;

        // Utilisé par le héro
        case 2:
            exploLapin = chargerImage("images/personnages/exploLapin.png");
            link1 = chargerImage("images/personnages/link1.png");
            spin = chargerImage("images/personnages/spin.png");
            break;

        // Armes du héro et équipements visibles du héro
        case 3:
            bouclier1 = chargerImage("images/personnages/bouclier1.png");
            epee1 = chargerImage("images/personnages/epee1.png");
            epee3 = chargerImage("images/personnages/epee3.png");
            epee5 = chargerImage("images/personnages/epee5.png");
            objets1 = chargerImage("images/personnages/objets.png");
            break;

        // Objets divers utilisés par la carte
        case 4:
            bombe = chargerImage("images/objets/bombe.png");
            boomerang = chargerImage("images/objets/boomerang.png");
            buisson = chargerImage("images/objets/buisson.png");
            caisse = chargerImage("images/objets/caisse.png");
            coeurs = chargerImage("images/objets/coeurs.png");
            feu = chargerImage("images/objets/feu.png");
            grappin = chargerImage("images/objets/grappin.png");
            magie = chargerImage("images/objets/magie.png");
            objets1 = chargerImage("images/personnages/objets.png");
            objets2 = chargerImage("images/objets/objets.png");
            objets4 = chargerImage("images/objets/objets4.png");
            plouf = chargerImage("images/objets/plouf.png");
            projectiles1 = chargerImage("images/objets/projectiles1.png");
            projectiles2 = chargerImage("images/objets/projectiles2.png");
            projectiles3 = chargerImage("images/objets/projectiles3.png");
            projectiles4 = chargerImage("images/objets/projectiles4.png");
            projectiles5 = chargerImage("images/objets/projectiles5.png");
            projectiles6 = chargerImage("images/objets/projectiles6.png");
            projectiles7 = chargerImage("images/objets/projectiles7.png");
            projectiles8 = chargerImage("images/objets/projectiles8.png");
            projectiles9 = chargerImage("images/objets/projectiles9.png");
            projectiles10 = chargerImage("images/objets/projectiles10.png");
            break;

        // Statut du joueur & menu de la partie
        case 5:
            bord = chargerImage("images/statut/bord.png");
            chiffres = chargerImage("images/statut/chiffres.png");
            coeur = chargerImage("images/statut/coeur.png");
            inventaire = chargerImage("images/statut/inventaire.png");
            level = chargerImage("images/statut/level.png");
            objets3 = chargerImage("images/statut/objets.png");
            statut = chargerImage("images/statut/statut.png");
            break;

        // Texte
        case 6:
            font = chargerImage("images/texte/font.png");
            break;

        default:

            break;
    }

}

void ImagesPartie::initSDLRect(int numSelectionSprite) // NE PAS OUBLIER D'UTILISER LE numSelectionSprite POUR OPTIMISER LE TEMPS NÉCESSAIRE À L'INIT
{
    int i,j;


    // À remplir/compléter
    switch(numSelectionSprite)
    {
        case 0: // Tileset
            break;


        case 1: // PNJ

            // Explosion
            for (i=0;i<7;i++)
            {
                explosion[i].x = 28*i;
                explosion[i].y = 0;
                explosion[i].w = 28;
                explosion[i].h = 28;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjA[i][j].x = i*16; // dimensions : x = 16 et y = 26 (64*52) ; E(1;29;37)
                    PnjA[i][j].y = j*26;
                    PnjA[i][j].w = 16;
                    PnjA[i][j].h = 26;
                }
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjB[i][j].x = i*21; // dimensions : x = 21 et y = 26 (84*52) ; E(2)
                    PnjB[i][j].y = j*26;
                    PnjB[i][j].w = 21;
                    PnjB[i][j].h = 26;
                }
            }

            for (i=0;i<3;i++)
            {
                PnjC[i].x = i*28; // dimensions : x = 28 et y = 17 (84*17) ; E(3)
                PnjC[i].y = 0;
                PnjC[i].w = 28;
                PnjC[i].h = 17;
            }

            for (i=0;i<3;i++)
            {
                PnjD[i].x = i*16; // dimensions : x = 16 et y = 17 (48*17) ; E(4;17)
                PnjD[i].y = 0;
                PnjD[i].w = 16;
                PnjD[i].h = 17;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjE[i][j].x = i*16; // dimensions : x = 16 et y = 26 (77*52) -> (64*52) ; E(5)
                    PnjE[i][j].y = j*26;
                    PnjE[i][j].w = 16;
                    PnjE[i][j].h = 26;
                }
            }

            for (i=0;i<11;i++)
            {
                PnjF[i].x = i*16; // dimensions : x = 16 et y = 37 (176*37) ; E(6)
                PnjF[i].y = 0;
                PnjF[i].w = 16;
                PnjF[i].h = 37;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjG[i][j].x = i*16; // dimensions : x = 16 et y = 16 (64*32) ; E(7)
                    PnjG[i][j].y = j*16;
                    PnjG[i][j].w = 16;
                    PnjG[i][j].h = 16;
                }
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjH[i][j].x = i*16; // dimensions : x = 16 et y = 24 (64*48) ; E(8;9;16;30;41) P(11)
                    PnjH[i][j].y = j*24;
                    PnjH[i][j].w = 16;
                    PnjH[i][j].h = 24;
                }
            }

            for (i=0;i<2;i++)
            {
                PnjI[i].x = i*16; // dimensions : x = 16 et y = 16 (32*16) ; E(10;20)
                PnjI[i].y = 0;
                PnjI[i].w = 16;
                PnjI[i].h = 16;
            }

            for (i=0;i<4;i++)
            {
                PnjJ[i].x = i*16; // dimensions : x = 16 et y = 21 (64*21) ; E(11)
                PnjJ[i].y = 0;
                PnjJ[i].w = 16;
                PnjJ[i].h = 21;
            }

            for (i=0;i<2;i++)
            {
                for (j=0;j<3;j++)
                {
                    PnjK[i][j].x = i*17; // dimensions : x = 17 et y = 17 (34*51) ; E(12)
                    PnjK[i][j].y = j*17;
                    PnjK[i][j].w = 17;
                    PnjK[i][j].h = 17;
                }
            }

            for (i=0;i<3;i++)
            {
                PnjL[i].x = i*32; // dimensions : x = 32 et y = 15 (96*15) ; E(13)
                PnjL[i].y = 0;
                PnjL[i].w = 32;
                PnjL[i].h = 15;
            }

            for (i=0;i<2;i++)
            {
                PnjM[i].x = i*41; // dimensions : x = 41 et y = 16 (82*16) ; E(14)
                PnjM[i].y = 0;
                PnjM[i].w = 41;
                PnjM[i].h = 16;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<3;j++)
                {
                    PnjN[i][j].x = i*24; // dimensions : x = 24 et y = 31 (96*93) ; E(15)
                    PnjN[i][j].y = j*31;
                    PnjN[i][j].w = 24;
                    PnjN[i][j].h = 31;
                }
            }

            for (i=0;i<3;i++)
            {
                PnjO[i].x = i*16; // dimensions : x = 16 et y = 14 (48*14) ; E(18)
                PnjO[i].y = 0;
                PnjO[i].w = 16;
                PnjO[i].h = 14;
            }

            for (i=0;i<3;i++)
            {
                PnjP[i].x = i*64; // dimensions : x = 64 et y = 32 (192*32) ; E(19 - BOSS)
                PnjP[i].y = 0;
                PnjP[i].w = 64;
                PnjP[i].h = 32;
            }

            for (i=0;i<2;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjQ[i][j].x = i*16; // dimensions : x = 16 et y = 15 (32*30) ; E(21)
                    PnjQ[i][j].y = j*15;
                    PnjQ[i][j].w = 16;
                    PnjQ[i][j].h = 15;
                }
            }

            for (i=0;i<2;i++)
            {
                PnjR[i].x = i*16; // dimensions : x = 16 et y = 23 (32*23) ; E(22)
                PnjR[i].y = 0;
                PnjR[i].w = 16;
                PnjR[i].h = 23;
            }

            for (i=0;i<2;i++)
            {
                PnjS[i].x = i*26; // dimensions : x = 26 et y = 17 (52*17) ; E(23)
                PnjS[i].y = 0;
                PnjS[i].w = 26;
                PnjS[i].h = 17;
            }

            for (i=0;i<2;i++)
            {
                PnjT[i].x = i*32; // dimensions : x = 32 et y = 53 (64*53) ; E(24 - BOSS)
                PnjT[i].y = 0;
                PnjT[i].w = 32;
                PnjT[i].h = 53;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjU[i][j].x = i*28; // dimensions : x = 28 et y = 30 (112*60) ; E(25)
                    PnjU[i][j].y = j*30;
                    PnjU[i][j].w = 28;
                    PnjU[i][j].h = 30;
                }
            }

            for (i=0;i<3;i++)
            {
                PnjV[i].x = i*14; // dimensions : x = 14 et y = 16 (42*16) ; E(26)
                PnjV[i].y = 0;
                PnjV[i].w = 14;
                PnjV[i].h = 16;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<4;j++)
                {
                    PnjW[i][j].x = i*16; // dimensions : x = 16 et y = 17 (64*68) ; E(27)
                    PnjW[i][j].y = j*17;
                    PnjW[i][j].w = 16;
                    PnjW[i][j].h = 17;
                }
            }

            for (i=0;i<2;i++)
            {
                PnjX[i].x = i*32; // dimensions : x = 32 et y = 38 (64*38) ; E(28 - BOSS)
                PnjX[i].y = 0;
                PnjX[i].w = 32;
                PnjX[i].h = 38;
            }

            for (i=0;i<4;i++)
            {
                PnjY[i].x = i*32; // dimensions : x = 32 et y = 32 (128*32) ; E(31 - BOSS)
                PnjY[i].y = 0;
                PnjY[i].w = 32;
                PnjY[i].h = 32;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjZ[i][j].x = i*16; // dimensions : x = 16 et y = 23 (48*23) ; E(32)
                    PnjZ[i][j].y = j*23;
                    PnjZ[i][j].w = 16;
                    PnjZ[i][j].h = 23;
                }
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAA[i][j].x = i*22; // dimensions : x = 22 et y = 25 (88*50) ; E(33;38)
                    PnjAA[i][j].y = j*25;
                    PnjAA[i][j].w = 22;
                    PnjAA[i][j].h = 25;
                }
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAB[i][j].x = i*24; // dimensions : x = 24 et y = 25 (96*50) ; E(34;39)
                    PnjAB[i][j].y = j*25;
                    PnjAB[i][j].w = 24;
                    PnjAB[i][j].h = 25;
                }
            }

            for (i=0;i<3;i++)
            {
                PnjAC[i].x = i*32; // dimensions : x = 32 et y = 32 (96*32) ; E(35 - BOSS)
                PnjAC[i].y = 0;
                PnjAC[i].w = 32;
                PnjAC[i].h = 32;
            }

            for (i=0;i<3;i++)
            {
                PnjAD[i].x = i*54; // dimensions : x = 54 et y = 42 (162*42) ; E(40 - BOSS)
                PnjAD[i].y = 0;
                PnjAD[i].w = 54;
                PnjAD[i].h = 42;
            }

            for (i=0;i<2;i++)
            {
                PnjAE[i].x = i*26; // dimensions : x = 26 et y = 19 (52*19) ; E(42)
                PnjAE[i].y = 0;
                PnjAE[i].w = 26;
                PnjAE[i].h = 19;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<3;j++)
                {
                    PnjAF[i][j].x = i*32; // dimensions : x = 32 et y = 32 (128*96) ; E(43 - BOSS)
                    PnjAF[i][j].y = j*32;
                    PnjAF[i][j].w = 32;
                    PnjAF[i][j].h = 32;
                }
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAG[i][j].x = i*22; // dimensions : x = 22 et y = 26 (103*52) -> (88*52) ; E(44)
                    PnjAG[i][j].y = j*26;
                    PnjAG[i][j].w = 22;
                    PnjAG[i][j].h = 26;
                }
            }

            for (i=0;i<4;i++)
            {
                PnjAH[i].x = i*69; // dimensions : x = 69 et y = 76 (276*76) ; E(45 - BOSS)
                PnjAH[i].y = 0;
                PnjAH[i].w = 69;
                PnjAH[i].h = 76;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAI[i][j].x = i*22; // dimensions : x = 22 et y = 27 (104*54) -> (88*54) ; E(46;47)
                    PnjAI[i][j].y = j*27;
                    PnjAI[i][j].w = 22;
                    PnjAI[i][j].h = 27;
                }
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAJ[i][j].x = i*22; // dimensions : x = 22 et y = 27 (105*54) -> (88*54) ; E(48)
                    PnjAJ[i][j].y = j*27;
                    PnjAJ[i][j].w = 22;
                    PnjAJ[i][j].h = 27;
                }
            }

            for (i=0;i<2;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAK[i][j].x = i*52; // dimensions : x = 52 et y = 36 (104*72) ; E(50 - BOSS)
                    PnjAK[i][j].y = j*36;
                    PnjAK[i][j].w = 52;
                    PnjAK[i][j].h = 36;
                }
            }

            for (i=0;i<2;i++)
            {
                PnjAL[i].x = i*24; // dimensions : x = 24 et y = 28 (48*28) ; E(51)
                PnjAL[i].y = 0;
                PnjAL[i].w = 24;
                PnjAL[i].h = 28;
            }

            for (i=0;i<2;i++)
            {
                for (j=0;j<4;j++)
                {
                    PnjAM[i][j].x = i*16; // dimensions : x = 16 et y = 17 (32*68) ; E(52)
                    PnjAM[i][j].y = j*17;
                    PnjAM[i][j].w = 16;
                    PnjAM[i][j].h = 17;
                }
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAN[i][j].x = i*16; // dimensions : x = 16 et y = 25 (64*50) ; P(1;13;14;23)
                    PnjAN[i][j].y = j*25;
                    PnjAN[i][j].w = 16;
                    PnjAN[i][j].h = 25;
                }
            }

            for (i=0;i<2;i++)
            {
                PnjAO[i].x = i*16; // dimensions : x = 16 et y = 24 (16*48) ; P(2;18)
                PnjAO[i].y = 0;
                PnjAO[i].w = 16;
                PnjAO[i].h = 24;
            }

            for (i=0;i<2;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAP[i][j].x = i*16; // dimensions : x = 16 et y = 23 (32*46) ; P(3)
                    PnjAP[i][j].y = j*23;
                    PnjAP[i][j].w = 16;
                    PnjAP[i][j].h = 23;
                }
            }

            for (i=0;i<4;i++)
            {
                PnjAQ[i].x = i*16; // dimensions : x = 16 et y = 17 (16*68) ; P(4)
                PnjAQ[i].y = 0;
                PnjAQ[i].w = 16;
                PnjAQ[i].h = 17;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAR[i][j].x = i*17; // dimensions : x = 17 et y = 16 (52*32) ; P(5)
                    PnjAR[i][j].y = j*16;
                    PnjAR[i][j].w = 17;
                    PnjAR[i][j].h = 16;
                }
            }

            for (i=0;i<2;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAS[i][j].x = i*15; // dimensions : x = 15 et y = 16 (30*32) ; P(6)
                    PnjAS[i][j].y = j*16;
                    PnjAS[i][j].w = 15;
                    PnjAS[i][j].h = 16;
                }
            }

            for (i=0;i<2;i++)
            {
                PnjAT[i].x = i*16; // dimensions : x = 16 et y = 25 (16*50) ; P(7;12;19;25)
                PnjAT[i].y = 0;
                PnjAT[i].w = 16;
                PnjAT[i].h = 25;
            }

            for (i=0;i<2;i++)
            {
                PnjAU[i].x = i*16; // dimensions : x = 16 et y = 23 (16*46) ; P(8;32)
                PnjAU[i].y = 0;
                PnjAU[i].w = 16;
                PnjAU[i].h = 23;
            }

            for (i=0;i<2;i++)
            {
                PnjAV[i].x = i*16; // dimensions : x = 16 et y = 27 (16*54) ; P(9)
                PnjAV[i].y = 0;
                PnjAV[i].w = 16;
                PnjAV[i].h = 27;
            }

            for (i=0;i<4;i++)
            {
                PnjAW[i].x = i*16; // dimensions : x = 16 et y = 26 (16*104) ; P(10;20)
                PnjAW[i].y = 0;
                PnjAW[i].w = 16;
                PnjAW[i].h = 26;
            }

            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    PnjAX[i][j].x = i*16; // dimensions : x = 16 et y = 23 (64*46) ; P(15)
                    PnjAX[i][j].y = j*23;
                    PnjAX[i][j].w = 16;
                    PnjAX[i][j].h = 23;
                }
            }

            for (i=0;i<2;i++)
            {
                PnjAY[i].x = i*32; // dimensions : x = 32 et y = 48 (32*96) ; P(16)
                PnjAY[i].y = 0;
                PnjAY[i].w = 32;
                PnjAY[i].h = 48;
            }

            for (i=0;i<2;i++)
            {
                PnjAZ[i].x = i*16; // dimensions : x = 16 et y = 25 (18*50) ; P(17)
                PnjAZ[i].y = 0;
                PnjAZ[i].w = 16;
                PnjAZ[i].h = 25;
            }

            for (i=0;i<2;i++)
            {
                PnjBA[i].x = i*17; // dimensions : x = 17 et y = 25 (17*50) ; P(21)
                PnjBA[i].y = 0;
                PnjBA[i].w = 17;
                PnjBA[i].h = 25;
            }

            for (i=0;i<4;i++)
            {
                PnjBB[i].x = i*16; // dimensions : x = 16 et y = 29 (16*116) ; P(22)
                PnjBB[i].y = 0;
                PnjBB[i].w = 16;
                PnjBB[i].h = 29;
            }

            for (i=0;i<2;i++)
            {
                PnjBC[i].x = i*16; // dimensions : x = 16 et y = 21 (16*42) ; P(24)
                PnjBC[i].y = 0;
                PnjBC[i].w = 16;
                PnjBC[i].h = 21;
            }

            for (i=0;i<4;i++)
            {
                PnjBD[i].x = i*39; // dimensions : x = 39 et y = 30 (39*120) ; P(26)
                PnjBD[i].y = 0;
                PnjBD[i].w = 39;
                PnjBD[i].h = 30;
            }

            for (i=0;i<2;i++)
            {
                PnjBE[i].x = i*16; // dimensions : x = 16 et y = 17 (16*34) ; P(27;30;31)
                PnjBE[i].y = 0;
                PnjBE[i].w = 16;
                PnjBE[i].h = 17;
            }

            for (i=0;i<5;i++)
            {
                PnjBF[i].x = i*25; // dimensions : x = 25 et y = 35 (25*175) ; P(28)
                PnjBF[i].y = 0;
                PnjBF[i].w = 25;
                PnjBF[i].h = 35;
            }

            for (i=0;i<5;i++)
            {
                PnjBG[i].x = i*21; // dimensions : x = 21 et y = 27 (21*135) ; P(29)
                PnjBG[i].y = 0;
                PnjBG[i].w = 21;
                PnjBG[i].h = 27;
            }


/*
        SDL_Rect PnjBH[13]; // spin et explosions

        SDL_Rect PnjEArme[4]; // dimensions : fonction de l'orientation ; E(5)
        SDL_Rect PnjAGArme[4]; // dimensions : fonction de l'orientation ; E(44)
        SDL_Rect PnjAIArme[4]; // dimensions : fonction de l'orientation ; E(46)
        SDL_Rect PnjAJArme[4][2]; // dimensions : fonction de l'orientation ; E(48)

*/

        break;


        case 2: // Link

            // Pour sélectionner la bonne position de link dans sa feuille de sprites
            for (i=0;i<26;i++)
            {
                for (j=0;j<9;j++)
                {
                    persoLink[i][j].x = LARGEUR_PERSO*i;
                    persoLink[i][j].y = HAUTEUR_PERSO*j;
                    persoLink[i][j].w = LARGEUR_PERSO;
                    persoLink[i][j].h = HAUTEUR_PERSO;
                }
            }

            // Position de Link lorsqu'il nage
            for (j=0;j<5;j++)
            {
                for (i=0;i<3;i++)
                {
                    persoLinkNage[i][j].x = LARGEUR_PERSO*(i+12);
                    persoLinkNage[i][j].y = HAUTEUR_PERSO*(j+3);
                    persoLinkNage[i][j].w = LARGEUR_PERSO+8*(i/2);
                    persoLinkNage[i][j].h = HAUTEUR_PERSO;
                }

                persoLinkNage[3][j].x = LARGEUR_PERSO*15+8;
                persoLinkNage[3][j].y = HAUTEUR_PERSO*(j+3);
                persoLinkNage[3][j].w = LARGEUR_PERSO+8;
                persoLinkNage[3][j].h = HAUTEUR_PERSO;
            }

            // Position de Link  lorsqu'il porte un objet
            for (j=0;j<5;j++)
            {
                for (i=0;i<3;i++)
                {
                    persoLinkPorte[i][j].x = LARGEUR_PERSO*(i+17);
                    persoLinkPorte[i][j].y = HAUTEUR_PERSO*(j+2);
                    persoLinkPorte[i][j].w = LARGEUR_PERSO;
                    persoLinkPorte[i][j].h = HAUTEUR_PERSO;
                }

                persoLinkPorte[3][j].x = LARGEUR_PERSO*20+16;
                persoLinkPorte[3][j].y = HAUTEUR_PERSO*(j+2);
                persoLinkPorte[3][j].w = LARGEUR_PERSO;
                persoLinkPorte[3][j].h = HAUTEUR_PERSO;
            }

            // Positions spéciales de Link sur la feuille de sprite
            persoLinkSpecial[0].x = 192;
            persoLinkSpecial[0].y = 200;
            persoLinkSpecial[0].w = HAUTEUR_PERSO;
            persoLinkSpecial[0].h = LARGEUR_PERSO;

            break;



        // Armes du héro et équipements visibles du héro
        case 3:

            // Pour sélectionner la bonne image du bouclier dans la feuille de sprites correspondante
            bouclierLink[0].x = 0;
            bouclierLink[0].y = 0;
            bouclierLink[0].w = 8;
            bouclierLink[0].h = 10;

            bouclierLink[1].x = 8;
            bouclierLink[1].y = 0;
            bouclierLink[1].w = 8;
            bouclierLink[1].h = 10;

            bouclierLink[2].x = 16;
            bouclierLink[2].y = 0;
            bouclierLink[2].w = 4;
            bouclierLink[2].h = 10;

            bouclierLink[3].x = 20;
            bouclierLink[3].y = 0;
            bouclierLink[3].w = 4;
            bouclierLink[3].h = 10;


            // Pour sélectionner la bonne image de l'épée dans la feuille de sprites correspondante
            for (i=0;i<6;i++)
            {
                for (j=0;j<6;j++)
                {
                    epeeLink[i][j].x = 14*i;
                    epeeLink[i][j].y = 14*j;
                    epeeLink[i][j].w = 14;
                    epeeLink[i][j].h = 14;
                }
            }


            // Pour sélectionner la bonne image de l'épée dans la feuille de sprites correspondante
            for (i=0;i<6;i++)
            {
                for (j=0;j<6;j++)
                {
                    epeeLinkLongue[i][j].x = 20*i;
                    epeeLinkLongue[i][j].y = 20*j;
                    epeeLinkLongue[i][j].w = 20;
                    epeeLinkLongue[i][j].h = 20;
                }
            }

            // Pour sélectionner la bonne image du marteau dans la feuille de sprites correspondante
            for (i=0;i<4;i++)
            {
                for (j=0;j<3;j++)
                {
                    mailletLink[i][j].x = 14*i;
                    mailletLink[i][j].y = 14*j+74;
                    mailletLink[i][j].w = 14;
                    mailletLink[i][j].h = 14;
                }
            }

            // Pour sélectionner la bonne image du marteau dans la feuille de sprites correspondante
            for (i=0;i<4;i++)
            {
                for (j=0;j<3;j++)
                {
                    baguetteFeuLink[i][j].x = 14*i;
                    baguetteFeuLink[i][j].y = 14*j+32;
                    baguetteFeuLink[i][j].w = 14;
                    baguetteFeuLink[i][j].h = 14;
                }
            }

            // Pour sélectionner la bonne image du marteau dans la feuille de sprites correspondante
            for (i=0;i<4;i++)
            {
                for (j=0;j<3;j++)
                {
                    baguetteGlaceLink[i][j].x = 14*i+56;
                    baguetteGlaceLink[i][j].y = 14*j+32;
                    baguetteGlaceLink[i][j].w = 14;
                    baguetteGlaceLink[i][j].h = 14;
                }
            }

            // Pour sélectionner la bonne image de l'arc dans la feuille de sprites correspondante
            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    arcLink[i][j].x = 16*i;
                    arcLink[i][j].y = 16*j;
                    arcLink[i][j].w = 16;
                    arcLink[i][j].h = 16;
                }
            }

            break;


        // Objets divers utilisés sur la carte
        case 4:
            // Flèches
            for (i=0;i<4;i++)
            {
                for (j=0;j<4;j++)
                {
                    flecheP[i][j].x = 16*i;
                    flecheP[i][j].y = 16*j;
                    flecheP[i][j].w = 16;
                    flecheP[i][j].h = 16;
                }
            }

            // Bombe qui explose
            for(i=0;i<8;i++)
            {
                bombeExplose[i].x = 16*i;
                bombeExplose[i].y = 8;
                bombeExplose[i].w = 16;
                bombeExplose[i].h = 16;
            }
            bombeExplose[9].x = 144;
            bombeExplose[9].y = 0;
            bombeExplose[9].w = 32;
            bombeExplose[9].h = 32;

            // Boomrang
            for (i=0;i<4;i++)
            {
                for (j=0;j<2;j++)
                {
                    boomR[i][j].x = 10*i;
                    boomR[i][j].y = 10*j;
                    boomR[i][j].w = 10;
                    boomR[i][j].h = 10;
                }
            }

            // Feu de la lampe
            for (i=0;i<3;i++)
            {
                feuMagie[i].x = 16*i;
                feuMagie[i].y = 0;
                feuMagie[i].w = 16;
                feuMagie[i].h = 16;
            }

            // Feu baguette de feu
            for (i=0;i<4;i++)
            {
                feuMagieMv[i].x = 51 + 17*i;
                feuMagieMv[i].y = 0;
                feuMagieMv[i].w = 17;
                feuMagieMv[i].h = 17;
            }

            // Glace baguette de glace
            for (i=0;i<3;i++)
            {
                glaceMagieMv[i].x = 17*i;
                glaceMagieMv[i].y = 0;
                glaceMagieMv[i].w = 17;
                glaceMagieMv[i].h = 17;
            }



            // Pour sélectionner la bonne position des feuilles dans sa feuille de sprites
            for (i=0;i<7;i++)
            {
                for (j=0;j<4;j++)
                {
                    feuillesExplose[i][j].x = 27*i;
                    feuillesExplose[i][j].y = 43*j;
                    feuillesExplose[i][j].w = 27;
                    feuillesExplose[i][j].h = 43;
                }
            }

            // Explosion d'un objet lancé
            objetExplose[0].x = 64;
            objetExplose[0].y = 74;
            objetExplose[0].w = 16;
            objetExplose[0].h = 16;

            objetExplose[1].x = 56;
            objetExplose[1].y = 90;
            objetExplose[1].w = 24;
            objetExplose[1].h = 24;

            objetExplose[2].x = 80;
            objetExplose[2].y = 74;
            objetExplose[2].w = 32;
            objetExplose[2].h = 32;




            // Objet uniques ou non que l'on peut récolter sur la carte

            OR_Bombe.x = 0;
            OR_Bombe.y = 0;
            OR_Bombe.w = 16;
            OR_Bombe.h = 16;

            OR_GrandeCle.x = 16;
            OR_GrandeCle.y = 0;
            OR_GrandeCle.w = 16;
            OR_GrandeCle.h = 16;

            OR_PetiteCle.x = 32;
            OR_PetiteCle.y = 0;
            OR_PetiteCle.w = 16;
            OR_PetiteCle.h = 16;

            OR_Parchemin.x = 48;
            OR_Parchemin.y = 0;
            OR_Parchemin.w = 16;
            OR_Parchemin.h = 16;

            OR_CoeurComplet.x = 64;
            OR_CoeurComplet.y = 0;
            OR_CoeurComplet.w = 16;
            OR_CoeurComplet.h = 16;

            OR_QuartDeCoeur.x = 80;
            OR_QuartDeCoeur.y = 0;
            OR_QuartDeCoeur.w = 16;
            OR_QuartDeCoeur.h = 16;

            OR_Boussole.x = 96;
            OR_Boussole.y = 0;
            OR_Boussole.w = 16;
            OR_Boussole.h = 16;

            OR_Cristal.x = 112;
            OR_Cristal.y = 0;
            OR_Cristal.w = 16;
            OR_Cristal.h = 16;

            OR_Lampe.x = 128;
            OR_Lampe.y = 0;
            OR_Lampe.w = 16;
            OR_Lampe.h = 16;

            OR_BaguetteDeFeu.x = 144;
            OR_BaguetteDeFeu.y = 0;
            OR_BaguetteDeFeu.w = 16;
            OR_BaguetteDeFeu.h = 16;

            OR_GrandeEpeePlantee.x = 160;
            OR_GrandeEpeePlantee.y = 0;
            OR_GrandeEpeePlantee.w = 16;
            OR_GrandeEpeePlantee.h = 22;

            OR_LivreVert.x = 176;
            OR_LivreVert.y = 0;
            OR_LivreVert.w = 16;
            OR_LivreVert.h = 16;

            OR_MedaillonRouge.x = 192;
            OR_MedaillonRouge.y = 0;
            OR_MedaillonRouge.w = 16;
            OR_MedaillonRouge.h = 16;

            OR_Gant1.x = 208;
            OR_Gant1.y = 0;
            OR_Gant1.w = 16;
            OR_Gant1.h = 16;

            for (i=0;i<3;i++)
            {
                OR_RubisVert[i].x = 16*i;
                OR_RubisVert[i].y = 17;
                OR_RubisVert[i].w = 16;
                OR_RubisVert[i].h = 16;
            }

            for (i=0;i<3;i++)
            {
                OR_RubisBleu[i].x = 48 + 16*i; // 3
                OR_RubisBleu[i].y = 17; // 3
                OR_RubisBleu[i].w = 16; // 3
                OR_RubisBleu[i].h = 16; // 3
            }

            OR_PetiteEpeePlantee.x = 96;
            OR_PetiteEpeePlantee.y = 17;
            OR_PetiteEpeePlantee.w = 16;
            OR_PetiteEpeePlantee.h = 16;

            OR_Gant2.x = 112;
            OR_Gant2.y = 17;
            OR_Gant2.w = 16;
            OR_Gant2.h = 16;

            OR_Palme.x = 128;
            OR_Palme.y = 17;
            OR_Palme.w = 16;
            OR_Palme.h = 17;

            OR_BaguetteDeGlace.x = 144;
            OR_BaguetteDeGlace.y = 17;
            OR_BaguetteDeGlace.w = 16;
            OR_BaguetteDeGlace.h = 16;

            OR_MedaillonVert.x = 192;
            OR_MedaillonVert.y = 17;
            OR_MedaillonVert.w = 16;
            OR_MedaillonVert.h = 16;

            OR_FlecheArgent.x = 208;
            OR_FlecheArgent.y = 17;
            OR_FlecheArgent.w = 16;
            OR_FlecheArgent.h = 16;

            for (i=0;i<3;i++)
            {
                OR_RubisRouge[i].x = 16*i; // 3
                OR_RubisRouge[i].y = 34; // 3
                OR_RubisRouge[i].w = 16; // 3
                OR_RubisRouge[i].h = 16; // 3
            }

            OR_PetitCoeur.x = 48;
            OR_PetitCoeur.y = 34;
            OR_PetitCoeur.w = 16;
            OR_PetitCoeur.h = 16;

            OR_Fleche.x = 64;
            OR_Fleche.y = 34;
            OR_Fleche.w = 16;
            OR_Fleche.h = 16;

            OR_PetiteReserveMagie.x = 80;
            OR_PetiteReserveMagie.y = 34;
            OR_PetiteReserveMagie.w = 16;
            OR_PetiteReserveMagie.h = 16;

            OR_GrandeReserveMagie.x = 96;
            OR_GrandeReserveMagie.y = 34;
            OR_GrandeReserveMagie.w = 16;
            OR_GrandeReserveMagie.h = 16;

            OR_Grappin.x = 112;
            OR_Grappin.y = 34;
            OR_Grappin.w = 16;
            OR_Grappin.h = 16;

            OR_Maillet.x = 128;
            OR_Maillet.y = 34;
            OR_Maillet.w = 16;
            OR_Maillet.h = 17;

            OR_Arc.x = 144;
            OR_Arc.y = 35;
            OR_Arc.w = 16;
            OR_Arc.h = 16;

            OR_MedaillonBleu.x = 192;
            OR_MedaillonBleu.y = 34;
            OR_MedaillonBleu.w = 16;
            OR_MedaillonBleu.h = 16;

            OR_Bottes.x = 208;
            OR_Bottes.y = 34;
            OR_Bottes.w = 16;
            OR_Bottes.h = 16;

            for (i=0;i<8;i++)
            {
                OR_Coupe[i].x = 16*i;
                OR_Coupe[i].y = 52;
                OR_Coupe[i].w = 16;
                OR_Coupe[i].h = 16;
            }

            OR_TuniqueBleue.x = 128;
            OR_TuniqueBleue.y = 52;
            OR_TuniqueBleue.w = 18;
            OR_TuniqueBleue.h = 16;

            OR_TuniqueRouge.x = 146;
            OR_TuniqueRouge.y = 52;
            OR_TuniqueRouge.w = 18;
            OR_TuniqueRouge.h = 16;

            OR_ArcArgent.x = 164;
            OR_ArcArgent.y = 52;
            OR_ArcArgent.w = 16;
            OR_ArcArgent.h = 16;

            OR_Flute.x = 180;
            OR_Flute.y = 52;
            OR_Flute.w = 14;
            OR_Flute.h = 16;

            OR_Masque.x = 194;
            OR_Masque.y = 52;
            OR_Masque.w = 14;
            OR_Masque.h = 16;

            OR_Bourse.x = 208;
            OR_Bourse.y = 52;
            OR_Bourse.w = 16;
            OR_Bourse.h = 16;
            /*
            OR_EpeeBleue;
            OR_EpeeBleue.y = 64;
            OR_EpeeBleue;
            OR_EpeeBleue.h = 16;

            OR_PetiteEpeeBlanche;
            OR_PetiteEpeeBlanche.y = 64;
            OR_PetiteEpeeBlanche;
            OR_PetiteEpeeBlanche.h = 16;

            OR_PetitBouclier;
            OR_PetitBouclier.y = 64;
            OR_PetitBouclier;
            OR_PetitBouclier.h = 16;

            OR_FlaconVide;
            OR_FlaconVide.y = 64;
            OR_FlaconVide.w = 16;
            OR_FlaconVide.h = 16;

            OR_FlaconRouge;
            OR_FlaconRouge.y = 64;
            OR_FlaconRouge.w = 16;
            OR_FlaconRouge.h = 16;

            OR_FlaconVert;
            OR_FlaconVert.y = 64;
            OR_FlaconVert.w = 16;
            OR_FlaconVert.h = 16;

            OR_FlaconBleu;
            OR_FlaconBleu.y = 64;
            OR_FlaconBleu.w = 16;
            OR_FlaconBleu.h = 16;

            OR_Bouclier2;
            OR_Bouclier2.y = 64;
            OR_Bouclier2.w = 16;
            OR_Bouclier2.h = 16;

            OR_EpeeRouge;
            OR_EpeeRouge.y = 64;
            OR_EpeeRouge;
            OR_EpeeRouge.h = 16;

            OR_EpeeJaune;
            OR_EpeeJaune.y = 64;
            OR_EpeeJaune;
            OR_EpeeJaune.h = 16;

            OR_Bouclier3;
            OR_Bouclier3.y = 64;
            OR_Bouclier3.w = 16;
            OR_Bouclier3.h = 16;

            OR_PetiteEpeeBleue;
            OR_PetiteEpeeBleue.y = 64;
            OR_PetiteEpeeBleue;
            OR_PetiteEpeeBleue.h = 16;

            OR_TriForce;
*/



            //400
            O_PierreNoire.x = 0;
            O_PierreNoire.y = 0;
            O_PierreNoire.w = 16;
            O_PierreNoire.h = 16;

            O_BuissonVert.x = 16;
            O_BuissonVert.y = 0;
            O_BuissonVert.w = 16;
            O_BuissonVert.h = 16;

            O_BuissonMarron.x = 32;
            O_BuissonMarron.y = 0;
            O_BuissonMarron.w = 16;
            O_BuissonMarron.h = 16;

            O_BuissonMauve.x = 48;
            O_BuissonMauve.y = 0;
            O_BuissonMauve.w = 16;
            O_BuissonMauve.h = 16;



            O_Jarre.x = 0;
            O_Jarre.y = 16;
            O_Jarre.w = 16;
            O_Jarre.h = 16;

            O_PetiteTeteDeMort.x = 16;
            O_PetiteTeteDeMort.y = 16;
            O_PetiteTeteDeMort.w = 16;
            O_PetiteTeteDeMort.h = 16;

            O_PierreGrise.x = 32;
            O_PierreGrise.y = 16;
            O_PierreGrise.w = 16;
            O_PierreGrise.h = 16;

            O_Pancarte.x = 48;
            O_Pancarte.y = 16;
            O_Pancarte.w = 16;
            O_Pancarte.h = 16;





            O_JarreSurSocle.x = 0;
            O_JarreSurSocle.y = 32;
            O_JarreSurSocle.w = 16;
            O_JarreSurSocle.h = 16;

            O_PetiteTeteDeMortSurSocle.x = 16;
            O_PetiteTeteDeMortSurSocle.y = 32;
            O_PetiteTeteDeMortSurSocle.w = 16;
            O_PetiteTeteDeMortSurSocle.h = 16;

            O_PetitCoffreFerme.x = 32;
            O_PetitCoffreFerme.y = 32;
            O_PetitCoffreFerme.w = 16;
            O_PetitCoffreFerme.h = 16;

            O_PetitCoffreOuvert.x = 48;
            O_PetitCoffreOuvert.y = 32;
            O_PetitCoffreOuvert.w = 16;
            O_PetitCoffreOuvert.h = 16;




            O_GrandCoffreOuvert.x = 0;
            O_GrandCoffreOuvert.y = 48;
            O_GrandCoffreOuvert.w = 32;
            O_GrandCoffreOuvert.h = 32;

            O_GrandCoffreFerme.x = 32;
            O_GrandCoffreFerme.y = 48;
            O_GrandCoffreFerme.w = 32;
            O_GrandCoffreFerme.h = 32;




            O_GrosseTeteDeMort.x = 0;
            O_GrosseTeteDeMort.y = 80;
            O_GrosseTeteDeMort.w = 16;
            O_GrosseTeteDeMort.h = 16;

            O_PancarteSurFondDesert.x = 16;
            O_PancarteSurFondDesert.y = 80;
            O_PancarteSurFondDesert.w = 16;
            O_PancarteSurFondDesert.h = 16;

            O_PancarteSurFondVert.x = 32;
            O_PancarteSurFondVert.y = 80;
            O_PancarteSurFondVert.w = 16;
            O_PancarteSurFondVert.h = 16;

            O_PancarteSurFondMarron.x = 48;
            O_PancarteSurFondMarron.y = 80;
            O_PancarteSurFondMarron.w = 16;
            O_PancarteSurFondMarron.h = 16;

            break;


        // Statut du joueur & menu de la partie
        case 5:

            // Jauge de magie
            for(i=0;i<2;i++)
            {
                jaugeLink[i].x = i*120;
                jaugeLink[i].y = 3;
                jaugeLink[i].w = 16;
                jaugeLink[i].h = 42;
            }

            // Inscription "1/2" sous la jauge de magie
            inscriptionDemiLink.x = 186;
            inscriptionDemiLink.y = 36;
            inscriptionDemiLink.w = 16;
            inscriptionDemiLink.h = 8;

            // Cadre dans lequel est indiqué l'objet actif
            cadreArmeLink.x = 17;
            cadreArmeLink.y = 5;
            cadreArmeLink.w = 22;
            cadreArmeLink.h = 22;

            // Objet actif à afficher dans le cadre

            // Icone rubis
            rubisLink.x = 52;
            rubisLink.y = 0;
            rubisLink.w = 8;
            rubisLink.h = 8;

            // Nombre de rubis du joueur

            // Icone bombe
            iconeBombeLink.x = 80;
            iconeBombeLink.y = 0;
            iconeBombeLink.w = 8;
            iconeBombeLink.h = 8;

            // Nombre de bombes du joueur

            // Icone arc
            for(i=0;i<2;i++)
            {
                iconeArcLink[i].x = 101;
                iconeArcLink[i].y = i*21;
                iconeArcLink[i].w = 14;
                iconeArcLink[i].h = 8;
            }

            // Nombre de flêches dont dispose le joueur

            // Icone petite clé
            cleLink.x = 50;
            cleLink.y = 28;
            cleLink.w = 8;
            cleLink.h = 8;

            // Nombre de clés du joueur

            // Inscription "vies" à placer au dessus des cœurs
            inscriptionVieLink.x = 164;
            inscriptionVieLink.y = 17;
            inscriptionVieLink.w = 32;
            inscriptionVieLink.h = 10;

            // Pour sélectionner la bonne image de cœurs dans la feuille de statut
            for (i=0;i<3;i++)
            {
                petitsCoeursLink[i].x = 141 + 8*i;
                petitsCoeursLink[i].y = 9;
                petitsCoeursLink[i].w = 7;
                petitsCoeursLink[i].h = 7;
            }

            // Inventaire des objets affichables dans la liste des items disponible
            for(i=0;i<4;i++)
            {
                for(j=0;j<6;j++)
                {
                    inventaireMenu[i][j].x = i*16;
                    inventaireMenu[i][j].y = j*17;
                    inventaireMenu[i][j].w = 16;
                    inventaireMenu[i][j].h = 17;
                }
            }

            // Cadre et fond du menu
            for(i=0;i<3;i++)
            {
                for(j=0;j<3;j++)
                {
                    bordMenu[i][j].x = i*16;
                    bordMenu[i][j].y = j*16;
                    bordMenu[i][j].w = 16;
                    bordMenu[i][j].h = 16;
                }
            }

            // Curseur de sélection item
            for(i=0;i<2;i++)
            {
                selecteurItem[i].x = 48 + i*32;
                selecteurItem[i].y = 0;
                selecteurItem[i].w = 32;
                selecteurItem[i].h = 32;
            }


            for(i=0;i<5;i++)
            {
                for(j=0;j<4;j++)
                {
                    spritesChiffres[i][j].x = 8*i;
                    spritesChiffres[i][j].y = 8*j;
                    spritesChiffres[i][j].w = 8;
                    spritesChiffres[i][j].h = 8;
                }
            }



            break;



        // Texte
        case 6:

            // police de caractères
            for(i=0;i<10;i++)
            {
                for(j=0;j<10;j++)
                {
                    if(j < 3)
                    {
                        police[i][j].x = 2 + i*16;
                        police[i][j].y = 2 + j*16;
                        police[i][j].w = 16;
                        police[i][j].h = 16;
                    }

                    else if(j<6)
                    {
                        police[i][j].x = i*16;
                        police[i][j].y = 52 + (j-3)*16;
                        police[i][j].w = 16;
                        police[i][j].h = 16;
                    }

                    else
                    {
                        police[i][j].x = i*16;
                        police[i][j].y = 54 + (j-3)*16;
                        police[i][j].w = 16;
                        police[i][j].h = 16;
                    }
                }
            }

            // bordu texte

            bordure[0][0].x = 103;
            bordure[0][0].y = 100;
            bordure[0][0].w = 8;
            bordure[0][0].h = 8;

            bordure[1][0].x = 112;
            bordure[1][0].y = 100;
            bordure[1][0].w = 16;
            bordure[1][0].h = 8;

            bordure[2][0].x = 129;
            bordure[2][0].y = 100;
            bordure[2][0].w = 8;
            bordure[2][0].h = 8;

            bordure[0][1].x = 103;
            bordure[0][1].y = 109;
            bordure[0][1].w = 8;
            bordure[0][1].h = 16;

            // bordure[1][1] non traité (vide)

            bordure[2][1].x = 129;
            bordure[2][1].y = 109;
            bordure[2][1].w = 8;
            bordure[2][1].h = 16;

            bordure[0][2].x = 103;
            bordure[0][2].y = 126;
            bordure[0][2].w = 8;
            bordure[0][2].h = 8;

            bordure[1][2].x = 112;
            bordure[1][2].y = 126;
            bordure[1][2].w = 16;
            bordure[1][2].h = 8;

            bordure[2][2].x = 129;
            bordure[2][2].y = 126;
            bordure[2][2].w = 8;
            bordure[2][2].h = 8;

            break;




        default:

            break;


    }





	// Pour sélectionner la bonne image de l'objet à lancer
	for (i=0;i<4;i++)
	{
	    for (j=0;j<3;j++)
	    {
            objetALancer[i][j].x = LARGEUR_TILE*(i+40);
            objetALancer[i][j].y = HAUTEUR_TILE*(j+6);
            objetALancer[i][j].w = LARGEUR_TILE;
            objetALancer[i][j].h = HAUTEUR_TILE;
	    }
	}


	// Pour sélectionner l'image de l'ombre lorsqu'on lance un objet
    ombreObjetLance.x = 0;
    ombreObjetLance.y = 9;
    ombreObjetLance.w = 8;
    ombreObjetLance.h = 4;







}

// Renvoi le SDL_Rect demandée
SDL_Rect ImagesPartie::getSDLRect(int numSurface,int xRect, int yRect)
{

    //cout << "numSurface = " << numSurface << endl;

    switch(numSurface)
    {
        case 0:
            return persoLink[xRect][yRect];
            break;

        case 1:
            return persoLinkNage[xRect][yRect];
            break;

        case 2:
            return persoLinkPorte[xRect][yRect];
            break;

        case 3:
            return persoLinkSpecial[xRect];
            break;

        case 4:
            return epeeLink[xRect][yRect];
            break;

        case 5:
            return bouclierLink[xRect];
            break;

        case 6:
            return mailletLink[xRect][yRect];
            break;

        case 7:
            return baguetteFeuLink[xRect][yRect];
            break;

        case 8:
            return baguetteGlaceLink[xRect][yRect];
            break;

        case 9:
            return objetALancer[xRect][yRect];
            break;

        case 10:
            return ombreObjetLance;
            break;

        case 11:
            return jaugeLink[xRect];
            break;

        case 12:
            return inscriptionDemiLink;
            break;

        case 13:
            return cadreArmeLink;
            break;

        case 14:
            return rubisLink;
            break;

        case 15:
            return iconeBombeLink;
            break;

        case 16:
            return iconeArcLink[xRect];
            break;

        case 17:
            return cleLink;
            break;

        case 18:
            return inscriptionVieLink;
            break;

        case 19:
            return petitsCoeursLink[xRect];
            break;

        case 20:
            return feuillesExplose[xRect][yRect];
            break;

        case 21:
            return epeeLinkLongue[xRect][yRect];
            break;

        case 22:
            return inventaireMenu[xRect][yRect];
            break;

        case 23:
            return bordMenu[xRect][yRect];
            break;

        case 24:
            return selecteurItem[xRect];
            break;

        case 25:
            return arcLink[xRect][yRect];
            break;

        case 26:
            return explosion[xRect];
            break;

        case 27:
            return bombeExplose[xRect];
            break;

        case 28:
            return police[xRect][yRect];
            break;

        case 29:
            return bordure[xRect][yRect];
            break;

        case 30:
            return spritesChiffres[xRect][yRect];
            break;

        case 31:
            return feuMagie[xRect];
            break;

        case 32:
            return boomR[xRect][yRect];
            break;

        case 33:
            return flecheP[xRect][yRect];
            break;

        case 34:
            return feuMagieMv[xRect];
            break;

        case 35:
            return glaceMagieMv[xRect];
            break;

        case 36:
            return objetExplose[xRect];
            break;

        // PNJ

        case 101: // Ennemi 1
        case 129: // Ennemi 29
        case 137: // Ennemi 37
            return PnjA[xRect][yRect];
            break;

        case 102: // Ennemi 2
            return PnjB[xRect][yRect];
            break;

        case 103: // Ennemi 3
            return PnjC[xRect];
            break;

        case 104: // Ennemi 4
        case 117: // Ennemi 17 (toujours le 0)
            return PnjD[xRect];
            break;

        case 105: // Ennemi 5
            return PnjE[xRect][yRect];
            break;

        case 106: // Ennemi 6
            return PnjF[xRect];
            break;

        case 107: // Ennemi 7
            return PnjG[xRect][yRect];
            break;

        case 108: // Ennemi 8
        case 109: // Ennemi 9
        case 116: // Ennemi 16
        case 130: // Ennemi 30
        case 141: // Ennemi 41
        case 211: // Ami 11
            return PnjH[xRect][yRect];
            break;

        case 110: // Ennemi 10
        case 120: // Ennemi 20
            return PnjI[xRect];
            break;

        case 111: // Ennemi 11
            return PnjJ[xRect];
            break;

        case 112: // Ennemi 12
            return PnjK[xRect][yRect];
            break;

        case 113: // Ennemi 13
            return PnjL[xRect];
            break;

        case 114: // Ennemi 14
            return PnjM[xRect];
            break;

        case 115: // Ennemi 15
            return PnjN[xRect][yRect];
            break;

        case 118: // Ennemi 18
            return PnjO[xRect];
            break;

        case 119: // Ennemi 19
            return PnjP[xRect];
            break;

        case 121: // Ennemi 21
            return PnjQ[xRect][yRect];
            break;

        case 122: // Ennemi 22
            return PnjR[xRect];
            break;

        case 123: // Ennemi 23
            return PnjS[xRect];
            break;

        case 124: // Ennemi 24
            return PnjT[xRect];
            break;

        case 125: // Ennemi 25
            return PnjU[xRect][yRect];
            break;

        case 126: // Ennemi 26
            return PnjV[xRect];
            break;

        case 127: // Ennemi 27
            return PnjW[xRect][yRect];
            break;

        case 128: // Ennemi 28
            return PnjX[xRect];
            break;

        case 131: // Ennemi 31
            return PnjY[xRect];
            break;

        case 132: // Ennemi 32
            return PnjZ[xRect][yRect];
            break;

        case 133: // Ennemi 33
        case 138: // Ennemi 38
            return PnjAA[xRect][yRect];
            break;

        case 134: // Ennemi 34
        case 139: // Ennemi 39
            return PnjAB[xRect][yRect];
            break;

        case 135: // Ennemi 35
            return PnjAC[xRect];
            break;

        case 136: // Ennemi 36
            return PnjAD[xRect];
            break;

        case 142: // Ennemi 42
            return PnjAE[xRect];
            break;

        case 143: // Ennemi 43
            return PnjAF[xRect][yRect];
            break;

        case 144: // Ennemi 44
            return PnjAG[xRect][yRect];
            break;

        case 145: // Ennemi 45
            return PnjAH[xRect];
            break;

        case 146: // Ennemi 46
        case 147: // Ennemi 47
            return PnjAI[xRect][yRect];
            break;

        case 148: // Ennemi 48
            return PnjAJ[xRect][yRect];
            break;

        case 150: // Ennemi 50
            return PnjAK[xRect][yRect];
            break;

        case 151: // Ennemi 51
            return PnjAL[xRect];
            break;

        case 152: // Ennemi 52
            return PnjAM[xRect][yRect];
            break;

/*

        SDL_Rect PnjEArme[4]; // dimensions : fonction de l'orientation ; E(5)
        SDL_Rect PnjAGArme[4]; // dimensions : fonction de l'orientation ; E(44)
        SDL_Rect PnjAIArme[4]; // dimensions : fonction de l'orientation ; E(46;47)
        SDL_Rect PnjAJArme[4][2]; // dimensions : fonction de l'orientation ; E(48)

*/
        case 201: // Ami 1
        case 213: // Ami 13
        case 214: // Ami 14
        case 223: // Ami 23
            return PnjAN[xRect][yRect];
            break;

        case 202: // Ami 2
        case 218: // Ami 18
            return PnjAO[xRect];
            break;

        case 203: // Ami 3
            return PnjAP[xRect][yRect];
            break;

        case 204: // Ami 4
            return PnjAQ[xRect];
            break;

        case 205: // Ami 5
            return PnjAR[xRect][yRect];
            break;

        case 206: // Ami 6
            return PnjAS[xRect][yRect];
            break;

        case 207: // Ami 7
        case 212: // Ami 12
        case 219: // Ami 19
        case 225: // Ami 25
            return PnjAT[xRect];
            break;

        case 208: // Ami 8
        case 232: // Ami 32
            return PnjAU[xRect];
            break;

        case 209: // Ami 9
            return PnjAV[xRect];
            break;

        case 210: // Ami 10
        case 220: // Ami 20
            return PnjAW[xRect];
            break;

        case 215: // Ami 15
            return PnjAX[xRect][yRect];
            break;

        case 216: // Ami 16
            return PnjAY[xRect];
            break;

        case 217: // Ami 17
            return PnjAZ[xRect];
            break;

        case 221: // Ami 21
            return PnjBA[xRect];
            break;

        case 222: // Ami 22
            return PnjBB[xRect];
            break;

        case 224: // Ami 24
            return PnjBC[xRect];
            break;

        case 226: // Ami 26
            return PnjBD[xRect];
            break;

        case 227: // Ami 27
        case 230: // Ami 30
        case 231: // Ami 31
            return PnjBE[xRect];
            break;

        case 228: // Ami 28
            return PnjBF[xRect];
            break;

        case 229: // Ami 29
            return PnjBG[xRect];
            break;


        // Objets récoltés

        case 300:
            return OR_Bombe;
            break;

        case 301:
            return OR_GrandeCle;
            break;

        case 302:
            return OR_PetiteCle;
            break;

        case 303:
            return OR_Parchemin;
            break;

        case 304:
            return OR_CoeurComplet;
            break;

        case 305:
            return OR_QuartDeCoeur;
            break;

        case 306:
            return OR_Boussole;
            break;

        case 307:
            return OR_Cristal;
            break;

        case 308:
            return OR_Lampe;
            break;

        case 309:
            return OR_BaguetteDeFeu;
            break;

        case 310:
            return OR_GrandeEpeePlantee;
            break;

        case 311:
            return OR_LivreVert;
            break;

        case 312:
            return OR_MedaillonRouge;
            break;

        case 313:
            return OR_Gant1;
            break;

        case 314:
            return OR_RubisVert[xRect];
            break;

        case 315:
            return OR_RubisBleu[xRect];
            break;

        case 316:
            return OR_PetiteEpeePlantee;
            break;

        case 317:
            return OR_Gant2;
            break;

        case 318:
            return OR_Palme;
            break;

        case 319:
            return OR_BaguetteDeGlace;
            break;

        case 320:
            return OR_MedaillonVert;
            break;

        case 321:
            return OR_FlecheArgent;
            break;

        case 322:
            return OR_RubisRouge[xRect];
            break;

        case 323:
            return OR_PetitCoeur;
            break;

        case 324:
            return OR_Fleche;
            break;

        case 325:
            return OR_PetiteReserveMagie;
            break;

        case 326:
            return OR_GrandeReserveMagie;
            break;

        case 327:
            return OR_Grappin;
            break;

        case 328:
            return OR_Maillet;
            break;

        case 329:
            return OR_Arc;
            break;

        case 330:
            return OR_MedaillonBleu;
            break;

        case 331:
            return OR_Bottes;
            break;

        case 332:
            return OR_Coupe[xRect];
            break;

        case 333:
            return OR_TuniqueBleue;
            break;

        case 334:
            return OR_TuniqueRouge;
            break;

        case 335:
            return OR_ArcArgent;
            break;

        case 336:
            return OR_Flute;
            break;

        case 337:
            return OR_Masque;
            break;

        case 338:
            return OR_Bourse;
            break;

        case 339:
            return OR_EpeeBleue;
            break;

        case 340:
            return OR_PetiteEpeeBlanche;
            break;

        case 341:
            return OR_PetitBouclier;
            break;

        case 342:
            return OR_FlaconVide;
            break;

        case 343:
            return OR_FlaconRouge;
            break;

        case 344:
            return OR_FlaconVert;
            break;

        case 345:
            return OR_FlaconBleu;
            break;

        case 346:
            return OR_Bouclier2;
            break;

        case 347:
            return OR_EpeeRouge;
            break;

        case 348:
            return OR_EpeeJaune;
            break;

        case 349:
            return OR_Bouclier3;
            break;

        case 351:
            return OR_PetiteEpeeBleue;
            break;

        case 352:
            return OR_TriForce;
            break;




        // Objets manipulables

        case 400:
            return O_PierreNoire;
            break;

        case 401:
            return O_BuissonVert;
            break;

        case 402:
            return O_BuissonMarron;
            break;

        case 403:
            return O_BuissonMauve;
            break;

        case 404:
            return O_Jarre;
            break;

        case 405:
            return O_PetiteTeteDeMort;
            break;

        case 406:
            return O_PierreGrise;
            break;

        case 407:
            return O_Pancarte;
            break;

        case 408:
            return O_JarreSurSocle;
            break;

        case 409:
            return O_PetiteTeteDeMortSurSocle;
            break;

        case 410:
            return O_PetitCoffreFerme;
            break;

        case 411:
            return O_PetitCoffreOuvert;
            break;

        case 412:
            return O_GrandCoffreOuvert;
            break;

        case 413:
            return O_GrandCoffreFerme;
            break;

        case 414:
            return O_GrosseTeteDeMort;
            break;

        case 415:
            return O_PancarteSurFondDesert;
            break;

        case 416:
            return O_PancarteSurFondVert;
            break;

        case 417:
            return O_PancarteSurFondMarron;
            break;






        default:
            return cleLink;
            break;

    }

}

void ImagesPartie::freeMem(int numSelectionSprite)
{
    switch(numSelectionSprite)
    {
        // Tileset
        case 0:
            SDL_FreeSurface(tilesetA);
            SDL_FreeSurface(tilesetB);
            SDL_FreeSurface(tilesetC);
            SDL_FreeSurface(tilesetD);
            break;

        // PNJ et autres images présents sur la carte
        case 1:
            SDL_FreeSurface(ennemi1);
            SDL_FreeSurface(ennemi2);
            SDL_FreeSurface(ennemi3);
            SDL_FreeSurface(ennemi4);
            SDL_FreeSurface(ennemi5);
            SDL_FreeSurface(ennemi6);
            SDL_FreeSurface(ennemi7);
            SDL_FreeSurface(ennemi8);
            SDL_FreeSurface(ennemi9);
            SDL_FreeSurface(ennemi10);
            SDL_FreeSurface(ennemi11);
            SDL_FreeSurface(ennemi12);
            SDL_FreeSurface(ennemi13);
            SDL_FreeSurface(ennemi14);
            SDL_FreeSurface(ennemi15);
            SDL_FreeSurface(ennemi16);
            SDL_FreeSurface(ennemi17);
            SDL_FreeSurface(ennemi18);
            SDL_FreeSurface(ennemi19);
            SDL_FreeSurface(ennemi20);
            SDL_FreeSurface(ennemi21);
            SDL_FreeSurface(ennemi22);
            SDL_FreeSurface(ennemi23);
            SDL_FreeSurface(ennemi24);
            SDL_FreeSurface(ennemi25);
            SDL_FreeSurface(ennemi26);
            SDL_FreeSurface(ennemi27);
            SDL_FreeSurface(ennemi28);
            SDL_FreeSurface(ennemi29);
            SDL_FreeSurface(ennemi30);
            SDL_FreeSurface(ennemi31);
            SDL_FreeSurface(ennemi32);
            SDL_FreeSurface(ennemi33);
            SDL_FreeSurface(ennemi34);
            SDL_FreeSurface(ennemi35);
            SDL_FreeSurface(ennemi36);
            SDL_FreeSurface(ennemi37);
            SDL_FreeSurface(ennemi38);
            SDL_FreeSurface(ennemi39);
            SDL_FreeSurface(ennemi40);
            SDL_FreeSurface(ennemi41);
            SDL_FreeSurface(ennemi42);
            SDL_FreeSurface(ennemi43);
            SDL_FreeSurface(ennemi44);
            SDL_FreeSurface(ennemi45);
            SDL_FreeSurface(ennemi46);
            SDL_FreeSurface(ennemi47);
            SDL_FreeSurface(ennemi48);
            SDL_FreeSurface(ennemi49);
            SDL_FreeSurface(ennemi50);
            SDL_FreeSurface(ennemi51);
            SDL_FreeSurface(ennemi52);
            SDL_FreeSurface(personnage1);
            SDL_FreeSurface(personnage2);
            SDL_FreeSurface(personnage3);
            SDL_FreeSurface(personnage4);
            SDL_FreeSurface(personnage5);
            SDL_FreeSurface(personnage6);
            SDL_FreeSurface(personnage7);
            SDL_FreeSurface(personnage8);
            SDL_FreeSurface(personnage9);
            SDL_FreeSurface(personnage10);
            SDL_FreeSurface(personnage11);
            SDL_FreeSurface(personnage12);
            SDL_FreeSurface(personnage13);
            SDL_FreeSurface(personnage14);
            SDL_FreeSurface(personnage15);
            SDL_FreeSurface(personnage16);
            SDL_FreeSurface(personnage17);
            SDL_FreeSurface(personnage18);
            SDL_FreeSurface(personnage19);
            SDL_FreeSurface(personnage20);
            SDL_FreeSurface(personnage21);
            SDL_FreeSurface(personnage22);
            SDL_FreeSurface(personnage23);
            SDL_FreeSurface(personnage24);
            SDL_FreeSurface(personnage25);
            SDL_FreeSurface(personnage26);
            SDL_FreeSurface(personnage27);
            SDL_FreeSurface(personnage28);
            SDL_FreeSurface(personnage29);
            SDL_FreeSurface(personnage30);
            SDL_FreeSurface(personnage31);
            SDL_FreeSurface(personnage32);
            SDL_FreeSurface(explo);
            break;

        // Utilisé par le héro
        case 2:
            SDL_FreeSurface(exploLapin);
            SDL_FreeSurface(link1);
            SDL_FreeSurface(spin);
            break;

        // Armes du héro et équipements visibles du héro
        case 3:
            SDL_FreeSurface(bouclier1);
            SDL_FreeSurface(epee1);
            SDL_FreeSurface(epee3);
            SDL_FreeSurface(epee5);
            SDL_FreeSurface(objets1);
            break;

        // Objets divers utilisés sur la carte
        case 4:
            SDL_FreeSurface(bombe);
            SDL_FreeSurface(boomerang);
            SDL_FreeSurface(buisson);
            SDL_FreeSurface(caisse);
            SDL_FreeSurface(coeurs);
            SDL_FreeSurface(feu);
            SDL_FreeSurface(grappin);
            SDL_FreeSurface(magie);
            SDL_FreeSurface(objets2);
            SDL_FreeSurface(plouf);
            SDL_FreeSurface(projectiles1);
            SDL_FreeSurface(projectiles2);
            SDL_FreeSurface(projectiles3);
            SDL_FreeSurface(projectiles4);
            SDL_FreeSurface(projectiles5);
            SDL_FreeSurface(projectiles6);
            SDL_FreeSurface(projectiles7);
            SDL_FreeSurface(projectiles8);
            SDL_FreeSurface(projectiles9);
            SDL_FreeSurface(projectiles10);
            break;

        // Statut du joueur & menu de la partie
        case 5:
            SDL_FreeSurface(bord);
            SDL_FreeSurface(chiffres);
            SDL_FreeSurface(coeur);
            SDL_FreeSurface(inventaire);
            SDL_FreeSurface(level);
            SDL_FreeSurface(objets3);
            SDL_FreeSurface(statut);
            break;

        // Texte
        case 6:
            SDL_FreeSurface(font);
            break;

        default:

            break;
    }

}
