/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Jeu.h"

Jeu::Jeu()
{
    gen = new Generique();
    compteurDebut = 0;
    selectPartie = new SelectionPartie();
    partieEnCours = new DeroulementPartie();
}

Jeu::~Jeu()
{
    delete gen;
    delete selectPartie;
    delete partieEnCours;
}

void Jeu::miseAJour(SDL_Surface* screenJeu, Mix_Chunk* musique, bool* sortieProgramme)
{
    //bool reboucleJeu = false;


    // Affichage de l'écran de présentation (début du jeu)
    if(gen->getDebut())
    {
        gen->afficherDebut(screenJeu);
        compteurDebut++;
        if(compteurDebut > 100)
        {
            gen->setDebut(false);
            compteurDebut = 0;

            // Efface l'écran (remplissage noir)
            SDL_FillRect(screenJeu, 0, SDL_MapRGB(screenJeu->format, 0, 0, 0));
        }
    }


    // Affichage du menu de sélection de partie
    else if(selectPartie->getMenuSelectionActif())
    {
        // Affichage de l'écran de saisie du nom (uniquement dans le cas d'une nouvelle partie)
        if(selectPartie->getMenuCreationNouvellePartie()) selectPartie->afficherMenuCreationNouvellePartie(screenJeu,sortieProgramme);

        // Affichage du menu de suppression de partie
        else if(selectPartie->getMenuSuppressionPartie()) selectPartie->afficherMenuSuppressionPartie(screenJeu,sortieProgramme);

        // Affichage du menu d'options (A définir)
        else if(selectPartie->getMenuOptionsPartie()) selectPartie->afficherMenuOptionsPartie(screenJeu,sortieProgramme);

        // Affichage du menu de sélection d'une nouvelle partie ou d'une partie existante
        else selectPartie->afficherMenuSelectionActif(screenJeu,sortieProgramme);

        // Chargement de la partie sélectionnée (uniquement lorsque getMenuSelectionActif() == false)
        if(!selectPartie->getMenuSelectionActif())
        {
            partieEnCours->chargerSauvegarde(selectPartie->getNumero());

            // Gestion de la musique
            Mix_FreeChunk(musique); //Libération de la musique
            musique = Mix_LoadWAV("musique/Plaine.ogg"); //Chargement de la musique
            //Mix_PlayMusic(musique, -1); //Jouer infiniment la musique
            Mix_PlayChannel(1,musique, -1); //Jouer infiniment la musique
        }
    }


    // Affichage du jeu si la partie est commencée
    else if(partieEnCours->getPartieEnCours())
    {
        partieEnCours->gestionPartieEnCours(screenJeu,musique,sortieProgramme);
    }

    // Affichage de la séquence d'introduction du jeu (suite de planches permettant de mettre en place l'histoire du jeu)
    else if(gen->getIntro()) gen->afficherIntro(screenJeu);

    // Affichage de la séquence de fin du jeu
    else {}

}
