/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "ObjetRecolte.h"

ObjetRecolte::ObjetRecolte()
{
    positionX = 0;
    positionY = 0;

    // Initialisation des SDL_Rect spécifiques aux PNJ et aux objets récolter sur la carte
    initSDLSurface(1);
    initSDLSurface(4);
    initSDLRect(1);
    initSDLRect(4);

    initCompteurAnim();
}

ObjetRecolte::~ObjetRecolte()
{

}

void ObjetRecolte::setValeur(int V) {valeur = V;}
int ObjetRecolte::getValeur() {return valeur;}

void ObjetRecolte::initType()
{
    setFeuilleSprite(objets2);
    setTimeToLive(-1); // Valeur à -1 signifie que l'objet recoltable sera affiché tant que le joueur sera sur la carte où se trouve cet objet recoltable et qu'il n'aura pas encore récolté cet objet recoltable

    switch(getType())
    {
        case 0: // bombe
        case 1: // grande clé
        case 2: // petite clé
        case 3: // parchemin
        case 4: // grand cœur
        case 5: // quart de cœur
        case 6: // bousole
        case 7: // cristal
        case 8: // lampe
        case 9: // baguette de feu
        case 11: // livre vert
        case 12: // médaillon rouge
        case 13: // gant 1
        case 14: // rubis vert
        case 16: // petite épée plantée
        case 17: // gant 2
        case 18: // palme
        case 19: // baguette de glace
        case 20: // médaillon vert
        case 21: // flèche argent
        case 23: // petit cœur
        case 24: // fleche normale
        case 25: // petite réserve de magie
        case 26: // grande réserve de magie
        //case 27: // grappin
        case 28: // maillet
        //case 29: // arc normal
        case 30: // médaillon bleu
        case 31: // bottes
        case 35: // arc argent
        case 38: // bourse

            setDimensionX(16);
            setDimensionY(16);
            setValeur(1);
            break;

        case 15: // rubis bleu
            setDimensionX(16);
            setDimensionY(16);
            setValeur(5);
            break;

        case 22: // rubis rouge
            setDimensionX(16);
            setDimensionY(16);
            setValeur(20);
            break;

        case 10: // grande épée plantée
            setDimensionX(16);
            setDimensionY(22);
            break;

        case 33: // tunique bleue
        case 34: // tunique rouge
            break;

        case 36: // flute
        case 37: // masque
            break;

        default:
            break;
    }

}

void ObjetRecolte::intelligenceArtificielle()
{
    // Lorsque l'objet recolté a une durée de vie limitée, il se déplace légèrement vers le haut de l'écran
    if(getTimeToLive() > 0)
    {
        setPositionY(-(getTimeToLive()%3)/2);
    }
}


SDL_Rect ObjetRecolte::selectionSprite(bool pause)
{
    SDL_Rect sprite;

    // Gestion du compteur d'animation (incrémentation, pause et initialisation)
    if(!pause) setIncrCompteurAnim();

    switch(getType())
    {
        case 0: // bombe
        case 1: // grande clé
        case 2: // petite clé
        case 3: // parchemin
        case 4: // grand cœur
        case 5: // quart de cœur
        case 6: // bousole
        case 7: // cristal
        case 8: // lampe
        case 9: // baguette de feu
        case 10: // grande épée plantée
        case 11: // livre vert
        case 12: // médaillon rouge
        case 13: // gant 1
        case 16: // petite épée plantée
        case 17: // gant 2
        case 18: // palme
        case 19: // baguette de glace
        case 20: // médaillon vert
        case 21: // flèche argent
        case 23: // petit cœur
        case 24: // fleche normale
        case 25: // petite réserve de magie
        case 26: // grande réserve de magie
        case 27: // grappin
        case 28: // maillet
        case 29: // arc normal
        case 30: // médaillon bleu
        case 31: // bottes
        case 33: // tunique bleue
        case 34: // tunique rouge
        case 35: // arc argent
        case 36: // flute
        case 37: // masque
        case 38: // bourse
            sprite = getSDLRect(300+getType(),0,0);
            break;

        case 14: // rubis vert
        case 15: // rubis bleu
        case 22: // rubis rouge
            if(getCompteurAnim()%200 < 180) sprite = getSDLRect(300+getType(),0,0);
            else if(getCompteurAnim()%200 < 190) sprite = getSDLRect(300+getType(),1,0);
            else sprite = getSDLRect(300+getType(),2,0);
            break;

        case 32: //
            if(getCompteurAnim()%300 < 265) sprite = getSDLRect(300+getType(),0,0);
            else if(getCompteurAnim()%300 < 270) sprite = getSDLRect(300+getType(),1,0);
            else if(getCompteurAnim()%300 < 275) sprite = getSDLRect(300+getType(),2,0);
            else if(getCompteurAnim()%300 < 280) sprite = getSDLRect(300+getType(),3,0);
            else if(getCompteurAnim()%300 < 285) sprite = getSDLRect(300+getType(),4,0);
            else if(getCompteurAnim()%300 < 290) sprite = getSDLRect(300+getType(),5,0);
            else if(getCompteurAnim()%300 < 295) sprite = getSDLRect(300+getType(),6,0);
            else sprite = getSDLRect(300+getType(),7,0);
            break;

        default:
            break;
    }

    return sprite;
}

void ObjetRecolte::setTimeToLive(int TTL) {timeToLive = TTL;}
int ObjetRecolte::getTimeToLive() {return timeToLive;}
