/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Personnage.h"

Personnage::Personnage()
{

    setDimensionX(16);
    setDimensionY(24);

    setDeplacementRapide(false);
}

Personnage::~Personnage()
{

}

void Personnage::setBougeHaut(bool H){deplacementVersLeHaut = H;}
bool Personnage::getBougeHaut() {return deplacementVersLeHaut;}

void Personnage::setBougeBas(bool B){deplacementVersLeBas = B;}
bool Personnage::getBougeBas() {return deplacementVersLeBas;}

void Personnage::setBougeGauche(bool G) {deplacementVersLaGauche = G;}
bool Personnage::getBougeGauche() {return deplacementVersLaGauche;}

void Personnage::setBougeDroite(bool D) {deplacementVersLaDroite = D;}
bool Personnage::getBougeDroite() {return deplacementVersLaDroite;}

    /*** Traitement des colisions ***/

void Personnage::gestionCollisions(int decalageFenetreX,int decalageFenetreY,position **table,bool personnageNonJoueur,bool testVertical, int vitessePersoSurX, int vitessePersoSurY)
{
    // Protection afin que les personnages ne quittent pas la carte
    if(getPositionX() < 0) setBougeGauche(false);
    if(getPositionY() < 0) setBougeHaut(false);

    if(!personnageNonJoueur)
    {
        if(getPositionX() > LARGEUR_ECRAN - getDimensionX()) setBougeDroite(false);
        if(getPositionY() > HAUTEUR_ECRAN - getDimensionY()) setBougeBas(false);

        // Protection lorsque l'on est au bord de la carte et que les collisions ne sont pas gérées
        if(getPositionX() < 0 || getPositionX() > LARGEUR_ECRAN - getDimensionX()) {setBougeHaut(false);setBougeBas(false);}
        if(getPositionY() < 0 || getPositionY() > HAUTEUR_ECRAN - getDimensionY()) {setBougeGauche(false);setBougeDroite(false);}
    }

    //if(getPositionX() > -1 && getPositionY() > -1 && ((getPositionX() < LARGEUR_ECRAN - 15 && getPositionY() < HAUTEUR_ECRAN - 23 && !personnageNonJoueur) || personnageNonJoueur))
    if(getPositionX() > -1 && getPositionY() > -1 && ((getPositionX() < LARGEUR_ECRAN - (getDimensionX() - 1) && getPositionY() < HAUTEUR_ECRAN - (getDimensionY() - 1))));// && !personnageNonJoueur) || personnageNonJoueur))
    {
        int pX0,pY0; // Coordonnée du personnage
        pX0 = getPositionX()+decalageFenetreX;
        pY0 = getPositionY()+decalageFenetreY;

        if(testVertical)
        {
            if(getBougeHaut())
            {
                gestionCollisionsHaut(pX0, pY0, table[pX0/LARGEUR_TILE][(pY0+8-vitessePersoSurY)/HAUTEUR_TILE],table[(pX0+getDimensionX()-1)/LARGEUR_TILE][(pY0+8-vitessePersoSurY)/HAUTEUR_TILE]);
            }

            if(getBougeBas())
            {
                gestionCollisionsBas(pX0, pY0, table[pX0/LARGEUR_TILE][(pY0+getDimensionY()-1+vitessePersoSurY)/HAUTEUR_TILE], table[(pX0+getDimensionX()-1)/LARGEUR_TILE][(pY0+getDimensionY()-1+vitessePersoSurY)/HAUTEUR_TILE]);
            }
        }

        else
        {
            if(getBougeGauche())
            {
                gestionCollisionsGauche(pX0, pY0, table[(pX0-vitessePersoSurX)/16][(pY0+8)/HAUTEUR_TILE], table[(pX0-vitessePersoSurX)/16][(pY0+getDimensionY()-1)/HAUTEUR_TILE]);
            }

            if(getBougeDroite())
            {
                gestionCollisionsDroite(pX0, pY0, table[(pX0+getDimensionX()-1+vitessePersoSurX)/16][(pY0+8)/HAUTEUR_TILE], table[(pX0+getDimensionX()-1+vitessePersoSurX)/16][(pY0+getDimensionY()-1)/HAUTEUR_TILE]);
            }
        }
    }
}

void Personnage::gestionCollisionsHaut(int pX0, int pY0, position P0, position P1)
{
    int typeBlocTraversable0,typeBlocTraversable1; // Variables indiquant le type traversable des deux tuiles à tester
    bool bloc0Traversable = true,bloc1Traversable = true; // Indique l'état traversable ou non de la partie de chaque tuile testée

    typeBlocTraversable0 = P0.typeBlocTraversable;
    typeBlocTraversable1 = P1.typeBlocTraversable;

    // Bouge haut ne se trouve pas dans le cas 1 (cas 1 == les deux blocs du dessus traversables totalement, càd dont le type du bloc traversable est 0)
    if(typeBlocTraversable0 != 0 || typeBlocTraversable1 !=0)
    {

        if((pY0+7)%16 < 8) // Partie basse des tuiles à tester
        {
            // test du typeBlocTraversable0 #1
            // case 1
            if(pX0%LARGEUR_TILE < 8)
            {
                switch(typeBlocTraversable0)
                {
                    case 1:
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 11:
                    case 13:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // case 8
            else
            {
                switch(typeBlocTraversable0)
                {
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // test du typeBlocTraversable1 #1
            // case 1
            if((pX0+15)%LARGEUR_TILE < 8)
            {
                switch(typeBlocTraversable1)
                {
                    case 1:
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 11:
                    case 13:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }
            // case 8
            else
            {
                switch(typeBlocTraversable1)
                {
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }

            // Test complémentaire si les typeBlocTraversables sont tout deux à vrai, pour détecter le dernier cas de blocage possible (la tuile la plus occupée par le personnage n'est pas testée entièrement dans les tests ci-dessus)
            if(bloc0Traversable && bloc1Traversable)
            {
                if(pX0%LARGEUR_TILE < 8)
                {
                    switch(typeBlocTraversable0)
                    {
                        case 4:
                        case 5:
                        case 8:
                        case 12:
                        case 13:
                        bloc0Traversable = false;
                        break;
                    }
                }
                else if(!((pX0+15)%LARGEUR_TILE < 8))
                {
                    switch(typeBlocTraversable1)
                    {
                        case 1:
                        case 2:
                        case 3:
                        case 10:
                        case 11:
                        bloc1Traversable = false;
                        break;
                    }
                }
            }
        }
        else // sinon, partie haute des tuiles à tester
        {
            // test du typeBlocTraversable0 #2
            // case 2
            if(pX0%LARGEUR_TILE < 8)
            {
                switch(typeBlocTraversable0)
                {
                    case 2:
                    case 3:
                    case 6:
                    case 7:
                    case 10:
                    case 11:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // case 4
            else
            {
                switch(typeBlocTraversable0)
                {
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // test du typeBlocTraversable1 #2
            // case 2
            if((pX0+15)%LARGEUR_TILE < 8)
            {
                switch(typeBlocTraversable1)
                {
                    case 2:
                    case 3:
                    case 6:
                    case 7:
                    case 10:
                    case 11:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }
            // case 4
            else
            {
                switch(typeBlocTraversable1)
                {
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }

            // Test complémentaire si les typeBlocTraversables sont tout deux à vrai, pour détecter le dernier cas de blocage possible (la tuile la plus occupée par le personnage n'est pas testée entièrement dans les tests ci-dessus)
            if(bloc0Traversable && bloc1Traversable)
            {
                if(pX0%LARGEUR_TILE < 8)
                {
                    switch(typeBlocTraversable0)
                    {
                        case 4:
                        case 5:
                        //case 8:
                        case 12:
                        case 13:
                        bloc0Traversable = false;
                        break;
                    }
                }
                else if(!((pX0+15)%LARGEUR_TILE < 8))
                {
                    switch(typeBlocTraversable1)
                    {
                        //case 1:
                        case 2:
                        case 3:
                        case 10:
                        case 11:
                        bloc1Traversable = false;
                        break;
                    }
                }
            }
        }

        traitementDeplacementSuiteGestionCollisionsHaut(bloc0Traversable, bloc1Traversable, pX0, P0, P1);
    }
}

void Personnage::traitementDeplacementSuiteGestionCollisionsHaut(bool bloc0Traversable, bool bloc1Traversable, int pX0, position P0, position P1)
{
    int blocTraversable0_X = 0,blocTraversable0_Y = 0,blocTraversable1_X = 0,blocTraversable1_Y = 0; // Coordonnées des tuiles à tester

    blocTraversable0_X = P0.x;
    blocTraversable0_Y = P0.y;

    blocTraversable1_X = P1.x;
    blocTraversable1_Y = P1.y;

    if(!bloc0Traversable || !bloc1Traversable)
    {
        setBougeHaut(false);
    }
    if(bloc0Traversable && !bloc1Traversable && !getBougeDroite())
    {
        setBougeGauche(true);
        setDeplacementDemandeParCorrection(true);
    }
    if(!bloc0Traversable && bloc1Traversable && !getBougeGauche())
    {
        setBougeDroite(true);
        setDeplacementDemandeParCorrection(true);
    }

    // Gestion des exceptions aux corrections automatiques des déplacements
    if(((bloc1Traversable && (pX0)%LARGEUR_TILE < 8)) || (bloc0Traversable && ((pX0)%LARGEUR_TILE > 7)))
    {
        if(((blocTraversable0_X == 23 || blocTraversable0_X == 24) && blocTraversable0_Y == 7) || ((blocTraversable1_X == 23 || blocTraversable1_X == 24) && blocTraversable1_Y == 7)) // petit coffre (ouvert ou fermé) vers le haut
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 8 && blocTraversable0_Y == 5) || (blocTraversable1_X == 8 && blocTraversable1_Y == 5)) // pancarte sur sol marron clair
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 6 && blocTraversable0_Y == 3) || (blocTraversable1_X == 6 && blocTraversable1_Y == 3)) // plot
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 8 && blocTraversable0_Y == 10) || (blocTraversable1_X == 8 && blocTraversable1_Y == 10)) // grosse pierre noire
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 11 && blocTraversable0_Y == 61) || (blocTraversable1_X == 11 && blocTraversable1_Y == 61)) // inter rouge
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 12 && blocTraversable0_Y == 61) || (blocTraversable1_X == 12 && blocTraversable1_Y == 61)) // inter bleu
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 26 && blocTraversable0_Y == 0) || (blocTraversable1_X == 26 && blocTraversable1_Y == 0)) //
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X >= 30 && blocTraversable0_X <= 39 && blocTraversable0_Y == 0) || (blocTraversable1_X >= 30 && blocTraversable1_X <= 39 && blocTraversable1_Y == 00)) // panneau dans l'herbe
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 8 && blocTraversable0_Y == 1) || (blocTraversable1_X == 8 && blocTraversable1_Y == 1)) // bosquet marron
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 25 && blocTraversable0_Y == 9) || (blocTraversable1_X == 25 && blocTraversable1_Y == 9)) // bosquet mauve
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }
    }
}

void Personnage::gestionCollisionsBas(int pX0, int pY0, position P0, position P1)
{
    int typeBlocTraversable0,typeBlocTraversable1; // Variables indiquant le type traversable des deux tuiles à tester
    bool bloc0Traversable = true,bloc1Traversable = true; // Indique l'état traversable ou non de la partie de chaque tuile testée

    typeBlocTraversable0 = P0.typeBlocTraversable;
    typeBlocTraversable1 = P1.typeBlocTraversable;

    // Bouge bas ne se trouve pas dans le cas 1 (cas 1 == les deux blocs du dessous traversables totalement, càd dont le type du bloc traversable est 0)
    if(typeBlocTraversable0 != 0 || typeBlocTraversable1 !=0)
    {
        if((pY0+24)%HAUTEUR_TILE < 8) // Première partie de la tuile suivant le sens de déplacement
        {
            // test du typeBlocTraversable0 #1
            // case 1
            if(pX0%LARGEUR_TILE < 8)
            {
                switch(typeBlocTraversable0)
                {
                    case 1:
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 11:
                    case 13:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // case 8
            else
            {
                switch(typeBlocTraversable0)
                {
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // test du typeBlocTraversable1 #1
            // case 1
            if((pX0+15)%LARGEUR_TILE < 8)
            {
                switch(typeBlocTraversable1)
                {
                    case 1:
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 11:
                    case 13:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }
            // case 8
            else
            {
                switch(typeBlocTraversable1)
                {
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }

            // Test complémentaire si les typeBlocTraversables sont tout deux à vrai, pour détecter le dernier cas de blocage possible (la tuile la plus occupée par le personnage n'est pas testée entièrement dans les tests ci-dessus)
            if(bloc0Traversable && bloc1Traversable)
            {
                if(pX0%LARGEUR_TILE < 8)
                {
                    switch(typeBlocTraversable0)
                    {
                        case 8:
                        case 10:
                        case 12:
                        case 14:
                        bloc0Traversable = false;
                        break;
                    }
                }
                else if(!((pX0+15)%LARGEUR_TILE < 8))
                {
                    switch(typeBlocTraversable1)
                    {
                        case 1:
                        case 3:
                        case 5:
                        case 7:
                        bloc1Traversable = false;
                        break;
                    }
                }
            }
        }
        else // seconde partie de la tuile (en fonction du sens de déplacement vers le bas)
        {

            // test du typeBlocTraversable0 #2
            // case 2
            if(pX0%LARGEUR_TILE < 8)
            {
                switch(typeBlocTraversable0)
                {
                    case 2:
                    case 3:
                    case 6:
                    case 7:
                    case 10:
                    case 11:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // case 4
            else
            {
                switch(typeBlocTraversable0)
                {
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // test du typeBlocTraversable1 #2
            // case 2
            if((pX0+15)%LARGEUR_TILE < 8)
            {
                switch(typeBlocTraversable1)
                {
                    case 2:
                    case 3:
                    case 6:
                    case 7:
                    case 10:
                    case 11:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }
            // case 4
            else
            {
                switch(typeBlocTraversable1)
                {
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }

            // Test complémentaire si les typeBlocTraversables sont tout deux à vrai, pour détecter le dernier cas de blocage possible (la tuile la plus occupée par le personnage n'est pas testée entièrement dans les tests ci-dessus)
            if(bloc0Traversable && bloc1Traversable)
            {
                if(pX0%LARGEUR_TILE < 8)
                {
                    switch(typeBlocTraversable0)
                    {
                        case 4:
                        bloc0Traversable = false;
                        break;
                    }
                }
                else if(!((pX0+15)%LARGEUR_TILE < 8))
                {
                    switch(typeBlocTraversable1)
                    {
                        case 2:
                        bloc1Traversable = false;
                        break;
                    }
                }
            }
        }

        traitementDeplacementSuiteGestionCollisionsBas(bloc0Traversable, bloc1Traversable, pX0, P0, P1);
    }
}

void Personnage::traitementDeplacementSuiteGestionCollisionsBas(bool bloc0Traversable, bool bloc1Traversable, int pX0, position P0, position P1)
{
    int blocTraversable0_X = 0,blocTraversable0_Y = 0,blocTraversable1_X = 0,blocTraversable1_Y = 0; // Coordonnées des tuiles à tester

    blocTraversable0_X = P0.x;
    blocTraversable0_Y = P0.y;

    blocTraversable1_X = P1.x;
    blocTraversable1_Y = P1.y;

    if(!bloc0Traversable || !bloc1Traversable)
    {
        setBougeBas(false);
    }
    if(bloc0Traversable && !bloc1Traversable && !getBougeDroite())
    {
        setBougeGauche(true);
        setDeplacementDemandeParCorrection(true);
    }
    if(!bloc0Traversable && bloc1Traversable && !getBougeGauche())
    {
        setBougeDroite(true);
        setDeplacementDemandeParCorrection(true);
    }


    // Gestion des exceptions aux corrections automatiques des déplacements
    if(((bloc1Traversable && (pX0)%LARGEUR_TILE < 8)) || (bloc0Traversable && ((pX0)%LARGEUR_TILE > 7)))
    {
        if((blocTraversable0_X == 8 && blocTraversable0_Y == 10) || (blocTraversable1_X == 8 && blocTraversable1_Y == 10)) // grosse pierre noire
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 11 && blocTraversable0_Y == 61) || (blocTraversable1_X == 11 && blocTraversable1_Y == 61)) // inter rouge
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 6 && blocTraversable0_Y == 3) || (blocTraversable1_X == 6 && blocTraversable1_Y == 3)) // plot
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 12 && blocTraversable0_Y == 61) || (blocTraversable1_X == 12 && blocTraversable1_Y == 61)) // inter bleu
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 26 && blocTraversable0_Y == 0) || (blocTraversable1_X == 26 && blocTraversable1_Y == 0)) //
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X >= 30 && blocTraversable0_X <= 39 && blocTraversable0_Y == 0) || (blocTraversable1_X >= 30 && blocTraversable1_X <= 39 && blocTraversable1_Y == 00)) // panneau dans l'herbe
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 8 && blocTraversable0_Y == 1) || (blocTraversable1_X == 8 && blocTraversable1_Y == 1)) // bosquet marron
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 25 && blocTraversable0_Y == 9) || (blocTraversable1_X == 25 && blocTraversable1_Y == 9)) // bosquet mauve
        {
            setBougeGauche(false);
            setBougeDroite(false);
            setDeplacementDemandeParCorrection(false);
        }
    }
}

void Personnage::gestionCollisionsGauche(int pX0, int pY0, position P0, position P1)
{
    int typeBlocTraversable0,typeBlocTraversable1; // Variables indiquant le type traversable des deux tuiles à tester
    bool bloc0Traversable = true,bloc1Traversable = true; // Indique l'état traversable ou non de la partie de chaque tuile testée

    typeBlocTraversable0 = P0.typeBlocTraversable;
    typeBlocTraversable1 = P1.typeBlocTraversable;

    // Bouge gauche ne se trouve pas dans le cas 1 (cas 1 == les deux blocs de gauche traversables totalement, càd dont le type du bloc traversable est 0)
    if(typeBlocTraversable0 != 0 || typeBlocTraversable1 !=0)
    {
        if((pX0-1)%LARGEUR_TILE < 8) // Première partie de la tuile en fonction du sens de déplacement (ici vers la gauche)
        {
            // test du typeBlocTraversable0 #1
            // case 1
            if((pY0+8)%HAUTEUR_TILE < 8)
            {
                switch(typeBlocTraversable0)
                {
                    case 1:
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 11:
                    case 13:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // case 2
            else
            {
                switch(typeBlocTraversable0)
                {
                    case 2:
                    case 3:
                    case 6:
                    case 7:
                    case 10:
                    case 11:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // test du typeBlocTraversable1 #1
            // case 1
            if((pY0+23)%HAUTEUR_TILE < 8)
            {
                switch(typeBlocTraversable1)
                {
                    case 1:
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 11:
                    case 13:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }
            // case 2
            else
            {
                switch(typeBlocTraversable1)
                {
                    case 2:
                    case 3:
                    case 6:
                    case 7:
                    case 10:
                    case 11:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }

            // Test complémentaire si les typeBlocTraversables sont tout deux à vrai, pour détecter le dernier cas de blocage possible (la tuile la plus occupée par le personnage n'est pas testée entièrement dans les tests ci-dessus)
            if(bloc0Traversable && bloc1Traversable)
            {
                if((pY0+8)%HAUTEUR_TILE < 8)
                {
                    switch(typeBlocTraversable0)
                    {
                        case 2:
                        case 4:
                        case 5:
                        case 6:
                        case 7:
                        bloc0Traversable = false;
                        break;
                    }
                }
                else if(!((pY0+23)%HAUTEUR_TILE < 8))
                {
                    switch(typeBlocTraversable1)
                    {
                        case 1:
                        case 8:
                        case 9:
                        case 10:
                        case 11:
                        bloc1Traversable = false;
                        break;
                    }
                }
            }
        }
        else
        {
            // test du typeBlocTraversable0 #2
            // case 8
            if((pY0+8)%HAUTEUR_TILE < 8)
            {
                switch(typeBlocTraversable0)
                {
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // case 4
            else
            {
                switch(typeBlocTraversable0)
                {
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // test du typeBlocTraversable1 #2
            // case 8
            if((pY0+23)%HAUTEUR_TILE < 8)
            {
                switch(typeBlocTraversable1)
                {
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }
            // case 4
            else
            {
                switch(typeBlocTraversable1)
                {
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }

            // Test complémentaire si les typeBlocTraversables sont tout deux à vrai, pour détecter le dernier cas de blocage possible (la tuile la plus occupée par le personnage n'est pas testée entièrement dans les tests ci-dessus)
            if(bloc0Traversable && bloc1Traversable)
            {
                if((pY0+8)%HAUTEUR_TILE < 8)
                {
                    switch(typeBlocTraversable0)
                    {
                        //case 2:
                        case 4:
                        case 5:
                        case 6:
                        case 7:
                        bloc0Traversable = false;
                        break;
                    }
                }
                else if(!((pY0+23)%HAUTEUR_TILE < 8))
                {
                    switch(typeBlocTraversable1)
                    {
                        //case 1:
                        case 8:
                        case 9:
                        case 10:
                        case 11:
                        bloc1Traversable = false;
                        break;
                    }
                }
            }
        }

        traitementDeplacementSuiteGestionCollisionsGauche(bloc0Traversable, bloc1Traversable, pY0, P0, P1);
    }
}

void Personnage::traitementDeplacementSuiteGestionCollisionsGauche(bool bloc0Traversable, bool bloc1Traversable, int pY0, position P0, position P1)
{
    int blocTraversable0_X = 0,blocTraversable0_Y = 0,blocTraversable1_X = 0,blocTraversable1_Y = 0; // Coordonnées des tuiles à tester

    blocTraversable0_X = P0.x;
    blocTraversable0_Y = P0.y;

    blocTraversable1_X = P1.x;
    blocTraversable1_Y = P1.y;

    if(!bloc0Traversable || !bloc1Traversable)
    {
        setBougeGauche(false);
    }
    if(bloc0Traversable && !bloc1Traversable && !getBougeBas())
    {
        setBougeHaut(true);
        setDeplacementDemandeParCorrection(true);
    }
    if(!bloc0Traversable && bloc1Traversable && !getBougeHaut())
    {
        setBougeBas(true);
        setDeplacementDemandeParCorrection(true);
    }

    // Gestion des exceptions aux corrections automatiques des déplacements
    if(((bloc1Traversable && (pY0+8)%HAUTEUR_TILE < 8)) || (bloc0Traversable && ((pY0+8)%HAUTEUR_TILE > 7)))
    {
        if((blocTraversable0_X == 8 && blocTraversable0_Y == 10) || (blocTraversable1_X == 8 && blocTraversable1_Y == 10)) // grosse pierre noire
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 6 && blocTraversable0_Y == 3) || (blocTraversable1_X == 6 && blocTraversable1_Y == 3)) // plot
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 11 && blocTraversable0_Y == 61) || (blocTraversable1_X == 11 && blocTraversable1_Y == 61)) // inter rouge
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 12 && blocTraversable0_Y == 61) || (blocTraversable1_X == 12 && blocTraversable1_Y == 61)) // inter bleu
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 26 && blocTraversable0_Y == 0) || (blocTraversable1_X == 26 && blocTraversable1_Y == 0)) //
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X >= 30 && blocTraversable0_X <= 39 && blocTraversable0_Y == 0) || (blocTraversable1_X >= 30 && blocTraversable1_X <= 39 && blocTraversable1_Y == 00)) // panneau dans l'herbe
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 8 && blocTraversable0_Y == 1) || (blocTraversable1_X == 8 && blocTraversable1_Y == 1)) // bosquet marron
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 25 && blocTraversable0_Y == 9) || (blocTraversable1_X == 25 && blocTraversable1_Y == 9)) // bosquet mauve
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }
    }
}

void Personnage::gestionCollisionsDroite(int pX0, int pY0, position P0, position P1)
{
    int typeBlocTraversable0,typeBlocTraversable1; // Variables indiquant le type traversable des deux tuiles à tester
    bool bloc0Traversable = true,bloc1Traversable = true; // Indique l'état traversable ou non de la partie de chaque tuile testée

    typeBlocTraversable0 = P0.typeBlocTraversable;
    typeBlocTraversable1 = P1.typeBlocTraversable;

    if(typeBlocTraversable0 != 0 || typeBlocTraversable1 !=0)
    {
       if((pX0+16)%LARGEUR_TILE < 8)
        {
            // test du typeBlocTraversable0 #1
            // case 1
            if((pY0+8)%HAUTEUR_TILE < 8)
            {
                switch(typeBlocTraversable0)
                {
                    case 1:
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 11:
                    case 13:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // case 2
            else
            {
                switch(typeBlocTraversable0)
                {
                    case 2:
                    case 3:
                    case 6:
                    case 7:
                    case 10:
                    case 11:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // test du typeBlocTraversable1 #1
            // case 1
            if((pY0+23)%HAUTEUR_TILE < 8)
            {
                switch(typeBlocTraversable1)
                {
                    case 1:
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 11:
                    case 13:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }
            // case 2
            else
            {
                switch(typeBlocTraversable1)
                {
                    case 2:
                    case 3:
                    case 6:
                    case 7:
                    case 10:
                    case 11:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }

            // Test complémentaire si les typeBlocTraversables sont tout deux à vrai, pour détecter le dernier cas de blocage possible (la tuile la plus occupée par le personnage n'est pas testée entièrement dans les tests ci-dessus)
            if(bloc0Traversable && bloc1Traversable)
            {
                if((pY0+8)%HAUTEUR_TILE < 8)
                {
                    switch(typeBlocTraversable0)
                    {
                        case 2:
                        case 6:
                        case 10:
                        case 14:
                        bloc0Traversable = false;
                        break;
                    }
                }
                else if(!((pY0+23)%HAUTEUR_TILE < 8))
                {
                    switch(typeBlocTraversable1)
                    {
                        case 1:
                        case 5:
                        case 9:
                        case 13:
                        bloc1Traversable = false;
                        break;
                    }
                }
            }
        }

        else
        {
            // test du typeBlocTraversable0 #2
            // case 8
            if((pY0+8)%HAUTEUR_TILE < 8)
            {
                switch(typeBlocTraversable0)
                {
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // case 4
            else
            {
                switch(typeBlocTraversable0)
                {
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc0Traversable = false;
                        break;
                }
            }
            // test du typeBlocTraversable1 #2
            // case 8
            if((pY0+23)%HAUTEUR_TILE < 8)
            {
                switch(typeBlocTraversable1)
                {
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }
            // case 4
            else
            {
                switch(typeBlocTraversable1)
                {
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        bloc1Traversable = false;
                        break;
                }
            }

            // Test complémentaire si les typeBlocTraversables sont tout deux à vrai, pour détecter le dernier cas de blocage possible (la tuile la plus occupée par le personnage n'est pas testée entièrement dans les tests ci-dessus)
            if(bloc0Traversable && bloc1Traversable)
            {
                if((pY0+8)%HAUTEUR_TILE < 8)
                {
                    switch(typeBlocTraversable0)
                    {
                        case 4:
                        bloc0Traversable = false;
                        break;
                    }
                }
                else if(!((pY0+23)%HAUTEUR_TILE < 8))
                {
                    switch(typeBlocTraversable1)
                    {
                        case 8:
                        bloc1Traversable = false;
                        break;
                    }
                }
            }
        }

        traitementDeplacementSuiteGestionCollisionsDroite(bloc0Traversable, bloc1Traversable, pY0, P0, P1);
    }
}

void Personnage::traitementDeplacementSuiteGestionCollisionsDroite(bool bloc0Traversable, bool bloc1Traversable, int pY0, position P0, position P1)
{
    int blocTraversable0_X = 0,blocTraversable0_Y = 0,blocTraversable1_X = 0,blocTraversable1_Y = 0; // Coordonnées des tuiles à tester

    blocTraversable0_X = P0.x;
    blocTraversable0_Y = P0.y;

    blocTraversable1_X = P1.x;
    blocTraversable1_Y = P1.y;

    if(!bloc0Traversable || !bloc1Traversable)
    {
        setBougeDroite(false);
    }
    if(bloc0Traversable && !bloc1Traversable && !getBougeBas())
    {
        setBougeHaut(true);
        setDeplacementDemandeParCorrection(true);
    }
    if(!bloc0Traversable && bloc1Traversable && !getBougeHaut())
    {
        setBougeBas(true);
        setDeplacementDemandeParCorrection(true);
    }

    // Gestion des exceptions aux corrections automatiques des déplacements
    if(((bloc1Traversable && (pY0+8)%HAUTEUR_TILE < 8)) || (bloc0Traversable && ((pY0+8)%HAUTEUR_TILE > 7)))
    {
        if((blocTraversable0_X == 8 && blocTraversable0_Y == 10) || (blocTraversable1_X == 8 && blocTraversable1_Y == 10)) // grosse pierre noire
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 6 && blocTraversable0_Y == 3) || (blocTraversable1_X == 6 && blocTraversable1_Y == 3)) // plot
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 11 && blocTraversable0_Y == 61) || (blocTraversable1_X == 11 && blocTraversable1_Y == 61)) // inter rouge
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 12 && blocTraversable0_Y == 61) || (blocTraversable1_X == 12 && blocTraversable1_Y == 61)) // inter bleu
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 26 && blocTraversable0_Y == 0) || (blocTraversable1_X == 26 && blocTraversable1_Y == 0)) //
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X >= 30 && blocTraversable0_X <= 39 && blocTraversable0_Y == 0) || (blocTraversable1_X >= 30 && blocTraversable1_X <= 39 && blocTraversable1_Y == 00)) // panneau dans l'herbe
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 8 && blocTraversable0_Y == 1) || (blocTraversable1_X == 8 && blocTraversable1_Y == 1)) // bosquet marron
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }

        else if((blocTraversable0_X == 25 && blocTraversable0_Y == 9) || (blocTraversable1_X == 25 && blocTraversable1_Y == 9)) // bosquet mauve
        {
            setBougeHaut(false);
            setBougeBas(false);
            setDeplacementDemandeParCorrection(false);
        }
    }
}

void Personnage::setImmobiliser()
{
    setBougeHaut(false);
    setBougeBas(false);
    setBougeGauche(false);
    setBougeDroite(false);
}

void Personnage::setDeplacementDemandeParCorrection(bool DDPC) {deplacementDemandeParCorrection = DDPC;}
bool Personnage::getDeplacementDemandeParCorrection() {return deplacementDemandeParCorrection;}

void Personnage::setDeplacementRapide(bool DR) {deplacementRapide = DR;}
bool Personnage::getDeplacementRapide() {return deplacementRapide;}

void Personnage::setVitesseDeplacementSurX(int VDX) {vitesseSurX = VDX;}
int Personnage::getVitesseDeplacementSurX() {return vitesseSurX;}

void Personnage::setVitesseDeplacementSurY(int VDY) {vitesseSurY = VDY;}
int Personnage::getVitesseDeplacementSurY() {return vitesseSurY;}

void Personnage::setCompteurDeplacement(int CD) {compteurDeplacementAutomatique = CD;}
void Personnage::incrementationCompteurDeplacement() {compteurDeplacementAutomatique++;}
int Personnage::getCompteurDeplacement() {return compteurDeplacementAutomatique;}

void Personnage::setDimensionXZoneCollision(int DXZC) {dimensionXZoneCollision = DXZC;}
int Personnage::getDimensionXZoneCollision() {return dimensionXZoneCollision;}
void Personnage::setDimensionYZoneCollision(int DYZC) {dimensionYZoneCollision = DYZC;}
int Personnage::getDimensionYZoneCollision() {return dimensionYZoneCollision;}

void Personnage::setDecalageXZoneCollision(int DXZC) {decalageXZoneCollision = DXZC;}
int Personnage::getDecalageXZoneCollision() {return decalageXZoneCollision;}
void Personnage::setDecalageYZoneCollision(int DYZC) {decalageYZoneCollision = DYZC;}
int Personnage::getDecalageYZoneCollision() {return decalageYZoneCollision;}
