/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Projectile.h"

Projectile::Projectile()
{
    positionX = 0;
    positionY = 0;
}

Projectile::~Projectile()
{

}

void Projectile::initType(SDL_Surface* P)
{
    setFeuilleSprite(P);

    // Valeur de déplacement par défaut ; préciser dans l'init si ces valeurs sont différentes ou évoluent
    setVitesseDeplacementSurX(0);
    setVitesseDeplacementSurY(0);
    setCompteurDeplacement(0);
    setDeplacementDemandeParCorrection(false);

    setArretProjectile(false);

    switch(getType())
    {
        // De 1 à 6 ---> projectiles emis par le joueur
        // DIMENSIONS A REVOIR
        case 1: // Flèche
            setDimensionX(16);
            setDimensionY(8); // Bricolage pour tromper le système de gestion des collisions (taille = 16 normalement)
            setVitesseDeplacementSurX(8); // A définir précisement
            setVitesseDeplacementSurY(8); // A définir précisement
            setTimeToLive(60); // A définir précisement
            break;

        case 2: // Grappin
            //setDimensionX(16);
            //setDimensionY(16);
            break;

        case 3: // Bombe
            setDimensionX(16);
            setDimensionY(16);
            setTimeToLive(120);
            break;

        case 4: // Boomerang
            setDimensionX(8); // 10
            setDimensionY(8); // 10
            {
                setVitesseDeplacementSurX(4);
                setVitesseDeplacementSurY(4);
                setTimeToLive(80);
            }
            break;

        case 5: // Flamme qui se déplace
            setDimensionX(16);
            setDimensionY(8);
            setVitesseDeplacementSurX(8); // A définir précisement
            setVitesseDeplacementSurY(8); // A définir précisement
            setTimeToLive(80);
            break;

        case 6: // Glace
            setDimensionX(16);
            setDimensionY(8);
            setVitesseDeplacementSurX(8); // A définir précisement
            setVitesseDeplacementSurY(8); // A définir précisement
            setTimeToLive(80);
            break;

        case 7: // Flamme immobile
            setDimensionX(16);
            setDimensionY(16);
            setTimeToLive(10);
            break;

        default:
            break;
    }
}

void Projectile::intelligenceArtificielle()
{
    if(getPauseAnimation()) {setImmobiliser();}
    else {incrementationCompteurDeplacement();}

    incrementationCompteurDeplacement();

    switch(getType())
    {
        case 1: // Flèche
        case 5: // Flamme qui se déplace
        case 6: // Glace
            if(getArretProjectile() || getDeplacementDemandeParCorrection())
            {
                cout << "arrêt projectile" << endl;
                setTimeToLive(0);
                setImmobiliser(); // Comportement à redéfinir
            }
            else
            {
                switch(getOrientation())
                {
                    case 0:
                        setBougeHaut(true);
                        setBougeBas(false);
                        break;
                    case 1:
                        setBougeHaut(false);
                        setBougeBas(true);
                        break;
                    case 2:
                        setBougeGauche(true);
                        setBougeDroite(false);
                        break;
                    case 3:
                        setBougeGauche(false);
                        setBougeDroite(true);
                        break;
                    default:
                        break;
                }
            }
            break;

        case 2: // Grappin
            break;

        case 3: // Bombe
            // Ajustement de la position de la bombe lorsqu'elle explose
            if(getTimeToLive() == 10)
            {
                setPositionX(-8);
                setPositionY(-8);
            }
            break;

        case 4: // Boomerang

            if(getTimeToLive() >= getCompteurDeplacement()) // Aller
            {
                if(getArretProjectile() || getDeplacementDemandeParCorrection()) // Si bloquage dans la phase aller
                {
                    setTimeToLive(getCompteurDeplacement() - 1);
                }

                else
                {
                    switch(getOrientation())
                    {
                        case 0:
                            setBougeHaut(true);
                            setBougeBas(false);
                            break;
                        case 1:
                            setBougeHaut(false);
                            setBougeBas(true);
                            break;
                        case 2:
                            setBougeGauche(true);
                            setBougeDroite(false);
                            break;
                        case 3:
                            setBougeGauche(false);
                            setBougeDroite(true);
                            break;
                        default:
                            break;
                    }
                }
            }

            else // Retour du boomerang vers le héro
            {
                if(getDecalageXAvecLeHero()>15)
                {
                    setBougeGauche(true);
                    setBougeDroite(false);
                }
                else if(getDecalageXAvecLeHero()<0)
                {
                    setBougeGauche(false);
                    setBougeDroite(true);
                }
                else
                {
                    setBougeGauche(false);
                    setBougeDroite(false);
                }

                if(getDecalageYAvecLeHero()>10)
                {
                    setBougeHaut(true);
                    setBougeBas(false);
                }
                else if(getDecalageYAvecLeHero()<0)
                {
                    setBougeHaut(false);
                    setBougeBas(true);
                }
                else
                {
                    setBougeHaut(false);
                    setBougeBas(false);
                }
            }
            break;

        default:
            break;
    }
}

SDL_Rect Projectile::selectionSprite(bool pause)
{
    SDL_Rect sprite;

    // Gestion du compteur d'animation (incrémentation, pause et initialisation)
    if(!pause) setIncrCompteurAnim();

    switch(getType())
    {
        // De 1 à 7 ---> projectiles emis par le joueur
        case 1: // Flèche
            if(!getDeplacementDemandeParCorrection()) sprite = getSDLRect(33,getOrientation(),0);
            break;

        case 2: // Grappin
            break;

        case 3: // Bombe
            if(getTimeToLive() < 1) sprite.w = 0;
            else if(getTimeToLive() < 11) sprite = getSDLRect(27,9,0);
            else if(getTimeToLive() < 60) sprite = getSDLRect(27,(getCompteurAnim()/2)%8,0);
            else sprite = getSDLRect(27,0,0);
            break;

        case 4: // Boomerang - Y à 0 ---> bleu ; à 1 ---> rouge
            sprite = getSDLRect(32,(getCompteurAnim()/12)%4,0);
            break;

        case 5: // Flamme qui se déplace
            sprite = getSDLRect(34,(getCompteurAnim()/8)%4,0);
            break;

        case 6: // Glace
            sprite = getSDLRect(35,(getCompteurAnim()/2)%3,0);
            break;

        case 7: // Flamme de la lampe
            sprite = getSDLRect(31,2-(getCompteurAnim()/6),0);
            break;

        default:
            break;
    }

    return sprite;
}

void Projectile::setPuissance(int P) {puissance = P;}
int Projectile::getPuissance() {return puissance;}

void Projectile::setTimeToLive(int TTL) {timeToLive = TTL;}
int Projectile::getTimeToLive() {return timeToLive;}

void Projectile::setArretProjectile(bool AP) {arretProjectile = AP;}
bool Projectile::getArretProjectile() {return arretProjectile;}
