/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "SelectionPartie.h"

SelectionPartie::SelectionPartie()
{
    int i,j;
    // Initialisation des propriétés traversantes des tiles
    refTileset = mapMenu.initRefTileset();

    // Chargement des divers éléments graphiques (A VIRER AUSSI)
    pointeurHero = chargerImage("images/texte/fee.png");
    imageCoeur = chargerImage("images/objets/coeurs.png");

    setMenuSelectionActif(true);
    setMenuCreationNouvellePartie(false);
    setMenuOptionsPartie(false);
    setMenuSuppressionPartie(false);
    setCurseurSurEffacerPartie(false);
    mapMenu.selectionCarte(0);

    // Positionnement du curseur dans le menu de création de partie
    setPositionXCurseurLettre(0);
    setPositionYCurseurLettre(0);

    // Afin de positionner le curseur devant la première partie
    setNumero(0);

    // Initialisation du nombre de cœurs sur chaque partie ; Ces valeurs sont lues dans chaque sauvegarde.
    for(i=0;i<3;i++)
    {
        nbCoeursPartie[i] = 0;
        lectureSauvegardes(&nbCoeursPartie[i],&nomJoueur[i],i);
    }

    // Initialisation des SDL_Rect figés (PARTIE A EFFECTUER UNE SEULE FOIS ET AILLEURS UNE FOIS LES TESTS FINIS)
    for(i=0;i<2;i++)
    {
        rectPointeur[i].x = 0;
        rectPointeur[i].y = 0;
        rectPointeur[i].w = 16;
        rectPointeur[i].h = 16;
    }

    for(i=0;i<3;i++)
    {
        rectCoeur[i].x = 12*i;
        rectCoeur[i].y = 0;
        rectCoeur[i].w = 12;
        rectCoeur[i].h = 12;
    }

    for(i=0;i<3;i++)
    {
        for(j=0;j<nbCoeursPartie[i];j++)
        {
            rectPositionCoeur[i][j].x = 140 + 7*j;
            rectPositionCoeur[i][j].y = 70 + 34*i + 6*(j%2);
            rectPositionCoeur[i][j].w = 16;
            rectPositionCoeur[i][j].h = 16;
        }
    }

    // Initialisation des SDL_Surface du tileset uniquement
    initSDLSurface(0);

}

SelectionPartie::~SelectionPartie()
{
    freeMem(0);

    // TEMPORAIRE
    SDL_FreeSurface(pointeurHero);
    SDL_FreeSurface(imageCoeur);
}

void SelectionPartie::setNumero(int N) {numero = N;}
int SelectionPartie::getNumero() {return numero;}

void SelectionPartie::setCurseurSurEffacerPartie(bool CSEP) {curseurSurEffacerPartie = CSEP;}
bool SelectionPartie::getCurseurSurEffacerPartie() {return curseurSurEffacerPartie;}

void SelectionPartie::setMenuSelectionActif(bool MSA) {menuSelectionActif = MSA;}
bool SelectionPartie::getMenuSelectionActif() {return menuSelectionActif;}
void SelectionPartie::afficherMenuSelectionActif(SDL_Surface* screenMenuSelectionPartie,bool* sortieProgramme)
{
    int i,j;
    int numPartie;
    bool effacerPartie; // A REVOIR IMPERATIVEMENT
    bool validation = false;

    // Gestion des événements clavier
    numPartie = getNumero();
    effacerPartie = getCurseurSurEffacerPartie();
    clav.ecouteEventMenuSelectionActif(event,*sortieProgramme,numPartie,effacerPartie,validation);

    // Mise à jour des données du menu
    if(numPartie<0) numPartie = 3;
    if(numPartie>3) numPartie = 0;
    setNumero(numPartie);
    setCurseurSurEffacerPartie(effacerPartie);

    rectPointeurHero.x = 30 + 160*getCurseurSurEffacerPartie();
    rectPointeurHero.y = 70 + 34*getNumero() + (getNumero()/3)*28;
    rectPointeurHero.w = 16;
    rectPointeurHero.h = 16;

    // Affichage de la carte et des autres éléments à afficher
    mapMenu.afficherCarte(screenMenuSelectionPartie,tilesetA,tilesetB,tilesetC,tilesetD,mapMenu.getTableauSol());
    mapMenu.afficherCarte(screenMenuSelectionPartie,tilesetA,tilesetB,tilesetC,tilesetD,mapMenu.getTableauTable());

    // Afficher les informations de chaque parties
    for(i=0;i<3;i++)
    {
        // Coeurs
        for(j=0;j<nbCoeursPartie[i];j++)
        {
            SDL_BlitSurface(imageCoeur,&rectCoeur[2],screenMenuSelectionPartie,&rectPositionCoeur[i][j]);
        }
    }

    // Afficher le texte dans ce menu
    texteMenu.afficherTexteSelectionPartie(screenMenuSelectionPartie,nomJoueur[0],nomJoueur[1],nomJoueur[2],"Parties","Options","Effacer");

    // Afficher le pointeur de partie (personnage s'affichant devant la partie sélectionnée)
    SDL_BlitSurface(pointeurHero,&rectPointeur[1],screenMenuSelectionPartie,&rectPointeurHero);

    // Prise en compte de la validation du choix de la partie par le joueur (sauf si on veut créer ou effacer une partie
    if(validation && numPartie < 3)
    {
        // Si partie existante -> on continue la partie
        if(nomJoueur[numPartie] != "") setMenuSelectionActif(false);

        // Sinon, aller dans le menu de sélection de partie
        else
        {
            setMenuCreationNouvellePartie(true);
            mapMenu.selectionCarte(1);
        }
    }

    // Actions à effectuer en cas de demande d'une nouvelle partie ou d'un effacement de partie existante
    if(validation && numPartie == 3)
    {
        // Options
        if(!effacerPartie)
        {
            setMenuOptionsPartie(true);
            mapMenu.selectionCarte(2); // (ajouter une autre carte que la n°2)
        }
        // Effacement de partie
        else
        {
            setMenuSuppressionPartie(true);
            mapMenu.selectionCarte(2);
        }
    }

}

void SelectionPartie::setMenuCreationNouvellePartie(bool MCNP) {menuCreationNouvellePartie = MCNP;}
bool SelectionPartie::getMenuCreationNouvellePartie() {return menuCreationNouvellePartie;}
void SelectionPartie::afficherMenuCreationNouvellePartie(SDL_Surface* screenMenuCreationNouvellePartie,bool* sortieProgramme)
{
    bool sortieMenu = false;
    bool valider = false;
    bool suppr = false;
    int x = getPositionXCurseurLettre();
    int y = getPositionYCurseurLettre();

    // Gestion des événements clavier
    clav.ecouteEventMenuCreationNouvellePartie(event,*sortieProgramme,sortieMenu,valider,suppr,x,y);

    // Correction eventuelle et mise à jour de la position du curseur
    if(x<0) x = 12;
    else if(x>12) x = 0;

    if(y<0) y = 5;
    else if(y>5) y = 0;

    setPositionXCurseurLettre(x);
    setPositionYCurseurLettre(y);

    // Mise à jour du nom du futur joueur si suppr
    if(suppr && nomJoueur[getNumero()] != "")
    {
        string::iterator it = nomJoueur[getNumero()].end();
        nomJoueur[getNumero()].replace(it-1,it,"");
    }

    // Mise à jour du nom du futur joueur si validation (limitation de la longueur du nom du joueur à 8 caractères)
    if(valider && nomJoueur[getNumero()].length() < 8)
    {
        // MODIFIER LE NOM DU JOUEUR AU FUR ET A MESURE
        switch(y)
        {
            case 0:
                switch(x)
                {
                    case 0:
                        nomJoueur[getNumero()] += "A";
                        break;
                    case 1:
                        nomJoueur[getNumero()] += "B";
                        break;
                    case 2:
                        nomJoueur[getNumero()] += "C";
                        break;
                    case 3:
                        nomJoueur[getNumero()] += "D";
                        break;
                    case 4:
                        nomJoueur[getNumero()] += "E";
                        break;
                    case 5:
                        nomJoueur[getNumero()] += "F";
                        break;
                    case 6:
                        nomJoueur[getNumero()] += "G";
                        break;
                    case 7:
                        nomJoueur[getNumero()] += "H";
                        break;
                    case 8:
                        nomJoueur[getNumero()] += "I";
                        break;
                    case 9:
                        nomJoueur[getNumero()] += "J";
                        break;
                    case 10:
                        nomJoueur[getNumero()] += "K";
                        break;
                    case 11:
                        nomJoueur[getNumero()] += "L";
                        break;
                    default:
                        nomJoueur[getNumero()] += "M";
                        break;
                }
                break;

            case 1:
                switch(x)
                {
                    case 0:
                        nomJoueur[getNumero()] += "N";
                        break;
                    case 1:
                        nomJoueur[getNumero()] += "O";
                        break;
                    case 2:
                        nomJoueur[getNumero()] += "P";
                        break;
                    case 3:
                        nomJoueur[getNumero()] += "Q";
                        break;
                    case 4:
                        nomJoueur[getNumero()] += "R";
                        break;
                    case 5:
                        nomJoueur[getNumero()] += "S";
                        break;
                    case 6:
                        nomJoueur[getNumero()] += "T";
                        break;
                    case 7:
                        nomJoueur[getNumero()] += "U";
                        break;
                    case 8:
                        nomJoueur[getNumero()] += "V";
                        break;
                    case 9:
                        nomJoueur[getNumero()] += "W";
                        break;
                    case 10:
                        nomJoueur[getNumero()] += "X";
                        break;
                    case 11:
                        nomJoueur[getNumero()] += "Y";
                        break;
                    default:
                        nomJoueur[getNumero()] += "Z";
                        break;
                }
                break;

            case 2:
                switch(x)
                {
                    case 0:
                        nomJoueur[getNumero()] += "a";
                        break;
                    case 1:
                        nomJoueur[getNumero()] += "b";
                        break;
                    case 2:
                        nomJoueur[getNumero()] += "c";
                        break;
                    case 3:
                        nomJoueur[getNumero()] += "d";
                        break;
                    case 4:
                        nomJoueur[getNumero()] += "e";
                        break;
                    case 5:
                        nomJoueur[getNumero()] += "f";
                        break;
                    case 6:
                        nomJoueur[getNumero()] += "g";
                        break;
                    case 7:
                        nomJoueur[getNumero()] += "h";
                        break;
                    case 8:
                        nomJoueur[getNumero()] += "i";
                        break;
                    case 9:
                        nomJoueur[getNumero()] += "j";
                        break;
                    case 10:
                        nomJoueur[getNumero()] += "k";
                        break;
                    case 11:
                        nomJoueur[getNumero()] += "l";
                        break;
                    default:
                        nomJoueur[getNumero()] += "m";
                        break;
                }
                break;

            case 3:
                switch(x)
                {
                    case 0:
                        nomJoueur[getNumero()] += "n";
                        break;
                    case 1:
                        nomJoueur[getNumero()] += "o";
                        break;
                    case 2:
                        nomJoueur[getNumero()] += "p";
                        break;
                    case 3:
                        nomJoueur[getNumero()] += "q";
                        break;
                    case 4:
                        nomJoueur[getNumero()] += "r";
                        break;
                    case 5:
                        nomJoueur[getNumero()] += "s";
                        break;
                    case 6:
                        nomJoueur[getNumero()] += "t";
                        break;
                    case 7:
                        nomJoueur[getNumero()] += "u";
                        break;
                    case 8:
                        nomJoueur[getNumero()] += "v";
                        break;
                    case 9:
                        nomJoueur[getNumero()] += "w";
                        break;
                    case 10:
                        nomJoueur[getNumero()] += "x";
                        break;
                    case 11:
                        nomJoueur[getNumero()] += "y";
                        break;
                    default:
                        nomJoueur[getNumero()] += "z";
                        break;
                }
                break;

            case 4:
                switch(x)
                {
                    case 0:
                        nomJoueur[getNumero()] += "0";
                        break;
                    case 1:
                        nomJoueur[getNumero()] += "1";
                        break;
                    case 2:
                        nomJoueur[getNumero()] += "2";
                        break;
                    case 3:
                        nomJoueur[getNumero()] += "3";
                        break;
                    case 4:
                        nomJoueur[getNumero()] += "4";
                        break;
                    case 5:
                        nomJoueur[getNumero()] += "5";
                        break;
                    case 6:
                        nomJoueur[getNumero()] += "6";
                        break;
                    case 7:
                        nomJoueur[getNumero()] += "7";
                        break;
                    case 8:
                        nomJoueur[getNumero()] += "8";
                        break;
                    case 9:
                        nomJoueur[getNumero()] += "9";
                        break;
                    case 10:
                        nomJoueur[getNumero()] += "-";
                        break;
                    case 11:
                        nomJoueur[getNumero()] += ".";
                        break;
                    default:
                        nomJoueur[getNumero()] += ",";
                        break;
                }
                break;

            default:
                switch(x)
                {
                    case 0:
                        nomJoueur[getNumero()] += "!";
                        break;
                    case 1:
                        nomJoueur[getNumero()] += "?";
                        break;
                    case 2:
                        nomJoueur[getNumero()] += "(";
                        break;
                    case 3:
                        nomJoueur[getNumero()] += ")";
                        break;
                    case 4:
                        nomJoueur[getNumero()] += "'";
                        break;
                    case 5:
                        nomJoueur[getNumero()] += ":";
                        break;
                    case 6:
                        nomJoueur[getNumero()] += " ";
                        break;
                    case 7:
                        nomJoueur[getNumero()] += "~";
                        break;
                    case 8:
                        nomJoueur[getNumero()] += "/";
                        break;
                    case 9:
                        nomJoueur[getNumero()] += "£";
                        break;
                    case 10:
                        nomJoueur[getNumero()] += "$";
                        break;

                    default:
                        break;
                }
                break;
        }

        if(x == 12 && y == 5 && nomJoueur[getNumero()].length() > 0)
        {
            // Création d'une sauvegarde initiale
            initSauvegarde();

            setMenuCreationNouvellePartie(false);
            setMenuSelectionActif(false);
        }
    }

    // Affichage de la carte
    mapMenu.afficherCarte(screenMenuCreationNouvellePartie,tilesetA,tilesetB,tilesetC,tilesetD,mapMenu.getTableauSol());
    mapMenu.afficherCarte(screenMenuCreationNouvellePartie,tilesetA,tilesetB,tilesetC,tilesetD,mapMenu.getTableauTable());

    // Afficher le texte dans ce menu
    texteMenu.afficherTexteCreationNouvellePartie(screenMenuCreationNouvellePartie,"Creer",nomJoueur[getNumero()],x,y);

    if(sortieMenu)
    {
        nomJoueur[getNumero()] = "";
        setMenuCreationNouvellePartie(false);
        mapMenu.selectionCarte(0);
    }
}

void SelectionPartie::setMenuOptionsPartie(bool MOP) {menuOptionsPartie = MOP;}
bool SelectionPartie::getMenuOptionsPartie() {return menuOptionsPartie;}
void SelectionPartie::afficherMenuOptionsPartie(SDL_Surface* screenMenuOptionsPartie,bool* sortieProgramme)
{
    bool sortieMenu = false;

    // Gestion des événemments clavier
    clav.ecouteEventMenuOptionsPartie(event,*sortieProgramme,sortieMenu);

    // Affichage de la carte
    mapMenu.afficherCarte(screenMenuOptionsPartie,tilesetA,tilesetB,tilesetC,tilesetD,mapMenu.getTableauSol());
    mapMenu.afficherCarte(screenMenuOptionsPartie,tilesetA,tilesetB,tilesetC,tilesetD,mapMenu.getTableauTable());

    // Afficher le texte dans ce menu
    texteMenu.afficherTexteSelectionPartie(screenMenuOptionsPartie,"","","","Options","","");

    if(sortieMenu)
    {
        setMenuOptionsPartie(false);
        mapMenu.selectionCarte(0);
    }
}

void SelectionPartie::setMenuSuppressionPartie(bool MSP) {menuSuppressionPartie = MSP;}
bool SelectionPartie::getMenuSuppressionPartie() {return menuSuppressionPartie;}
void SelectionPartie::afficherMenuSuppressionPartie(SDL_Surface* screenMenuSuppressionPartie,bool* sortieProgramme)
{
    int i,j;
    int numPartie;
    bool sortieMenu = false;
    bool validation = false;
    std::string nomFichier = "sauvegardes/sauvegarde";

    numPartie = getNumero();

    // Gestion des événements clavier
    clav.ecouteEventMenuSuppressionPartie(event,*sortieProgramme,numPartie,sortieMenu,validation);

    if(numPartie < 0) numPartie = 2;
    else if(numPartie > 2) numPartie = 0;
    setNumero(numPartie);


    rectPointeurHero.x = 30;
    rectPointeurHero.y = 70 + 34*getNumero();
    rectPointeurHero.w = 16;
    rectPointeurHero.h = 16;

    // Affichage de la carte
    mapMenu.afficherCarte(screenMenuSuppressionPartie,tilesetA,tilesetB,tilesetC,tilesetD,mapMenu.getTableauSol());
    mapMenu.afficherCarte(screenMenuSuppressionPartie,tilesetA,tilesetB,tilesetC,tilesetD,mapMenu.getTableauTable());

    // Afficher les informations de chaque parties
    for(i=0;i<3;i++)
    {
        // Coeurs
        for(j=0;j<nbCoeursPartie[i];j++)
        {
            SDL_BlitSurface(imageCoeur,&rectCoeur[2],screenMenuSuppressionPartie,&rectPositionCoeur[i][j]);
        }
    }

    // Afficher le texte dans ce menu
    texteMenu.afficherTexteSelectionPartie(screenMenuSuppressionPartie,nomJoueur[0],nomJoueur[1],nomJoueur[2],"Effacer","","");

    // Afficher le pointeur de partie (personnage s'affichant devant la partie sélectionnée)
    SDL_BlitSurface(pointeurHero,&rectPointeur[1],screenMenuSuppressionPartie,&rectPointeurHero);

    if(validation)
    {
        switch (numPartie)
        {
            case 0:
            {
                nomFichier += "0.txt";
                break;
            }
            case 1:
            {
                nomFichier += "1.txt";
                break;
            }
            case 2:
            {
                nomFichier += "2.txt";
                break;
            }
            default:
                break;
        }

        // Permet de ne supprimer que les partie existant réellement
        if(nomJoueur[numPartie] != "") remove(nomFichier.c_str());

        // Initialisation du nombre de cœurs sur chaque partie ; Ces valeurs sont lues dans chaque sauvegarde.
        for(i=0;i<3;i++)
        {
            nomJoueur[i] = "";
            nbCoeursPartie[i] = 0;
            lectureSauvegardes(&nbCoeursPartie[i],&nomJoueur[i],i);
        }
    }

    if(sortieMenu)
    {
        setNumero(3); // Pour se repositionner correctement
        setMenuSuppressionPartie(false);
        mapMenu.selectionCarte(0);
    }
}

void SelectionPartie::lectureSauvegardes(int* nbCoeurs, string* nomJ, int numPartie)
{
    std::string nomFichier = "sauvegardes/sauvegarde";

    switch (numPartie)
    {
        case 0:
        {
            nomFichier += "0.txt";
            break;
        }
        case 1:
        {
            nomFichier += "1.txt";
            break;
        }
        case 2:
        {
            nomFichier += "2.txt";
            break;
        }
        default:
            break;
    }

    std::ifstream fichier(nomFichier.c_str());

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        std::string ligne; // variable contenant chaque ligne lue

        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( std::getline( fichier, ligne ) )
        {
            // Tester si l'une des lignes contient la séquence "coeur " (espace compris)
            if(ligne.find("nom_joueur ") == 0)
            {
                // Si c'est le cas, éliminer cette séquence de la ligne et convertir ce qui reste en int
                ligne.replace(0,11,"");

                // Attribuer cette valeur à *nomJ
                stringstream flux(ligne);
                flux >> *nomJ;

            }

            // Tester si l'une des lignes contient la séquence "coeur " (espace compris)
            if(ligne.find("coeur ") == 0)
            {
                // Si c'est le cas, éliminer cette séquence de la ligne et convertir ce qui reste en int
                ligne.replace(0,6,"");

                // Attribuer cette valeur à *nbCoeurs
                stringstream flux(ligne);
                flux >> *nbCoeurs;

            }

            // afficher la ligne à l'écran
            //std::cout << ligne << std::endl;
        }
    }

    fichier.close();

    // Protection sur *nbCoeurs (utile en cas de modification manuelle des fichiers de sauvegarde)
    if(*nbCoeurs > 20) *nbCoeurs = 20;

}

void SelectionPartie::initSauvegarde()
{
    string ligne = ""; // variable contenant chaque ligne lue

    // Détermination du bon fichier pour mémoriser la sauvegarde
    string nomFichierDeBase = "sauvegardes/sauvegardeDeBase.txt";
    string nomFichier = "sauvegardes/sauvegarde";
    switch (getNumero())
    {
        case 0:
        {
            nomFichier += "0.txt";
            break;
        }
        case 1:
        {
            nomFichier += "1.txt";
            break;
        }
        default:
        {
            nomFichier += "2.txt";
            break;
        }
    }

    // Traitement du fichier de sauvegarde et extraction des données
    ofstream fichier(nomFichier.c_str());
    ifstream fichierDeBase(nomFichierDeBase.c_str());

    if ( fichier && fichierDeBase ) // ce test échoue si le fichier n'est pas ouvert
    {
        // Ecriture du minimum de données à sauvegarder pour démarrer une partie normalement.
        fichier << "nom_joueur " << nomJoueur[getNumero()] << "\n" ;
        fichier << "coeur 3" << "\n" ;
        fichier << "magie 0" << "\n" ;
        fichier << "jaugeMagieDoublee 0" << "\n" ;

        fichier << "nbRubis 0" << "\n" ;
        fichier << "nbBombes 0" << "\n" ;
        fichier << "nbFleches 0" << "\n" ;

        fichier << "quete1 0" << "\n" ;
        fichier << "quete2 0" << "\n" ;
        fichier << "num epee 1" << "\n" ;

        fichier << "num arc 1" << "\n" ;
        fichier << "grappin 0" << "\n" ;
        fichier << "bombe 1" << "\n" ;
        fichier << "boomerang 0" << "\n" ;

        fichier << "feu 0" << "\n" ;
        fichier << "glace 1" << "\n" ;
        fichier << "lampe 1" << "\n" ;
        fichier << "flute 0" << "\n" ;

        fichier << "marteau 0" << "\n" ;
        fichier << "palme 0" << "\n" ;
        fichier << "num gant 0" << "\n" ;
        fichier << "filet papillon 0" << "\n" ;

        fichier << "flacon 1 0" << "\n" ;
        fichier << "flacon 2 0" << "\n" ;
        fichier << "flacon 3 0" << "\n" ;
        fichier << "livre 0" << "\n\n" ;

        // Chargement de la sauvegarde de base du jeu
        while ( std::getline( fichierDeBase, ligne ) )
        {
            fichier << ligne << endl;
        }

    }

    fichier.close();
    fichierDeBase.close();

}

void SelectionPartie::setPositionXCurseurLettre(int PXCL) {positionXCurseurLettre = PXCL;}
int SelectionPartie::getPositionXCurseurLettre() {return positionXCurseurLettre;}

void SelectionPartie::setPositionYCurseurLettre(int PYCL) {positionYCurseurLettre = PYCL;}
int SelectionPartie::getPositionYCurseurLettre() {return positionYCurseurLettre;}
