/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Texte.h"

Texte::Texte()
{
    int i,j;

    // sélection partie (+ création et suppression partie)
    for(i=0;i<8;i++)
    {
        // pour placer le nom des joueurs mémorisés dans les sauvegardes
        for(j=0;j<3;j++)
        {
            rectTexteSelectionPartie[i][j].x = 48 + 8*i;
            rectTexteSelectionPartie[i][j].y = 68 + 34*j;
            rectTexteSelectionPartie[i][j].w = 8;
            rectTexteSelectionPartie[i][j].h = 14;
        }

        // Nom du menu de sélection de partie
        rectTexteSelectionPartie[i][3].x = 48 + 8*i;
        rectTexteSelectionPartie[i][3].y = 16;
        rectTexteSelectionPartie[i][3].w = 8;
        rectTexteSelectionPartie[i][3].h = 14;

        // Nouvelle partie
        rectTexteSelectionPartie[i][4].x = 48 + 8*i;
        rectTexteSelectionPartie[i][4].y = 200;
        rectTexteSelectionPartie[i][4].w = 8;
        rectTexteSelectionPartie[i][4].h = 14;

        // Effacer partie
        rectTexteSelectionPartie[i][5].x = 208 + 8*i;
        rectTexteSelectionPartie[i][5].y = 200;
        rectTexteSelectionPartie[i][5].w = 8;
        rectTexteSelectionPartie[i][5].h = 14;

    }

    // Spécifique au menu de création d'une nouvelle partie
    for(i=0;i<13;i++)
    {
        // pour placer le nom des joueurs mémorisés dans les sauvegardes
        for(j=0;j<6;j++)
        {
            rectTexteCreationNouvellePartie[i][j].x = 30 + 20*i;
            rectTexteCreationNouvellePartie[i][j].y = 50 + 22*j;
            rectTexteCreationNouvellePartie[i][j].w = 8;
            rectTexteCreationNouvellePartie[i][j].h = 14;
        }
    }

    // déroulement partie
    for(i=0;i<26;i++)
    {
        for(j=0;j<6;j++)
        {
            rectTexte[i][j].x = 52 + 8*i;
            rectTexte[i][j].y = 130 + 16*j;
            rectTexte[i][j].w = 8;
            rectTexte[i][j].h = 14;
        }
    }

    // Initialisation du SDL_Rect spécifique au texte
    initSDLSurface(6);
    initSDLRect(6);

    setFinAffichageTexte(true);
    setFinAffichagePage(false);
    page = 0;
    compteurAffichageLettreParLettre = 0;

}

Texte::~Texte()
{
    freeMem(6);
}

void Texte::setNumTexte(int NT) {numTexte = NT;}
int Texte::getNumTexte() {return numTexte;}

void Texte::setPageSuivante()
{
    compteurAffichageLettreParLettre = 0;
    if(!getFinAffichageTexte())
    {
        page++;
    }
    else
    {
        page = 0;
        //setNumTexte(-1);
    }
}

int Texte::getPage() {return page;}

void Texte::setFinAffichageTexte(bool FAT)
{
    finAffichageTexte = FAT;
    if(finAffichageTexte)
    {
        page = 0;
        compteurAffichageLettreParLettre = 0;
        setAutorisationAffichageTexte(false);
    }
}
bool Texte::getFinAffichageTexte() {return finAffichageTexte;}

void Texte::setAutorisationAffichageTexte(bool AAT) {autorisationAffichageTexte = AAT;}
bool Texte::getAutorisationAffichageTexte() {return autorisationAffichageTexte;}

void Texte::setFinAffichagePage(bool FAP)
{
    finAffichagePage = FAP;
    if(finAffichagePage) compteurAffichageLettreParLettre = 288;
}
bool Texte::getFinAffichagePage() {return finAffichagePage;}

void Texte::afficherTexte(SDL_Surface* screenTexte)
{
    unsigned int i,j,k;
    string pancarte;
    int tempo = 2; // gère la vitesse de défilement du texte

    // UNIQUEMENT POUR TEST
    /*cout << "compteurAffichageLettreParLettre : " << compteurAffichageLettreParLettre << endl;
    cout << "getFinAffichageTexte() : " << getFinAffichageTexte() << endl;
    cout << "getFinAffichagePage() : " << getFinAffichagePage() << endl;
    cout << "getNumTexte() : " << getNumTexte() << endl;
    cout << "getPage() : " << getPage() << endl;
    cout << "getAutorisationAffichageTexte() : " << getAutorisationAffichageTexte() << endl;
    cout << "getNarratif() : " << getNarratif() << endl;*/

    if(getNumTexte() == -1) {compteurAffichageLettreParLettre = 0;} // Initialisation du compteur lorsqu'il n'y a pas de texte à afficher
    else // Traitement lorsqu'il y a un texte à afficher
    {
        compteurAffichageLettreParLettre++;
        if(!(compteurAffichageLettreParLettre < 96*tempo)) setFinAffichagePage(true);

        // Gestion du déroulement de l'affichage du texte en fonction du type de texte (naratif ou non) ainsi que des actions du joueur lors de l'affichage du texte
        if(getTexte(getNumTexte()).size() < abs(getPage()*96)) // S'il ne faut plus afficher le texte
        {
            setFinAffichageTexte(true);
            compteurAffichageLettreParLettre = 0; // Initialisation du compteur lorsqu'il n'y a pas de texte à afficher
        }
        else if(!getNarratif()) // Affichage directement complet de la page si elle n'est pas narrative
        {
            pancarte = getTexte(getNumTexte());
            setFinAffichagePage(true);
        }
        else if(getFinAffichagePage()) // Affichage complet de la page si elle est narrative et si la fin de page est demandée
        {
            pancarte = getTexte(getNumTexte()).substr(getPage()*96,97);
        }
        else
        {
            pancarte = getTexte(getNumTexte()).substr(getPage()*96,(compteurAffichageLettreParLettre/tempo)%97);
            if(pancarte == getTexte(getNumTexte()).substr(getPage()*96,97)) setFinAffichagePage(true); // Permet un retour à la ligne sans attendre que le compteur soit arrivé au 120ème caractère
        }


        // Affichage à l'écran
        for(j=1;j<5;j++)
        {
            for(i=1;i<25;i++)
            {
                k = (i-1 + (j-1)*24);
                gestionLettres(screenTexte,pancarte,rectTexte[i][j],k);
            }
        }
    }
}

void Texte::afficherTexteSelectionPartie(SDL_Surface* screen, string nom1, string nom2, string nom3, string nomMenu, string nouveau, string effacer)
{
    int i;

    for(i=0;i<8;i++)
    {
        gestionLettres(screen,nom1,rectTexteSelectionPartie[i][0],i);
        gestionLettres(screen,nom2,rectTexteSelectionPartie[i][1],i);
        gestionLettres(screen,nom3,rectTexteSelectionPartie[i][2],i);
        gestionLettres(screen,nomMenu,rectTexteSelectionPartie[i][3],i);
        gestionLettres(screen,nouveau,rectTexteSelectionPartie[i][4],i);
        gestionLettres(screen,effacer,rectTexteSelectionPartie[i][5],i);
    }
}

void Texte::afficherTexteCreationNouvellePartie(SDL_Surface* screen, string nomMenu, string nomJoueur, int X, int Y)
{
    int i;

    // Initialisation curseur de sélection lettre (en 4 parties)
    SDL_Rect curseur[4];
    curseur[0] = getSDLRect(29,0,0);
    curseur[1] = getSDLRect(29,2,0);
    curseur[2] = getSDLRect(29,0,2);
    curseur[3] = getSDLRect(29,2,2);

    // Initialisation position du curseur à l'écran
    SDL_Rect positionCurseur[4];
    for(i=0;i<4;i++) positionCurseur[i] = rectTexteCreationNouvellePartie[X][Y];
    positionCurseur[0].x -= 3;
    positionCurseur[0].y -= 5;
    positionCurseur[1].x += 11;
    positionCurseur[1].y -= 5;
    positionCurseur[2].x -= 3;
    positionCurseur[2].y += 13;
    positionCurseur[3].x += 11;
    positionCurseur[3].y += 13;

    // Affichage du nom du menu
    for(i=0;i<8;i++) gestionLettres(screen,nomMenu,rectTexteSelectionPartie[i][3],i);

    // Affichage du nom du joueur
    for(i=0;i<8;i++) gestionLettres(screen,nomJoueur,rectTexteSelectionPartie[i][5],i);

    // Affichage des lettres unes à unes
    gestionLettres(screen,"A",rectTexteCreationNouvellePartie[0][0],0);
    gestionLettres(screen,"B",rectTexteCreationNouvellePartie[1][0],0);
    gestionLettres(screen,"C",rectTexteCreationNouvellePartie[2][0],0);
    gestionLettres(screen,"D",rectTexteCreationNouvellePartie[3][0],0);
    gestionLettres(screen,"E",rectTexteCreationNouvellePartie[4][0],0);
    gestionLettres(screen,"F",rectTexteCreationNouvellePartie[5][0],0);
    gestionLettres(screen,"G",rectTexteCreationNouvellePartie[6][0],0);
    gestionLettres(screen,"H",rectTexteCreationNouvellePartie[7][0],0);
    gestionLettres(screen,"I",rectTexteCreationNouvellePartie[8][0],0);
    gestionLettres(screen,"J",rectTexteCreationNouvellePartie[9][0],0);
    gestionLettres(screen,"K",rectTexteCreationNouvellePartie[10][0],0);
    gestionLettres(screen,"L",rectTexteCreationNouvellePartie[11][0],0);
    gestionLettres(screen,"M",rectTexteCreationNouvellePartie[12][0],0);

    gestionLettres(screen,"N",rectTexteCreationNouvellePartie[0][1],0);
    gestionLettres(screen,"O",rectTexteCreationNouvellePartie[1][1],0);
    gestionLettres(screen,"P",rectTexteCreationNouvellePartie[2][1],0);
    gestionLettres(screen,"Q",rectTexteCreationNouvellePartie[3][1],0);
    gestionLettres(screen,"R",rectTexteCreationNouvellePartie[4][1],0);
    gestionLettres(screen,"S",rectTexteCreationNouvellePartie[5][1],0);
    gestionLettres(screen,"T",rectTexteCreationNouvellePartie[6][1],0);
    gestionLettres(screen,"U",rectTexteCreationNouvellePartie[7][1],0);
    gestionLettres(screen,"V",rectTexteCreationNouvellePartie[8][1],0);
    gestionLettres(screen,"W",rectTexteCreationNouvellePartie[9][1],0);
    gestionLettres(screen,"X",rectTexteCreationNouvellePartie[10][1],0);
    gestionLettres(screen,"Y",rectTexteCreationNouvellePartie[11][1],0);
    gestionLettres(screen,"Z",rectTexteCreationNouvellePartie[12][1],0);

    gestionLettres(screen,"a",rectTexteCreationNouvellePartie[0][2],0);
    gestionLettres(screen,"b",rectTexteCreationNouvellePartie[1][2],0);
    gestionLettres(screen,"c",rectTexteCreationNouvellePartie[2][2],0);
    gestionLettres(screen,"d",rectTexteCreationNouvellePartie[3][2],0);
    gestionLettres(screen,"e",rectTexteCreationNouvellePartie[4][2],0);
    gestionLettres(screen,"f",rectTexteCreationNouvellePartie[5][2],0);
    gestionLettres(screen,"g",rectTexteCreationNouvellePartie[6][2],0);
    gestionLettres(screen,"h",rectTexteCreationNouvellePartie[7][2],0);
    gestionLettres(screen,"i",rectTexteCreationNouvellePartie[8][2],0);
    gestionLettres(screen,"j",rectTexteCreationNouvellePartie[9][2],0);
    gestionLettres(screen,"k",rectTexteCreationNouvellePartie[10][2],0);
    gestionLettres(screen,"l",rectTexteCreationNouvellePartie[11][2],0);
    gestionLettres(screen,"m",rectTexteCreationNouvellePartie[12][2],0);

    gestionLettres(screen,"n",rectTexteCreationNouvellePartie[0][3],0);
    gestionLettres(screen,"o",rectTexteCreationNouvellePartie[1][3],0);
    gestionLettres(screen,"p",rectTexteCreationNouvellePartie[2][3],0);
    gestionLettres(screen,"q",rectTexteCreationNouvellePartie[3][3],0);
    gestionLettres(screen,"r",rectTexteCreationNouvellePartie[4][3],0);
    gestionLettres(screen,"s",rectTexteCreationNouvellePartie[5][3],0);
    gestionLettres(screen,"t",rectTexteCreationNouvellePartie[6][3],0);
    gestionLettres(screen,"u",rectTexteCreationNouvellePartie[7][3],0);
    gestionLettres(screen,"v",rectTexteCreationNouvellePartie[8][3],0);
    gestionLettres(screen,"w",rectTexteCreationNouvellePartie[9][3],0);
    gestionLettres(screen,"x",rectTexteCreationNouvellePartie[10][3],0);
    gestionLettres(screen,"y",rectTexteCreationNouvellePartie[11][3],0);
    gestionLettres(screen,"z",rectTexteCreationNouvellePartie[12][3],0);

    gestionLettres(screen,"0",rectTexteCreationNouvellePartie[0][4],0);
    gestionLettres(screen,"1",rectTexteCreationNouvellePartie[1][4],0);
    gestionLettres(screen,"2",rectTexteCreationNouvellePartie[2][4],0);
    gestionLettres(screen,"3",rectTexteCreationNouvellePartie[3][4],0);
    gestionLettres(screen,"4",rectTexteCreationNouvellePartie[4][4],0);
    gestionLettres(screen,"5",rectTexteCreationNouvellePartie[5][4],0);
    gestionLettres(screen,"6",rectTexteCreationNouvellePartie[6][4],0);
    gestionLettres(screen,"7",rectTexteCreationNouvellePartie[7][4],0);
    gestionLettres(screen,"8",rectTexteCreationNouvellePartie[8][4],0);
    gestionLettres(screen,"9",rectTexteCreationNouvellePartie[9][4],0);
    gestionLettres(screen,"-",rectTexteCreationNouvellePartie[10][4],0);
    gestionLettres(screen,".",rectTexteCreationNouvellePartie[11][4],0);
    gestionLettres(screen,",",rectTexteCreationNouvellePartie[12][4],0);

    gestionLettres(screen,"!",rectTexteCreationNouvellePartie[0][5],0);
    gestionLettres(screen,"?",rectTexteCreationNouvellePartie[1][5],0);
    gestionLettres(screen,"(",rectTexteCreationNouvellePartie[2][5],0);
    gestionLettres(screen,")",rectTexteCreationNouvellePartie[3][5],0);
    gestionLettres(screen,"'",rectTexteCreationNouvellePartie[4][5],0);
    gestionLettres(screen,":",rectTexteCreationNouvellePartie[5][5],0);
    gestionLettres(screen," ",rectTexteCreationNouvellePartie[6][5],0);
    gestionLettres(screen,"~",rectTexteCreationNouvellePartie[7][5],0);
    gestionLettres(screen,"/",rectTexteCreationNouvellePartie[8][5],0);
    gestionLettres(screen,"£",rectTexteCreationNouvellePartie[9][5],0);
    gestionLettres(screen,"$",rectTexteCreationNouvellePartie[10][5],0);

    gestionLettres(screen,"Ö",rectTexteCreationNouvellePartie[12][5],0);

    // Affichage du curseur
    for(i=0;i<4;i++) SDL_BlitSurface(font,&curseur[i],screen,&positionCurseur[i]);

}


void Texte::gestionLettres(SDL_Surface* screen, string Txt, SDL_Rect rect,int k)
{
    SDL_Rect tmp;
    tmp.x = 0;
    tmp.y = 0;
    tmp.w = 0;
    tmp.h = 0;

    if(Txt.find("A",k) == abs(k)) tmp = getSDLRect(28,0,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("B",k) == abs(k)) tmp = getSDLRect(28,1,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("C",k) == abs(k)) tmp = getSDLRect(28,2,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("D",k) == abs(k)) tmp = getSDLRect(28,3,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("E",k) == abs(k)) tmp = getSDLRect(28,4,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("F",k) == abs(k)) tmp = getSDLRect(28,5,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("G",k) == abs(k)) tmp = getSDLRect(28,6,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("H",k) == abs(k)) tmp = getSDLRect(28,7,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("I",k) == abs(k)) tmp = getSDLRect(28,8,0); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("J",k) == abs(k)) tmp = getSDLRect(28,9,0); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("K",k) == abs(k)) tmp = getSDLRect(28,0,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("L",k) == abs(k)) tmp = getSDLRect(28,1,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("M",k) == abs(k)) tmp = getSDLRect(28,2,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("N",k) == abs(k)) tmp = getSDLRect(28,3,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("O",k) == abs(k)) tmp = getSDLRect(28,4,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("P",k) == abs(k)) tmp = getSDLRect(28,5,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("Q",k) == abs(k)) tmp = getSDLRect(28,6,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("R",k) == abs(k)) tmp = getSDLRect(28,7,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("S",k) == abs(k)) tmp = getSDLRect(28,8,1); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("T",k) == abs(k)) tmp = getSDLRect(28,9,1); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("U",k) == abs(k)) tmp = getSDLRect(28,0,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("V",k) == abs(k)) tmp = getSDLRect(28,1,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("W",k) == abs(k)) tmp = getSDLRect(28,2,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("X",k) == abs(k)) tmp = getSDLRect(28,3,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("Y",k) == abs(k)) tmp = getSDLRect(28,4,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("Z",k) == abs(k)) tmp = getSDLRect(28,5,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("-",k) == abs(k)) tmp = getSDLRect(28,6,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find(".",k) == abs(k)) tmp = getSDLRect(28,7,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find(",",k) == abs(k)) tmp = getSDLRect(28,8,2); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ç",k) == abs(k)) tmp = getSDLRect(28,9,2); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("a",k) == abs(k)) tmp = getSDLRect(28,0,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("b",k) == abs(k)) tmp = getSDLRect(28,1,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("c",k) == abs(k)) tmp = getSDLRect(28,2,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("d",k) == abs(k)) tmp = getSDLRect(28,3,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("e",k) == abs(k)) tmp = getSDLRect(28,4,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("f",k) == abs(k)) tmp = getSDLRect(28,5,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("g",k) == abs(k)) tmp = getSDLRect(28,6,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("h",k) == abs(k)) tmp = getSDLRect(28,7,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("i",k) == abs(k)) tmp = getSDLRect(28,8,3); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("j",k) == abs(k)) tmp = getSDLRect(28,9,3); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("k",k) == abs(k)) tmp = getSDLRect(28,0,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("l",k) == abs(k)) tmp = getSDLRect(28,1,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("m",k) == abs(k)) tmp = getSDLRect(28,2,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("n",k) == abs(k)) tmp = getSDLRect(28,3,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("o",k) == abs(k)) tmp = getSDLRect(28,4,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("p",k) == abs(k)) tmp = getSDLRect(28,5,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("q",k) == abs(k)) tmp = getSDLRect(28,6,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("r",k) == abs(k)) tmp = getSDLRect(28,7,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("s",k) == abs(k)) tmp = getSDLRect(28,8,4); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("t",k) == abs(k)) tmp = getSDLRect(28,9,4); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("u",k) == abs(k)) tmp = getSDLRect(28,0,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("v",k) == abs(k)) tmp = getSDLRect(28,1,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("w",k) == abs(k)) tmp = getSDLRect(28,2,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("x",k) == abs(k)) tmp = getSDLRect(28,3,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("y",k) == abs(k)) tmp = getSDLRect(28,4,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("z",k) == abs(k)) tmp = getSDLRect(28,5,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("é",k) == abs(k)) tmp = getSDLRect(28,6,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ê",k) == abs(k)) tmp = getSDLRect(28,7,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("è",k) == abs(k)) tmp = getSDLRect(28,8,5); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("à",k) == abs(k)) tmp = getSDLRect(28,9,5); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("0",k) == abs(k)) tmp = getSDLRect(28,0,6); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("1",k) == abs(k)) tmp = getSDLRect(28,1,6); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("2",k) == abs(k)) tmp = getSDLRect(28,2,6); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("3",k) == abs(k)) tmp = getSDLRect(28,3,6); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("4",k) == abs(k)) tmp = getSDLRect(28,4,6); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("û",k) == abs(k)) tmp = getSDLRect(28,5,6); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("â",k) == abs(k)) tmp = getSDLRect(28,9,6); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("5",k) == abs(k)) tmp = getSDLRect(28,0,7); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("6",k) == abs(k)) tmp = getSDLRect(28,1,7); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("7",k) == abs(k)) tmp = getSDLRect(28,2,7); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("8",k) == abs(k)) tmp = getSDLRect(28,3,7); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("9",k) == abs(k)) tmp = getSDLRect(28,4,7); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("î",k) == abs(k)) tmp = getSDLRect(28,5,7); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ô",k) == abs(k)) tmp = getSDLRect(28,9,7); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("!",k) == abs(k)) tmp = getSDLRect(28,0,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("?",k) == abs(k)) tmp = getSDLRect(28,1,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("(",k) == abs(k)) tmp = getSDLRect(28,2,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find(")",k) == abs(k)) tmp = getSDLRect(28,3,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("'",k) == abs(k)) tmp = getSDLRect(28,4,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find(":",k) == abs(k)) tmp = getSDLRect(28,5,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ẍ",k) == abs(k)) tmp = getSDLRect(28,6,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ü",k) == abs(k)) tmp = getSDLRect(28,7,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("Ö",k) == abs(k)) tmp = getSDLRect(28,8,8); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ä",k) == abs(k)) tmp = getSDLRect(28,9,8); SDL_BlitSurface(font,&tmp,screen,&rect); // OK

    if(Txt.find("~",k) == abs(k)) tmp = getSDLRect(28,0,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("£",k) == abs(k)) tmp = getSDLRect(28,1,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("$",k) == abs(k)) tmp = getSDLRect(28,2,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("/",k) == abs(k)) tmp = getSDLRect(28,3,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find(" ",k) == abs(k)) tmp = getSDLRect(28,4,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find(">",k) == abs(k)) tmp = getSDLRect(28,5,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("<",k) == abs(k)) tmp = getSDLRect(28,6,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ï",k) == abs(k)) tmp = getSDLRect(28,7,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ë",k) == abs(k)) tmp = getSDLRect(28,8,9); SDL_BlitSurface(font,&tmp,screen,&rect);
    if(Txt.find("ù",k) == abs(k)) tmp = getSDLRect(28,9,9); SDL_BlitSurface(font,&tmp,screen,&rect); // OK


}

void Texte::setNarratif(bool N) {narratif = N;}
bool Texte::getNarratif() {return narratif;}

string Texte::getTexte(int NT)
{
    string texte = "";
    int narratif = 0;

    // créer un flux de sortie, écrire le numéro de carte passé en paramètre de la fonction dans ce flux
    std::ostringstream oss;
    oss << NT;

    // Traitement du fichier de sauvegarde et extraction des données
    ifstream fichier( FICHIER_DIALOGUES );

    if ( fichier ) // ce test échoue si le fichier n'est pas ouvert
    {
        string ligne; // variable contenant chaque ligne lue
        string mot; // variable contenant tout à tour les différentes infoscontenues dans chaque ligne
        vector<string> infosUnitaires; // Vecteur regroupant l'ensemble des informations contenues dans chaque ligne


        // cette boucle s'arrête dès qu'une erreur de lecture survient
        while ( getline( fichier, ligne ) )
        {
            infosUnitaires.clear();

            // Tester si l'une des lignes contient la séquence "ennemi " (espace compris)
            if(ligne.find( oss.str() + "_") == 0)
            {
                // Transformer la ligne en un vector contenant chaque info
                istringstream iss( ligne );
                while ( getline( iss, mot, '_' ) )
                {
                    infosUnitaires.push_back(mot);
                }

                // Attribuer ces informations à l'ennemi
                stringstream flux1(infosUnitaires.at(1));
                flux1 >> narratif;

                setNarratif(narratif);
                texte = infosUnitaires.at(2);
            }
        }
    }

    return texte;
}
