/*
    This program is a clone of Zelda 3
    Copyright (C) 2012  Frédéric Micout - fred@sujets-libres.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef POSITION_H_INCLUDED
#define POSITION_H_INCLUDED

// On place ici toutes les constantes du programme
#define LARGEUR_TILE 16
#define HAUTEUR_TILE 16
#define LARGEUR_PERSO 16
#define HAUTEUR_PERSO 24
#define LARGEUR_ECRAN 320
#define HAUTEUR_ECRAN 240
#define M_PI  3.14159265358979323846
#define TEMPS_INTOUCHABLE 50 // Nombre de boucles du programme durant lesquelles le joueur ou le PNJ est intouchable
#define TEMPS_RECUL 15 // Nombre de boucles du programme durant lesquelles le joueur ou le PNJ recule rapidement
#define TEMPS_IMMOBILISATION 45 // Nombre de boucles du programme durant lesquelles le PNJ ne peut plus bouger
#define TEMPS_ANIMATION_ARME 3
#define FICHIER_SAUVEGARDE_TMP "sauvegardes/sauvegardeTMP.txt"
#define FICHIER_REFTILESET "sauvegardes/refTileset.txt"
#define FICHIER_CORRESPONDANCE_REFTILESET "images/motifs/correspondance.bmp"
#define FICHIER_DIALOGUES "sauvegardes/dialogues.txt"
#define CARTE_DEPART 2000

#include <iostream>
#include <string>
#include <fstream>
#include <sstream>
#include <list>
#include <vector>
#include <iterator>
#include <typeinfo>
#include <algorithm>
#include <cmath>

// librairies externes
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_rotozoom.h>

using namespace std;

// Une position stocke les informations relatives à chaque tuile affichées à l'écran
struct position
{
    int x;
    int y;

    /*
    Permet de contrôler les animations non prévisibles sur chaque tile

    USAGE :
    La valeur 0 est à employer lorsqu'il n'y a aucune animation
    Passer cette valeur à 1 (ou une autre valeur différente de 0) lorsqu'il y a une animation à lancer sur une tile
    A la fin de l'animation, repasser cette valeur à 0
    */
    int compteurTile;

    /*
    Chaque bloc est divisé en quatre parties
    Chaque partie peut être traversable ou non - Il y a donc 16 combinaisons possible sur chaque case

    USAGE :
    partie 1 = d: en haut à gauche
    partie 2 = c: en bas à gauche
    partie 3 = b: en bas à droite
    partie 4 = a: en haut à droite

    si coeff a,b,c ou d à 0, partie traversable
    si coeff a,b,c ou d à 1, partie non traversable

    typeBlocTraversable = a*2^3 + b*2^2 + c*2^1 + d*2^0
    */
    int typeBlocTraversable;

    /*
    La nature du sol peut engendrer des contraintes supplémentaires au sujet des déplacements du perso.
    La nature du sol peut aussi peut aussi empêcher certains ennemmis d'avancer.

    USAGE :
    La valeur 0 est à utiliser par défaut, y compris pour les tiles ne représentant pas le sol.
    La valeur 1 est à utiliser pour les tiles d'eau. Les tiles à la fois d'eau et normales doivent être du type par défaut.
    */
    int typeSol;
};

// Structure permettant de définir des liens unidirectionnels entre les cartes
struct lienCarte
{
    int positionDepartX0; // Position la plus à gauche sur X au départ du lien
    int positionDepartX1; // Position la plus à droite sur X au départ du lien
    int positionDepartY0; // Position la plus haute sur y au départ du lien
    int positionDepartY1; // Position la plus basse sur y au départ du lien
    int decalageDepartX0; // Décalage le plus à gauche sur X au départ du lien
    int decalageDepartX1; // Décalage le plus à droite sur X au départ du lien
    int decalageDepartY0; // Décalage le plus haut sur Y au départ du lien
    int decalageDepartY1; // Décalage le plus bas sur Y au départ du lien

    int carteArrive; // Carte où le lien mène
    int positionArriveX; // Position la plus à gauche sur X à l'arrivée du lien
    int positionArriveY; // Position la plus haute sur y à l'arrivée du lien
    int decalageArriveX; // Décalage le plus à gauche sur X à l'arrivée du lien
    int decalageArriveY; // Décalage le plus haut sur Y à l'arrivée du lien
};

// Permet d'afficher chaque pièce correctement sur le plan du donjon
struct blocDonjon
{
    /*
    Numéro de la carte associée à ce bloc de donjon
    (une carte peut être associée à plusieurs blocs du donjon car une pièce du donjon peut couvrir plusieurs blocs)
    */
    int numCarte;

    /*
    Miniature représentant la surface au sol du bloc de donjon
    */
    int miniature;
};

#endif // POSITION_H_INCLUDED
